package com.glxp.api.admin.controller.basic;

import com.github.pagehelper.PageInfo;
import com.glxp.api.admin.annotation.AuthRuleAnnotation;
import com.glxp.api.admin.constant.BussinessTypeUtils;
import com.glxp.api.admin.entity.basic.BussinessTypeEntity;
import com.glxp.api.admin.req.inout.DeleteRequest;
import com.glxp.api.admin.req.receipt.BussinessTypeFilterRequest;
import com.glxp.api.admin.res.PageSimpleResponse;
import com.glxp.api.admin.res.basic.BasicThirdSysResponse;
import com.glxp.api.admin.res.basic.BussinessTypResponse;
import com.glxp.api.admin.res.basic.SrBtResponse;
import com.glxp.api.admin.service.basic.BussinessTypeService;
import com.glxp.api.common.enums.ResultEnum;
import com.glxp.api.common.res.BaseResponse;
import com.glxp.api.common.util.ResultVOUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.validation.BindingResult;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RestController;

import javax.annotation.Resource;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * 业务类型维护
 */

@RestController
public class BussinessTypeController {


    @Resource
    private BussinessTypeService bussinessTypeService;


    @GetMapping("/udiwms/bussinessType/filter")
    public BaseResponse filterBussinessType(BussinessTypeFilterRequest bussinessTypeFilterRequest,
                                            BindingResult bindingResult) {

        if (bindingResult.hasErrors()) {
            return ResultVOUtils.error(ResultEnum.PARAM_VERIFY_FALL, bindingResult.getFieldError().getDefaultMessage());
        }
        List<BussinessTypeEntity> bussinessTypeEntities;
        bussinessTypeEntities = bussinessTypeService.filterList(bussinessTypeFilterRequest);
        PageInfo<BussinessTypeEntity> pageInfo;
        pageInfo = new PageInfo<>(bussinessTypeEntities);
        PageSimpleResponse<BussinessTypeEntity> pageSimpleResponse = new PageSimpleResponse<>();
        pageSimpleResponse.setTotal(pageInfo.getTotal());
        pageSimpleResponse.setList(bussinessTypeEntities);
        return ResultVOUtils.success(pageSimpleResponse);
    }

    @GetMapping("/udiwms/bussinessType/joinFilter")
    public BaseResponse filterJoinBussinessType(BussinessTypeFilterRequest bussinessTypeFilterRequest,
                                            BindingResult bindingResult) {

        if (bindingResult.hasErrors()) {
            return ResultVOUtils.error(ResultEnum.PARAM_VERIFY_FALL, bindingResult.getFieldError().getDefaultMessage());
        }
        List<BussinessTypResponse> bussinessTypeEntities;
        bussinessTypeEntities = bussinessTypeService.filterJoinList(bussinessTypeFilterRequest);
        PageInfo<BussinessTypResponse> pageInfo;
        pageInfo = new PageInfo<>(bussinessTypeEntities);
        PageSimpleResponse<BussinessTypResponse> pageSimpleResponse = new PageSimpleResponse<>();
        pageSimpleResponse.setTotal(pageInfo.getTotal());
        pageSimpleResponse.setList(bussinessTypeEntities);
        return ResultVOUtils.success(pageSimpleResponse);
    }

//    @AuthRuleAnnotation("")
    @GetMapping("/udiwms/bussinessType/download")
    public BaseResponse downloadJoinBussinessType(BussinessTypeFilterRequest bussinessTypeFilterRequest,
                                                BindingResult bindingResult) {

        if (bindingResult.hasErrors()) {
            return ResultVOUtils.error(ResultEnum.PARAM_VERIFY_FALL, bindingResult.getFieldError().getDefaultMessage());
        }
        bussinessTypeFilterRequest.setEnabled(true);
        List<BussinessTypResponse> bussinessTypeEntities;
        bussinessTypeEntities = bussinessTypeService.filterJoinList(bussinessTypeFilterRequest);

        List<SrBtResponse> basicThirdSysResponses = bussinessTypeEntities.stream().map(item ->
        {
            SrBtResponse basicThirdSysResponse = new SrBtResponse();
            BeanUtils.copyProperties(item, basicThirdSysResponse);
            return basicThirdSysResponse;
        }).collect(Collectors.toList());

        PageInfo<BussinessTypResponse> pageInfo;
        pageInfo = new PageInfo<>(bussinessTypeEntities);
        PageSimpleResponse<SrBtResponse> pageSimpleResponse = new PageSimpleResponse<>();
        pageSimpleResponse.setTotal(pageInfo.getTotal());
        pageSimpleResponse.setList(basicThirdSysResponses);
        return ResultVOUtils.success(pageSimpleResponse);
    }


    @PostMapping("/udiwms/bussinessType/delete")
    public BaseResponse deleteBussinessType(@RequestBody DeleteRequest deleteRequest, BindingResult bindingResult) {

        if (bindingResult.hasErrors()) {
            return ResultVOUtils.error(ResultEnum.PARAM_VERIFY_FALL, bindingResult.getFieldError().getDefaultMessage());
        }
        String id = deleteRequest.getId();
        bussinessTypeService.deleteById(id);
        return ResultVOUtils.success("删除成功");
    }

    @PostMapping("/udiwms/bussinessType/update")
    public BaseResponse updateBussinessType(@RequestBody BussinessTypeEntity bussinessTypeEntity,
                                            BindingResult bindingResult) {

        if (bindingResult.hasErrors()) {
            return ResultVOUtils.error(ResultEnum.PARAM_VERIFY_FALL, bindingResult.getFieldError().getDefaultMessage());
        }
        if (bussinessTypeEntity != null) {
            bussinessTypeService.updateBussinessType(bussinessTypeEntity);
        } else {
            ResultVOUtils.error(999, "参数错误");
        }
        return ResultVOUtils.success("更新成功");
    }


    @PostMapping("/udiwms/bussinessType/insert")
    public BaseResponse insertBussinessType(@RequestBody BussinessTypeEntity bussinessTypeEntity, BindingResult bindingResult) {

        if (bindingResult.hasErrors()) {
            return ResultVOUtils.error(ResultEnum.PARAM_VERIFY_FALL, bindingResult.getFieldError().getDefaultMessage());
        }
        if (bussinessTypeEntity != null) {
            bussinessTypeService.insertBussinessType(bussinessTypeEntity);
        } else {
            ResultVOUtils.error(999, "参数错误");
        }
        return ResultVOUtils.success("添加成功");
    }

}
