package com.glxp.api.admin.controller.basic;

import com.github.pagehelper.PageInfo;
import com.glxp.api.admin.constant.BasicProcessStatus;
import com.glxp.api.admin.entity.basic.BasicUnitMaintainEntity;
import com.glxp.api.admin.entity.basic.CorpExportLogEntity;
import com.glxp.api.admin.req.basic.*;
import com.glxp.api.admin.req.inout.DeleteRequest;
import com.glxp.api.admin.res.PageSimpleResponse;
import com.glxp.api.admin.service.basic.BasicUnitMaintainService;
import com.glxp.api.admin.service.basic.CorpExportLogService;
import com.glxp.api.admin.thread.BasicGenExcelService;
import com.glxp.api.admin.util.CustomUtil;
import com.glxp.api.common.enums.ResultEnum;
import com.glxp.api.common.res.BaseResponse;
import com.glxp.api.common.util.ResultVOUtils;
import org.springframework.validation.BindingResult;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RestController;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletResponse;
import java.io.File;
import java.io.FileInputStream;
import java.io.OutputStream;
import java.util.*;

@RestController
public class CorpExportLogController {
    @Resource
    BasicGenExcelService basicGenExcelService;
    @Resource
    CorpExportLogService corpExportLogService;
    @Resource
    private BasicUnitMaintainService basicUnitMaintainService;

    @GetMapping("/udiwms/corps/exportLog/filter")
    public BaseResponse filter(FilterUdiEpLogRequest filterUdiEpLogReques,
                               BindingResult bindingResult) {

        if (bindingResult.hasErrors()) {
            return ResultVOUtils.error(ResultEnum.PARAM_VERIFY_FALL, bindingResult.getFieldError().getDefaultMessage());
        }
        List<CorpExportLogEntity> corpExportLogEntities
                = corpExportLogService.filterCorpExportLog(filterUdiEpLogReques);
        PageInfo<CorpExportLogEntity> pageInfo;
        pageInfo = new PageInfo<>(corpExportLogEntities);
        PageSimpleResponse<CorpExportLogEntity> pageSimpleResponse = new PageSimpleResponse<>();
        pageSimpleResponse.setTotal(pageInfo.getTotal());
        pageSimpleResponse.setList(corpExportLogEntities);
        return ResultVOUtils.success(pageSimpleResponse);
    }

    @PostMapping("/udiwms/corps/exportLog/deleteLog")
    public BaseResponse deleteLog(@RequestBody DeleteRequest deleteRequest, BindingResult bindingResult) {

        if (bindingResult.hasErrors()) {
            return ResultVOUtils.error(ResultEnum.PARAM_VERIFY_FALL, bindingResult.getFieldError().getDefaultMessage());
        }
        String id = deleteRequest.getId();
        corpExportLogService.deleteById(id + "");
        return ResultVOUtils.success("删除成功");
    }

    //往来单位excel文件下载
    @GetMapping("/udiwms/corps/exportLog/download")
    public void download(FilterUdiEpLogRequest filterUdiEpLogRequest, HttpServletResponse res) {

        List<CorpExportLogEntity> corpExportLogEntities = corpExportLogService.filterCorpExportLog(filterUdiEpLogRequest);
        CorpExportLogEntity corpExportLogEntity = corpExportLogEntities.get(0);
        try {
            String filePath = corpExportLogEntity.getFilePath();
            String fileName = "temp";
            if (filePath != null && filePath.length() > 0) {
                String[] data = filePath.split("/");
                fileName = data[data.length - 1];
            }
            res.setHeader("Content-disposition", "attachment;fileName=" + fileName);
            res.setContentType("text/plain;charset=UTF-8");
            FileInputStream input = new FileInputStream(new File(filePath));
            OutputStream out = res.getOutputStream();
            byte[] b = new byte[2048];
            int len;
            while ((len = input.read(b)) != -1) {
                out.write(b, 0, len);
            }
            input.close();
            if (corpExportLogEntity.getDlCount() == 0) {
                corpExportLogEntity.setDlCount(1);
            } else {
                corpExportLogEntity.setDlCount(corpExportLogEntity.getDlCount() + 1);
            }
            corpExportLogService.updateCorpExportLog(corpExportLogEntity);
        } catch (Exception ex) {
            System.out.println("下载失败！");
        }


    }

    //往来单位导出生成Excel，并创建生成记录
    @PostMapping("/udiwms/corps/importLog/export")
    public BaseResponse excelDownload(@RequestBody CorpsExportRequest corpsExportRequest) {
        String fileName = "D:\\udiwms\\exportFile\\" + "往来单位信息导出" + System.currentTimeMillis() + ".xls";
        CorpExportLogEntity corpExportLogEntity = new CorpExportLogEntity();
        corpExportLogEntity.setStatus(BasicProcessStatus.UDIINFO_IMPORT_PROCESS);
        String genKey = CustomUtil.getId();
        corpExportLogEntity.setGenKey(genKey);
        corpExportLogEntity.setFilePath(fileName);
        corpExportLogEntity.setUpdateTime(new Date());
        corpExportLogEntity.setDlCount(0);
        corpExportLogEntity.setType(BasicProcessStatus.EXPORT_EXCEL);
        corpExportLogService.insertCorpExportLog(corpExportLogEntity);
        basicGenExcelService.exportCorp(genKey, corpsExportRequest);
        return ResultVOUtils.success("后台正在导出生成excel文件，请稍后刷新查看!");
    }

    //往来单位上传SMP，并创建生成记录
    @PostMapping("/udiwms/corps/importLog/uploadSmp")
    public BaseResponse uploadSmp(@RequestBody CorpsExportRequest corpsExportRequest) {
        CorpExportLogEntity corpExportLogEntity = new CorpExportLogEntity();
        corpExportLogEntity.setStatus(BasicProcessStatus.UDIINFO_IMPORT_PROCESS);
        String genKey = CustomUtil.getId();
        corpExportLogEntity.setGenKey(genKey);
        corpExportLogEntity.setUpdateTime(new Date());
        corpExportLogEntity.setDlCount(0);
        corpExportLogEntity.setType(BasicProcessStatus.EXPORT_SMP);
        corpExportLogService.insertCorpExportLog(corpExportLogEntity);
        basicGenExcelService.uploadCorpSmp(genKey, corpsExportRequest);
        return ResultVOUtils.success("后台正在上传，请稍后刷新查看!");
    }
    
    //往来单位接口下载（第三方使用）
    @GetMapping("/udiwms/basic/corps/download")
    public BaseResponse filterBasicUnitMaintain(BasicUnitMaintainFilterRequest basicUnitMaintainFilterRequest,
                                                BindingResult bindingResult) {
        if (bindingResult.hasErrors()) {
            return ResultVOUtils.error(ResultEnum.PARAM_VERIFY_FALL, bindingResult.getFieldError().getDefaultMessage());
        }
        List<BasicUnitMaintainEntity> basicUnitMaintainEntities;
        basicUnitMaintainEntities = basicUnitMaintainService.filterList(basicUnitMaintainFilterRequest);
        PageInfo<BasicUnitMaintainEntity> pageInfo = new PageInfo<>(basicUnitMaintainEntities);
        PageSimpleResponse<BasicUnitMaintainEntity> pageSimpleResponse = new PageSimpleResponse<>();
        pageSimpleResponse.setTotal(pageInfo.getTotal());
        pageSimpleResponse.setList(basicUnitMaintainEntities);
        return ResultVOUtils.success(pageSimpleResponse);
    }
    


}
