package com.glxp.api.admin.controller.basic;

import com.alibaba.fastjson.JSONObject;
import com.alibaba.fastjson.TypeReference;
import com.github.pagehelper.PageInfo;
import com.glxp.api.admin.entity.basic.*;
import com.glxp.api.admin.entity.info.SystemParamConfigEntity;
import com.glxp.api.admin.entity.thrsys.ThrProductsEntity;
import com.glxp.api.admin.httpclient.ErpBasicClient;
import com.glxp.api.admin.httpclient.ErpOrderClient;
import com.glxp.api.admin.req.basic.FilterErpGoodsRequest;
import com.glxp.api.admin.req.basic.FilterUdiInfoRequest;
import com.glxp.api.admin.req.info.SystemParamConfigRequest;
import com.glxp.api.admin.req.thrsys.FilterThrProductsRequest;
import com.glxp.api.admin.res.PageSimpleResponse;
import com.glxp.api.admin.res.basic.ErpProductsResponse;
import com.glxp.api.admin.service.basic.BasicThirdSysDetailService;
import com.glxp.api.admin.service.basic.BasicThirdSysService;
import com.glxp.api.admin.service.basic.UdiInfoService;
import com.glxp.api.admin.service.basic.UdiRelevanceService;
import com.glxp.api.admin.service.info.SystemParamConfigService;
import com.glxp.api.admin.service.thrsys.ThrProductsService;
import com.glxp.api.admin.util.FilterUdiUtils;
import com.glxp.api.admin.util.HttpClient;
import com.glxp.api.common.enums.ResultEnum;
import com.glxp.api.common.res.BaseResponse;
import com.glxp.api.common.util.ResultVOUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.validation.BindingResult;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.annotation.Resource;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

//基础信息维护
@RestController
public class UdiInfoController {

    @Resource
    UdiRelevanceService udiRelevanceService;
    @Resource
    UdiInfoService udiInfoService;
    @Resource
    BasicThirdSysDetailService basicThirdSysDetailService;
    @Resource
    private ThrProductsService thrProductsService;
    @Value("${UDI_SERVER_URL}")
    private String udiUrl;

    //获取同步库UDI信息
    @GetMapping("udiwms/udiinfo/filterUdi")
    public BaseResponse filterUdi(FilterUdiInfoRequest productInfoFilterRequest) {


        String nameCode = null;
        if (productInfoFilterRequest.getUdiCode() != null && !productInfoFilterRequest.getUdiCode().equals("")) {
            UdiEntity udiEntity = FilterUdiUtils.getUdi(productInfoFilterRequest.getUdiCode());
            if (udiEntity != null) {
                nameCode = udiEntity.getUdi();
            } else {
                return ResultVOUtils.error(500, "无效UDI码！");

//                nameCode = productInfoFilterRequest.getNameCode();
            }
        } else
            nameCode = productInfoFilterRequest.getNameCode();
        Map<String, Object> paramMap = new HashMap<>(16);
        paramMap.put("page", productInfoFilterRequest.getPage());
        paramMap.put("limit", productInfoFilterRequest.getLimit());
        paramMap.put("ylqxzcrbarmc", productInfoFilterRequest.getYlqxzcrbarmc());
        paramMap.put("cpmctymc", productInfoFilterRequest.getCpmctymc());
        paramMap.put("nameCode", nameCode);
        paramMap.put("ggxh", productInfoFilterRequest.getGgxh());
        paramMap.put("zczbhhzbapzbh", productInfoFilterRequest.getZczbhhzbapzbh());
        paramMap.put("uuid", productInfoFilterRequest.getUuid());

        String response = HttpClient.mipsGet(udiUrl + "/udidl/device/filterUdi", paramMap);
        try {
            BaseResponse<PageSimpleResponse<UdiInfoEntity>> udiDlDeviceResponse =
                    JSONObject.parseObject(response, new TypeReference<BaseResponse<PageSimpleResponse<UdiInfoEntity>>>() {
                    });


            if (productInfoFilterRequest.getIsCheck() != null && productInfoFilterRequest.getIsCheck()) {
                List<UdiInfoEntity> udiInfoEntities = udiDlDeviceResponse.getData().getList();

                if (udiInfoEntities != null && udiInfoEntities.size() > 0) {
                    for (UdiInfoEntity udiInfoEntity : udiInfoEntities) {
                        List<UdiRelevanceEntity> udiRelevanceEntities = udiRelevanceService.selectByUuid(udiInfoEntity.getUuid());
                        if (udiRelevanceEntities != null && udiRelevanceEntities.size() > 0) {

                            for(UdiRelevanceEntity udiRelevanceEntity:udiRelevanceEntities){
                                if (udiRelevanceEntity.getThirdId() == null &&
                                        udiRelevanceEntity.getThirdId2() == null && udiRelevanceEntity.getThirdId1() == null &&
                                        udiRelevanceEntity.getThirdId3() == null && udiRelevanceEntity.getThirdId4() == null) {
                                    //不能添加
                                    udiInfoEntity.setCheck(true);
                                    break;
                                } else
                                    udiInfoEntity.setCheck(false);
                            }

                        } else {
                            udiInfoEntity.setCheck(false);
                        }
                    }
                }
            }


            return udiDlDeviceResponse;
        } catch (Exception e) {
            e.printStackTrace();
            return ResultVOUtils.error(500, "连接UDI数据下载服务出错！");
        }
    }


    //获取同步库医疗器械注册人信息
    @GetMapping("udiwms/udiinfo/filterCompany")
    public BaseResponse filterCompany(FilterUdiInfoRequest productInfoFilterRequest) {

        Map<String, Object> paramMap = new HashMap<>(16);
        paramMap.put("page", productInfoFilterRequest.getPage());
        paramMap.put("limit", productInfoFilterRequest.getLimit());
        paramMap.put("ylqxzcrbarmc", productInfoFilterRequest.getYlqxzcrbarmc());

        String response = HttpClient.mipsGet(udiUrl + "/udi/company/filter", paramMap);
        try {
            BaseResponse<PageSimpleResponse<UdiCompanyEntity>> udiDlDeviceResponse = (BaseResponse<PageSimpleResponse<UdiCompanyEntity>>) JSONObject.parseObject(response, BaseResponse.class);
            return udiDlDeviceResponse;
        } catch (Exception e) {
            e.printStackTrace();
            return ResultVOUtils.error(500, "连接UDI数据下载服务出错！");
        }
    }

    //获取本地已维护UDI信息
    @GetMapping("udiwms/udiinfo/local/filterByUuid")
    public BaseResponse filterLocalByUuid(FilterUdiInfoRequest productInfoFilterRequest) {

        List<UdiInfoEntity> udiInfoEntities = udiInfoService.filterAllUdiInfo(productInfoFilterRequest);
        return ResultVOUtils.success(udiInfoEntities);
    }

    //获取ERP产品信息
    @GetMapping("udiwms/udiinfo/erp/products")
    public BaseResponse getErpProducts(FilterErpGoodsRequest filterErpGoodsRequest,
                                       BindingResult bindingResult) {

        if (bindingResult.hasErrors()) {
            return ResultVOUtils.error(ResultEnum.PARAM_VERIFY_FALL, bindingResult.getFieldError().getDefaultMessage());
        }
        BasicThirdSysDetailEntity basicThirdSysDetailEntity = basicThirdSysDetailService.selectByKey("piQueryUrl", filterErpGoodsRequest.getThirdSys());
        if (basicThirdSysDetailEntity == null || basicThirdSysDetailEntity.getValue() == null)
            return ResultVOUtils.error(500, "ERP产品信息接口地址未定义");

        if (basicThirdSysDetailEntity.getEnabled()) {
            if (basicThirdSysDetailEntity.getFromType() == 0 || (filterErpGoodsRequest.getIsDownThrSys() != null && filterErpGoodsRequest.getIsDownThrSys())) {
                ErpBasicClient erpBasicHttpClient = new ErpBasicClient();
                BaseResponse<PageSimpleResponse<ErpProductsResponse>> udiDlDeviceResponse = erpBasicHttpClient.getErpProducts(filterErpGoodsRequest, basicThirdSysDetailEntity.getValue());
                if (udiDlDeviceResponse.getCode() == 20000) {
                    List<ErpProductsResponse> invmandocResponses = udiDlDeviceResponse.getData().getList();
                    if (invmandocResponses != null) {
                        for (ErpProductsResponse invmandocResponse : invmandocResponses) {
                            UdiRelevanceEntity udiRelevanceEntity = udiRelevanceService.selectByThirdId(invmandocResponse.getCode(), filterErpGoodsRequest.getThirdSys());
                            invmandocResponse.setThirdSys(filterErpGoodsRequest.getThirdSys());
                            if (udiRelevanceEntity != null) {
                                invmandocResponse.setChecked(true);
                            } else {
                                invmandocResponse.setChecked(false);
                            }
                        }
                    }

                }
                return udiDlDeviceResponse;
            } else {
                FilterThrProductsRequest filterThrProductsRequest = new FilterThrProductsRequest();
                BeanUtils.copyProperties(filterErpGoodsRequest, filterThrProductsRequest);
                filterThrProductsRequest.setThirdSysFk(filterErpGoodsRequest.getThirdSys());
                filterThrProductsRequest.setCode(null);

                List<ThrProductsEntity> thrCorpEntities
                        = thrProductsService.filterThrProductsRequest(filterThrProductsRequest);
                for (ThrProductsEntity thrProductsEntity : thrCorpEntities) {
                    UdiRelevanceEntity udiRelevanceEntity = udiRelevanceService.selectByThirdId(thrProductsEntity.getCode(), thrProductsEntity.getThirdSysFk());
                    if (udiRelevanceEntity != null) {
                        thrProductsEntity.setChecked(true);
                    } else {
                        thrProductsEntity.setChecked(false);
                    }
                }


                PageInfo<ThrProductsEntity> pageInfo;
                pageInfo = new PageInfo<>(thrCorpEntities);
                PageSimpleResponse<ThrProductsEntity> pageSimpleResponse = new PageSimpleResponse<>();
                pageSimpleResponse.setTotal(pageInfo.getTotal());
                pageSimpleResponse.setList(thrCorpEntities);
                return ResultVOUtils.success(pageSimpleResponse);
            }
        } else {
            return ResultVOUtils.error(500, "第三方产品信息服务未启用");
        }


    }
}