package com.glxp.api.admin.controller.basic;

import com.github.pagehelper.PageInfo;
import com.glxp.api.admin.constant.BasicProcessStatus;
import com.glxp.api.admin.entity.basic.UdiInfoExportLogEntity;
import com.glxp.api.admin.req.basic.FilterUdiEpLogRequest;
import com.glxp.api.admin.req.basic.FilterUdiInfoRequest;
import com.glxp.api.admin.req.basic.UdiInfoExportRequest;
import com.glxp.api.admin.req.inout.DeleteRequest;
import com.glxp.api.admin.res.PageSimpleResponse;
import com.glxp.api.admin.res.basic.UdiRelevanceResponse;
import com.glxp.api.admin.res.basic.UdiRlDlResponse;
import com.glxp.api.admin.res.inout.WarehouseResponse;
import com.glxp.api.admin.service.basic.UdiInfoExportLogService;
import com.glxp.api.admin.service.basic.UdiRelevanceService;
import com.glxp.api.admin.thread.BasicGenExcelService;
import com.glxp.api.admin.util.CustomUtil;
import com.glxp.api.common.enums.ResultEnum;
import com.glxp.api.common.res.BaseResponse;
import com.glxp.api.common.util.ResultVOUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.validation.BindingResult;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RestController;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletResponse;
import java.io.File;
import java.io.FileInputStream;
import java.io.OutputStream;
import java.util.Date;
import java.util.List;
import java.util.stream.Collectors;

/**
 * 医疗器械信息导出
 */
@RestController
public class UdiInfoExportLogController {

    @Resource
    UdiInfoExportLogService udiInfoExportLogService;
    @Resource
    UdiRelevanceService udiRelevanceService;
    @Resource
    BasicGenExcelService basicGenExcelService;


    @GetMapping("/udiwms/products/exportLog/filter")
    public BaseResponse filter(FilterUdiEpLogRequest filterUdiEpLogReques,
                               BindingResult bindingResult) {

        if (bindingResult.hasErrors()) {
            return ResultVOUtils.error(ResultEnum.PARAM_VERIFY_FALL, bindingResult.getFieldError().getDefaultMessage());
        }
        List<UdiInfoExportLogEntity> udiInfoExportLogEntities
                = udiInfoExportLogService.filterUdiInfoExportLog(filterUdiEpLogReques);
        PageInfo<UdiInfoExportLogEntity> pageInfo;
        pageInfo = new PageInfo<>(udiInfoExportLogEntities);
        PageSimpleResponse<UdiInfoExportLogEntity> pageSimpleResponse = new PageSimpleResponse<>();
        pageSimpleResponse.setTotal(pageInfo.getTotal());
        pageSimpleResponse.setList(udiInfoExportLogEntities);
        return ResultVOUtils.success(pageSimpleResponse);
    }

    @PostMapping("/udiwms/products/exportLog/deleteLog")
    public BaseResponse deleteLog(@RequestBody DeleteRequest deleteRequest, BindingResult bindingResult) {

        if (bindingResult.hasErrors()) {
            return ResultVOUtils.error(ResultEnum.PARAM_VERIFY_FALL, bindingResult.getFieldError().getDefaultMessage());
        }
        String id = deleteRequest.getId();
        udiInfoExportLogService.deleteById(id + "");
        return ResultVOUtils.success("删除成功");
    }

    //产品信息excel文件下载
    @GetMapping("/udiwms/products/exportLog/download")
    public void download(FilterUdiEpLogRequest filterUdiEpLogRequest, HttpServletResponse res) {

        List<UdiInfoExportLogEntity> udiInfoExportLogEntities = udiInfoExportLogService.filterUdiInfoExportLog(filterUdiEpLogRequest);
        UdiInfoExportLogEntity udiInfoExportLogEntity = udiInfoExportLogEntities.get(0);
        try {
            String filePath = udiInfoExportLogEntity.getFilePath();
            String fileName = "temp";
            if (filePath != null && filePath.length() > 0) {
                String[] data = filePath.split("/");
                fileName = data[data.length - 1];
            }
            res.setHeader("Content-disposition", "attachment;fileName=" + fileName);
            res.setContentType("text/plain;charset=UTF-8");
            FileInputStream input = new FileInputStream(new File(filePath));
            OutputStream out = res.getOutputStream();
            byte[] b = new byte[2048];
            int len;
            while ((len = input.read(b)) != -1) {
                out.write(b, 0, len);
            }
            input.close();
            if (udiInfoExportLogEntity.getDlCount() == 0) {
                udiInfoExportLogEntity.setDlCount(1);
            } else {
                udiInfoExportLogEntity.setDlCount(udiInfoExportLogEntity.getDlCount() + 1);
            }
            udiInfoExportLogService.updateUdiInfoExportLog(udiInfoExportLogEntity);
        } catch (Exception ex) {
            System.out.println("下载失败！");
        }


    }

    //产品信息导出生成Excel，并创建生成记录
    @PostMapping("/udiwms/products/importLog/export")
    public BaseResponse excelDownload(@RequestBody UdiInfoExportRequest udiInfoExportRequest) {

        String fileName = "D:\\udiwms\\exportFile\\" + "产品信息导出" + System.currentTimeMillis() + ".xls";
        UdiInfoExportLogEntity udiInfoExportLogEntity = new UdiInfoExportLogEntity();
        udiInfoExportLogEntity.setStatus(BasicProcessStatus.UDIINFO_IMPORT_PROCESS);
        String genKey = CustomUtil.getId();
        udiInfoExportLogEntity.setGenKey(genKey);
        udiInfoExportLogEntity.setFilePath(fileName);
        udiInfoExportLogEntity.setUpdateTime(new Date());
        udiInfoExportLogEntity.setDlCount(0);
        udiInfoExportLogEntity.setType(BasicProcessStatus.EXPORT_EXCEL);
        udiInfoExportLogService.insertUdiInfoExportLog(udiInfoExportLogEntity);
        basicGenExcelService.exportUdiInfo(genKey, udiInfoExportRequest);
        return ResultVOUtils.success("后台正在导出生成excel文件，请稍后刷新查看!");

    }

    //产品信息上传SMP，并创建生成记录
    @PostMapping("/udiwms/products/importLog/uploadSmp")
    public BaseResponse uploadSmp(@RequestBody UdiInfoExportRequest udiInfoExportRequest) {

        UdiInfoExportLogEntity udiInfoExportLogEntity = new UdiInfoExportLogEntity();
        udiInfoExportLogEntity.setStatus(BasicProcessStatus.UDIINFO_IMPORT_PROCESS);
        String genKey = CustomUtil.getId();
        udiInfoExportLogEntity.setGenKey(genKey);
        udiInfoExportLogEntity.setUpdateTime(new Date());
        udiInfoExportLogEntity.setType(BasicProcessStatus.EXPORT_SMP);
        udiInfoExportLogEntity.setDlCount(0);
        udiInfoExportLogService.insertUdiInfoExportLog(udiInfoExportLogEntity);
        basicGenExcelService.uploadProductsSmp(genKey, udiInfoExportRequest);
        return ResultVOUtils.success("后台正在上传，请稍后刷新查看!");

    }


    //产品信息接口下载（第三方使用）
    @GetMapping("/udiwms/basic/products/download")
    public BaseResponse downloadProducts(FilterUdiInfoRequest filterUdiInfoRequest,
                                         BindingResult bindingResult) {

        if (bindingResult.hasErrors()) {
            return ResultVOUtils.error(ResultEnum.PARAM_VERIFY_FALL, bindingResult.getFieldError().getDefaultMessage());
        }

        if (filterUdiInfoRequest.getThirdSys() != null && filterUdiInfoRequest.getThirdCode() != null) {
            if ("thirdId1".equals(filterUdiInfoRequest.getThirdSys())) {
                filterUdiInfoRequest.setThirdId1(filterUdiInfoRequest.getThirdCode());
            } else if ("thirdId2".equals(filterUdiInfoRequest.getThirdSys())) {
                filterUdiInfoRequest.setThirdId2(filterUdiInfoRequest.getThirdCode());
            } else if ("thirdId3".equals(filterUdiInfoRequest.getThirdSys())) {
                filterUdiInfoRequest.setThirdId3(filterUdiInfoRequest.getThirdCode());
            } else if ("thirdId4".equals(filterUdiInfoRequest.getThirdSys())) {
                filterUdiInfoRequest.setThirdId4(filterUdiInfoRequest.getThirdCode());
            } else {
                filterUdiInfoRequest.setThirdId(filterUdiInfoRequest.getThirdCode());
            }
        }

        List<UdiRelevanceResponse> udiRelevanceResponses = null;
        udiRelevanceResponses = udiRelevanceService.filterUdiRelevance(filterUdiInfoRequest);
        List<UdiRlDlResponse> udiRlDlResponses = udiRelevanceResponses.stream().map(item ->
        {
            UdiRlDlResponse udiRlDlResponse = new UdiRlDlResponse();
            BeanUtils.copyProperties(item, udiRlDlResponse);
            return udiRlDlResponse;
        }).collect(Collectors.toList());
        PageInfo<UdiRelevanceResponse> pageInfo;
        pageInfo = new PageInfo<>(udiRelevanceResponses);
        PageSimpleResponse<UdiRlDlResponse> pageSimpleResponse = new PageSimpleResponse<>();
        pageSimpleResponse.setTotal(pageInfo.getTotal());
        pageSimpleResponse.setList(udiRlDlResponses);
        return ResultVOUtils.success(pageSimpleResponse);
    }

}
