package com.glxp.api.admin.controller.basic;

import com.github.pagehelper.PageInfo;
import com.glxp.api.admin.constant.BasicProcessStatus;
import com.glxp.api.admin.entity.basic.UdiInfoImportDetailEntity;
import com.glxp.api.admin.entity.basic.UdiInfoImportLogEntity;
import com.glxp.api.admin.req.basic.FilterUdiIpLogRequest;
import com.glxp.api.admin.req.basic.PostUdiInfoRequest;
import com.glxp.api.admin.req.inout.DeleteRequest;
import com.glxp.api.admin.res.PageSimpleResponse;
import com.glxp.api.admin.service.basic.UdiInfoImportLogService;
import com.glxp.api.admin.service.basic.UdiInfoImportDetailService;
import com.glxp.api.admin.thread.BasicUdiInfoImportService;
import com.glxp.api.admin.util.CustomUtil;
import com.glxp.api.common.enums.ResultEnum;
import com.glxp.api.common.res.BaseResponse;
import com.glxp.api.common.util.ResultVOUtils;
import org.apache.poi.hssf.usermodel.HSSFWorkbook;
import org.apache.poi.ss.usermodel.*;
import org.apache.poi.xssf.usermodel.XSSFWorkbook;
import org.springframework.validation.BindingResult;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import javax.annotation.Resource;
import java.io.File;
import java.io.IOException;
import java.io.InputStream;
import java.sql.Date;
import java.util.ArrayList;
import java.util.List;

/**
 * 医疗器械信息导入
 */

@RestController
public class UdiInfoImportLogController {

    @Resource
    UdiInfoImportLogService udiInfoImportLogService;
    @Resource
    UdiInfoImportDetailService udiInfoImportService;
    @Resource
    BasicUdiInfoImportService basicUdiInfoImportService;

    @GetMapping("/udiwms/products/importLog/filter")
    public BaseResponse filter(FilterUdiIpLogRequest filterUdiIpLogRequest,
                               BindingResult bindingResult) {

        if (bindingResult.hasErrors()) {
            return ResultVOUtils.error(ResultEnum.PARAM_VERIFY_FALL, bindingResult.getFieldError().getDefaultMessage());
        }
        List<UdiInfoImportLogEntity> udiInfoImportLogEntities
                = udiInfoImportLogService.filterImportLog(filterUdiIpLogRequest);
        PageInfo<UdiInfoImportLogEntity> pageInfo;
        pageInfo = new PageInfo<>(udiInfoImportLogEntities);
        PageSimpleResponse<UdiInfoImportLogEntity> pageSimpleResponse = new PageSimpleResponse<>();
        pageSimpleResponse.setTotal(pageInfo.getTotal());
        pageSimpleResponse.setList(udiInfoImportLogEntities);
        return ResultVOUtils.success(pageSimpleResponse);
    }

    @GetMapping("/udiwms/products/importLog/filterDetail")
    public BaseResponse filterDetail(FilterUdiIpLogRequest filterUdiIpLogRequest,
                                     BindingResult bindingResult) {

        if (bindingResult.hasErrors()) {
            return ResultVOUtils.error(ResultEnum.PARAM_VERIFY_FALL, bindingResult.getFieldError().getDefaultMessage());
        }
        List<UdiInfoImportDetailEntity> udiInfoImportEntities
                = udiInfoImportService.filterUdiInfoImport(filterUdiIpLogRequest);
        PageInfo<UdiInfoImportDetailEntity> pageInfo;
        pageInfo = new PageInfo<>(udiInfoImportEntities);
        PageSimpleResponse<UdiInfoImportDetailEntity> pageSimpleResponse = new PageSimpleResponse<>();
        pageSimpleResponse.setTotal(pageInfo.getTotal());
        pageSimpleResponse.setList(udiInfoImportEntities);
        return ResultVOUtils.success(pageSimpleResponse);
    }


    @PostMapping("/udiwms/products/importLog/deleteLog")
    public BaseResponse deleteLog(@RequestBody DeleteRequest deleteRequest, BindingResult bindingResult) {

        if (bindingResult.hasErrors()) {
            return ResultVOUtils.error(ResultEnum.PARAM_VERIFY_FALL, bindingResult.getFieldError().getDefaultMessage());
        }
        String id = deleteRequest.getId();
        FilterUdiIpLogRequest filterInCodeLogRequest = new FilterUdiIpLogRequest();
        filterInCodeLogRequest.setId(Integer.parseInt(id));
        List<UdiInfoImportLogEntity> importLogEntities = udiInfoImportLogService.filterImportLog(filterInCodeLogRequest);
        if (importLogEntities != null && importLogEntities.size() > 0) {
            UdiInfoImportLogEntity importLogEntity = importLogEntities.get(0);
            udiInfoImportLogService.deleteById(importLogEntity.getId() + "");
            udiInfoImportService.deleteByGenkey(importLogEntity.getGenKey());
        }

        return ResultVOUtils.success("删除成功");
    }

    @PostMapping("/udiwms/products/importLog/delete")
    public BaseResponse delete(@RequestBody DeleteRequest deleteRequest, BindingResult bindingResult) {

        if (bindingResult.hasErrors()) {
            return ResultVOUtils.error(ResultEnum.PARAM_VERIFY_FALL, bindingResult.getFieldError().getDefaultMessage());
        }
        if (udiInfoImportService.deleteById(deleteRequest.getId()))
            return ResultVOUtils.success("删除成功");
        else
            return ResultVOUtils.error(500, "删除成功");
    }


    //产品信息文件导入
    @PostMapping("/udiwms/products/importLog/upload")
    public BaseResponse uploadProducts(@RequestParam("file") List<MultipartFile> files, @RequestParam("thirdSys") String thirdSys) {
        String filePath = "D:\\udiwms\\filePath\\";
        File createFile = new File(filePath);
        if (!createFile.exists()) {
            createFile.mkdirs();
        }
        for (int i = 0; i < files.size(); i++) {
            MultipartFile file = files.get(i);
            if (file.isEmpty()) {
                return ResultVOUtils.error(500, "上传第" + (i++) + "个文件失败");
            }
            String fileName = file.getOriginalFilename();
            try {
                InputStream inputStream = file.getInputStream();
                Workbook workbook = null;
                String filetype = fileName.substring(fileName.lastIndexOf("."));

                if (".xls".equals(filetype)) {
                    workbook = new HSSFWorkbook(inputStream);
                } else if (".xlsx".equals(filetype)) {
                    workbook = new XSSFWorkbook(inputStream);
                } else {
                    return ResultVOUtils.error(500, "请上传excel文件");
                }
                Sheet sheet = null;
                Row row = null;
                UdiInfoImportLogEntity udiInfoImportLogEntity = new UdiInfoImportLogEntity();
                String genKey = CustomUtil.getId();
                udiInfoImportLogEntity.setGenKey(genKey);
                udiInfoImportLogEntity.setFromType("文件导入");
                udiInfoImportLogEntity.setStatus(BasicProcessStatus.UDIINFO_IMPORT_UNPROCESS);
                udiInfoImportLogEntity.setUpdateTime(new Date(System.currentTimeMillis()));
                udiInfoImportLogService.insertImportLog(udiInfoImportLogEntity);
                List<UdiInfoImportDetailEntity> udiInfoImportEntities = new ArrayList<>();
                for (int j = 0; j < 1; j++) {
                    sheet = workbook.getSheetAt(j);
                    if (sheet == null) {
                        continue;
                    }
                    // 滤过第一行标题
                    row = sheet.getRow(0);
                    if ((row.getCell(0) == null || row.getCell(1) == null || row.getCell(2) == null
                            || row.getCell(3) == null || row.getCell(4) == null || row.getCell(5) == null|| row.getCell(6) == null) ||
                            (!"udiCode".equals(row.getCell(0).getStringCellValue())
                                    || !"医保码".equals(row.getCell(1).getStringCellValue())
                                    || !"商品条码".equals(row.getCell(2).getStringCellValue())
                                    || !"thirdId".equals(row.getCell(3).getStringCellValue())
                                    || !"thirdId1".equals(row.getCell(4).getStringCellValue())
                                    || !"thirdId2".equals(row.getCell(5).getStringCellValue())
                                    || !"thirdId3".equals(row.getCell(6).getStringCellValue()))) {
                        return ResultVOUtils.error(500, "文件格式错误！");
                    }
                    for (int k = sheet.getFirstRowNum()+1; k <= sheet.getLastRowNum(); k++) {
                        row = sheet.getRow(k);
                        if (row.getRowNum() <= 0) {
                            continue;
                        }
                        if ((row.getCell(0) == null && row.getCell(1) == null && row.getCell(2) == null
                                && row.getCell(3) == null && row.getCell(4) == null && row.getCell(5) == null&& row.getCell(6) == null) ||
                                ("".equals(row.getCell(0).getStringCellValue())
                                        && "".equals(row.getCell(1).getStringCellValue())
                                        && "".equals(row.getCell(2).getStringCellValue())
                                        && "".equals(row.getCell(3).getStringCellValue())
                                        && "".equals(row.getCell(4).getStringCellValue())
                                        && "".equals(row.getCell(5).getStringCellValue())
                                        && "".equals(row.getCell(6).getStringCellValue()))) {
                            continue;
                        }
                        UdiInfoImportDetailEntity udiInfoImportEntity = new UdiInfoImportDetailEntity();
                        Cell cell1 = row.getCell(0);
                        if (cell1 != null) {
                            cell1.setCellType(CellType.STRING);
                            udiInfoImportEntity.setUdiCode(cell1.getStringCellValue());
                        }


                        Cell cell2 = row.getCell(1);
                        if (cell2 != null) {
                            cell2.setCellType(CellType.STRING);
                            udiInfoImportEntity.setYbbm(cell2.getStringCellValue());
                        }
                        Cell cell3 = row.getCell(2);
                        if (cell3 != null) {
                            cell3.setCellType(CellType.STRING);
                            udiInfoImportEntity.setSptm(cell3.getStringCellValue());

                        }
                        Cell cell4 = row.getCell(3);
                        if (cell4 != null) {
                            cell4.setCellType(CellType.STRING);
                            udiInfoImportEntity.setThirdId(cell4.getStringCellValue());

                        }
                        Cell cell5 = row.getCell(4);
                        if (cell5 != null) {
                            cell5.setCellType(CellType.STRING);
                            udiInfoImportEntity.setThirdId1(cell5.getStringCellValue());

                        }
                        Cell cell6 = row.getCell(5);
                        if (cell6 != null) {
                            cell6.setCellType(CellType.STRING);
                            udiInfoImportEntity.setThirdId2(cell6.getStringCellValue());
                        }

                        Cell cell7 = row.getCell(6);
                        if (cell7 != null) {
                            cell7.setCellType(CellType.STRING);
                            udiInfoImportEntity.setThirdId3(cell7.getStringCellValue());
                        }
                        Cell cell8 = row.getCell(7);
                        if (cell8 != null) {
                            cell8.setCellType(CellType.STRING);
                            udiInfoImportEntity.setThirdId4(cell8.getStringCellValue());
                        }
                        udiInfoImportEntity.setGenKeyFk(genKey);
                        udiInfoImportEntity.setStatus(0);
                        udiInfoImportEntity.setUpdateTime(new Date(System.currentTimeMillis()));
                        udiInfoImportEntities.add(udiInfoImportEntity);
                    }
                }
                udiInfoImportService.insertUdiInfoImports(udiInfoImportEntities);
                udiInfoImportLogEntity.setStatus(BasicProcessStatus.UDIINFO_IMPORT_UNPROCESS);
                udiInfoImportLogService.updateImportLog(udiInfoImportLogEntity);
                workbook.close();
                basicUdiInfoImportService.importUdiInfo(genKey);
            } catch (IOException e) {
                e.printStackTrace();
            }
        }
        return ResultVOUtils.success("上传成功");
    }


    //产品信息接口上传
    @PostMapping("/udiwms/basic/products/upload")
    public BaseResponse postUdiInfo(@RequestBody PostUdiInfoRequest postUdiInfoRequest, BindingResult bindingResult) {
        if (bindingResult.hasErrors()) {
            return ResultVOUtils.error(ResultEnum.PARAM_VERIFY_FALL, bindingResult.getFieldError().getDefaultMessage());
        }

        String genKey = postUdiInfoRequest.getGenKey();
        if (genKey == null) {
            genKey = CustomUtil.getId();
        }
        UdiInfoImportLogEntity udiInfoImportLogEntity = udiInfoImportLogService.selectByGenKey(genKey);
        List<UdiInfoImportDetailEntity> udiInfoImportEntities = postUdiInfoRequest.getDatas();
        if(udiInfoImportEntities!=null){
            for(UdiInfoImportDetailEntity udiInfoImportEntity:udiInfoImportEntities){
                udiInfoImportEntity.setGenKeyFk(genKey);
                udiInfoImportEntity.setStatus(BasicProcessStatus.UDIINFO_IMPORT_CODE_UNPROCESS);
                udiInfoImportEntity.setUpdateTime(new java.util.Date());
                udiInfoImportService.insertUdiInfoImport(udiInfoImportEntity);
            }
        }
        if(udiInfoImportLogEntity==null){
            udiInfoImportLogEntity = new UdiInfoImportLogEntity();
            udiInfoImportLogEntity.setGenKey(genKey);
            udiInfoImportLogEntity.setFromType("接口上传");
            udiInfoImportLogEntity.setStatus(BasicProcessStatus.UDIINFO_IMPORT_UNPROCESS);
            udiInfoImportLogEntity.setUpdateTime(new Date(System.currentTimeMillis()));
            udiInfoImportLogService.insertImportLog(udiInfoImportLogEntity);
        }else {
            udiInfoImportLogEntity.setStatus(BasicProcessStatus.UDIINFO_IMPORT_UNPROCESS);
            udiInfoImportLogService.updateImportLog(udiInfoImportLogEntity);
        }
        basicUdiInfoImportService.importUdiInfo(genKey);
        return ResultVOUtils.success("上传成功");
    }


    //扫码精灵产品信息接口上传
    @PostMapping("/udiwms/basic/products/srUpload")
    public BaseResponse postSrUdiInfo(@RequestBody PostUdiInfoRequest postUdiInfoRequest, BindingResult bindingResult) {
        if (bindingResult.hasErrors()) {
            return ResultVOUtils.error(ResultEnum.PARAM_VERIFY_FALL, bindingResult.getFieldError().getDefaultMessage());
        }

        String genKey = postUdiInfoRequest.getGenKey();
        if (genKey == null) {
            genKey = CustomUtil.getId();
        }
        UdiInfoImportLogEntity udiInfoImportLogEntity = udiInfoImportLogService.selectByGenKey(genKey);
        List<UdiInfoImportDetailEntity> udiInfoImportEntities = postUdiInfoRequest.getDatas();
        if(udiInfoImportEntities!=null){
            for(UdiInfoImportDetailEntity udiInfoImportEntity:udiInfoImportEntities){
                udiInfoImportEntity.setGenKeyFk(genKey);
                udiInfoImportEntity.setStatus(BasicProcessStatus.UDIINFO_IMPORT_CODE_UNPROCESS);
                udiInfoImportEntity.setUpdateTime(new java.util.Date());
                udiInfoImportService.insertUdiInfoImport(udiInfoImportEntity);
            }
        }
        if(udiInfoImportLogEntity==null){
            udiInfoImportLogEntity = new UdiInfoImportLogEntity();
            udiInfoImportLogEntity.setGenKey(genKey);
            udiInfoImportLogEntity.setFromType("扫码精灵上传");
            udiInfoImportLogEntity.setStatus(BasicProcessStatus.UDIINFO_IMPORT_UNPROCESS);
            udiInfoImportLogEntity.setUpdateTime(new Date(System.currentTimeMillis()));
            udiInfoImportLogService.insertImportLog(udiInfoImportLogEntity);
        }else {
            udiInfoImportLogEntity.setStatus(BasicProcessStatus.UDIINFO_IMPORT_UNPROCESS);
            udiInfoImportLogService.updateImportLog(udiInfoImportLogEntity);
        }
        basicUdiInfoImportService.importUdiInfo(genKey);
        return ResultVOUtils.success("上传成功");
    }

    //供应商平台产品信息下载
    @GetMapping("/udiwms/products/smp/download")
    public BaseResponse downloadSmpUdi(FilterUdiIpLogRequest filterUdiIpLogRequest,
                               BindingResult bindingResult) {

        if (bindingResult.hasErrors()) {
            return ResultVOUtils.error(ResultEnum.PARAM_VERIFY_FALL, bindingResult.getFieldError().getDefaultMessage());
        }
        String genKey = CustomUtil.getId();
        UdiInfoImportLogEntity udiInfoImportLogEntity = new UdiInfoImportLogEntity();
        udiInfoImportLogEntity.setGenKey(genKey);
        udiInfoImportLogEntity.setFromType("供应商平台下载");
        udiInfoImportLogEntity.setStatus(BasicProcessStatus.UDIINFO_IMPORT_CODE_UNPROCESS);
        udiInfoImportLogEntity.setUpdateTime(new java.util.Date());
        udiInfoImportLogService.insertImportLog(udiInfoImportLogEntity);
        basicUdiInfoImportService.downloadSmpUdiInfo(genKey);
        return ResultVOUtils.success("后台已开始下载并更新，请稍后刷新查看!");

    }

}
