package com.glxp.api.admin.controller.basic;

import com.alibaba.fastjson.JSONObject;
import com.alibaba.fastjson.TypeReference;
import com.github.pagehelper.PageInfo;
import com.glxp.api.admin.config.WebSocketServer;
import com.glxp.api.admin.entity.basic.*;
import com.glxp.api.admin.entity.info.SystemParamConfigEntity;
import com.glxp.api.admin.entity.inout.WarehouseEntity;
import com.glxp.api.admin.entity.thrsys.ThrProductsEntity;
import com.glxp.api.admin.httpclient.UdiDlHttpClient;
import com.glxp.api.admin.req.basic.*;
import com.glxp.api.admin.req.inout.DeleteRequest;
import com.glxp.api.admin.res.PageSimpleResponse;
import com.glxp.api.admin.res.basic.BasicProductThirdSysResponse;
import com.glxp.api.admin.res.basic.UdiRelevanceResponse;
import com.glxp.api.admin.service.basic.BasicThirdSysDetailService;
import com.glxp.api.admin.service.basic.BasicThirdSysService;
import com.glxp.api.admin.service.basic.UdiInfoService;
import com.glxp.api.admin.service.basic.UdiRelevanceService;
import com.glxp.api.admin.service.info.SystemParamConfigService;
import com.glxp.api.admin.service.inout.CodesService;
import com.glxp.api.admin.service.thrsys.ThrDataService;
import com.glxp.api.admin.thread.BasicBackUpdateProductService;
import com.glxp.api.admin.thread.BasicUpdateProductService;
import com.glxp.api.admin.util.CustomUtil;
import com.glxp.api.admin.util.DateUtil;
import com.glxp.api.admin.util.FilterUdiUtils;
import com.glxp.api.admin.util.HttpClient;
import com.glxp.api.common.enums.ResultEnum;
import com.glxp.api.common.res.BaseResponse;
import com.glxp.api.common.util.ResultVOUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.validation.BindingResult;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RestController;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

@RestController
public class UdiRelevanceController {

    @Value("${UDI_SERVER_URL}")
    private String udiUrl;
    @Resource
    UdiRelevanceService udiRelevanceService;
    @Resource
    UdiInfoService udiInfoService;
    @Resource
    BasicBackUpdateProductService updateproductService;
    @Resource
    BasicUpdateProductService basicUpdateProductService;
    @Resource
    CodesService warehouseService;
    @Resource
    private SystemParamConfigService systemParamConfigService;
    @Resource
    private BasicThirdSysService basicThirdSysService;
    @Resource
    BasicThirdSysDetailService basicThirdSysDetailService;
    @Resource
    ThrDataService thrDataService;

    //客户端器械信息下载
    @GetMapping("/udi/udirel/download")
    public BaseResponse downloadProducts(FilterUdiInfoRequest filterUdiInfoRequest,
                                         BindingResult bindingResult) {

        if (bindingResult.hasErrors()) {
            return ResultVOUtils.error(ResultEnum.PARAM_VERIFY_FALL, bindingResult.getFieldError().getDefaultMessage());
        }
        List<UdiRelevanceResponse> udiRelevanceResponses = null;
        udiRelevanceResponses = udiRelevanceService.filterUdiRelevance(filterUdiInfoRequest);

        PageInfo<UdiRelevanceResponse> pageInfo;
        pageInfo = new PageInfo<>(udiRelevanceResponses);
        PageSimpleResponse<UdiRelevanceResponse> pageSimpleResponse = new PageSimpleResponse<>();
        pageSimpleResponse.setTotal(pageInfo.getTotal());
        pageSimpleResponse.setList(udiRelevanceResponses);
        return ResultVOUtils.success(pageSimpleResponse);
    }


    //器械信息维护首页
    @GetMapping("/udi/udirel/filterUdi")
    public BaseResponse filterUdiGp(FilterUdiInfoRequest filterUdiInfoRequest,
                                    BindingResult bindingResult) {

        if (bindingResult.hasErrors()) {
            return ResultVOUtils.error(ResultEnum.PARAM_VERIFY_FALL, bindingResult.getFieldError().getDefaultMessage());
        }
        List<UdiRelevanceResponse> udiRelevanceResponses = null;

        filterUdiInfoRequest.setDiType(1);

        if (filterUdiInfoRequest.getUdiCode() != null && !filterUdiInfoRequest.getUdiCode().isEmpty()) {
            UdiEntity udiEntity = FilterUdiUtils.getUdi(filterUdiInfoRequest.getUdiCode());
            filterUdiInfoRequest.setNameCode(udiEntity.getUdi());
        }


        udiRelevanceResponses = udiRelevanceService.filterUdiGp(filterUdiInfoRequest);
//        if (udiRelevanceResponses != null && udiRelevanceResponses.size() > 0) {
//            for (UdiRelevanceResponse udiRelevanceResponse : udiRelevanceResponses) {
//                if (udiRelevanceResponse.getDiType() == 2 && udiRelevanceResponse.getIsUseDy() != 1) {
//                    udiRelevanceResponse.setNameCode(udiRelevanceResponse.getSjcpbm());
//                }
//            }
//        }
        PageInfo<UdiRelevanceResponse> pageInfo;
        pageInfo = new PageInfo<>(udiRelevanceResponses);
        PageSimpleResponse<UdiRelevanceResponse> pageSimpleResponse = new PageSimpleResponse<>();
        pageSimpleResponse.setTotal(pageInfo.getTotal());
        pageSimpleResponse.setList(udiRelevanceResponses);
        return ResultVOUtils.success(pageSimpleResponse);
    }


    //更新下载ERP存货产品
    @GetMapping("/udi/udirel/updateErp")
    public BaseResponse updateErp() {
        SystemParamConfigEntity thirdSys = systemParamConfigService.selectByParamKey("erp_updateErpData_thirdSys");
        String thirdSysId = null;
        if (thirdSys != null && thirdSys.getParamValue() != null) {
            thirdSysId = thirdSys.getParamValue();
        } else
            thirdSysId = "thirdId";
        BasicThirdSysDetailEntity piDetailEntity = basicThirdSysDetailService.selectByKey("piQueryUrl", thirdSysId);
        BasicThirdSysDetailEntity invPiDetailEntity = basicThirdSysDetailService.selectByKey("invPiUrl", thirdSysId);
        if (piDetailEntity == null || piDetailEntity.getValue() == null)
            return ResultVOUtils.error(500, "ERP产品信息接口地址未定义");
        if (invPiDetailEntity == null || invPiDetailEntity.getValue() == null)
            return ResultVOUtils.error(500, "ERP库存产品信息接口地址未定义");
//        updateproductService.(piDetailEntity.getValue(), invPiDetailEntity.getValue(), thirdSysId,true);
        return ResultVOUtils.success("后台已开启下载更新，请稍后刷新查看！");
    }


    //多选删除
    @PostMapping("/udi/udirel/deleteAll")
    public BaseResponse deleteAll(@RequestBody DeleteRequest deleteRequest, BindingResult bindingResult) {

        if (bindingResult.hasErrors()) {
            return ResultVOUtils.error(ResultEnum.PARAM_VERIFY_FALL, bindingResult.getFieldError().getDefaultMessage());
        }
        List<String> ids = deleteRequest.getIds();
        udiRelevanceService.deleteByIds(ids);
        return ResultVOUtils.success("删除成功");
    }

    @PostMapping("/udi/udirel/deleteById")
    public BaseResponse deleteById(@RequestBody DeleteRequest deleteRequest, BindingResult bindingResult) {

        if (bindingResult.hasErrors()) {
            return ResultVOUtils.error(ResultEnum.PARAM_VERIFY_FALL, bindingResult.getFieldError().getDefaultMessage());
        }
        String ids = deleteRequest.getId();
        UdiRelevanceEntity udiRelevanceEntity = udiRelevanceService.selectById(ids);
        if (udiRelevanceEntity != null && udiRelevanceEntity.getUuid() != null) {
            List<UdiRelevanceEntity> udiRelevanceEntities = udiRelevanceService.selectByUuid(udiRelevanceEntity.getUuid());
            if (udiRelevanceEntities != null && !udiRelevanceEntities.isEmpty()) {
                for (UdiRelevanceEntity udiRelevanceEntity1 : udiRelevanceEntities) {
                    List<UdiInfoEntity> udiInfoEntities = udiInfoService.findByUuids(udiRelevanceEntity1.getUuid());
                    if (udiInfoEntities != null && udiInfoEntities.size() > 0) {
                        for (int i = 0; i < udiInfoEntities.size(); i++) {
                            List<WarehouseEntity> warehouseEntityList = warehouseService.searchCode(udiInfoEntities.get(i).getNameCode());
                            if (warehouseEntityList != null && warehouseEntityList.size() > 0) {
                                return ResultVOUtils.error(500, "改产品已被" + warehouseEntityList.get(0).getOrderId() + "订单关联不能删除");
                            }
                        }
                    }
                }
            }
        }
        udiRelevanceService.deleteById(ids);
        List<UdiRelevanceEntity> udiRelevanceEntities = udiRelevanceService.selectByUuid(udiRelevanceEntity.getUuid());
        if (udiRelevanceEntities == null || udiRelevanceEntities.isEmpty()) {
            udiInfoService.deleteByUuid(udiRelevanceEntity.getUuid());
        }
        return ResultVOUtils.success("删除成功");
    }

    @PostMapping("/udi/udirel/update")
    public BaseResponse updateRelevanceEntity(@RequestBody UdiRelevanceEntity udiRelevanceEntity, BindingResult bindingResult) {

        if (bindingResult.hasErrors()) {
            return ResultVOUtils.error(ResultEnum.PARAM_VERIFY_FALL, bindingResult.getFieldError().getDefaultMessage());
        }
        udiRelevanceEntity.setUpdateTime(DateUtil.getDateTime());
        udiRelevanceService.updateUdiRelevance(udiRelevanceEntity);
        return ResultVOUtils.success("更新成功");
    }

    //新增或修改关联
    @PostMapping("/udi/udirel/revleance")
    public BaseResponse revleanceUdi(@RequestBody CombineRequest combineRequest) {

        List<String> keys = combineRequest.getKeys();
        if (keys != null && keys.size() > 0) {
            for (String key : keys) {
                FilterUdiInfoRequest filterUdiInfoRequest = new FilterUdiInfoRequest();
                filterUdiInfoRequest.setUuid(key);
                List<UdiInfoEntity> udiInfoEntities = udiInfoService.filterAllUdiInfo(filterUdiInfoRequest);
                UdiRelevanceEntity udiRelevanceEntity = new UdiRelevanceEntity();
                if (udiInfoEntities == null || udiInfoEntities.isEmpty()) {
                    udiInfoEntities = new UdiDlHttpClient(udiUrl).getUdiByUuid(key);
                    if (udiInfoEntities != null) {
                        udiInfoService.insertUdiInfos(udiInfoEntities);
                    } else {
                        return ResultVOUtils.error(500, "连接UDI数据下载服务出错！");
                    }
                }
                if (combineRequest.getRelId() != null && !combineRequest.getRelId().equals("")) {

                    //判断DI是否被使用，若使用则新增一条
                    UdiRelevanceEntity temp = udiRelevanceService.selectById(combineRequest.getRelId());
                    List<UdiRelevanceEntity> udiRelevanceEntities = udiRelevanceService.selectByUuid(temp.getUuid());
                    if (udiRelevanceEntities != null && !udiRelevanceEntities.isEmpty()) {
                        for (UdiRelevanceEntity udiRelevanceEntity1 : udiRelevanceEntities) {
                            List<UdiInfoEntity> temps = udiInfoService.findByUuids(udiRelevanceEntity1.getUuid());
                            if (temps != null && temps.size() > 0) {
                                for (int i = 0; i < temps.size(); i++) {
                                    List<WarehouseEntity> warehouseEntityList = warehouseService.searchCode(temps.get(i).getNameCode());
                                    if (warehouseEntityList != null && warehouseEntityList.size() > 0) {

                                        //新增一条记录
                                        UdiRelevanceEntity newRlEntity = new UdiRelevanceEntity();
                                        newRlEntity = setThird(combineRequest, newRlEntity);
                                        newRlEntity.setUpdateTime(DateUtil.getDateTime());
                                        newRlEntity.setUuid(key);
                                        udiRelevanceService.insertUdiRelevance(newRlEntity);

                                        return ResultVOUtils.success("合并成功");
                                    }
                                }
                            }
                        }
                    }


                    udiRelevanceEntity.setId(combineRequest.getRelId());
                    udiRelevanceEntity.setUuid(key);
                    udiRelevanceEntity.setUpdateTime(DateUtil.getDateTime());
                    udiRelevanceService.updateUdiRelevance(udiRelevanceEntity);
                } else {
                    udiRelevanceEntity = setThird(combineRequest, udiRelevanceEntity);
                    udiRelevanceEntity.setUpdateTime(DateUtil.getDateTime());
                    udiRelevanceEntity.setUuid(key);
                    udiRelevanceService.insertUdiRelevance(udiRelevanceEntity);
                }


            }
        }
        return ResultVOUtils.success("合并成功");
    }


    //选入查询结果的所有DI
    @GetMapping("/udi/udirel/combineAll")
    public BaseResponse combineAll(FilterUdiInfoRequest productInfoFilterRequest) {
        basicUpdateProductService.relAllUdi(productInfoFilterRequest);
        return ResultVOUtils.success("后台已开始导入，请稍后刷新查看");
    }

    //更换关联ERP产品
    @PostMapping("/udi/udirel/modifyRel")
    public BaseResponse modifyRel(@RequestBody CombineRequest combineRequest) {

        UdiRelevanceEntity udiRelevanceEntity = new UdiRelevanceEntity();
        udiRelevanceEntity.setId(combineRequest.getRelId());
        udiRelevanceEntity = setThird(combineRequest, udiRelevanceEntity);
        udiRelevanceEntity.setUpdateTime(DateUtil.getDateTime());
        udiRelevanceService.updateUdiRelevance(udiRelevanceEntity);
        return ResultVOUtils.success("更新成功");
    }

    //单独选入ERP产品
    @PostMapping("/udi/udirel/addErp")
    public BaseResponse addErp(@RequestBody CombineRequest combineRequest) {
        BasicThirdSysDetailEntity basicThirdSysDetailEntity = basicThirdSysDetailService.selectByKey("piQueryUrl", combineRequest.getThirdSys());
        if (basicThirdSysDetailEntity == null || basicThirdSysDetailEntity.getValue() == null)
            return ResultVOUtils.error(500, "ERP产品信息接口地址未定义");
        String url = basicThirdSysDetailEntity.getValue();
        if (basicThirdSysDetailEntity.getFromType() == 0) {
            String errMsg = basicUpdateProductService.addProduct(combineRequest, url);
            if (errMsg == null)
                return ResultVOUtils.success("添加成功！");
            else {
                return ResultVOUtils.error(500, errMsg);
            }
        } else {
            String errMsg = basicUpdateProductService.addLocalProduct(combineRequest);
            if (errMsg == null)
                return ResultVOUtils.success("添加成功！");
            else {
                return ResultVOUtils.error(500, errMsg);
            }
        }

    }


    //一键选入ERP产品
    @PostMapping("/udi/udirel/addAllErp")
    public BaseResponse addAllErp(@RequestBody CombineRequest combineRequest) {
        BasicThirdSysDetailEntity basicThirdSysDetailEntity = basicThirdSysDetailService.selectByKey("piQueryUrl", combineRequest.getThirdSys());
        if (basicThirdSysDetailEntity == null || basicThirdSysDetailEntity.getValue() == null)
            return ResultVOUtils.error(500, "ERP产品信息接口地址未定义");
        String url = basicThirdSysDetailEntity.getValue();
        BasicThirdSysDetailEntity inQuery = basicThirdSysDetailService.selectByKey("invPiUrl", combineRequest.getThirdSys());
        if (basicThirdSysDetailEntity.getFromType() == 0) {


            if (combineRequest.getIsRlInv() != null && combineRequest.getIsRlInv()) {//是否关联库存
                updateproductService.addAllProducts(combineRequest.getQuery(), basicThirdSysDetailEntity.getValue(), inQuery.getValue(), combineRequest.getThirdSys(), true);
            } else {
                updateproductService.addAllProducts(combineRequest.getQuery(), basicThirdSysDetailEntity.getValue(), inQuery.getValue(), combineRequest.getThirdSys(), null);
            }

        } else {

            if (combineRequest.getIsRlInv() != null && combineRequest.getIsRlInv()) {
                updateproductService.loadLocal(combineRequest.getQuery(), combineRequest.getThirdSys(), inQuery.getValue(), true);
            } else {
                updateproductService.loadLocal(combineRequest.getQuery(), combineRequest.getThirdSys(), inQuery.getValue(), null);
            }

        }
        return ResultVOUtils.success("后台正在添加，请稍后刷新查看！");
    }

    //上传产品信息至供应商管理平台
    @PostMapping("/udiwms/smp/udirl/upload")
    public BaseResponse uploadSMP(@RequestBody CombineRequest combineRequest) {
        SystemParamConfigEntity systemParamConfigEntity = systemParamConfigService.selectByParamKey("smp_uploadProduct_url");
        String postUrl;
        if (systemParamConfigEntity == null)
            return ResultVOUtils.error(500, "上传供应商平台接口地址未定义");
        else
            postUrl = systemParamConfigEntity.getParamValue();
        List<String> ids = combineRequest.getKeys();
        if (ids != null && ids.size() > 0) {
            List<UdiRelevanceResponse> udiRelevanceResponses = new ArrayList<>();
            for (int i = 0; i < ids.size(); i++) {
                FilterUdiInfoRequest filterUdiInfoRequest = new FilterUdiInfoRequest();
                filterUdiInfoRequest.setId(ids.get(i));
                List<UdiRelevanceResponse> temps = udiRelevanceService.filterUdiRelevance(filterUdiInfoRequest);
                if (temps != null && temps.size() > 0) {
                    udiRelevanceResponses.addAll(temps);
                }
            }
            List<PostSmpUdiInfoRequest> postDatas = udiRelevanceResponses.stream().map(item ->
            {
                PostSmpUdiInfoRequest postSmpUdiInfoRequest = new PostSmpUdiInfoRequest();
                BeanUtils.copyProperties(item, postSmpUdiInfoRequest);
                postSmpUdiInfoRequest.setErpId(item.getThirdId());
                postSmpUdiInfoRequest.setErpName(item.getThirdName());
                return postSmpUdiInfoRequest;
            }).collect(Collectors.toList());
            String response = HttpClient.uCloudPost(postUrl, postDatas);
            BaseResponse baseResponse =
                    JSONObject.parseObject(response, new TypeReference<BaseResponse>() {
                    });
            if (baseResponse.getCode() == 20000) {
                return ResultVOUtils.success("上传成功！");
            } else {
                return ResultVOUtils.error(baseResponse.getCode(), baseResponse.getMessage());
            }
        } else {
            return ResultVOUtils.error(500, "未选择产品信息");
        }


    }

    //接收供应商或第三方服务上传产品信息
    @PostMapping("/udiwms/udirl/thirdSys/upload")
    public BaseResponse uploadThirdServcie(@RequestBody PostThirdSysPiRequest postThirdSysPiRequest) {

        List<PostSmpUdiInfoRequest> postSmpUdiInfoRequests = postThirdSysPiRequest.getDatas();
        if (postSmpUdiInfoRequests != null && postSmpUdiInfoRequests.size() > 0) {
            for (PostSmpUdiInfoRequest postSmpUdiInfoRequest : postSmpUdiInfoRequests) {
                UdiRelevanceEntity udiRelevanceEntity = new UdiRelevanceEntity();
                if ("thirdId1".equals(postThirdSysPiRequest.getThirdSys())) {
                    udiRelevanceEntity.setThirdId1(postSmpUdiInfoRequest.getErpId());
                    udiRelevanceEntity.setThirdName1(postSmpUdiInfoRequest.getErpName());
                } else if ("thirdId2".equals(postThirdSysPiRequest.getThirdSys())) {
                    udiRelevanceEntity.setThirdId2(postSmpUdiInfoRequest.getErpId());
                    udiRelevanceEntity.setThirdName2(postSmpUdiInfoRequest.getErpName());
                } else if ("thirdId3".equals(postThirdSysPiRequest.getThirdSys())) {
                    udiRelevanceEntity.setThirdId3(postSmpUdiInfoRequest.getErpId());
                    udiRelevanceEntity.setThirdName3(postSmpUdiInfoRequest.getErpName());
                } else if ("thirdId4".equals(postThirdSysPiRequest.getThirdSys())) {
                    udiRelevanceEntity.setThirdId4(postSmpUdiInfoRequest.getErpId());
                    udiRelevanceEntity.setThirdName4(postSmpUdiInfoRequest.getErpName());
                } else {
                    udiRelevanceEntity.setThirdId(postSmpUdiInfoRequest.getErpId());
                    udiRelevanceEntity.setThirdName(postSmpUdiInfoRequest.getErpName());
                }
                if (postSmpUdiInfoRequest.getUuid() == null) {
                    udiRelevanceEntity.setUuid(CustomUtil.getUUId());
                } else {
                    udiRelevanceEntity.setUuid(postSmpUdiInfoRequest.getUuid());
                }

                udiRelevanceEntity.setUpdateTime(DateUtil.getDateTime());
                UdiInfoEntity udiInfoEntity = new UdiInfoEntity();
                udiInfoEntity.setYbbm(postSmpUdiInfoRequest.getYbbm());
                udiInfoEntity.setSptm(postSmpUdiInfoRequest.getSptm());
                BeanUtils.copyProperties(postSmpUdiInfoRequest, udiInfoEntity);
                udiInfoEntity.setUuid(udiRelevanceEntity.getUuid());
                udiInfoService.insertUdiInfo(udiInfoEntity);
                udiRelevanceService.insertUdiRelevance(udiRelevanceEntity);
            }
        }
        WebSocketServer.sendInfo("产品信息上传解析完成，请刷新查看！", "sid");
        return ResultVOUtils.success("产品信息上传成功！");
    }

    //获取产品信息详情（多平台）
    @GetMapping("/udiwms/udirel/thirdSys/filter")
    public BaseResponse getThirdSysfilter(FilterUdiInfoRequest filterUdiInfoRequest,
                                          BindingResult bindingResult) {

        if (bindingResult.hasErrors()) {
            return ResultVOUtils.error(ResultEnum.PARAM_VERIFY_FALL, bindingResult.getFieldError().getDefaultMessage());
        }
        FilterBasicThirdSysRequest filterBasicThirdSysRequest = new FilterBasicThirdSysRequest();
        filterBasicThirdSysRequest.setEnabled(true);
        List<BasicThirdSysEntity> basicThirdSysEntities = basicThirdSysService.filterBasicThiSys(filterBasicThirdSysRequest);
        List<FilterSysEntity> filterSysEntityList = new ArrayList<>();

        filterSysEntityList.add(new FilterSysEntity(0, "全部"));
        filterSysEntityList.add(new FilterSysEntity(1, "对照完成"));
        filterSysEntityList.add(new FilterSysEntity(2, "未对照DI"));
        if (basicThirdSysEntities != null && basicThirdSysEntities.size() > 0) {
            for (int i = 0; i < basicThirdSysEntities.size(); i++) {
                filterSysEntityList.add(new FilterSysEntity(i + 3, "未对照" + basicThirdSysEntities.get(i).getThirdName()));
            }
        }
        PageInfo<FilterSysEntity> pageInfo;
        pageInfo = new PageInfo<>(filterSysEntityList);
        PageSimpleResponse<FilterSysEntity> pageSimpleResponse = new PageSimpleResponse<>();
        pageSimpleResponse.setTotal(pageInfo.getTotal());
        pageSimpleResponse.setList(filterSysEntityList);
        return ResultVOUtils.success(pageSimpleResponse);
    }


    //获取对照过滤列表
    @GetMapping("/udiwms/udirel/thirdSys/detail")
    public BaseResponse getThirdSysDetail(FilterUdiInfoRequest filterUdiInfoRequest,
                                          BindingResult bindingResult) {

        if (bindingResult.hasErrors()) {
            return ResultVOUtils.error(ResultEnum.PARAM_VERIFY_FALL, bindingResult.getFieldError().getDefaultMessage());
        }
        UdiRelevanceEntity udiRelevanceEntity = udiRelevanceService.selectById(filterUdiInfoRequest.getId());
        FilterBasicThirdSysRequest filterBasicThirdSysRequest = new FilterBasicThirdSysRequest();
        filterBasicThirdSysRequest.setEnabled(true);
        List<BasicThirdSysEntity> basicThirdSysEntities = basicThirdSysService.filterBasicThiSys(filterBasicThirdSysRequest);
        List<BasicProductThirdSysResponse> basicProductThirdSysResponses = new ArrayList<>();

        if (basicThirdSysEntities != null && basicThirdSysEntities.size() > 0) {
            for (BasicThirdSysEntity basicThirdSysEntity : basicThirdSysEntities) {
                if (basicThirdSysEntity.getThirdId().equals("thirdId")) {
                    BasicProductThirdSysResponse basicProductThirdSysResponse = new BasicProductThirdSysResponse();
                    basicProductThirdSysResponse.setThirdId(udiRelevanceEntity.getThirdId());
                    basicProductThirdSysResponse.setThirdName(udiRelevanceEntity.getThirdName());
                    basicProductThirdSysResponse.setSysName(basicThirdSysEntity.getThirdName());
                    basicProductThirdSysResponse.setSysId(basicThirdSysEntity.getThirdId());
                    if (udiRelevanceEntity.getThirdId() != null && !udiRelevanceEntity.getThirdId().isEmpty()) {
                        if (udiRelevanceEntity.getThirdId() != null) {
                            ThrProductsEntity thrProductsEntity = thrDataService.getProducts(udiRelevanceEntity.getThirdId(), "thirdId");
                            if (thrProductsEntity != null) {
                                basicProductThirdSysResponse.setManufactory(thrProductsEntity.getManufactory());
                                basicProductThirdSysResponse.setRegisterNo(thrProductsEntity.getRegisterNo());
                                basicProductThirdSysResponse.setSpec(thrProductsEntity.getSpec());
                            }
                        }

                    }
                    basicProductThirdSysResponses.add(basicProductThirdSysResponse);
                }
                if (basicThirdSysEntity.getThirdId().equals("thirdId1")) {
                    BasicProductThirdSysResponse basicProductThirdSysResponse = new BasicProductThirdSysResponse();
                    basicProductThirdSysResponse.setThirdId(udiRelevanceEntity.getThirdId1());
                    basicProductThirdSysResponse.setThirdName(udiRelevanceEntity.getThirdName1());
                    basicProductThirdSysResponse.setSysName(basicThirdSysEntity.getThirdName());
                    basicProductThirdSysResponse.setSysId(basicThirdSysEntity.getThirdId());
                    if (udiRelevanceEntity.getThirdId1() != null && !udiRelevanceEntity.getThirdId1().isEmpty()) {
                        ThrProductsEntity thrProductsEntity = thrDataService.getProducts(udiRelevanceEntity.getThirdId1(), "thirdId1");
                        if (thrProductsEntity != null) {
                            basicProductThirdSysResponse.setManufactory(thrProductsEntity.getManufactory());
                            basicProductThirdSysResponse.setRegisterNo(thrProductsEntity.getRegisterNo());
                            basicProductThirdSysResponse.setSpec(thrProductsEntity.getSpec());
                        }
                    }
                    basicProductThirdSysResponses.add(basicProductThirdSysResponse);
                }
                if (basicThirdSysEntity.getThirdId().equals("thirdId2")) {
                    BasicProductThirdSysResponse basicProductThirdSysResponse = new BasicProductThirdSysResponse();
                    basicProductThirdSysResponse.setThirdId(udiRelevanceEntity.getThirdId2());
                    basicProductThirdSysResponse.setThirdName(udiRelevanceEntity.getThirdName2());
                    basicProductThirdSysResponse.setSysName(basicThirdSysEntity.getThirdName());
                    basicProductThirdSysResponse.setSysId(basicThirdSysEntity.getThirdId());
                    if (udiRelevanceEntity.getThirdId2() != null && !udiRelevanceEntity.getThirdId2().isEmpty()) {
                        ThrProductsEntity thrProductsEntity = thrDataService.getProducts(udiRelevanceEntity.getThirdId2(), "thirdId2");
                        if (thrProductsEntity != null) {
                            basicProductThirdSysResponse.setManufactory(thrProductsEntity.getManufactory());
                            basicProductThirdSysResponse.setRegisterNo(thrProductsEntity.getRegisterNo());
                            basicProductThirdSysResponse.setSpec(thrProductsEntity.getSpec());
                        }
                    }
                    basicProductThirdSysResponses.add(basicProductThirdSysResponse);
                }
                if (basicThirdSysEntity.getThirdId().equals("thirdId3")) {
                    BasicProductThirdSysResponse basicProductThirdSysResponse = new BasicProductThirdSysResponse();
                    basicProductThirdSysResponse.setThirdId(udiRelevanceEntity.getThirdId3());
                    basicProductThirdSysResponse.setThirdName(udiRelevanceEntity.getThirdName3());
                    basicProductThirdSysResponse.setSysName(basicThirdSysEntity.getThirdName());
                    basicProductThirdSysResponse.setSysId(basicThirdSysEntity.getThirdId());
                    if (udiRelevanceEntity.getThirdId3() != null && !udiRelevanceEntity.getThirdId3().isEmpty()) {
                        ThrProductsEntity thrProductsEntity = thrDataService.getProducts(udiRelevanceEntity.getThirdId3(), "thirdId3");
                        if (thrProductsEntity != null) {
                            basicProductThirdSysResponse.setManufactory(thrProductsEntity.getManufactory());
                            basicProductThirdSysResponse.setRegisterNo(thrProductsEntity.getRegisterNo());
                            basicProductThirdSysResponse.setSpec(thrProductsEntity.getSpec());
                        }
                    }

                    basicProductThirdSysResponses.add(basicProductThirdSysResponse);
                }
                if (basicThirdSysEntity.getThirdId().equals("thirdId4")) {
                    BasicProductThirdSysResponse basicProductThirdSysResponse = new BasicProductThirdSysResponse();
                    basicProductThirdSysResponse.setThirdId(udiRelevanceEntity.getThirdId4());
                    basicProductThirdSysResponse.setThirdName(udiRelevanceEntity.getThirdName4());
                    basicProductThirdSysResponse.setSysName(basicThirdSysEntity.getThirdName());
                    basicProductThirdSysResponse.setSysId(basicThirdSysEntity.getThirdId());
                    if (udiRelevanceEntity.getThirdId4() != null && !udiRelevanceEntity.getThirdId4().isEmpty()) {
                        ThrProductsEntity thrProductsEntity = thrDataService.getProducts(udiRelevanceEntity.getThirdId4(), "thirdId4");
                        if (thrProductsEntity != null) {
                            basicProductThirdSysResponse.setManufactory(thrProductsEntity.getManufactory());
                            basicProductThirdSysResponse.setRegisterNo(thrProductsEntity.getRegisterNo());
                            basicProductThirdSysResponse.setSpec(thrProductsEntity.getSpec());
                        }
                    }

                    basicProductThirdSysResponses.add(basicProductThirdSysResponse);
                }
            }
        }
        PageInfo<BasicProductThirdSysResponse> pageInfo;
        pageInfo = new PageInfo<>(basicProductThirdSysResponses);
        PageSimpleResponse<BasicProductThirdSysResponse> pageSimpleResponse = new PageSimpleResponse<>();
        pageSimpleResponse.setTotal(pageInfo.getTotal());
        pageSimpleResponse.setList(basicProductThirdSysResponses);
        return ResultVOUtils.success(pageSimpleResponse);
    }

    //解除绑定
    @GetMapping("/udiwms/udirl/thirdSys/remove")
    public BaseResponse uploadThirdServcie(RemoveRelRequest removeRelRequest) {
        UdiRelevanceEntity udiRelevanceEntity = udiRelevanceService.selectById(removeRelRequest.getRelId());
        List<UdiRelevanceEntity> udiRelevanceEntities = udiRelevanceService.selectByUuid(udiRelevanceEntity.getUuid());
        if (udiRelevanceEntities != null && udiRelevanceEntities.size() > 0) {
            for (UdiRelevanceEntity udiRelevanceEntity1 : udiRelevanceEntities) {
                if (udiRelevanceEntity1.getThirdId() == null && udiRelevanceEntity1.getThirdId1() == null &&
                        udiRelevanceEntity1.getThirdId2() == null && udiRelevanceEntity1.getThirdId3() == null && udiRelevanceEntity1.getThirdId4() == null) {
                    return ResultVOUtils.error(500, "解除绑定失败！");
                }
            }
        }
        if (removeRelRequest.getThirdSys().equals("thirdId")) {
            udiRelevanceEntity.setThirdId("");
            udiRelevanceEntity.setThirdName("");
        } else if (removeRelRequest.getThirdSys().equals("thirdId1")) {
            udiRelevanceEntity.setThirdId1("");
            udiRelevanceEntity.setThirdName1("");
        } else if (removeRelRequest.getThirdSys().equals("thirdId2")) {
            udiRelevanceEntity.setThirdId2("");
            udiRelevanceEntity.setThirdName2("");
        } else if (removeRelRequest.getThirdSys().equals("thirdId3")) {
            udiRelevanceEntity.setThirdId3("");
            udiRelevanceEntity.setThirdName3("");
        } else if (removeRelRequest.getThirdSys().equals("thirdId4")) {
            udiRelevanceEntity.setThirdId4("");
            udiRelevanceEntity.setThirdName4("");
        }
        try {
            udiRelevanceService.updateUdiRelevance(udiRelevanceEntity);
        } catch (Exception e) {
            e.printStackTrace();
            return ResultVOUtils.error(500, "解除绑定失败！");
        }
        return ResultVOUtils.success("解除绑定成功！");
    }

    public UdiRelevanceEntity setThird(CombineRequest combineRequest, UdiRelevanceEntity udiRelevanceEntity) {
        if ("thirdId".equals(combineRequest.getThirdSys())) {
            udiRelevanceEntity.setThirdId(combineRequest.getThirdId());
            udiRelevanceEntity.setThirdName(combineRequest.getErpName());
        } else if ("thirdId1".equals(combineRequest.getThirdSys())) {
            udiRelevanceEntity.setThirdId1(combineRequest.getThirdId());
            udiRelevanceEntity.setThirdName1(combineRequest.getErpName());
        } else if ("thirdId2".equals(combineRequest.getThirdSys())) {
            udiRelevanceEntity.setThirdId2(combineRequest.getThirdId());
            udiRelevanceEntity.setThirdName2(combineRequest.getErpName());
        } else if ("thirdId3".equals(combineRequest.getThirdSys())) {
            udiRelevanceEntity.setThirdId3(combineRequest.getThirdId());
            udiRelevanceEntity.setThirdName3(combineRequest.getErpName());
        } else if ("thirdId4".equals(combineRequest.getThirdSys())) {
            udiRelevanceEntity.setThirdId4(combineRequest.getThirdId());
            udiRelevanceEntity.setThirdName4(combineRequest.getErpName());
        }
        if (combineRequest.getErpProductsResponse() != null) {
            udiRelevanceEntity.setMeasname(combineRequest.getErpProductsResponse().getMeasname());
            udiRelevanceEntity.setManufactory(combineRequest.getErpProductsResponse().getManufactory());
        }

        return udiRelevanceEntity;
    }

    public BasicThirdSysEntity filterThirdSys(String thirdId, List<BasicThirdSysEntity> basicThirdSysEntities) {
        if (basicThirdSysEntities != null && basicThirdSysEntities.size() > 0) {
            for (BasicThirdSysEntity basicThirdSysEntity : basicThirdSysEntities) {
                if (thirdId.equals(basicThirdSysEntity.getThirdId())) {
                    return basicThirdSysEntity;
                }
            }
        }
        return null;
    }
}
