package com.glxp.api.admin.controller.info;

import com.github.pagehelper.PageInfo;
import com.glxp.api.admin.annotation.AuthRuleAnnotation;
import com.glxp.api.admin.entity.info.SystemPDFModuleEntity;
import com.glxp.api.admin.entity.info.SystemPDFTemplateEntity;
import com.glxp.api.admin.req.info.SystemPDFModuleRequest;
import com.glxp.api.admin.req.info.SystemPDFModuleSaveRequest;
import com.glxp.api.admin.res.PageSimpleResponse;
import com.glxp.api.admin.res.info.SystemPDFModuleResponse;
import com.glxp.api.admin.res.info.SystemPDFModuleSelectResponse;
import com.glxp.api.admin.service.info.SystemPDFModuleService;
import com.glxp.api.admin.service.info.SystemPDFTemplateService;
import com.glxp.api.common.enums.ResultEnum;
import com.glxp.api.common.res.BaseResponse;
import com.glxp.api.common.util.ResultVOUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.validation.BindingResult;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import javax.validation.Valid;
import java.util.Date;
import java.util.List;
import java.util.stream.Collectors;

@RestController
public class SystemPDFModuleController {

    @Resource
    private SystemPDFModuleService systemPDFModuleService;
    @Resource
    private SystemPDFTemplateService systemPDFTemplateService;

    @AuthRuleAnnotation("system/pdf/module/list")
    @GetMapping("/system/pdf/module/list")
    public BaseResponse list(@Valid SystemPDFModuleRequest systemPDFModuleRequest,
                             BindingResult bindingResult) {

        if (bindingResult.hasErrors()) {
            return ResultVOUtils.error(ResultEnum.PARAM_VERIFY_FALL, bindingResult.getFieldError().getDefaultMessage());
        }

        List<SystemPDFModuleEntity> systemPDFModuleEntityList = systemPDFModuleService.queryPage(systemPDFModuleRequest);
        List<SystemPDFModuleResponse> systemPDFModuleResponseList = systemPDFModuleEntityList.stream().map(item -> {
            SystemPDFModuleResponse systemPDFModuleResponse = new SystemPDFModuleResponse();
            BeanUtils.copyProperties(item, systemPDFModuleResponse);
            if (item.getTemplateId() != 0) {
                SystemPDFTemplateEntity systemPDFTemplateEntity =
                        systemPDFTemplateService.selectById(String.valueOf(item.getTemplateId()));
                if (systemPDFTemplateEntity != null) {
                    systemPDFModuleResponse.setTemplateName(systemPDFTemplateEntity.getName());
                }
            }
            return systemPDFModuleResponse;
        }).collect(Collectors.toList());

        PageInfo<SystemPDFModuleEntity> pageInfo = new PageInfo<>(systemPDFModuleEntityList);
        PageSimpleResponse<SystemPDFModuleResponse> pageSimpleResponse = new PageSimpleResponse<>();
        pageSimpleResponse.setTotal(pageInfo.getTotal());
        pageSimpleResponse.setList(systemPDFModuleResponseList);
        return ResultVOUtils.success(pageSimpleResponse);
    }

    @AuthRuleAnnotation("system/pdf/module/list")
    @GetMapping("/system/pdf/module/pdfModules")
    public BaseResponse pdfModules(@RequestParam(value = "page", defaultValue = "1") Integer page,
                                     @RequestParam(value = "limit", defaultValue = "100") Integer limit) {

        List<SystemPDFModuleEntity> systemPDFModuleEntityList = systemPDFModuleService.listPDFModules(page, limit);
        PageInfo<SystemPDFModuleEntity> pageInfo = new PageInfo<>(systemPDFModuleEntityList);
        PageSimpleResponse<SystemPDFModuleSelectResponse> pageSimpleResponse = new PageSimpleResponse<>();
        pageSimpleResponse.setTotal(pageInfo.getTotal());
        List<SystemPDFModuleSelectResponse> systemPDFModuleSelectResponses = systemPDFModuleEntityList.stream().map(e -> {
            SystemPDFModuleSelectResponse systemPDFModuleSelectResponse = new SystemPDFModuleSelectResponse();
            BeanUtils.copyProperties(e, systemPDFModuleSelectResponse);
            systemPDFModuleSelectResponse.setIdStr(String.valueOf(e.getId()));
            return systemPDFModuleSelectResponse;
        }).collect(Collectors.toList());
        pageSimpleResponse.setList(systemPDFModuleSelectResponses);

        return ResultVOUtils.success(pageSimpleResponse);
    }

    @AuthRuleAnnotation("system/pdf/module/save")
    @PostMapping("/system/pdf/module/save")
    public BaseResponse save(@RequestBody @Valid SystemPDFModuleSaveRequest systemPDFModuleSaveRequest,
                             BindingResult bindingResult) {

        if (bindingResult.hasErrors()) {
            return ResultVOUtils.error(ResultEnum.PARAM_VERIFY_FALL, bindingResult.getFieldError().getDefaultMessage());
        }

        SystemPDFModuleEntity systemPDFModuleEntity = new SystemPDFModuleEntity();
        BeanUtils.copyProperties(systemPDFModuleSaveRequest, systemPDFModuleEntity);

        systemPDFModuleEntity.setCreate_time(new Date());
        systemPDFModuleEntity.setUpdate_time(new Date());

        boolean b = systemPDFModuleService.insert(systemPDFModuleEntity);
        if (!b) {
            return ResultVOUtils.error(ResultEnum.NOT_NETWORK);
        }

        return ResultVOUtils.success();
    }

    @AuthRuleAnnotation("system/pdf/module/update")
    @PostMapping("/system/pdf/module/update")
    public BaseResponse update(@RequestBody SystemPDFModuleSaveRequest systemPDFModuleSaveRequest) {
        systemPDFModuleSaveRequest.setUpdate_time(new Date());
        systemPDFModuleService.updateById(systemPDFModuleSaveRequest);

        return ResultVOUtils.success("修改成功");
    }

    @AuthRuleAnnotation("system/pdf/module/delete")
    @GetMapping("/system/pdf/module/delete")
    public BaseResponse deleteById(@RequestParam("id") Integer id) {


        boolean b = systemPDFModuleService.deleteById(id);
        if (b)
            return ResultVOUtils.success("删除成功");
        else {
            return ResultVOUtils.error(500, "删除失败");
        }
    }
}
