package com.glxp.api.admin.controller.info;

import com.github.pagehelper.PageInfo;
import com.glxp.api.admin.annotation.AuthRuleAnnotation;
import com.glxp.api.admin.entity.info.SystemPDFTemplateEntity;
import com.glxp.api.admin.req.info.SystemPDFTemplateRequest;
import com.glxp.api.admin.req.info.SystemPDFTemplateSaveRequest;
import com.glxp.api.admin.res.PageSimpleResponse;
import com.glxp.api.admin.res.info.SystemPDFTemplateResponse;
import com.glxp.api.admin.res.info.SystemPDFTemplateSelectResponse;
import com.glxp.api.admin.service.info.SystemPDFTemplateService;
import com.glxp.api.common.enums.ResultEnum;
import com.glxp.api.common.res.BaseResponse;
import com.glxp.api.common.util.ResultVOUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.validation.BindingResult;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import javax.validation.Valid;
import java.io.File;
import java.util.Date;
import java.util.List;
import java.util.stream.Collectors;

@RestController
public class SystemPDFTemplateController {

    @Resource
    private SystemPDFTemplateService systemPDFTemplateService;

    @AuthRuleAnnotation("system/pdf/template/list")
    @GetMapping("/system/pdf/template/list")
    public BaseResponse list(@Valid SystemPDFTemplateRequest systemPDFTemplateRequest,
                             BindingResult bindingResult) {

        if (bindingResult.hasErrors()) {
            return ResultVOUtils.error(ResultEnum.PARAM_VERIFY_FALL, bindingResult.getFieldError().getDefaultMessage());
        }

        systemPDFTemplateRequest.setType(2);
        List<SystemPDFTemplateEntity> systemPDFTemplateEntityList = systemPDFTemplateService.queryPage(systemPDFTemplateRequest);
        List<SystemPDFTemplateResponse> systemPDFTemplateResponseList = systemPDFTemplateEntityList.stream().map(item -> {
            SystemPDFTemplateResponse systemPDFTemplateResponse = new SystemPDFTemplateResponse();
            BeanUtils.copyProperties(item, systemPDFTemplateResponse);
            return systemPDFTemplateResponse;
        }).collect(Collectors.toList());

        PageInfo<SystemPDFTemplateEntity> pageInfo = new PageInfo<>(systemPDFTemplateEntityList);
        PageSimpleResponse<SystemPDFTemplateResponse> pageSimpleResponse = new PageSimpleResponse<>();
        pageSimpleResponse.setTotal(pageInfo.getTotal());
        pageSimpleResponse.setList(systemPDFTemplateResponseList);
        return ResultVOUtils.success(pageSimpleResponse);
    }

    @AuthRuleAnnotation("system/pdf/template/list")
    @GetMapping("/system/pdf/template/pdfTemplates")
    public BaseResponse pdfTemplates(@RequestParam(value = "page", defaultValue = "1") Integer page,
                                     @RequestParam(value = "limit", defaultValue = "100") Integer limit) {

        List<SystemPDFTemplateEntity> systemPDFTemplateEntityList = systemPDFTemplateService.listPDFTemplates(page, limit);
        PageInfo<SystemPDFTemplateEntity> pageInfo = new PageInfo<>(systemPDFTemplateEntityList);
        PageSimpleResponse<SystemPDFTemplateSelectResponse> pageSimpleResponse = new PageSimpleResponse<>();
        pageSimpleResponse.setTotal(pageInfo.getTotal());
        List<SystemPDFTemplateSelectResponse> systemPDFTemplateSelectResponses = systemPDFTemplateEntityList.stream().map(e -> {
            SystemPDFTemplateSelectResponse systemPDFTemplateSelectResponse = new SystemPDFTemplateSelectResponse();
            BeanUtils.copyProperties(e, systemPDFTemplateSelectResponse);
            systemPDFTemplateSelectResponse.setIdStr(String.valueOf(e.getId()));
            return systemPDFTemplateSelectResponse;
        }).collect(Collectors.toList());
        pageSimpleResponse.setList(systemPDFTemplateSelectResponses);

        return ResultVOUtils.success(pageSimpleResponse);
    }

    @AuthRuleAnnotation("system/pdf/template/save")
    @PostMapping("/system/pdf/template/save")
    public BaseResponse save(@RequestBody @Valid SystemPDFTemplateSaveRequest systemPDFTemplateSaveRequest,
                             BindingResult bindingResult) {

        if (bindingResult.hasErrors()) {
            return ResultVOUtils.error(ResultEnum.PARAM_VERIFY_FALL, bindingResult.getFieldError().getDefaultMessage());
        }

        SystemPDFTemplateEntity systemPDFTemplateEntity = new SystemPDFTemplateEntity();
        BeanUtils.copyProperties(systemPDFTemplateSaveRequest, systemPDFTemplateEntity);

        systemPDFTemplateEntity.setCreate_time(new Date());
        systemPDFTemplateEntity.setUpdate_time(new Date());

        boolean b = systemPDFTemplateService.insert(systemPDFTemplateEntity);
        if (!b) {
            return ResultVOUtils.error(ResultEnum.NOT_NETWORK);
        }

        return ResultVOUtils.success();
    }

    @AuthRuleAnnotation("system/pdf/template/update")
    @PostMapping("/system/pdf/template/update")
    public BaseResponse update(@RequestBody SystemPDFTemplateSaveRequest systemPDFTemplateSaveRequest) {
//        systemPDFTemplateSaveRequest.setId(1);
        systemPDFTemplateSaveRequest.setUpdate_time(new Date());
        systemPDFTemplateService.updateById(systemPDFTemplateSaveRequest);

        return ResultVOUtils.success("修改成功");
    }

    @AuthRuleAnnotation("system/pdf/template/delete")
    @GetMapping("/system/pdf/template/delete")
    public BaseResponse deleteById(@RequestParam("id") Integer id,
                                   @RequestParam("path") String path) {

        if (StringUtils.isNotBlank(path)) {//删除文件
            File file = new File(path);
            file.delete();
        }

        boolean b = systemPDFTemplateService.deleteById(id);
        if (b)
            return ResultVOUtils.success("删除成功");
        else {
            return ResultVOUtils.error(500, "删除失败");
        }
    }
}
