package com.glxp.api.admin.controller.info;

import com.github.pagehelper.PageInfo;
import com.glxp.api.admin.annotation.AuthRuleAnnotation;
import com.glxp.api.admin.entity.info.SystemParamConfigEntity;
import com.glxp.api.admin.req.info.SystemParamConfigRequest;
import com.glxp.api.admin.req.info.SystemParamConfigSaveRequest;
import com.glxp.api.admin.res.PageSimpleResponse;
import com.glxp.api.admin.res.info.SystemParamConfigResponse;
import com.glxp.api.admin.service.info.SystemParamConfigService;
import com.glxp.api.admin.thread.IoModifyErpTask;
import com.glxp.api.common.enums.ResultEnum;
import com.glxp.api.common.res.BaseResponse;
import com.glxp.api.common.util.ResultVOUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.validation.BindingResult;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RestController;

import javax.annotation.Resource;
import javax.validation.Valid;
import java.util.List;
import java.util.stream.Collectors;

@RestController
public class SystemParamConfigController {

    @Resource
    private SystemParamConfigService systemParamConfigService;

    @AuthRuleAnnotation("system/param/config/list")
    @GetMapping("/system/param/config/list")
    public BaseResponse list(@Valid SystemParamConfigRequest systemParamConfigRequest,
                             BindingResult bindingResult) {

        if (bindingResult.hasErrors()) {
            return ResultVOUtils.error(ResultEnum.PARAM_VERIFY_FALL, bindingResult.getFieldError().getDefaultMessage());
        }

        systemParamConfigRequest.setParamStatus(1);
        List<SystemParamConfigEntity> systemParamConfigEntityList = systemParamConfigService.queryPage(systemParamConfigRequest);
        List<SystemParamConfigResponse> systemParamConfigResponseList = systemParamConfigEntityList.stream().map(item -> {
            SystemParamConfigResponse systemParamConfigResponse = new SystemParamConfigResponse();
            BeanUtils.copyProperties(item, systemParamConfigResponse);
            return systemParamConfigResponse;
        }).collect(Collectors.toList());

        PageInfo<SystemParamConfigEntity> pageInfo = new PageInfo<>(systemParamConfigEntityList);
        PageSimpleResponse<SystemParamConfigResponse> pageSimpleResponse = new PageSimpleResponse<>();
        pageSimpleResponse.setTotal(pageInfo.getTotal());
        pageSimpleResponse.setList(systemParamConfigResponseList);
        return ResultVOUtils.success(pageSimpleResponse);
    }

    //key搜索
    @GetMapping("/udiwms/sys/config/selectByKey")
    public BaseResponse selectByKey(@Valid SystemParamConfigRequest systemParamConfigRequest,
                                    BindingResult bindingResult) {

        if (bindingResult.hasErrors()) {
            return ResultVOUtils.error(ResultEnum.PARAM_VERIFY_FALL, bindingResult.getFieldError().getDefaultMessage());
        }
        SystemParamConfigEntity systemParamConfigEntity = systemParamConfigService.selectByParamKey(systemParamConfigRequest.getParamKey());
        return ResultVOUtils.success(systemParamConfigEntity);
    }


    @AuthRuleAnnotation("system/param/config/save")
    @PostMapping("/system/param/config/save")
    public BaseResponse save(@RequestBody @Valid SystemParamConfigSaveRequest systemParamConfigSaveRequest,
                             BindingResult bindingResult) {

        if (bindingResult.hasErrors()) {
            return ResultVOUtils.error(ResultEnum.PARAM_VERIFY_FALL, bindingResult.getFieldError().getDefaultMessage());
        }

        SystemParamConfigEntity systemParamConfigEntity = new SystemParamConfigEntity();
        BeanUtils.copyProperties(systemParamConfigSaveRequest, systemParamConfigEntity);

        boolean b = systemParamConfigService.insert(systemParamConfigEntity);
        if (!b) {
            return ResultVOUtils.error(ResultEnum.NOT_NETWORK);
        }

        return ResultVOUtils.success();
    }

    @AuthRuleAnnotation("system/param/config/update")
    @PostMapping("/system/param/config/update")
    public BaseResponse update(@RequestBody SystemParamConfigSaveRequest systemParamConfigSaveRequest) {
        systemParamConfigService.updateById(systemParamConfigSaveRequest);
        if (systemParamConfigSaveRequest.getParentId() == 0) {
            SystemParamConfigSaveRequest systemParamConfigSaveRequest1 = new SystemParamConfigSaveRequest();
            systemParamConfigSaveRequest1.setParentId(systemParamConfigSaveRequest.getId());
            if (systemParamConfigSaveRequest.getParamValue().equals("1")) {
                systemParamConfigSaveRequest1.setParamStatus(1);
            } else {
                systemParamConfigSaveRequest1.setParamStatus(0);
            }
            systemParamConfigService.updateParentId(systemParamConfigSaveRequest1);
        }

        //toDo  //各种设置所需执行操作

        return ResultVOUtils.success("修改成功");
    }

}
