package com.glxp.api.admin.controller.inout;

import com.github.pagehelper.PageInfo;
import com.glxp.api.admin.constant.ConstantStatus;
import com.glxp.api.admin.controller.inout.utils.ContrastErpUtil;
import com.glxp.api.admin.controller.inout.utils.InoutUtils;
import com.glxp.api.admin.entity.basic.BasicThirdSysDetailEntity;
import com.glxp.api.admin.entity.info.SystemParamConfigEntity;
import com.glxp.api.admin.httpclient.ErpOrderClient;
import com.glxp.api.admin.entity.inout.OrderEntity;
import com.glxp.api.admin.entity.inout.WarehouseEntity;
import com.glxp.api.admin.entity.inout.ErpOrderEntity;
import com.glxp.api.admin.entity.inout.FilterErpOrderRequest;
import com.glxp.api.admin.req.inout.*;
import com.glxp.api.admin.res.PageSimpleResponse;
import com.glxp.api.admin.res.basic.BussinessTypResponse;
import com.glxp.api.admin.service.basic.BussinessTypeService;
import com.glxp.api.admin.service.basic.UdiInfoService;
import com.glxp.api.admin.service.basic.UdiRelevanceService;
import com.glxp.api.admin.service.info.SystemParamConfigService;
import com.glxp.api.admin.service.inout.OrderDetailService;
import com.glxp.api.admin.service.inout.OrderService;
import com.glxp.api.admin.service.inout.CodesService;
import com.glxp.api.admin.service.thrsys.ThrOrderService;
import com.glxp.api.common.enums.ResultEnum;
import com.glxp.api.common.res.BaseResponse;
import com.glxp.api.common.util.ResultVOUtils;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.validation.BindingResult;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RestController;

import javax.annotation.Resource;
import javax.validation.Valid;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.TreeSet;
import java.util.stream.Collectors;

@RestController
public class OrderController {

    @Resource
    private OrderService orderService;
    @Resource
    private CodesService warehouseService;
    @Resource
    private OrderDetailService myErpOrderService;
    @Resource
    private UdiInfoService udiInfoService;
    @Resource
    private UdiRelevanceService udiRelevanceService;
    @Resource
    private SystemParamConfigService systemParamConfigService;
    @Resource
    private BussinessTypeService bussinessTypeService;
    @Resource
    private ThrOrderService thrOrderService;


    @GetMapping("udiwms/inout/order/status")
    public BaseResponse listOrderError(OrderFilterRequest orderFilterRequest) {
        List<OrderEntity> orderEntityList = orderService.filterList(orderFilterRequest);
        PageInfo<OrderEntity> pageInfo;
        pageInfo = new PageInfo<>(orderEntityList);
        PageSimpleResponse<OrderEntity> pageSimpleResponse = new PageSimpleResponse<>();
        pageSimpleResponse.setTotal(pageInfo.getTotal());
        pageSimpleResponse.setList(orderEntityList);
        return ResultVOUtils.success(pageSimpleResponse);
    }

    @GetMapping("udiwms/inout/order/repeat")
    public BaseResponse listOrderRepeat(OrderQueryRequest orderQueryRequest) {
        List<OrderEntity> orderEntityList = orderService.listOrderRepeat(orderQueryRequest);
        PageInfo<OrderEntity> pageInfo;
        pageInfo = new PageInfo<>(orderEntityList);
        PageSimpleResponse<OrderEntity> pageSimpleResponse = new PageSimpleResponse<>();
        pageSimpleResponse.setTotal(pageInfo.getTotal());
        pageSimpleResponse.setList(orderEntityList);
        return ResultVOUtils.success(pageSimpleResponse);
    }

    @PostMapping("udiwms/inout/order/delete")
    public BaseResponse deleteOrder(@RequestBody DeleteRequest deleteRequest, BindingResult bindingResult) {

        if (bindingResult.hasErrors()) {
            return ResultVOUtils.error(ResultEnum.PARAM_VERIFY_FALL, bindingResult.getFieldError().getDefaultMessage());
        }
        String id = deleteRequest.getId();

        orderService.deleteById(id);
        myErpOrderService.deleteByOrderId(id);
        return ResultVOUtils.success("删除成功");
    }

    @PostMapping("/udiwms/inout/order/updateUnit")
    public BaseResponse updateUnit(@RequestBody @Valid OrderEntity orderEntity,
                                   BindingResult bindingResult) {

        if (bindingResult.hasErrors()) {
            return ResultVOUtils.error(ResultEnum.PARAM_VERIFY_FALL, bindingResult.getFieldError().getDefaultMessage());
        }

        orderService.updateUnit(orderEntity);
        warehouseService.updateUnit(orderEntity.getId(), orderEntity.getFromCorpId(), orderEntity.getFromCorp());
        return ResultVOUtils.success("添加成功");
    }

    @GetMapping("/udiwms/inout/order/filter")
    public BaseResponse filterOrder(OrderFilterRequest orderFilterRequest,
                                    BindingResult bindingResult) {

        if (bindingResult.hasErrors()) {
            return ResultVOUtils.error(ResultEnum.PARAM_VERIFY_FALL, bindingResult.getFieldError().getDefaultMessage());
        }
        List<OrderEntity> orderEntityList;
        if (orderFilterRequest.getCode() != null && !orderFilterRequest.getCode().equals("")) {
            List<WarehouseEntity> warehouseEntityList = warehouseService.findByCode(orderFilterRequest.getCode());
            orderEntityList = orderService.filterListByCode(warehouseEntityList);
        } else {
            orderEntityList = orderService.filterList(orderFilterRequest);
        }
        PageInfo<OrderEntity> pageInfo;
        pageInfo = new PageInfo<>(orderEntityList);
        PageSimpleResponse<OrderEntity> pageSimpleResponse = new PageSimpleResponse<>();
        pageSimpleResponse.setTotal(pageInfo.getTotal());
        pageSimpleResponse.setList(orderEntityList);
        return ResultVOUtils.success(pageSimpleResponse);
    }

    @GetMapping("/udiwms/inout/order/filterCode")
    public BaseResponse filterOrderByCode(OrderFilterByCodeRequest orderFilterByCodeRequest,
                                          BindingResult bindingResult) {

        if (bindingResult.hasErrors()) {
            return ResultVOUtils.error(ResultEnum.PARAM_VERIFY_FALL, bindingResult.getFieldError().getDefaultMessage());
        }
        List<WarehouseEntity> warehouseEntityList = warehouseService.findByCode(orderFilterByCodeRequest.getCode());
        List<OrderEntity> orderEntityList = orderService.filterListByCode(warehouseEntityList);
        PageInfo<OrderEntity> pageInfo;
        pageInfo = new PageInfo<>(orderEntityList);
        PageSimpleResponse<OrderEntity> pageSimpleResponse = new PageSimpleResponse<>();
        pageSimpleResponse.setTotal(pageInfo.getTotal());
        pageSimpleResponse.setList(orderEntityList);
        return ResultVOUtils.success(pageSimpleResponse);
    }

    @PostMapping("/udiwms/inout/order/updateExportStatus")
    public BaseResponse updateExportStatus(@RequestBody UpdateExportStatusRequest updateExportStatusRequest,
                                           BindingResult bindingResult) {

        if (bindingResult.hasErrors()) {
            return ResultVOUtils.error(ResultEnum.PARAM_VERIFY_FALL, bindingResult.getFieldError().getDefaultMessage());
        }
        boolean b = orderService.updateExportStatus(updateExportStatusRequest.getOrderId(), updateExportStatusRequest.getExportStatus());
        if (!b) {
            return ResultVOUtils.error(ResultEnum.NOT_NETWORK);
        }
        return ResultVOUtils.success("更新成功");
    }

    @PostMapping("/warehouse/inout/order/mergeOrder")
    public BaseResponse mergeOrder(@RequestBody OrderMergeRequest orderMergeRequest, BindingResult bindingResult) {

        if (bindingResult.hasErrors()) {
            return ResultVOUtils.error(ResultEnum.PARAM_VERIFY_FALL, bindingResult.getFieldError().getDefaultMessage());
        }

        List<String> orderList = orderMergeRequest.getOrderList();
        List<OrderEntity> mergeList = new ArrayList<>();
        if (orderList != null && orderList.size() > 1) {
            for (String orderId : orderList) {
                FilterErpOrderRequest filterErpOrderRequest = new FilterErpOrderRequest();
                filterErpOrderRequest.setOrderId(orderId);
                OrderEntity orderEntity = orderService.findById(orderId);
                if (orderEntity != null) {
                    mergeList.add(orderEntity);
                }
                myErpOrderService.deleteByOrderId(orderId);

            }
            String mainOrder = orderList.get(0);
            for (int i = 1; i < orderList.size(); i++) {
                warehouseService.updateOrderId(orderList.get(i), mainOrder);
                orderService.deleteByOrderId(orderList.get(i));
            }
            OrderEntity mainOrderEntity = orderService.findById(mainOrder);
            List<String> docids = getDocids(mergeList);
            String docidStr = "";
            if (docids != null && docids.size() > 0) {
                for (String temp : docids) {
                    docidStr = docidStr + "," + temp;
                }
                docidStr = docidStr.substring(1);
            }
            mainOrderEntity.setErpFk(docidStr);
            orderService.updateOrder(mainOrderEntity);
            if (docidStr.length() > 0) {
                BasicThirdSysDetailEntity basicThirdSysDetailEntity = bussinessTypeService.findByActionKey(mainOrderEntity.getAction(), "orderQueryUrl");
                if (basicThirdSysDetailEntity == null) {
                    return ResultVOUtils.error(500, "ERP出入库单据查询接口地址未定义");
                }
                List<ErpOrderEntity> erpOrderEntities = null;
                if(basicThirdSysDetailEntity.getEnabled()){
                    if ( basicThirdSysDetailEntity.getFromType() == 0 ) {
                        erpOrderEntities = new ErpOrderClient(basicThirdSysDetailEntity.getValue()).getErpOrder(docids, mainOrderEntity.getAction());
                    } else {
                        erpOrderEntities = thrOrderService.filterAllOrders(docids, basicThirdSysDetailEntity.getThirdSysFk());
                    }
                }else {
                    return ResultVOUtils.error(500, "第三方业务单据服务未启用！");
                }


                if (erpOrderEntities != null && erpOrderEntities.size() > 0) {
                    List<ErpOrderEntity> returnOrders = new ContrastErpUtil().transErp(erpOrderEntities, udiRelevanceService, mainOrder);
                    myErpOrderService.insertErpOrders(returnOrders);
                    List<WarehouseEntity> warehouseEntityList = warehouseService.findByReceiptId(mainOrder);
                    new ContrastErpUtil().contrastErp(udiInfoService, udiRelevanceService, myErpOrderService, orderService, warehouseEntityList, systemParamConfigService);
                } else {
                    mainOrderEntity.setContrastStatus(ConstantStatus.ORDER_CHECK_FAIL);
                    mainOrderEntity.setRemark("获取ERP单失败");
                    orderService.updateOrder(mainOrderEntity);
                    List<WarehouseEntity> datas = warehouseService.findByReceiptId(mainOrder);
                    new InoutUtils().generateConstrastDetail(udiInfoService, udiRelevanceService, myErpOrderService, datas);
                    return ResultVOUtils.error(500, "获取ERP单失败");
                }
            } else {
                new Thread(() -> {
                    List<WarehouseEntity> datas = warehouseService.findByReceiptId(mainOrder);
                    new InoutUtils().generateConstrastDetail(udiInfoService, udiRelevanceService, myErpOrderService, datas);
                }).start();
            }

        } else {
            return ResultVOUtils.error(ResultEnum.DATA_ERROR);
        }


        return ResultVOUtils.success("合并成功,后台正在数据转换，请稍后刷新重试！");
    }


    public List<String> getDocids(List<OrderEntity> mergeList) {
        List<String> docids = new ArrayList<>();
        for (int i = 0; i < mergeList.size(); i++) {
            OrderEntity orderEntity = mergeList.get(i);
            if (orderEntity.getErpFk() != null && !"".equals(orderEntity.getErpFk())) {
                String[] erpFks = orderEntity.getErpFk().split(",");
                if (erpFks.length > 0) {
                    for (int j = 0; j < erpFks.length; j++) {
                        docids.add(erpFks[j]);
                    }
                }
            }
        }
        if (docids.size() > 1) {
            return new ArrayList<>(new TreeSet<>(docids));
        } else
            return docids;

    }


}
