package com.glxp.api.admin.controller.inout;

import com.github.pagehelper.PageInfo;
import com.glxp.api.admin.constant.ConstantStatus;
import com.glxp.api.admin.controller.inout.utils.ContrastErpUtil;
import com.glxp.api.admin.entity.basic.BasicThirdSysDetailEntity;
import com.glxp.api.admin.entity.basic.BussinessTypeEntity;
import com.glxp.api.admin.entity.info.SystemParamConfigEntity;
import com.glxp.api.admin.entity.inout.*;
import com.glxp.api.admin.entity.thrsys.ThrOrderDetailEntity;
import com.glxp.api.admin.entity.thrsys.ThrOrderEntity;
import com.glxp.api.admin.httpclient.ErpOrderClient;
import com.glxp.api.admin.req.thrsys.FilterThrOrderDetailRequest;
import com.glxp.api.admin.req.thrsys.FilterThrOrderRequest;
import com.glxp.api.admin.res.basic.BussinessTypResponse;
import com.glxp.api.admin.res.inout.ErpOrderResponse;
import com.glxp.api.admin.res.inout.PurchaseinResponse;
import com.glxp.api.admin.req.inout.*;
import com.glxp.api.admin.res.PageSimpleResponse;
import com.glxp.api.admin.service.basic.BussinessTypeService;
import com.glxp.api.admin.service.basic.UdiInfoService;
import com.glxp.api.admin.service.basic.UdiRelevanceService;
import com.glxp.api.admin.service.info.SystemParamConfigService;
import com.glxp.api.admin.service.inout.*;
import com.glxp.api.admin.service.thrsys.ThrOrderDetailService;
import com.glxp.api.admin.service.thrsys.ThrOrderService;
import com.glxp.api.admin.thread.IoTransInoutService;
import com.glxp.api.common.enums.ResultEnum;
import com.glxp.api.common.res.BaseResponse;
import com.glxp.api.common.util.ResultVOUtils;
import org.aspectj.lang.annotation.DeclareWarning;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.BindingResult;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RestController;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

@RestController
public class OrderDetailController {

    @Resource
    OrderDetailService myErpOrderService;
    @Resource
    UdiInfoService udiInfoService;
    @Resource
    CodesService warehouseService;
    @Resource
    OrderService orderService;
    @Resource
    private UdiRelevanceService udiRelevanceService;

    @Resource
    private IoTransInoutService transInoutService;
    @Autowired
    private SystemParamConfigService systemParamConfigService;
    @Resource
    private BussinessTypeService bussinessTypeService;
    @Resource
    private ThrOrderService thrOrderService;
    @Resource
    private ThrOrderDetailService thrOrderDetailService;

    @Resource
    private StockOrderService stockOrderService;
    @Resource
    private StockOrderDetailService stockOrderDetailService;


    @GetMapping("/udiwms/erpOrder/filter")
    public BaseResponse filterErpOrder(FilterOrderRequest filterErpOrderRequest,
                                       BindingResult bindingResult) {

        if (bindingResult.hasErrors()) {
            return ResultVOUtils.error(ResultEnum.PARAM_VERIFY_FALL, bindingResult.getFieldError().getDefaultMessage());
        }


        BasicThirdSysDetailEntity basicThirdSysDetailEntity = bussinessTypeService.findByActionKey(filterErpOrderRequest.getBillAction(), "orderQueryUrl");

        if (basicThirdSysDetailEntity == null || !basicThirdSysDetailEntity.getEnabled()) {
            return ResultVOUtils.error(500, "第三方业务单据服务未启用");
        }

        if (filterErpOrderRequest.getIsDownThrSys() != null && filterErpOrderRequest.getIsDownThrSys()) {
            return new ErpOrderClient(basicThirdSysDetailEntity.getValue()).getErpOrderResponse(filterErpOrderRequest);
        }

        if (filterErpOrderRequest.getIsLocal() != null && filterErpOrderRequest.getIsLocal()) {
            return getLoaclData(filterErpOrderRequest);
        }

        if (basicThirdSysDetailEntity.getFromType() == 0) {
            if (basicThirdSysDetailEntity.getValue() == null) {
                return ResultVOUtils.error(500, "业务单据查询接口未定义");
            }
            BaseResponse<PageSimpleResponse<ErpOrderResponse>> baseResponse = new ErpOrderClient(basicThirdSysDetailEntity.getValue()).getErpOrderResponse(filterErpOrderRequest);

//
//            if (basicThirdSysDetailEntity.getLocalAction() != null) {
//                if (baseResponse.getData().getList() != null) {
//                    BaseResponse<PageSimpleResponse<ErpOrderResponse>> loca = getLoaclData(filterErpOrderRequest);
//                    baseResponse.getData().getList().addAll(loca.getData().getList());
//                }
//            }
//
//            if ((baseResponse == null || baseResponse.getCode() != 20000 || baseResponse.getData().getList().size() == 0)
//                    && basicThirdSysDetailEntity.getIsLocal()) {
//                return
//            }

            return baseResponse;

        } else {
            FilterThrOrderRequest filterThrOrderRequest = new FilterThrOrderRequest();
            BeanUtils.copyProperties(filterErpOrderRequest, filterThrOrderRequest);
            filterThrOrderRequest.setPage(filterErpOrderRequest.getPage());
            filterThrOrderRequest.setLimit(filterErpOrderRequest.getLimit());
            List<ErpOrderResponse> erpOrderResponses = new ArrayList<>();
            List<ThrOrderEntity> data = thrOrderService.filterThrOrder(filterThrOrderRequest);
            if (data != null && data.size() > 0) {
                for (ThrOrderEntity thrOrderEntity : data) {
                    FilterThrOrderDetailRequest filterThrOrderDetailRequest = new FilterThrOrderDetailRequest();
                    filterThrOrderDetailRequest.setOrderIdFk(thrOrderEntity.getId() + "");
                    List<ThrOrderDetailEntity> thrOrderDetailEntities = thrOrderDetailService.filterThrOrderDetailDetail(filterThrOrderDetailRequest);
                    ErpOrderResponse erpOrderResponse = new ErpOrderResponse();
                    BeanUtils.copyProperties(thrOrderEntity, erpOrderResponse);
                    List<ErpOrderResponse.SubErpOrder> subErpOrders = new ArrayList<>();
                    if (thrOrderDetailEntities != null && thrOrderDetailEntities.size() > 0) {
                        for (ThrOrderDetailEntity thrOrderDetailEntity : thrOrderDetailEntities) {
                            ErpOrderResponse.SubErpOrder subErpOrder = new ErpOrderResponse.SubErpOrder();
                            BeanUtils.copyProperties(thrOrderDetailEntity, subErpOrder);
                            subErpOrders.add(subErpOrder);
                        }
                    }
                    erpOrderResponse.setSubErpOrders(subErpOrders);
                    erpOrderResponses.add(erpOrderResponse);
                }
            } else {
//                if (basicThirdSysDetailEntity.getIsLocal())
//                    return getLoaclData(filterErpOrderRequest);
            }
            PageInfo<ThrOrderEntity> pageInfo;
            pageInfo = new PageInfo<>(data);
            PageSimpleResponse<ErpOrderResponse> pageSimpleResponse = new PageSimpleResponse<>();
            pageSimpleResponse.setTotal(pageInfo.getTotal());
            pageSimpleResponse.setList(erpOrderResponses);
            return ResultVOUtils.success(pageSimpleResponse);

        }
    }


    public BaseResponse getLoaclData(FilterOrderRequest filterErpOrderRequest) {
        StockOrderFilterRequest stockOrderFilterRequest = new StockOrderFilterRequest();
        BeanUtils.copyProperties(filterErpOrderRequest, stockOrderFilterRequest);
        stockOrderFilterRequest.setBillType(filterErpOrderRequest.getLocalAction());
        stockOrderFilterRequest.setPage(filterErpOrderRequest.getPage());
        stockOrderFilterRequest.setLimit(filterErpOrderRequest.getLimit());
        List<ErpOrderResponse> erpOrderResponses = new ArrayList<>();
        List<StockOrderEntity> data = stockOrderService.filterListOr(stockOrderFilterRequest);
        if (data != null && data.size() > 0) {
            for (StockOrderEntity thrOrderEntity : data) {
                StockOrderDetailFilterRequest filterThrOrderDetailRequest = new StockOrderDetailFilterRequest();
                filterThrOrderDetailRequest.setOrderIdFk(thrOrderEntity.getId() + "");
                List<StockOrderDetailEntity> thrOrderDetailEntities = stockOrderDetailService.filterStockOrderDetail(filterThrOrderDetailRequest);
                ErpOrderResponse erpOrderResponse = new ErpOrderResponse();
                BeanUtils.copyProperties(thrOrderEntity, erpOrderResponse);
                List<ErpOrderResponse.SubErpOrder> subErpOrders = new ArrayList<>();
                if (thrOrderDetailEntities != null && thrOrderDetailEntities.size() > 0) {
                    for (StockOrderDetailEntity thrOrderDetailEntity : thrOrderDetailEntities) {
                        ErpOrderResponse.SubErpOrder subErpOrder = new ErpOrderResponse.SubErpOrder();
                        BeanUtils.copyProperties(thrOrderDetailEntity, subErpOrder);
                        subErpOrders.add(subErpOrder);
                    }
                }
                erpOrderResponse.setSubErpOrders(subErpOrders);
                erpOrderResponses.add(erpOrderResponse);
            }
        }
        PageInfo<StockOrderEntity> pageInfo;
        pageInfo = new PageInfo<>(data);
        PageSimpleResponse<ErpOrderResponse> pageSimpleResponse = new PageSimpleResponse<>();
        pageSimpleResponse.setTotal(pageInfo.getTotal());
        pageSimpleResponse.setList(erpOrderResponses);
        return ResultVOUtils.success(pageSimpleResponse);
    }

    @GetMapping("/warehouse/getMyErpOrders")
    public BaseResponse getMyErpOrders(FilterErpOrderRequest filterErpOrderRequest,
                                       BindingResult bindingResult) {

        if (bindingResult.hasErrors()) {
            return ResultVOUtils.error(ResultEnum.PARAM_VERIFY_FALL, bindingResult.getFieldError().getDefaultMessage());
        }
        if (filterErpOrderRequest.getErpOrderId() == null || filterErpOrderRequest.getErpOrderId().equals("")) {
            filterErpOrderRequest.setErpOrderId("null");
        }
        List<ErpOrderEntity> erpInfoEntityList = myErpOrderService.filterMyErpDetailOrder(filterErpOrderRequest);
        PageInfo<ErpOrderEntity> pageInfo;
        pageInfo = new PageInfo<>(erpInfoEntityList);
        PageSimpleResponse<ErpOrderEntity> pageSimpleResponse = new PageSimpleResponse<>();
        pageSimpleResponse.setTotal(pageInfo.getTotal());
        pageSimpleResponse.setList(erpInfoEntityList);
        return ResultVOUtils.success(pageSimpleResponse);
    }


    @PostMapping("/warehouse/deleteErpOrder")
    public BaseResponse deleteErpOrder(@RequestBody DeleteRequest deleteRequest, BindingResult bindingResult) {

        if (bindingResult.hasErrors()) {
            return ResultVOUtils.error(ResultEnum.PARAM_VERIFY_FALL, bindingResult.getFieldError().getDefaultMessage());
        }
        boolean b = myErpOrderService.deleteById(deleteRequest.getId());
        if (b) {
            return ResultVOUtils.success("删除成功");
        } else {
            return ResultVOUtils.error(500, "删除失败");
        }
    }

    @GetMapping(value = "/warehouse/inout/order/detail")
    public BaseResponse orderDetail(FilterErpOrderRequest filterErpOrderRequest,
                                    BindingResult bindingResult) {
        if (bindingResult.hasErrors()) {
            return ResultVOUtils.error(ResultEnum.PARAM_VERIFY_FALL, bindingResult.getFieldError().getDefaultMessage());
        }
        if (filterErpOrderRequest.getOrderId() == null || filterErpOrderRequest.getOrderId().equals("")) {
            filterErpOrderRequest.setOrderId("1");
        }
        filterErpOrderRequest.setPage(1);
        filterErpOrderRequest.setLimit(100);
        List<ErpOrderEntity> erpOrderEntities = myErpOrderService.filterMyErpOrder(filterErpOrderRequest);
        return ResultVOUtils.success(erpOrderEntities);
    }

    @GetMapping(value = "/warehouse/inout/order/uploadOrder")
    public BaseResponse uploadOrder(FilterErpOrderRequest filterErpOrderRequest,
                                    BindingResult bindingResult) {
        if (bindingResult.hasErrors()) {
            return ResultVOUtils.error(ResultEnum.PARAM_VERIFY_FALL, bindingResult.getFieldError().getDefaultMessage());
        }
        if (filterErpOrderRequest.getOrderId() == null || filterErpOrderRequest.getOrderId().equals("")) {
            return ResultVOUtils.error(500, "上传失败");
        }

        filterErpOrderRequest.setPage(1);
        filterErpOrderRequest.setLimit(100);
        List<ErpOrderEntity> erpOrderEntities = myErpOrderService.filterMyErpOrder(filterErpOrderRequest);
        OrderFilterRequest orderFilterRequest = new OrderFilterRequest();
        orderFilterRequest.setId(filterErpOrderRequest.getOrderId());
        OrderEntity orderEntity = orderService.findOne(orderFilterRequest);
        if (erpOrderEntities != null) {
            for (int i = 0; i < erpOrderEntities.size(); i++) {
                if (erpOrderEntities.get(i).getErpOrderId() == null) {
                    return ResultVOUtils.error(500, "批次号不匹配，请检查后修改！");
                }

                if (erpOrderEntities.get(i).getErpOrderId().contains(",")) {
                    return ResultVOUtils.error(500, "无法修改合并单据！");
                }

                if (erpOrderEntities.get(i).getCount() > erpOrderEntities.get(i).getErpCount()) {
                    return ResultVOUtils.error(500, "扫码数量大于实际数量,无法修改！");
                }
            }
        }
        BasicThirdSysDetailEntity basicThirdSysDetailEntity = bussinessTypeService.findByActionKey(orderEntity.getAction(), "orderModifyUrl");
        if (basicThirdSysDetailEntity == null) {
            return ResultVOUtils.error(500, "ERP出入库单据查询接口地址未定义");
        }
        String url = basicThirdSysDetailEntity.getValue();
        Map<String, List<ErpOrderEntity>> listMap = erpOrderEntities.stream().collect(Collectors.groupingBy(ErpOrderEntity::getErpOrderId));
        BaseResponse baseResponse = new ErpOrderClient().postBill(listMap, orderEntity.getAction(), url);
        if (baseResponse != null && baseResponse.getCode() == 20000) {
            orderService.updateExportStatus(orderEntity.getId(), ConstantStatus.ORDER_EXPORT_ED);
            return ResultVOUtils.success("修改成功");
        } else {
            if (baseResponse != null) {
                return ResultVOUtils.error(500, baseResponse.getMessage());
            }
            return ResultVOUtils.error(500, "修改失败");
        }
    }


    @GetMapping("/warehouse/dlErpStock1")
    public BaseResponse dlErpStock() {
//        dlErpStockService.dlErpStock();
        return ResultVOUtils.success("后台已自动更新，请稍后刷新查看！");
    }

    @PostMapping("/warehouse/selectErpOrder")
    public BaseResponse selectErpOrder(@RequestBody ImportErpOrderRequest importErpOrderRequest, BindingResult bindingResult) {

        if (bindingResult.hasErrors()) {
            return ResultVOUtils.error(ResultEnum.PARAM_VERIFY_FALL, bindingResult.getFieldError().getDefaultMessage());
        }
        List<PurchaseinResponse> docids = importErpOrderRequest.getPurchaseinResponses();
        String docidStr = "";
        if (docids != null && docids.size() > 0) {
            for (PurchaseinResponse temp : docids) {
                docidStr = docidStr + "," + temp.getVbillcode();
            }
            docidStr = docidStr.substring(1);
        }
        List<ErpOrderEntity> erpOrderEntities = new ArrayList<>();
        if (docids != null && docids.size() > 0) {
            for (PurchaseinResponse purchaseinResponse : docids) {
                for (PurchaseinResponse.SubPurchase subPurchase : purchaseinResponse.getSubPurchases()) {
                    ErpOrderEntity erpOrderEntity = new ErpOrderEntity();
                    erpOrderEntity.setErpOrderId(purchaseinResponse.getVbillcode());
                    erpOrderEntity.setCompanyid(purchaseinResponse.getProvider_code());
                    erpOrderEntity.setCompanyname(purchaseinResponse.getProvider_name());
                    erpOrderEntity.setCredate(purchaseinResponse.getDbilldate());
                    erpOrderEntity.setGoodsid(subPurchase.getInventory_code());
                    erpOrderEntity.setGoodsname(subPurchase.getInventory_name());
                    erpOrderEntity.setBatchNo(subPurchase.getVbatchcode());
                    erpOrderEntity.setErpCount(Math.abs(subPurchase.getDtlgoodsqty()));
                    erpOrderEntity.setReCount(Math.abs(subPurchase.getReCount()));
                    erpOrderEntity.setGoodsunit(subPurchase.getInvspec());
                    erpOrderEntities.add(erpOrderEntity);
                }
            }
        }
        if (erpOrderEntities != null && erpOrderEntities.size() > 0) {
            ErpOrderEntity temp = erpOrderEntities.get(0);
            for (int i = 0; i < erpOrderEntities.size(); i++) {
                if (!temp.getCompanyname().equals(erpOrderEntities.get(i).getCompanyname())) {
                    return ResultVOUtils.error(500, "选择Erp订单往来单位不一致！");
                }
            }

            OrderFilterRequest orderFilterRequest = new OrderFilterRequest();
            orderFilterRequest.setId(importErpOrderRequest.getOrderId());
            OrderEntity orderEntity = orderService.findOne(orderFilterRequest);
            ErpOrderEntity exitErp = new ContrastErpUtil().vailErpExit(myErpOrderService, erpOrderEntities, importErpOrderRequest.getOrderId());
            if (exitErp != null) {
                String errMsg = "所选ERP单" + exitErp.getErpOrderId() + "已被其他出入库单" + exitErp.getOrderIdFk() + "绑定";
                orderEntity.setRemark(errMsg);
                orderService.updateOrder(orderEntity);
                return ResultVOUtils.error(500, errMsg);
            } else {
                orderEntity.setErpFk(docidStr);
                orderService.updateOrder(orderEntity);
            }

            myErpOrderService.deleteByOrderId(importErpOrderRequest.getOrderId());
            List<ErpOrderEntity> returnOrders = new ContrastErpUtil().transErp(erpOrderEntities, udiRelevanceService, importErpOrderRequest.getOrderId());
            myErpOrderService.insertErpOrders(returnOrders);
            List<WarehouseEntity> warehouseEntityList = warehouseService.findByReceiptId(importErpOrderRequest.getOrderId());
            new ContrastErpUtil().contrastErp(udiInfoService, udiRelevanceService, myErpOrderService, orderService, warehouseEntityList, systemParamConfigService);


        } else {
            return ResultVOUtils.error(500, "ERP订单不能为空");
        }

        return ResultVOUtils.success("导入成功");
    }


    @PostMapping("/warehouse/repeatCheck")
    public BaseResponse repeatCheck(@RequestBody FilterErpOrderRequest filterErpOrderRequest, BindingResult bindingResult) {

        if (bindingResult.hasErrors()) {
            return ResultVOUtils.error(ResultEnum.PARAM_VERIFY_FALL, bindingResult.getFieldError().getDefaultMessage());
        }
        OrderFilterRequest orderFilterRequest = new OrderFilterRequest();
        orderFilterRequest.setId(filterErpOrderRequest.getOrderId());
        OrderEntity orderEntity = orderService.findOne(orderFilterRequest);
        transInoutService.repeatCheck(orderEntity.getId());
        return ResultVOUtils.success("后台已开始校验，请稍后刷新重试！");
    }


}
