package com.glxp.api.admin.controller.inout;

import com.github.pagehelper.PageInfo;
import com.glxp.api.admin.entity.basic.BussinessTypeEntity;
import com.glxp.api.admin.entity.inout.WarehouseEntity;
import com.glxp.api.admin.req.inout.*;
import com.glxp.api.admin.res.PageSimpleResponse;
import com.glxp.api.admin.res.inout.WarehouseResponse;
import com.glxp.api.admin.service.basic.BussinessTypeService;
import com.glxp.api.admin.service.inout.CodesTempService;
import com.glxp.api.admin.service.inout.CodesService;
import com.glxp.api.admin.thread.IoTransInoutService;
import com.glxp.api.admin.util.DateUtil;
import com.glxp.api.common.enums.ResultEnum;
import com.glxp.api.common.res.BaseResponse;
import com.glxp.api.common.util.ResultVOUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.validation.BindingResult;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.stream.Collectors;

@RestController
public class WareHouseController {
    @Resource
    private CodesService warehouseService;
    @Resource
    private IoTransInoutService transInoutService;
    @Resource
    private CodesTempService codesTempService;
    @Resource
    private BussinessTypeService bussinessTypeService;

    @PostMapping("/warehouse/inout/code/deleteCodes")
    public BaseResponse deleteCodes(@RequestBody DeleteCodeRequest deleteCodeRequest, BindingResult bindingResult) {

        if (bindingResult.hasErrors()) {
            return ResultVOUtils.error(ResultEnum.PARAM_VERIFY_FALL, bindingResult.getFieldError().getDefaultMessage());
        }
        List<Long> codes = deleteCodeRequest.getCodes();
        if (codes != null && codes.size() > 0) {
            codesTempService.deleteCodesTemp(codes);
        } else {
            return ResultVOUtils.error(ResultEnum.DATA_ERROR);
        }
        return ResultVOUtils.success("删除成功");
    }


    @GetMapping("admin/warehouse/inout/findByNo")
    public BaseResponse findByNo(WarehouseQueryRequest warehouseQueryRequest) {
        List<WarehouseEntity> warehouseEntityList = warehouseService.findByNo(warehouseQueryRequest);
        List<WarehouseResponse> warehouseResponses = warehouseEntityList.stream().map(item ->
        {
            WarehouseResponse warehouseResponse = new WarehouseResponse();
            BeanUtils.copyProperties(item, warehouseResponse);
            return warehouseResponse;
        }).collect(Collectors.toList());
        PageInfo<WarehouseEntity> pageInfo;
        pageInfo = new PageInfo<>(warehouseEntityList);
        PageSimpleResponse<WarehouseResponse> pageSimpleResponse = new PageSimpleResponse<>();
        pageSimpleResponse.setTotal(pageInfo.getTotal());
        pageSimpleResponse.setList(warehouseResponses);
        return ResultVOUtils.success(pageSimpleResponse);
    }

    @GetMapping("admin/warehouse/inout/findByOrderId")
    public BaseResponse findByOrderId(WarehouseQueryRequest warehouseQueryRequest) {
        List<WarehouseEntity> warehouseEntityList = warehouseService.findByOrderId(warehouseQueryRequest);
        List<WarehouseResponse> warehouseResponses = warehouseEntityList.stream().map(item ->
        {
            WarehouseResponse warehouseResponse = new WarehouseResponse();
            BeanUtils.copyProperties(item, warehouseResponse);
            return warehouseResponse;
        }).collect(Collectors.toList());
        PageInfo<WarehouseEntity> pageInfo;
        pageInfo = new PageInfo<>(warehouseEntityList);
        PageSimpleResponse<WarehouseResponse> pageSimpleResponse = new PageSimpleResponse<>();
        pageSimpleResponse.setTotal(pageInfo.getTotal());
        pageSimpleResponse.setList(warehouseResponses);
        return ResultVOUtils.success(pageSimpleResponse);
    }

    @GetMapping("admin/warehouse/inout/findAllByOrderId")
    public BaseResponse findAllByOrderId(WarehouseQueryRequest warehouseQueryRequest) {
        List<WarehouseEntity> warehouseEntityList = warehouseService.findAllByOrderId(warehouseQueryRequest);
        List<WarehouseResponse> warehouseResponses = warehouseEntityList.stream().map(item ->
        {
            WarehouseResponse warehouseResponse = new WarehouseResponse();
            BeanUtils.copyProperties(item, warehouseResponse);
            return warehouseResponse;
        }).collect(Collectors.toList());
        PageInfo<WarehouseEntity> pageInfo;
        pageInfo = new PageInfo<>(warehouseEntityList);
        PageSimpleResponse<WarehouseResponse> pageSimpleResponse = new PageSimpleResponse<>();
        pageSimpleResponse.setTotal(pageInfo.getTotal());
        pageSimpleResponse.setList(warehouseResponses);
        return ResultVOUtils.success(pageSimpleResponse);
    }


    @GetMapping("admin/warehouse/inout/findErrorByOrderId")
    public BaseResponse findErrorByOrderId(WarehouseQueryRequest warehouseQueryRequest) {
        List<WarehouseEntity> warehousetempList = codesTempService.findByOrderId(warehouseQueryRequest);
        PageInfo<WarehouseEntity> pageInfo;
        pageInfo = new PageInfo<>(warehousetempList);
        PageSimpleResponse<WarehouseEntity> pageSimpleResponse = new PageSimpleResponse<>();
        pageSimpleResponse.setTotal(pageInfo.getTotal());
        pageSimpleResponse.setList(warehousetempList);
        return ResultVOUtils.success(pageSimpleResponse);
    }


    @PostMapping("warehouse/inout/postOrders")
    public BaseResponse postOrders(@RequestBody PostOrderRequest postOrderRequest, BindingResult bindingResult) {

        if (bindingResult.hasErrors()) {
            return ResultVOUtils.error(ResultEnum.PARAM_VERIFY_FALL, bindingResult.getFieldError().getDefaultMessage());
        }
        try {
            transInoutService.creatOrder(postOrderRequest);
        } catch (Exception e) {
            return ResultVOUtils.error(ResultEnum.NOT_NETWORK);
        }

        return ResultVOUtils.success("导入成功");
    }


    //扫码精灵扫码上传
    @PostMapping("warehouse/inout/scanPost")
    public BaseResponse postOrders(@RequestBody ScPostOrderRequest postOrderRequest, BindingResult bindingResult) {

        if (bindingResult.hasErrors()) {
            return ResultVOUtils.error(ResultEnum.PARAM_VERIFY_FALL, bindingResult.getFieldError().getDefaultMessage());
        }
        try {
            PostOrderRequest.PostOrder postOrder = new PostOrderRequest.PostOrder();
            postOrder.setOrderId(postOrderRequest.getOrderId());
            postOrder.setBillType(postOrderRequest.getBillType());
            BussinessTypeEntity bussinessTypeEntity = bussinessTypeService.findBTByAction(postOrderRequest.getBillType());
            List<WarehouseEntity> warehouseEntityList = postOrderRequest.getCodes();
            String date = DateUtil.formatDateTime(new Date());
            for (WarehouseEntity warehouseEntity : warehouseEntityList) {
                warehouseEntity.setActDate(date);
                warehouseEntity.setMainAction(bussinessTypeEntity.getMainAction());
                warehouseEntity.setAction(postOrderRequest.getBillType());
                warehouseEntity.setCorpOrderId(postOrderRequest.getOrderId());
                warehouseEntity.setFromCorp(postOrderRequest.getFromCorp());
                warehouseEntity.setFromCorpId(postOrderRequest.getFromCorpId());
            }
            postOrder.setCodes(warehouseEntityList);
            PostOrderRequest commit = new PostOrderRequest();
            List<PostOrderRequest.PostOrder> postOrders = new ArrayList<>();
            postOrders.add(postOrder);
            commit.setPostOrder(postOrders);
            transInoutService.creatOrder(commit);
        } catch (Exception e) {
            return ResultVOUtils.error(ResultEnum.NOT_NETWORK);
        }

        return ResultVOUtils.success("导入成功");
    }

    @PostMapping("/warehouse/inout/commit")
    public BaseResponse commitOrder(@RequestBody CommitRequest commitRequest,
                                    BindingResult bindingResult) {

        if (bindingResult.hasErrors()) {
            return ResultVOUtils.error(ResultEnum.PARAM_VERIFY_FALL, bindingResult.getFieldError().getDefaultMessage());
        }
        transInoutService.repeatCommit(commitRequest);
        return ResultVOUtils.success("提交成功，后台已开启重码校验，请稍后刷新查看！");

    }

}
