package com.glxp.api.admin.controller.inout.utils;

import com.glxp.api.admin.constant.ConstantType;
import com.glxp.api.admin.entity.info.SystemParamConfigEntity;
import com.glxp.api.admin.httpclient.UdiDlHttpClient;
import com.glxp.api.admin.entity.basic.UdiInfoEntity;
import com.glxp.api.admin.entity.basic.UdiRelevanceEntity;
import com.glxp.api.admin.entity.inout.WarehouseEntity;
import com.glxp.api.admin.entity.basic.UdiEntity;
import com.glxp.api.admin.entity.inout.ErpOrderEntity;
import com.glxp.api.admin.service.basic.UdiInfoService;
import com.glxp.api.admin.service.basic.UdiRelevanceService;
import com.glxp.api.admin.service.info.SystemParamConfigService;
import com.glxp.api.admin.service.inout.OrderDetailService;
import com.glxp.api.admin.service.inout.CodesService;
import com.glxp.api.admin.util.DateUtil;
import com.glxp.api.admin.util.FilterUdiUtils;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

public class InoutUtils {
    String udiUrl = "";

    public InoutUtils() {

    }

    public InoutUtils(String udiUrl) {
        this.udiUrl = udiUrl;
    }

    public String filterCode(CodesService warehouseService, WarehouseEntity warehouseData, SystemParamConfigService setupService) {
        List<WarehouseEntity> warehouseEntityList = warehouseService.findByCode(warehouseData.getCode());
        if (warehouseEntityList == null || warehouseEntityList.size() == 0) { //数据库为null,可以添加
            return nullVail(warehouseData, setupService);
        }
        List<WarehouseEntity> inList = new ArrayList<>();
        List<WarehouseEntity> outList = new ArrayList<>();
        if (warehouseEntityList != null && warehouseEntityList.size() > 0) {
            for (int i = 0; i < warehouseEntityList.size(); i++) {
                WarehouseEntity temp = warehouseEntityList.get(i);
                if (temp.getAction().equals(ConstantType.TYPE_CODE_DESTORY)) {   //过滤已销毁的
                    return getErrMsg(temp);
                }
                if (temp.getAction().equals(ConstantType.TYPE_STOCK_CHECK)) {   //过滤已盘点
                    if (warehouseData.getAction().equals(ConstantType.TYPE_STOCK_CHECK)) {
                        return warehouseData.getCode() + "该码已盘点";
                    }
                }
                if (temp.getMainAction().equals(ConstantType.TYPE_OUT)) {
                    outList.add(temp);
                } else if (temp.getMainAction().equals(ConstantType.TYPE_PUT)) {
                    inList.add(temp);
                }
            }
            //如果序列号未null则不判断是否重码
            UdiEntity udiEntity = FilterUdiUtils.getUdi(warehouseData.getCode());
            if (udiEntity.getSerialNo() == null) {
                return null;
            }
            //如果数据库存在1个出库，则可以再录入1个入库;（同理：存在4个出库，3个入库，则可以再录入1个入库）
            if (outList.size() - inList.size() == 1) {
                if (warehouseData.getMainAction().equals(ConstantType.TYPE_PUT)) {
                    return null;
                }
            } else if (inList.size() - outList.size() == 1) {
                if (warehouseData.getMainAction().equals(ConstantType.TYPE_OUT)) {
                    return null;
                }
            } else if (inList.size() - outList.size() == 0) {
                return nullVail(warehouseData, setupService);
            } else {
                return "订单：" + warehouseData.getOrderId() + "已存在" + warehouseData.getCode();
            }
            return warehouseData.getCode() + "条码重复";
        }


        return null;
    }

    public String nullVail(WarehouseEntity warehousetemp, SystemParamConfigService configService) {
        if (warehousetemp.getAction().equals(ConstantType.TYPE_PUT_RETURN)) {
           SystemParamConfigEntity systemParamConfigEntity =  configService.selectByParamKey("io_isReturnWareHouseIn");
            if (systemParamConfigEntity==null || "0".equals(systemParamConfigEntity.getParamValue())) {
                return null;
            } else {
                return warehousetemp.getCode() + "不允许未销售出库可以退货入库";
            }
        } else if (warehousetemp.getAction().equals(ConstantType.TYPE_OUT_RETURN)) {
            SystemParamConfigEntity systemParamConfigEntity =  configService.selectByParamKey("io_isReturnWareHouseOut");
            if (systemParamConfigEntity==null || "0".equals(systemParamConfigEntity.getParamValue())) {
                return null;
            } else {
                return warehousetemp.getCode() + "不允许零库存退货出库";
            }
        } else if (warehousetemp.getAction().equals(ConstantType.TYPE_OUT_SALE)) {
            SystemParamConfigEntity systemParamConfigEntity =  configService.selectByParamKey("io_isSalesWareHouseOut");
            if (systemParamConfigEntity==null || "0".equals(systemParamConfigEntity.getParamValue())) {
                return null;
            } else {
                return warehousetemp.getCode() + "不允许零库存销售出库";
            }
        }
        return null;
    }

    public String getErrMsg(WarehouseEntity warehousetemp) {
        if (warehousetemp.getAction().equals(ConstantType.TYPE_PUT_PRODUCT)) {
            return warehousetemp.getCode() + "已生产入库";
        } else if (warehousetemp.getAction().equals(ConstantType.TYPE_PUT_PURCHASE)) {
            return warehousetemp.getCode() + "已采购入库";
        } else if (warehousetemp.getAction().equals(ConstantType.TYPE_PUT_RETURN)) {
            return warehousetemp.getCode() + "已退货入库";
        } else if (warehousetemp.getAction().equals(ConstantType.TYPE_PUT_ALLOT)) {
            return warehousetemp.getCode() + "已调拨入库";
        } else if (warehousetemp.getAction().equals(ConstantType.TYPE_OUT_SALE)) {
            return warehousetemp.getCode() + "已销售出库";
        } else if (warehousetemp.getAction().equals(ConstantType.TYPE_OUT_RETURN)) {
            return warehousetemp.getCode() + "已退货出库";
        } else if (warehousetemp.getAction().equals(ConstantType.TYPE_OUT_ALLOT)) {
            return warehousetemp.getCode() + "已调拨出库";
        } else if (warehousetemp.getAction().equals(ConstantType.TYPE_OUT_DESTORY)) {
            return warehousetemp.getCode() + "已销毁出库";
        } else if (warehousetemp.getAction().equals(ConstantType.TYPE_OUT_STRAIGHT)) {
            return warehousetemp.getCode() + "已直调出库";
        } else if (warehousetemp.getAction().equals(ConstantType.TYPE_OUT_REWORK)) {
            return warehousetemp.getCode() + "已返工出库";
        } else if (warehousetemp.getAction().equals(ConstantType.TYPE_OUT_CHECK)) {
            return warehousetemp.getCode() + "已抽检出库";
        } else if (warehousetemp.getAction().equals(ConstantType.TYPE_CODE_DESTORY)) {
            return warehousetemp.getCode() + "已销毁";
        }
        return warehousetemp.getCode() + "出错";
    }


    public UdiInfoEntity getUdiInfo(UdiInfoService udiInfoService, UdiRelevanceService udiRelevanceService, String code) {
        String prefix;
        prefix = FilterUdiUtils.getDiStr(code);
        UdiInfoEntity udiInfoEntity = udiInfoService.findByNameCode(prefix);
        if (udiInfoEntity == null) {
            List<UdiInfoEntity> froms = new UdiDlHttpClient(udiUrl).getUdiByDi(prefix);
            if (froms != null && !froms.isEmpty()) {
                udiInfoService.insertUdiInfos(froms);
                udiInfoEntity = udiInfoService.findByNameCode(prefix);
                UdiRelevanceEntity udiRelevanceEntity = new UdiRelevanceEntity();
                udiRelevanceEntity.setUuid(udiInfoEntity.getUuid());
                udiRelevanceEntity.setUpdateTime(DateUtil.getDateTime());
                udiRelevanceService.insertUdiRelevance(udiRelevanceEntity);
            } else {
                udiInfoEntity = new UdiInfoEntity();
            }
        }
        udiInfoEntity.setCode(code);
        udiInfoEntity = FilterUdiUtils.transUdi(udiInfoEntity);
        return udiInfoEntity;
    }

    public void generateConstrastDetail(UdiInfoService udiInfoService, UdiRelevanceService udiRelevanceService,
                                        OrderDetailService myErpOrderService,
                                        List<WarehouseEntity> warehouseEntityList) {
        if (warehouseEntityList != null && warehouseEntityList.size() > 0) {
            List<UdiInfoEntity> udiInfoEntities = new ArrayList<>();
            for (WarehouseEntity warehouseEntity : warehouseEntityList) {
                UdiInfoEntity udiInfoEntity = getUdiInfo(udiInfoService, udiRelevanceService, warehouseEntity.getCode());
                if (udiInfoEntity != null) {
                    if (warehouseEntity.getCount() != null)
                        udiInfoEntity.setCount(warehouseEntity.getCount());
                    else {
                        udiInfoEntity.setCount(getActCount(udiInfoEntity));
                    }
                    udiInfoEntities.add(udiInfoEntity);
                }
            }

            List<UdiInfoEntity> returnOrderDetail = new ArrayList<>();
            Map<String, UdiInfoEntity> sortMaps = new HashMap<>();
            for (UdiInfoEntity detailCodeEntity : udiInfoEntities) {
                String key = detailCodeEntity.getNameCode();
                UdiInfoEntity temp = sortMaps.get(key);
                if (temp != null) {
                    detailCodeEntity.setCount(temp.getCount() + detailCodeEntity.getCount());
                }
                sortMaps.put(key, detailCodeEntity);
            }

            for (Map.Entry<String, UdiInfoEntity> entry : sortMaps.entrySet()) {
                System.out.println("key = " + entry.getKey() + ", value = " + entry.getValue());
                returnOrderDetail.add(entry.getValue());
            }

            List<ErpOrderEntity> erpOrderEntities = new ArrayList<>();
            for (UdiInfoEntity erpDetailCodeEntity : returnOrderDetail) {
                ErpOrderEntity erpOrderEntity = new ErpOrderEntity();
                erpOrderEntity.setNameCode(erpDetailCodeEntity.getNameCode());
                erpOrderEntity.setPackRatio(erpDetailCodeEntity.getPackRatio());
                erpOrderEntity.setCoName(erpDetailCodeEntity.getCpmctymc());
                erpOrderEntity.setBatchNo(erpDetailCodeEntity.getBatchNo());
                erpOrderEntity.setCount(erpDetailCodeEntity.getCount());
                erpOrderEntity.setOrderIdFk(warehouseEntityList.get(0).getOrderId());
                erpOrderEntity.setProductCompany(erpDetailCodeEntity.getYlqxzcrbarmc());
                erpOrderEntity.setAuthCode(erpDetailCodeEntity.getZczbhhzbapzbh());
                erpOrderEntity.setPackSpec(erpDetailCodeEntity.getGgxh());
                erpOrderEntities.add(erpOrderEntity);
            }
            if (!erpOrderEntities.isEmpty())
                myErpOrderService.insertErpOrders(erpOrderEntities);
        }
    }

    public int getActCount(UdiInfoEntity udiInfoBean) {
        if (udiInfoBean != null) {
            if (udiInfoBean.getZxxsbzbhsydysl() > 1) {
                return udiInfoBean.getZxxsbzbhsydysl() * udiInfoBean.getBhzxxsbzsl();
            } else {
                return udiInfoBean.getBhzxxsbzsl();
            }
        }
        return 1;
    }
}
