package com.glxp.api.admin.controller.inventory;

import com.github.pagehelper.PageInfo;
import com.glxp.api.admin.entity.info.SystemParamConfigEntity;
import com.glxp.api.admin.entity.inventory.InCodeLogEntity;
import com.glxp.api.admin.req.inout.DeleteRequest;
import com.glxp.api.admin.req.inventory.FilterInCodeLogRequest;
import com.glxp.api.admin.res.PageSimpleResponse;
import com.glxp.api.admin.res.inventory.IncodeSysParmsResponse;
import com.glxp.api.admin.service.info.SystemParamConfigService;
import com.glxp.api.admin.service.inventory.InCodeLogService;
import com.glxp.api.admin.service.inventory.InvStockPrintService;
import com.glxp.api.common.enums.ResultEnum;
import com.glxp.api.common.res.BaseResponse;
import com.glxp.api.common.util.ResultVOUtils;
import org.springframework.validation.BindingResult;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RestController;

import javax.annotation.Resource;
import java.util.List;

@RestController
public class InCodeLogController {
    @Resource
    InCodeLogService inCodeLogService;
    @Resource
    InvStockPrintService invStockPrintService;
    @Resource
    SystemParamConfigService systemParamConfigService;


    @GetMapping("/udiwms/inv/incode/filter")
    public BaseResponse filter(FilterInCodeLogRequest filterInCodeLogRequest,
                               BindingResult bindingResult) {

        if (bindingResult.hasErrors()) {
            return ResultVOUtils.error(ResultEnum.PARAM_VERIFY_FALL, bindingResult.getFieldError().getDefaultMessage());
        }
        List<InCodeLogEntity> inCodeLogEntities
                = inCodeLogService.filterInCodeLog(filterInCodeLogRequest);
        PageInfo<InCodeLogEntity> pageInfo;
        pageInfo = new PageInfo<>(inCodeLogEntities);
        PageSimpleResponse<InCodeLogEntity> pageSimpleResponse = new PageSimpleResponse<>();
        pageSimpleResponse.setTotal(pageInfo.getTotal());
        pageSimpleResponse.setList(inCodeLogEntities);
        return ResultVOUtils.success(pageSimpleResponse);
    }


    @PostMapping("/udiwms/inv/incode/delete")
    public BaseResponse delete(@RequestBody DeleteRequest deleteRequest, BindingResult bindingResult) {

        if (bindingResult.hasErrors()) {
            return ResultVOUtils.error(ResultEnum.PARAM_VERIFY_FALL, bindingResult.getFieldError().getDefaultMessage());
        }
        String id = deleteRequest.getId();
        FilterInCodeLogRequest filterInCodeLogRequest = new FilterInCodeLogRequest();
        filterInCodeLogRequest.setId(Integer.parseInt(id));
        List<InCodeLogEntity> inCodeLogEntities = inCodeLogService.filterInCodeLog(filterInCodeLogRequest);
        if (inCodeLogEntities != null && inCodeLogEntities.size() > 0) {
            InCodeLogEntity inCodeLogEntity = inCodeLogEntities.get(0);
            inCodeLogService.deleteById(inCodeLogEntity.getId() + "");
            invStockPrintService.deleteByGenKey(inCodeLogEntity.getGenKey());
        }

        return ResultVOUtils.success("删除成功");
    }


    @GetMapping("/udiwms/inv/incode/getParams")
    public BaseResponse getParams() {
        IncodeSysParmsResponse incodeSysParmsResponse = new IncodeSysParmsResponse();
        incodeSysParmsResponse.setFromType(systemParamConfigService.selectValueByParamKey("incode_fromType"));
        incodeSysParmsResponse.setThirdSys(systemParamConfigService.selectValueByParamKey("incode_thirdSys_fromType"));
        return ResultVOUtils.success(incodeSysParmsResponse);
    }
}
