package com.glxp.api.admin.controller.inventory;

import com.github.pagehelper.PageInfo;
import com.glxp.api.admin.constant.ConstantType;
import com.glxp.api.admin.entity.inout.OrderEntity;
import com.glxp.api.admin.entity.inout.StockCheckEntity;
import com.glxp.api.admin.entity.inout.ErpOrderEntity;
import com.glxp.api.admin.entity.inout.FilterErpOrderRequest;
import com.glxp.api.admin.req.inout.DeleteRequest;
import com.glxp.api.admin.req.inout.FilterStockCheckRequest;
import com.glxp.api.admin.req.inout.OrderFilterRequest;
import com.glxp.api.admin.res.PageSimpleResponse;
import com.glxp.api.admin.service.inout.OrderDetailService;
import com.glxp.api.admin.service.inout.OrderService;
import com.glxp.api.admin.service.inventory.InvOrderCheckService;
import com.glxp.api.admin.thread.InvImportStockService;
import com.glxp.api.admin.util.DateUtil;
import com.glxp.api.common.enums.ResultEnum;
import com.glxp.api.common.res.BaseResponse;
import com.glxp.api.common.util.ResultVOUtils;
import org.springframework.validation.BindingResult;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RestController;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

@RestController
public class InvCheckController {

    @Resource
    private OrderService orderService;
    @Resource
    private InvOrderCheckService stockCheckService;
    @Resource
    private InvImportStockService importStockService;
    @Resource
    private OrderDetailService myErpOrderService;


    @GetMapping("/udiwarehouse/stockCheck/filter")
    public BaseResponse filter(FilterStockCheckRequest filterStockCheckRequest,
                               BindingResult bindingResult) {

        if (bindingResult.hasErrors()) {
            return ResultVOUtils.error(ResultEnum.PARAM_VERIFY_FALL, bindingResult.getFieldError().getDefaultMessage());
        }
        List<StockCheckEntity> stockCheckEntities
                = stockCheckService.filterStockCheck(filterStockCheckRequest);
        PageInfo<StockCheckEntity> pageInfo;
        pageInfo = new PageInfo<>(stockCheckEntities);
        PageSimpleResponse<StockCheckEntity> pageSimpleResponse = new PageSimpleResponse<>();
        pageSimpleResponse.setTotal(pageInfo.getTotal());
        pageSimpleResponse.setList(stockCheckEntities);
        return ResultVOUtils.success(pageSimpleResponse);
    }


    @GetMapping("/udiwarehouse/stockCheck/getDetail")
    public BaseResponse getDetail(FilterStockCheckRequest filterStockCheckRequest,
                                  BindingResult bindingResult) {

        if (bindingResult.hasErrors()) {
            return ResultVOUtils.error(ResultEnum.PARAM_VERIFY_FALL, bindingResult.getFieldError().getDefaultMessage());
        }
        List<String> orderIds = getOrderIds(filterStockCheckRequest.getId() + "");
        List<ErpOrderEntity> erpOrderEntities = new ArrayList<>();
        if (orderIds.size() > 0) {
            FilterErpOrderRequest filterErpOrderRequest = new FilterErpOrderRequest();
            filterErpOrderRequest.setPage(1);
            filterErpOrderRequest.setLimit(100);
            filterErpOrderRequest.setOrderIds(orderIds);
            erpOrderEntities = myErpOrderService.findByStockCheck(filterErpOrderRequest);
            if (isRepeat(erpOrderEntities)) {
                Map<String, List<ErpOrderEntity>> groupBy = erpOrderEntities.stream().collect(Collectors.groupingBy(ErpOrderEntity::getNameCode));
                for (List<ErpOrderEntity> combines : groupBy.values()) {
                    ErpOrderEntity erpOrderEntity = combines.get(0);
                    if (combines.size() > 1) {
                        for (int i = 1; i < combines.size(); i++) {
                            erpOrderEntity.setCount(erpOrderEntity.getCount() + combines.get(i).getCount());
                            myErpOrderService.deleteByPrimiyId(combines.get(i).getId() + "");
                        }
                    }
                    myErpOrderService.updateErpOrder(erpOrderEntity);
                }
                erpOrderEntities = myErpOrderService.findByStockCheck(filterErpOrderRequest);
            }
        }


        PageInfo<ErpOrderEntity> pageInfo;
        pageInfo = new PageInfo<>(erpOrderEntities);
        PageSimpleResponse<ErpOrderEntity> pageSimpleResponse = new PageSimpleResponse<>();
        pageSimpleResponse.setTotal(pageInfo.getTotal());
        pageSimpleResponse.setList(erpOrderEntities);
        return ResultVOUtils.success(pageSimpleResponse);
    }


    public List<String> getOrderIds(String checkId) {
        OrderFilterRequest orderFilterRequest = new OrderFilterRequest();
        orderFilterRequest.setAction(ConstantType.TYPE_STOCK_CHECK);
        orderFilterRequest.setStockCheckFk(checkId + "");
        List<OrderEntity> orderEntities = orderService.findAllOrders(orderFilterRequest);
        List<String> orderIds = new ArrayList<>();
        if (orderEntities != null) {
            for (OrderEntity orderEntity : orderEntities) {
                orderIds.add(orderEntity.getId());
            }
        }
        return orderIds;
    }

    private boolean isRepeat(List<ErpOrderEntity> erpOrderEntities) {
        List<String> stringList = erpOrderEntities.stream().map(ErpOrderEntity::getNameCode)
                .collect(Collectors.toList());
        long count = stringList.stream().distinct().count();
        if (stringList.size() == count) {
            return false;
        } else {
            return true;
        }
    }

    @GetMapping("/udiwarehouse/stockCheck/checkErp")
    public BaseResponse checkErp(FilterStockCheckRequest filterStockCheckRequest, BindingResult bindingResult) {
        if (bindingResult.hasErrors()) {
            return ResultVOUtils.error(ResultEnum.PARAM_VERIFY_FALL, bindingResult.getFieldError().getDefaultMessage());
        }
        importStockService.checkStock(filterStockCheckRequest);

        return ResultVOUtils.success("后台已开启库存盘点校验，请稍后刷新查看");
    }


    @GetMapping("/udiwarehouse/stockCheck/create")
    public BaseResponse createStockCheck() {

        FilterStockCheckRequest filterStockCheckRequest = new FilterStockCheckRequest();
        filterStockCheckRequest.setStatus("正在盘点");
        List<StockCheckEntity> stockCheckEntities = stockCheckService.filterStockCheck(filterStockCheckRequest);
        if (stockCheckEntities != null && stockCheckEntities.size() > 0) {
            return ResultVOUtils.error(500, "当前有盘点单正在进行，无法创建新盘点单！");
        }
        StockCheckEntity stockCheckEntity = new StockCheckEntity();
        stockCheckEntity.setActDate(DateUtil.getDateTime());
        stockCheckEntity.setAction(ConstantType.TYPE_STOCK_CHECK);
        stockCheckEntity.setStatus("正在盘点");
        stockCheckService.insertStockCheck(stockCheckEntity);
        return ResultVOUtils.success("创建成功");
    }

    @PostMapping("/udiwarehouse/stockCheck/finish")
    public BaseResponse finishStockCheck(@RequestBody StockCheckEntity stockCheckEntity, BindingResult bindingResult) {

        if (bindingResult.hasErrors()) {
            return ResultVOUtils.error(ResultEnum.PARAM_VERIFY_FALL, bindingResult.getFieldError().getDefaultMessage());
        }
        stockCheckEntity.setEndTime(DateUtil.getDateTime());
        stockCheckEntity.setStatus("盘点完成");
        stockCheckService.updateStockCheck(stockCheckEntity);
        return ResultVOUtils.success("更新成功");
    }


    @PostMapping("/udiwarehouse/stockCheck/delete")
    public BaseResponse deleteStockCheck(@RequestBody DeleteRequest deleteRequest, BindingResult bindingResult) {

        if (bindingResult.hasErrors()) {
            return ResultVOUtils.error(ResultEnum.PARAM_VERIFY_FALL, bindingResult.getFieldError().getDefaultMessage());
        }
        stockCheckService.deleteById(deleteRequest.getId());
        List<String> orderIds = getOrderIds(deleteRequest.getId() + "");
        if (orderIds.size() > 0) {
            for (String id : orderIds) {
                orderService.deleteById(id);
                myErpOrderService.deleteByOrderId(id);
            }
        }

        return ResultVOUtils.success("删除成功");
    }

}
