package com.glxp.api.admin.controller.inventory;

import com.github.pagehelper.PageInfo;
import com.glxp.api.admin.entity.basic.BasicThirdSysDetailEntity;
import com.glxp.api.admin.entity.basic.UdiEntity;
import com.glxp.api.admin.entity.thrsys.ThrInvProductsEntity;
import com.glxp.api.admin.httpclient.ErpInvClient;
import com.glxp.api.admin.req.basic.FilterUdiInfoRequest;
import com.glxp.api.admin.req.inventory.ErpOnhandRequest;
import com.glxp.api.admin.req.thrsys.FilterThrInvProductsRequest;
import com.glxp.api.admin.res.PageSimpleResponse;
import com.glxp.api.admin.res.basic.UdiRelevanceResponse;
import com.glxp.api.admin.res.inventory.ErpInvProductResponse;
import com.glxp.api.admin.service.basic.BasicThirdSysDetailService;
import com.glxp.api.admin.service.basic.UdiRelevanceService;
import com.glxp.api.admin.service.thrsys.ThrInvProductsService;
import com.glxp.api.admin.util.FilterUdiUtils;
import com.glxp.api.common.enums.ResultEnum;
import com.glxp.api.common.res.BaseResponse;
import com.glxp.api.common.util.ResultVOUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.validation.BindingResult;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.annotation.Resource;
import java.util.List;
import java.util.stream.Collectors;

@RestController
public class InvErpOnhandController {

    @Resource
    UdiRelevanceService udiRelevanceService;
    @Resource
    BasicThirdSysDetailService basicThirdSysDetailService;
    @Resource
    private ThrInvProductsService thrInvProductsService;

    @GetMapping("/udiwms/inv/getOnHand")
    public BaseResponse getOnhand(ErpOnhandRequest onhandQueryRequest,
                                  BindingResult bindingResult) {

        if (bindingResult.hasErrors()) {
            return ResultVOUtils.error(ResultEnum.PARAM_VERIFY_FALL, bindingResult.getFieldError().getDefaultMessage());
        }
        if (onhandQueryRequest.getUdiCode() != null) {
            UdiEntity udiEntity = FilterUdiUtils.getUdi(onhandQueryRequest.getUdiCode());
            if (udiEntity != null) {
                UdiRelevanceResponse udiRelevanceResponse = findByNamecode(udiEntity.getUdi());
                if (udiRelevanceResponse != null) {
                    String thirdId = udiRelevanceResponse.getThirdId();
                    onhandQueryRequest.setNameCode(thirdId);
                    onhandQueryRequest.setBatchNo(udiEntity.getBatchNo());

                } else {
                    return ResultVOUtils.error(500, "产品信息未维护！");
                }

            } else {
                return ResultVOUtils.error(500, "无效UDI码");

            }
        }
        BasicThirdSysDetailEntity basicThirdSysDetailEntity = basicThirdSysDetailService.selectByKey("invPiUrl", onhandQueryRequest.getThirdSys());
        String onHandUrl;
        if (basicThirdSysDetailEntity == null || basicThirdSysDetailEntity.getValue() == null)
            return ResultVOUtils.error(500, "ERP库存产品信息接口地址未定义");
        else
            onHandUrl = basicThirdSysDetailEntity.getValue();
        if (!basicThirdSysDetailEntity.getEnabled()) {
            return ResultVOUtils.error(500, "第三方库存信息服务未启用！");
        }
        if (basicThirdSysDetailEntity.getFromType() == 0 || (onhandQueryRequest.getIsDownThrSys() != null && onhandQueryRequest.getIsDownThrSys())) {

            BaseResponse<PageSimpleResponse<ErpInvProductResponse>> responseBaseResponse = new ErpInvClient().getInvPrdoductResponse(onhandQueryRequest, onHandUrl);
            return responseBaseResponse;
        } else {
            FilterThrInvProductsRequest filterThrInvProductsRequest = new FilterThrInvProductsRequest();
            BeanUtils.copyProperties(onhandQueryRequest, filterThrInvProductsRequest);
            filterThrInvProductsRequest.setThirdSysFk(onhandQueryRequest.getThirdSys());
            List<ThrInvProductsEntity> thrInvProductsEntities
                    = thrInvProductsService.filterThrInvProductss(filterThrInvProductsRequest);
            List<ErpInvProductResponse> erpInvProductResponseList = thrInvProductsEntities.stream().map(item ->
            {
                ErpInvProductResponse erpInvProductResponse = new ErpInvProductResponse();
                BeanUtils.copyProperties(item, erpInvProductResponse);
                erpInvProductResponse.setCode(item.getInventoryCode());
                erpInvProductResponse.setName(item.getInventoryName());
                return erpInvProductResponse;
            }).collect(Collectors.toList());
            PageInfo<ThrInvProductsEntity> pageInfo;
            pageInfo = new PageInfo<>(thrInvProductsEntities);
            PageSimpleResponse<ErpInvProductResponse> pageSimpleResponse = new PageSimpleResponse<>();
            pageSimpleResponse.setTotal(pageInfo.getTotal());
            pageSimpleResponse.setList(erpInvProductResponseList);
            return ResultVOUtils.success(pageSimpleResponse);
        }

    }


    public UdiRelevanceResponse findByNamecode(String udi) {
        FilterUdiInfoRequest filterUdiInfoRequest = new FilterUdiInfoRequest();
        filterUdiInfoRequest.setNameCode(udi);
        List<UdiRelevanceResponse> udiRelevanceResponses = udiRelevanceService.filterUdiGp(filterUdiInfoRequest);
        if (udiRelevanceResponses != null && udiRelevanceResponses.size() > 0) {
            return udiRelevanceResponses.get(0);
        } else {
            return null;
        }
    }


}
