package com.glxp.api.admin.controller.inventory;

import com.github.pagehelper.PageInfo;
import com.glxp.api.admin.constant.IncodeStatus;
import com.glxp.api.admin.entity.basic.BasicThirdSysDetailEntity;
import com.glxp.api.admin.entity.basic.BasicThirdSysEntity;
import com.glxp.api.admin.entity.info.SystemParamConfigEntity;
import com.glxp.api.admin.entity.inventory.InCodeLogEntity;
import com.glxp.api.admin.httpclient.ErpInvClient;
import com.glxp.api.admin.req.inventory.ErpOnhandRequest;
import com.glxp.api.admin.entity.inventory.StockPrintEntity;
import com.glxp.api.admin.req.basic.FilterStockprintRequest;
import com.glxp.api.admin.req.basic.UpdatePrintStatusRequest;
import com.glxp.api.admin.req.inout.DeleteRequest;
import com.glxp.api.admin.req.inventory.PostStockPrintRequest;
import com.glxp.api.admin.res.PageSimpleResponse;
import com.glxp.api.admin.res.inventory.ErpInvProductResponse;
import com.glxp.api.admin.res.inventory.ErpOnHandResponse;
import com.glxp.api.admin.service.basic.BasicThirdSysDetailService;
import com.glxp.api.admin.service.basic.BasicThirdSysService;
import com.glxp.api.admin.service.inventory.InCodeLogService;
import com.glxp.api.admin.service.inventory.InvStockPrintService;
import com.glxp.api.admin.service.info.SystemParamConfigService;
import com.glxp.api.admin.thread.InvImportStockService;
import com.glxp.api.admin.util.CustomUtil;
import com.glxp.api.common.enums.ResultEnum;
import com.glxp.api.common.res.BaseResponse;
import com.glxp.api.common.util.ResultVOUtils;
import org.springframework.validation.BindingResult;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import java.sql.Date;
import java.util.List;

@RestController
public class InvPrintController {
    @Resource
    InvStockPrintService stockPrintService;
    @Resource
    InvImportStockService importStockService;
    @Resource
    private SystemParamConfigService systemParamConfigService;
    @Resource
    BasicThirdSysDetailService basicThirdSysDetailService;
    @Resource
    InCodeLogService inCodeLogService;

    @PostMapping("/udiwarehouse/stockPrint/filter")
    public BaseResponse filterAll(@RequestBody FilterStockprintRequest stockprintRequest,
                                  BindingResult bindingResult) {

        if (bindingResult.hasErrors()) {
            return ResultVOUtils.error(ResultEnum.PARAM_VERIFY_FALL, bindingResult.getFieldError().getDefaultMessage());
        }
        String thirdSys = systemParamConfigService.selectValueByParamKey("incode_thirdSys_fromType");
        stockprintRequest.setThirdSysFk(thirdSys);
        List<StockPrintEntity> stockPrintEntities
                = stockPrintService.filterStockPrint(stockprintRequest);
        PageInfo<StockPrintEntity> pageInfo;
        pageInfo = new PageInfo<>(stockPrintEntities);
        PageSimpleResponse<StockPrintEntity> pageSimpleResponse = new PageSimpleResponse<>();
        pageSimpleResponse.setTotal(pageInfo.getTotal());
        pageSimpleResponse.setList(stockPrintEntities);
        return ResultVOUtils.success(pageSimpleResponse);
    }

    @GetMapping("/udiwms/stockPrint/getFilter")
    public BaseResponse getFilter(FilterStockprintRequest stockprintRequest,
                                  BindingResult bindingResult) {

        if (bindingResult.hasErrors()) {
            return ResultVOUtils.error(ResultEnum.PARAM_VERIFY_FALL, bindingResult.getFieldError().getDefaultMessage());
        }
        List<StockPrintEntity> stockPrintEntities
                = stockPrintService.filterStockPrint(stockprintRequest);
        PageInfo<StockPrintEntity> pageInfo;
        pageInfo = new PageInfo<>(stockPrintEntities);
        PageSimpleResponse<StockPrintEntity> pageSimpleResponse = new PageSimpleResponse<>();
        pageSimpleResponse.setTotal(pageInfo.getTotal());
        pageSimpleResponse.setList(stockPrintEntities);
        return ResultVOUtils.success(pageSimpleResponse);
    }


    @GetMapping("/udiwarehouse/stockPrint/filterNum")
    public BaseResponse filterNum(FilterStockprintRequest stockprintRequest,
                                  BindingResult bindingResult) {

        if (bindingResult.hasErrors()) {
            return ResultVOUtils.error(ResultEnum.PARAM_VERIFY_FALL, bindingResult.getFieldError().getDefaultMessage());
        }
        String onHandUrl;
        String thirdSys;
        if (stockprintRequest.getThirdSysFk() == null)
            thirdSys = "thirdId";
        else
            thirdSys = stockprintRequest.getThirdSysFk();
        BasicThirdSysDetailEntity basicThirdSysDetailEntity = basicThirdSysDetailService.selectByKey("invPiUrl", thirdSys);
        if (basicThirdSysDetailEntity == null || basicThirdSysDetailEntity.getValue() == null) {
            return ResultVOUtils.error(500, "ERP库存产品信息接口地址未定义");
        } else {
            onHandUrl = basicThirdSysDetailEntity.getValue();
        }
        List<StockPrintEntity> stockPrintEntities
                = stockPrintService.filterStockPrint(stockprintRequest);
        if (stockPrintEntities != null) {
            for (StockPrintEntity stockPrintEntity : stockPrintEntities) {
                ErpOnhandRequest erpOnhandRequest = new ErpOnhandRequest();
                erpOnhandRequest.setInventoryCode(stockPrintEntity.getThirdId());
                List<ErpInvProductResponse> erpOnHandResponses = new ErpInvClient().getInvPrdoduct(erpOnhandRequest, onHandUrl);
                if (erpOnHandResponses != null && erpOnHandResponses.size() > 0) {
                    for (ErpInvProductResponse onhandQueryResponse : erpOnHandResponses) {
                        if (onhandQueryResponse.getBatchNo().equals(stockPrintEntity.getBatchNo()) && onhandQueryResponse.getWarehouseCode().equals(stockPrintEntity.getWarehouseCode())) {
                            stockPrintEntity.setNnum(onhandQueryResponse.getCount() + "");
                            break;
                        }
                    }
                }
            }
        }
        PageInfo<StockPrintEntity> pageInfo;
        pageInfo = new PageInfo<>(stockPrintEntities);
        PageSimpleResponse<StockPrintEntity> pageSimpleResponse = new PageSimpleResponse<>();
        pageSimpleResponse.setTotal(pageInfo.getTotal());
        pageSimpleResponse.setList(stockPrintEntities);
        return ResultVOUtils.success(pageSimpleResponse);
    }

    @PostMapping("/udiwarehouse/stockPrint/update")
    public BaseResponse updatePrintStatus(@RequestBody UpdatePrintStatusRequest updatePrintStatusRequest, BindingResult bindingResult) {

        if (bindingResult.hasErrors()) {
            return ResultVOUtils.error(ResultEnum.PARAM_VERIFY_FALL, bindingResult.getFieldError().getDefaultMessage());
        }
        List<String> ids = updatePrintStatusRequest.getIds();
        if (ids != null && ids.size() > 0) {
            stockPrintService.updatePrintStatus(ids);
        } else {
            return ResultVOUtils.error(500, "更新数据不能为空");
        }
        return ResultVOUtils.success("更新成功");
    }


    @PostMapping("/udiwarehouse/stockPrint/modify")
    public BaseResponse updatePrintStatus(@RequestBody StockPrintEntity stockPrintEntity, BindingResult bindingResult) {

        if (bindingResult.hasErrors()) {
            return ResultVOUtils.error(ResultEnum.PARAM_VERIFY_FALL, bindingResult.getFieldError().getDefaultMessage());
        }
        if (stockPrintEntity.getProduceDate() == null) {
            stockPrintEntity.setProduceDate("");
        }

        if (stockPrintEntity.getExpireDate() == null) {
            stockPrintEntity.setExpireDate("");
        }

        if (stockPrintEntity.getBatchNo() == null) {
            stockPrintEntity.setBatchNo("");
        }
        String udiCode = "#" + stockPrintEntity.getNameCode() + "#" + stockPrintEntity.getProduceDate() + "#" + stockPrintEntity.getExpireDate() +
                "#" + stockPrintEntity.getBatchNo() + "#" + "#" + stockPrintEntity.getWarehouseCode();
        stockPrintEntity.setUdiCode(udiCode);
        stockPrintService.updateStockPrint(stockPrintEntity);
        return ResultVOUtils.success("更新成功");
    }

    @PostMapping("/udiwarehouse/stockPrint/delete")
    public BaseResponse deletePrintStatus(@RequestBody DeleteRequest deleteRequest, BindingResult bindingResult) {

        if (bindingResult.hasErrors()) {
            return ResultVOUtils.error(ResultEnum.PARAM_VERIFY_FALL, bindingResult.getFieldError().getDefaultMessage());
        }
        stockPrintService.deleteById(deleteRequest.getId());
        return ResultVOUtils.success("删除成功");
    }

    @GetMapping("/warehouse/stockPrint/dlErpStock")
    public BaseResponse dlErpStock() {
//        dlErpStockService.dlErpStock();
        return ResultVOUtils.success("后台已自动更新，请稍后刷新查看！");
    }


    //产品信息一键生成内部码
    @GetMapping("/udiwarehouse/stockPrint/import")
    public BaseResponse importStock(FilterStockprintRequest filterStockprintRequest) {
        InCodeLogEntity inCodeLogEntity = new InCodeLogEntity();
        inCodeLogEntity.setFromType(IncodeStatus.FROM_UDIINFO);
        inCodeLogEntity.setUpdateTime(new Date(System.currentTimeMillis()));
        String genKey = CustomUtil.getId();
        inCodeLogEntity.setGenKey(genKey);
        inCodeLogEntity.setStatus(IncodeStatus.PROCESS);
        inCodeLogEntity.setThirdSysFk(filterStockprintRequest.getThirdSysFk());
        inCodeLogService.insertInCodeLog(inCodeLogEntity);
        importStockService.updateStock(genKey, filterStockprintRequest.getThirdSysFk());
        return ResultVOUtils.success("后台已开始自动更新，请稍后刷新查看");
    }

    // 根据第三方产品ID生成内部码
    @GetMapping("/udiwarehouse/stockPrint/importBycode")
    public BaseResponse importBycode(FilterStockprintRequest stockprintRequest) {
        if (stockprintRequest.getThirdId() == null || stockprintRequest.getThirdId().equals("")) {
            return ResultVOUtils.error(500, "存货编码不能为空");
        }

        SystemParamConfigEntity systemParamConfigEntity = systemParamConfigService.selectByParamKey("erp_inv_product_query");
        String url;
        if (systemParamConfigEntity == null)
            return ResultVOUtils.error(500, "ERP库存产品信息查询接口地址未定义");
        else url = systemParamConfigEntity.getParamValue();

        importStockService.updateByCode(stockprintRequest.getThirdId(), url, stockprintRequest.getThirdSysFk());
        return ResultVOUtils.success("后台已开始自动更新，请稍后刷新查看");
    }

    //根据产品信息生成内部码
    @PostMapping("udiwms/stockprint/postByUdiInfo")
    public BaseResponse postByUdiInfo(@RequestBody PostStockPrintRequest postStockPrintRequest, BindingResult bindingResult) {
        if (bindingResult.hasErrors()) {
            return ResultVOUtils.error(ResultEnum.PARAM_VERIFY_FALL, bindingResult.getFieldError().getDefaultMessage());
        }
        if (postStockPrintRequest != null) {
            InCodeLogEntity inCodeLogEntity = new InCodeLogEntity();
            inCodeLogEntity.setFromType(IncodeStatus.FROM_UDIINFO);
            inCodeLogEntity.setUpdateTime(new Date(System.currentTimeMillis()));
            inCodeLogEntity.setStatus(IncodeStatus.PROCESS);
            String genKey = CustomUtil.getId();
            inCodeLogEntity.setGenKey(genKey);
            inCodeLogEntity.setThirdSysFk(postStockPrintRequest.getThirdSys());
            inCodeLogService.insertInCodeLog(inCodeLogEntity);
            importStockService.updateStockById(genKey, postStockPrintRequest.getIds(), postStockPrintRequest.getThirdSys());
        }
        return ResultVOUtils.success("后台已开始自动更新，请稍后刷新查看！");
    }

    //根据库存产品信息生成内部码
    @PostMapping("udiwms/stockprint/postByInvInfo")
    public BaseResponse postByInvInfo(@RequestBody PostStockPrintRequest postStockPrintRequest, BindingResult bindingResult) {
        if (bindingResult.hasErrors()) {
            return ResultVOUtils.error(ResultEnum.PARAM_VERIFY_FALL, bindingResult.getFieldError().getDefaultMessage());
        }
        if (postStockPrintRequest != null) {
            InCodeLogEntity inCodeLogEntity = new InCodeLogEntity();
            inCodeLogEntity.setFromType(IncodeStatus.FROM_INVINFO);
            inCodeLogEntity.setUpdateTime(new Date(System.currentTimeMillis()));
            inCodeLogEntity.setStatus(IncodeStatus.PROCESS);
            String genKey = CustomUtil.getId();
            inCodeLogEntity.setGenKey(genKey);
            inCodeLogEntity.setThirdSysFk(postStockPrintRequest.getThirdSys());
            inCodeLogService.insertInCodeLog(inCodeLogEntity);
            importStockService.updateByInvInfo(genKey, postStockPrintRequest.getIds(), postStockPrintRequest.getThirdSys());
        }
        return ResultVOUtils.success("后台已开始自动更新，请稍后刷新查看！");
    }

    //根据库存产品信息一键生成内部码
    @GetMapping("udiwms/stockprint/postAllByInvInfo")
    public BaseResponse postAllByInvInfo(FilterStockprintRequest filterStockprintRequest, BindingResult bindingResult) {
        if (bindingResult.hasErrors()) {
            return ResultVOUtils.error(ResultEnum.PARAM_VERIFY_FALL, bindingResult.getFieldError().getDefaultMessage());
        }
        if (filterStockprintRequest != null) {
            InCodeLogEntity inCodeLogEntity = new InCodeLogEntity();
            inCodeLogEntity.setFromType(IncodeStatus.FROM_INVINFO);
            inCodeLogEntity.setUpdateTime(new Date(System.currentTimeMillis()));
            inCodeLogEntity.setStatus(IncodeStatus.PROCESS);
            String genKey = CustomUtil.getId();
            inCodeLogEntity.setGenKey(genKey);
            inCodeLogEntity.setThirdSysFk(filterStockprintRequest.getThirdSysFk());
            inCodeLogService.insertInCodeLog(inCodeLogEntity);
            importStockService.updateAllByInvInfo(genKey, filterStockprintRequest.getThirdSysFk());
        }
        return ResultVOUtils.success("后台已开始自动更新，请稍后刷新查看！");
    }

    //其他系统上传参数生成内部码
    @PostMapping("udiwms/stockprint/post")
    public BaseResponse postInCode(@RequestBody PostStockPrintRequest postStockPrintRequest, BindingResult bindingResult) {

        if (bindingResult.hasErrors()) {
            return ResultVOUtils.error(ResultEnum.PARAM_VERIFY_FALL, bindingResult.getFieldError().getDefaultMessage());
        }
        String genKey = null;
        if (postStockPrintRequest != null) {

            InCodeLogEntity inCodeLogEntity = new InCodeLogEntity();
            inCodeLogEntity.setFromType(IncodeStatus.FROM_THRSYS);
            inCodeLogEntity.setUpdateTime(new Date(System.currentTimeMillis()));
            if (postStockPrintRequest.getGenKey() == null) {
                genKey = CustomUtil.getId();
                inCodeLogEntity.setGenKey(genKey);
            } else {
                genKey = postStockPrintRequest.getGenKey();
                inCodeLogEntity.setGenKey(postStockPrintRequest.getGenKey());
            }
            inCodeLogEntity.setStatus(IncodeStatus.PROCESS);
            inCodeLogEntity.setThirdSysFk(postStockPrintRequest.getThirdSys());
            inCodeLogService.insertInCodeLog(inCodeLogEntity);

            List<StockPrintEntity> stockPrintEntities = postStockPrintRequest.getDatas();
            if (stockPrintEntities != null && stockPrintEntities.size() > 0) {
                for (StockPrintEntity stockPrintEntity : stockPrintEntities) {
//                    FilterStockprintRequest filterStockprintRequest = new FilterStockprintRequest();
//                    filterStockprintRequest.setThirdSysFk(filterStockprintRequest.getThirdSysFk());
//                    filterStockprintRequest.setNameCode(stockPrintEntity.getNameCode());
//                    filterStockprintRequest.setWarehouseCode(stockPrintEntity.getWarehouseCode());
//                    filterStockprintRequest.setBatchNo(stockPrintEntity.getBatchNo());
//                    filterStockprintRequest.setThirdId(stockPrintEntity.getThirdId());
//                    filterStockprintRequest.setSpaceCode(stockPrintEntity.getSpaceCode());
//                    List<StockPrintEntity> stockPrintEntityList = stockPrintService.filterStockPrint(filterStockprintRequest);
//                    if (stockPrintEntityList != null && stockPrintEntityList.size() > 0) {
//                        StockPrintEntity temp = stockPrintEntities.get(0);
//                        stockPrintEntity.setId(temp.getId());
//                        stockPrintService.updateStockPrint(stockPrintEntity);
//                        StockPrintEntity stockPrintEntity1 = stockPrintService.findByNameCode(temp.getId() + "");
//                        stockPrintEntity = stockPrintEntity1;
//                    }
                    stockPrintEntity.setGenKeyFk(genKey);
                    String udiCode = "#" + stockPrintEntity.getNameCode() + "#" + stockPrintEntity.getProduceDate() + "#" + stockPrintEntity.getExpireDate() +
                            "#" + stockPrintEntity.getBatchNo() + "#" + stockPrintEntity.getSerialNo() + "#" + stockPrintEntity.getWarehouseCode() + "#" + stockPrintEntity.getSpaceCode();
                    stockPrintEntity.setUdiCode(udiCode);
                    stockPrintService.insertStockPrint(stockPrintEntity);
                }
            }
            inCodeLogEntity.setStatus(IncodeStatus.FINISH);
            inCodeLogService.updateInCodeLog(inCodeLogEntity);
        }
        return ResultVOUtils.success(genKey);
    }


}
