package com.glxp.api.admin.controller.thrsys;

import com.github.pagehelper.PageInfo;
import com.glxp.api.admin.constant.BasicProcessStatus;
import com.glxp.api.admin.entity.basic.BasicThirdSysDetailEntity;
import com.glxp.api.admin.entity.thrsys.*;
import com.glxp.api.admin.entity.thrsys.ThrCorpExportLogEntity;
import com.glxp.api.admin.httpclient.ErpBasicClient;
import com.glxp.api.admin.req.basic.BasicUnitMaintainFilterRequest;
import com.glxp.api.admin.req.basic.FilterUdiEpLogRequest;
import com.glxp.api.admin.req.inout.DeleteRequest;
import com.glxp.api.admin.req.thrsys.*;
import com.glxp.api.admin.res.PageSimpleResponse;
import com.glxp.api.admin.res.basic.ErpUnitsResponse;
import com.glxp.api.admin.service.basic.BasicThirdSysDetailService;
import com.glxp.api.admin.service.thrsys.ThrCorpExportLogService;
import com.glxp.api.admin.service.thrsys.ThrCorpService;
import com.glxp.api.admin.thread.ThrCorpsDlService;
import com.glxp.api.admin.util.CustomUtil;
import com.glxp.api.common.enums.ResultEnum;
import com.glxp.api.common.res.BaseResponse;
import com.glxp.api.common.util.ResultVOUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.validation.BindingResult;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RestController;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletResponse;
import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.OutputStream;
import java.util.Date;
import java.util.List;
import java.util.stream.Collectors;

/**
 * 第三方数据往来单位导出日志
 */
@RestController
public class ThrCorpExportLogController {

    @Resource
    ThrCorpExportLogService thrCorpExportLogService;
    @Resource
    ThrCorpService thrCorpService;
    @Resource
    ThrCorpsDlService thrCorpsDlService;
    @Resource
    BasicThirdSysDetailService basicThirdSysDetailService;

    @GetMapping("/udiwms/thrCorp/exportLog/filter")
    public BaseResponse filter(FilterUdiEpLogRequest filterUdiEpLogReques,
                               BindingResult bindingResult) {

        if (bindingResult.hasErrors()) {
            return ResultVOUtils.error(ResultEnum.PARAM_VERIFY_FALL, bindingResult.getFieldError().getDefaultMessage());
        }
        List<ThrCorpExportLogEntity> thrCorpExportLogEntities
                = thrCorpExportLogService.filterThrCorpExportLog(filterUdiEpLogReques);
        PageInfo<ThrCorpExportLogEntity> pageInfo;
        pageInfo = new PageInfo<>(thrCorpExportLogEntities);
        PageSimpleResponse<ThrCorpExportLogEntity> pageSimpleResponse = new PageSimpleResponse<>();
        pageSimpleResponse.setTotal(pageInfo.getTotal());
        pageSimpleResponse.setList(thrCorpExportLogEntities);
        return ResultVOUtils.success(pageSimpleResponse);
    }

    @PostMapping("/udiwms/thrCorp/exportLog/deleteLog")
    public BaseResponse deleteLog(@RequestBody DeleteRequest deleteRequest, BindingResult bindingResult) {

        if (bindingResult.hasErrors()) {
            return ResultVOUtils.error(ResultEnum.PARAM_VERIFY_FALL, bindingResult.getFieldError().getDefaultMessage());
        }
        String id = deleteRequest.getId();
        thrCorpExportLogService.deleteById(id + "");
        return ResultVOUtils.success("删除成功");
    }

    //往来单位excel文件下载
    @GetMapping("/udiwms/thrCorp/exportLog/download")
    public void download(FilterUdiEpLogRequest filterUdiEpLogRequest, HttpServletResponse res) {

        List<ThrCorpExportLogEntity> thrCorpExportLogEntities = thrCorpExportLogService.filterThrCorpExportLog(filterUdiEpLogRequest);
        ThrCorpExportLogEntity thrOrderExportLogEntity = thrCorpExportLogEntities.get(0);
        try {
            String filePath = thrOrderExportLogEntity.getFilePath();
            String fileName = "temp";
            if (filePath != null && filePath.length() > 0) {
                String[] data = filePath.split("/");
                fileName = data[data.length - 1];
            }
            res.setHeader("Content-disposition", "attachment;fileName=" + fileName);
            res.setContentType("text/plain;charset=UTF-8");
            FileInputStream input = new FileInputStream(new File(filePath));
            OutputStream out = res.getOutputStream();
            byte[] b = new byte[2048];
            int len;
            while ((len = input.read(b)) != -1) {
                out.write(b, 0, len);
            }
            input.close();
            if (thrOrderExportLogEntity.getDlCount() == 0) {
                thrOrderExportLogEntity.setDlCount(1);
            } else {
                thrOrderExportLogEntity.setDlCount(thrOrderExportLogEntity.getDlCount() + 1);
            }
            thrCorpExportLogService.updateThrCorpExportLog(thrOrderExportLogEntity);
        } catch (Exception ex) {
            System.out.println("下载失败！");
        }


    }

    //往来单位导出生成Excel，并创建生成记录
    @PostMapping("/udiwms/thrCorp/importLog/export")
    public BaseResponse excelDownload(@RequestBody ThrCorpExportRequest thrCorpExportRequest) {

        String fileName = "D:\\udiwms\\exportFile\\" + "第三方往来单位信息导出" + System.currentTimeMillis() + ".xls";
        File file = new File(fileName);
        if (!file.exists()) {
            try {
                file.createNewFile();
            } catch (IOException e) {
                e.printStackTrace();
            }
        }
        ThrCorpExportLogEntity thrCorpExportLogEntity = new ThrCorpExportLogEntity();
        thrCorpExportLogEntity.setStatus(BasicProcessStatus.UDIINFO_IMPORT_PROCESS);
        String genKey = CustomUtil.getId();
        thrCorpExportLogEntity.setGenKey(genKey);
        thrCorpExportLogEntity.setFilePath(fileName);
        thrCorpExportLogEntity.setUpdateTime(new Date());
        thrCorpExportLogEntity.setDlCount(0);
        thrCorpExportLogEntity.setType(1);
        thrCorpExportLogService.insertThrCorpExportLog(thrCorpExportLogEntity);
        thrCorpsDlService.genExcel(genKey, thrCorpExportRequest);
        return ResultVOUtils.success("后台正在导出生成excel文件，请稍后刷新查看!");

    }


    //第三方往来单位接口下载（第三方使用）
    @GetMapping("/udiwms/thrsys/thrCorp/download")
    public BaseResponse downloadThrCorps(BasicUnitMaintainFilterRequest unitMaintainFilterRequest,
                                         BindingResult bindingResult) {

        if (bindingResult.hasErrors()) {
            return ResultVOUtils.error(ResultEnum.PARAM_VERIFY_FALL, bindingResult.getFieldError().getDefaultMessage());
        }
        BasicThirdSysDetailEntity basicThirdSysDetailEntity = basicThirdSysDetailService.selectByKey("corpUrl", unitMaintainFilterRequest.getThirdSys());
        if (basicThirdSysDetailEntity == null || basicThirdSysDetailEntity.getValue() == null)
            return ResultVOUtils.error(500, "往来单位接口地址未定义");
        if (basicThirdSysDetailEntity.getEnabled()) {
            if (basicThirdSysDetailEntity.getFromType() == 0) {
                BaseResponse<PageSimpleResponse<ErpUnitsResponse>> udiDlDeviceResponse =
                        new ErpBasicClient().getErpCrop(unitMaintainFilterRequest, basicThirdSysDetailEntity.getValue());
                return udiDlDeviceResponse;
            } else {
                FilterThrCorpRequest filterThrCorpRequest = new FilterThrCorpRequest();
                BeanUtils.copyProperties(unitMaintainFilterRequest, filterThrCorpRequest);
                filterThrCorpRequest.setThirdSysFk(unitMaintainFilterRequest.getThirdSys());
                List<ThrCorpEntity> thrCorpEntities
                        = thrCorpService.filterThrCorps(filterThrCorpRequest);
                List<ThrCorpEntity> thrCorpEntityList = thrCorpEntities.stream().map(item ->
                {
                    ThrCorpEntity thrCorpEntity = new ThrCorpEntity();
                    BeanUtils.copyProperties(item, thrCorpEntity);
                    thrCorpEntity.setId(item.getUnitId());
                    return thrCorpEntity;
                }).collect(Collectors.toList());
                PageInfo<ThrCorpEntity> pageInfo;
                pageInfo = new PageInfo<>(thrCorpEntities);
                PageSimpleResponse<ThrCorpEntity> pageSimpleResponse = new PageSimpleResponse<>();
                pageSimpleResponse.setTotal(pageInfo.getTotal());
                pageSimpleResponse.setList(thrCorpEntityList);
                return ResultVOUtils.success(pageSimpleResponse);
            }
        } else {
            return ResultVOUtils.error(500, "第三方往来单位服务未启用");
        }


    }
}
