package com.glxp.api.admin.controller.thrsys;

import com.github.pagehelper.PageInfo;
import com.glxp.api.admin.entity.thrsys.ThrCorpImportDetailEntity;
import com.glxp.api.admin.entity.thrsys.ThrCorpImportLogEntity;
import com.glxp.api.admin.entity.thrsys.ThrProductsImportDetailEntity;
import com.glxp.api.admin.req.basic.FilterUdiIpLogRequest;
import com.glxp.api.admin.req.inout.DeleteRequest;
import com.glxp.api.admin.res.PageSimpleResponse;
import com.glxp.api.admin.service.thrsys.ThrCorpImportDetailService;
import com.glxp.api.admin.service.thrsys.ThrCorpImportLogService;
import com.glxp.api.common.enums.ResultEnum;
import com.glxp.api.common.res.BaseResponse;
import com.glxp.api.common.util.ResultVOUtils;
import org.springframework.validation.BindingResult;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RestController;

import javax.annotation.Resource;
import java.util.List;

/**
 * 第三方数据往来单位导入日志
 */
@RestController
public class ThrCorpImportLogController {

    @Resource
    ThrCorpImportLogService thrCorpImportLogService;
    @Resource
    ThrCorpImportDetailService thrCorpImportDetailService;

    @GetMapping("/udiwms/thrCorp/importLog/filter")
    public BaseResponse filter(FilterUdiIpLogRequest filterUdiIpLogRequest,
                               BindingResult bindingResult) {

        if (bindingResult.hasErrors()) {
            return ResultVOUtils.error(ResultEnum.PARAM_VERIFY_FALL, bindingResult.getFieldError().getDefaultMessage());
        }
        List<ThrCorpImportLogEntity> thrCorpImportLogEntities
                = thrCorpImportLogService.filterImportLog(filterUdiIpLogRequest);
        PageInfo<ThrCorpImportLogEntity> pageInfo;
        pageInfo = new PageInfo<>(thrCorpImportLogEntities);
        PageSimpleResponse<ThrCorpImportLogEntity> pageSimpleResponse = new PageSimpleResponse<>();
        pageSimpleResponse.setTotal(pageInfo.getTotal());
        pageSimpleResponse.setList(thrCorpImportLogEntities);
        return ResultVOUtils.success(pageSimpleResponse);
    }

    @GetMapping("/udiwms/thrCorp/importLog/filterDetail")
    public BaseResponse filterDetail(FilterUdiIpLogRequest filterUdiIpLogRequest,
                                     BindingResult bindingResult) {

        if (bindingResult.hasErrors()) {
            return ResultVOUtils.error(ResultEnum.PARAM_VERIFY_FALL, bindingResult.getFieldError().getDefaultMessage());
        }
        List<ThrCorpImportDetailEntity> thrCorpImportDetailEntities
                = thrCorpImportDetailService.filterCorpDetailImport(filterUdiIpLogRequest);
        PageInfo<ThrCorpImportDetailEntity> pageInfo;
        pageInfo = new PageInfo<>(thrCorpImportDetailEntities);
        PageSimpleResponse<ThrCorpImportDetailEntity> pageSimpleResponse = new PageSimpleResponse<>();
        pageSimpleResponse.setTotal(pageInfo.getTotal());
        pageSimpleResponse.setList(thrCorpImportDetailEntities);
        return ResultVOUtils.success(pageSimpleResponse);
    }

    @PostMapping("/udiwms/thrCorp/importLog/deleteLog")
    public BaseResponse deleteLog(@RequestBody DeleteRequest deleteRequest, BindingResult bindingResult) {

        if (bindingResult.hasErrors()) {
            return ResultVOUtils.error(ResultEnum.PARAM_VERIFY_FALL, bindingResult.getFieldError().getDefaultMessage());
        }
        String id = deleteRequest.getId();
        FilterUdiIpLogRequest filterInCodeLogRequest = new FilterUdiIpLogRequest();
        filterInCodeLogRequest.setId(Integer.parseInt(id));
        List<ThrCorpImportLogEntity> thrCorpImportLogEntities = thrCorpImportLogService.filterImportLog(filterInCodeLogRequest);
        if (thrCorpImportLogEntities != null && thrCorpImportLogEntities.size() > 0) {
            ThrCorpImportLogEntity thrCorpImportLogEntity = thrCorpImportLogEntities.get(0);
            thrCorpImportLogService.deleteById(thrCorpImportLogEntity.getId() + "");
            thrCorpImportDetailService.deleteByGenkey(thrCorpImportLogEntity.getGenKey());
        }
        return ResultVOUtils.success("删除成功");
    }


}
