package com.glxp.api.admin.controller.thrsys;

import com.github.pagehelper.PageInfo;
import com.glxp.api.admin.constant.BasicProcessStatus;
import com.glxp.api.admin.entity.thrsys.*;
import com.glxp.api.admin.req.basic.BasicUnitMaintainFilterRequest;
import com.glxp.api.admin.req.inout.DeleteRequest;
import com.glxp.api.admin.req.thrsys.FilterThrCorpRequest;
import com.glxp.api.admin.req.thrsys.FilterThrProductsRequest;
import com.glxp.api.admin.req.thrsys.PostThrCorpRequest;
import com.glxp.api.admin.res.PageSimpleResponse;
import com.glxp.api.admin.res.basic.ErpUnitsResponse;
import com.glxp.api.admin.service.thrsys.ThrCorpImportLogService;
import com.glxp.api.admin.service.thrsys.ThrCorpService;
import com.glxp.api.admin.thread.ThrCorpsDlService;
import com.glxp.api.admin.util.Constant;
import com.glxp.api.admin.util.CustomUtil;
import com.glxp.api.admin.util.RedisUtil;
import com.glxp.api.common.enums.ResultEnum;
import com.glxp.api.common.res.BaseResponse;
import com.glxp.api.common.util.ResultVOUtils;
import org.springframework.validation.BindingResult;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import java.util.Date;
import java.util.List;

@RestController
public class ThrCorpsController {
    @Resource
    RedisUtil redisUtil;
    @Resource
    private ThrCorpService thrCorpService;
    @Resource
    private ThrCorpsDlService thrCorpsDlService;
    @Resource
    private ThrCorpImportLogService thrCorpImportLogService;


    @GetMapping("/udiwms/thrsys/getCorps")
    public BaseResponse getCorps(FilterThrCorpRequest filterThrCorpRequest,
                                 BindingResult bindingResult) {

        if (bindingResult.hasErrors()) {
            return ResultVOUtils.error(ResultEnum.PARAM_VERIFY_FALL, bindingResult.getFieldError().getDefaultMessage());
        }
        List<ThrCorpEntity> thrCorpEntities
                = thrCorpService.filterThrCorps(filterThrCorpRequest);
        PageInfo<ThrCorpEntity> pageInfo;
        pageInfo = new PageInfo<>(thrCorpEntities);
        PageSimpleResponse<ThrCorpEntity> pageSimpleResponse = new PageSimpleResponse<>();
        pageSimpleResponse.setTotal(pageInfo.getTotal());
        pageSimpleResponse.setList(thrCorpEntities);
        return ResultVOUtils.success(pageSimpleResponse);
    }

    @PostMapping("/udiwms/thrsys/delCorps")
    public BaseResponse delCorps(@RequestBody DeleteRequest deleteRequest, BindingResult bindingResult) {

        if (bindingResult.hasErrors()) {
            return ResultVOUtils.error(ResultEnum.PARAM_VERIFY_FALL, bindingResult.getFieldError().getDefaultMessage());
        }
        String id = deleteRequest.getId();
        thrCorpService.deleteById(id);
        return ResultVOUtils.success("删除成功");
    }

    @RequestMapping("/udiwms/thrsys/corp/delAll")
    public BaseResponse delAll() {
        thrCorpService.deleteAll();
        return ResultVOUtils.success("删除成功");
    }


    //第三方服务上传（包括文件导入后上传）
    @PostMapping("/udiwms/thrsys/postCorps")
    public BaseResponse postCorps(@RequestBody PostThrCorpRequest postThrCorpRequest,
                                  BindingResult bindingResult) {

        if (bindingResult.hasErrors()) {
            return ResultVOUtils.error(ResultEnum.PARAM_VERIFY_FALL, bindingResult.getFieldError().getDefaultMessage());
        }
        List<ErpUnitsResponse> erpUnitsResponses = postThrCorpRequest.getCorps();
        if (erpUnitsResponses != null && erpUnitsResponses.size() > 0) {
            String genKey = postThrCorpRequest.getGenKey();
            if (genKey == null) {
                genKey = CustomUtil.getId();
            }
            ThrCorpImportLogEntity thrCorpImportLogEntity = thrCorpImportLogService.selectByGenKey(genKey);
            if (thrCorpImportLogEntity == null) {
                thrCorpImportLogEntity = new ThrCorpImportLogEntity();
                thrCorpImportLogEntity.setGenKey(genKey);
                if (postThrCorpRequest.getUploadType() != null) {
                    thrCorpImportLogEntity.setFromType(postThrCorpRequest.getUploadType());
                } else
                    thrCorpImportLogEntity.setFromType("第三方系统上传");
                thrCorpImportLogEntity.setStatus(BasicProcessStatus.UDIINFO_IMPORT_PROCESS);
                thrCorpImportLogEntity.setUpdateTime(new Date());
                thrCorpImportLogEntity.setThirdSysFk(postThrCorpRequest.getThirdSys());
                thrCorpImportLogService.insertImportLog(thrCorpImportLogEntity);
            }
            thrCorpsDlService.insertData(genKey, postThrCorpRequest.getThirdSys(), erpUnitsResponses);
            return ResultVOUtils.success("往来单位上传成功！后台正在处理，请稍后刷新查看！");
        }
        return ResultVOUtils.error(500, "上传数据为空");
    }

    //接口一键全部
    @RequestMapping("/udiwms/thrsys/corp/downloadAll")
    public BaseResponse downloadAll(FilterThrCorpRequest filterThrProductsRequest) {
        String data = (String) redisUtil.get(Constant.dlThrProducts);
        if (data != null && data.equals("true")) {
            return ResultVOUtils.error(500, "当前任务正在下载更新产品信息，请稍后重试！");
        } else {
            redisUtil.set(Constant.dlThrCorps, "true", 30);
            if (filterThrProductsRequest.getThirdSysFk() == null) {
                return ResultVOUtils.error(500, "未选择第三方系统！");
            }
            ThrCorpImportLogEntity thrCorpImportLogEntity = new ThrCorpImportLogEntity();
            String genKey = CustomUtil.getId();
            thrCorpImportLogEntity.setGenKey(genKey);
            thrCorpImportLogEntity.setFromType("第三方系统获取");
            thrCorpImportLogEntity.setStatus(BasicProcessStatus.UDIINFO_IMPORT_PROCESS);
            thrCorpImportLogEntity.setUpdateTime(new Date());
            thrCorpImportLogEntity.setThirdSysFk(filterThrProductsRequest.getThirdSysFk());
            thrCorpImportLogService.insertImportLog(thrCorpImportLogEntity);
            thrCorpsDlService.importCorps(genKey,null, filterThrProductsRequest.getThirdSysFk());
            return ResultVOUtils.success("后台开始下载更新，请稍后刷新查看");
        }
    }

    //选中或者下载全部
    @PostMapping("/udiwms/thrsys/corp/corpsDlAll")
    public BaseResponse corpsDlAll(@RequestBody BasicUnitMaintainFilterRequest filterThrCorpRequest) {
        filterThrCorpRequest.setThirdSysFk(filterThrCorpRequest.getThirdSys());
        String data = (String) redisUtil.get(Constant.dlThrProducts);
        if (data != null && data.equals("true")) {
            return ResultVOUtils.error(500, "当前任务正在下载更新产品信息，请稍后重试！");
        } else {
            redisUtil.set(Constant.dlThrCorps, "true", 30);
            if (filterThrCorpRequest.getThirdSysFk() == null) {
                return ResultVOUtils.error(500, "未选择第三方系统！");
            }
            ThrCorpImportLogEntity thrCorpImportLogEntity = new ThrCorpImportLogEntity();
            String genKey = CustomUtil.getId();
            thrCorpImportLogEntity.setGenKey(genKey);
            thrCorpImportLogEntity.setFromType("第三方系统获取");
            thrCorpImportLogEntity.setStatus(BasicProcessStatus.UDIINFO_IMPORT_PROCESS);
            thrCorpImportLogEntity.setUpdateTime(new Date());
            thrCorpImportLogEntity.setThirdSysFk(filterThrCorpRequest.getThirdSysFk());
            thrCorpImportLogService.insertImportLog(thrCorpImportLogEntity);

            if (filterThrCorpRequest.getThrCorpEntities() != null && filterThrCorpRequest.getThrCorpEntities().size() > 0) {//选中导出
                thrCorpsDlService.importSelectCorps(genKey, filterThrCorpRequest.getThrCorpEntities(), filterThrCorpRequest.getThirdSysFk());
            } else {        //结果导出
                thrCorpsDlService.importCorps(genKey,filterThrCorpRequest, filterThrCorpRequest.getThirdSysFk());
            }

//            thrCorpsDlService.importCorps(genKey, filterThrCorpRequest.getThirdSysFk());
            return ResultVOUtils.success("后台开始下载更新，请稍后刷新查看");
        }
    }
}
