package com.glxp.api.admin.controller.thrsys;

import com.github.pagehelper.PageInfo;
import com.glxp.api.admin.constant.BasicProcessStatus;
import com.glxp.api.admin.entity.thrsys.ThrInvProductsEntity;
import com.glxp.api.admin.entity.thrsys.ThrInvProductsImportLogEntity;
import com.glxp.api.admin.req.basic.BasicUnitMaintainFilterRequest;
import com.glxp.api.admin.req.inout.DeleteRequest;
import com.glxp.api.admin.req.inventory.ErpOnhandRequest;
import com.glxp.api.admin.req.thrsys.FilterThrInvProductsRequest;
import com.glxp.api.admin.req.thrsys.PostThrInvProductsRequest;
import com.glxp.api.admin.res.PageSimpleResponse;
import com.glxp.api.admin.res.inventory.ErpInvProductResponse;
import com.glxp.api.admin.service.thrsys.ThrInvProductsImportLogService;
import com.glxp.api.admin.service.thrsys.ThrInvProductsService;
import com.glxp.api.admin.thread.ThrInvProductsDlService;
import com.glxp.api.admin.util.Constant;
import com.glxp.api.admin.util.CustomUtil;
import com.glxp.api.admin.util.RedisUtil;
import com.glxp.api.common.enums.ResultEnum;
import com.glxp.api.common.res.BaseResponse;
import com.glxp.api.common.util.ResultVOUtils;
import org.springframework.validation.BindingResult;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import java.util.Date;
import java.util.List;

@RestController
public class ThrInvProductsController {

    @Resource
    RedisUtil redisUtil;
    @Resource
    private ThrInvProductsService thrInvProductsService;
    @Resource
    private ThrInvProductsDlService thrInvProductsDlService;
    @Resource
    private ThrInvProductsImportLogService thrInvProductsImportLogService;

    @GetMapping("/udiwms/thrsys/getInvProducts")
    public BaseResponse getInvProducts(FilterThrInvProductsRequest filterThrInvProductsRequest,
                                       BindingResult bindingResult) {

        if (bindingResult.hasErrors()) {
            return ResultVOUtils.error(ResultEnum.PARAM_VERIFY_FALL, bindingResult.getFieldError().getDefaultMessage());
        }
        List<ThrInvProductsEntity> thrInvProductsEntities
                = thrInvProductsService.filterThrInvProductss(filterThrInvProductsRequest);
        PageInfo<ThrInvProductsEntity> pageInfo;
        pageInfo = new PageInfo<>(thrInvProductsEntities);
        PageSimpleResponse<ThrInvProductsEntity> pageSimpleResponse = new PageSimpleResponse<>();
        pageSimpleResponse.setTotal(pageInfo.getTotal());
        pageSimpleResponse.setList(thrInvProductsEntities);
        return ResultVOUtils.success(pageSimpleResponse);
    }


    //第三方服务上传（包括文件导入后上传）
    @PostMapping("/udiwms/thrsys/postInvProducts")
    public BaseResponse postInvProducts(@RequestBody PostThrInvProductsRequest postThrInvProductsRequest,
                                        BindingResult bindingResult) {

        if (bindingResult.hasErrors()) {
            return ResultVOUtils.error(ResultEnum.PARAM_VERIFY_FALL, bindingResult.getFieldError().getDefaultMessage());
        }

        List<ErpInvProductResponse> erpOnHandResponses = postThrInvProductsRequest.getDatas();

        String genKey = postThrInvProductsRequest.getGenKey();
        if (genKey == null) {
            genKey = CustomUtil.getId();
        }
        ThrInvProductsImportLogEntity thrInvProductsImportLogEntity = thrInvProductsImportLogService.selectByGenKey(genKey);
        if (thrInvProductsImportLogEntity == null) {
            thrInvProductsImportLogEntity = new ThrInvProductsImportLogEntity();
            thrInvProductsImportLogEntity.setGenKey(genKey);
            if (postThrInvProductsRequest.getUploadType() != null) {
                thrInvProductsImportLogEntity.setFromType(postThrInvProductsRequest.getUploadType());
            } else
                thrInvProductsImportLogEntity.setFromType("第三方系统上传");
            thrInvProductsImportLogEntity.setStatus(BasicProcessStatus.UDIINFO_IMPORT_PROCESS);
            thrInvProductsImportLogEntity.setUpdateTime(new Date());
            thrInvProductsImportLogEntity.setThirdSysFk(postThrInvProductsRequest.getThirdSys());
            thrInvProductsImportLogService.insertImportLog(thrInvProductsImportLogEntity);
        }
        if (erpOnHandResponses != null && erpOnHandResponses.size() > 0) {
            thrInvProductsDlService.insertInvProducts(genKey, postThrInvProductsRequest.getThirdSys(), erpOnHandResponses);
            return ResultVOUtils.success("库存产品信息上传成功,后台正在处理，请稍后重试！");
        }
        thrInvProductsImportLogEntity.setStatus(BasicProcessStatus.UDIINFO_IMPORT_FAIL);
        thrInvProductsImportLogEntity.setUpdateTime(new Date());
        thrInvProductsImportLogEntity.setRemark("上传数据为空");
        thrInvProductsImportLogService.updateImportLog(thrInvProductsImportLogEntity);
        return ResultVOUtils.error(500, "上传数据为空");
    }

    @PostMapping("/udiwms/thrsys/delInvProducts")
    public BaseResponse delInvProducts(@RequestBody DeleteRequest deleteRequest, BindingResult bindingResult) {

        if (bindingResult.hasErrors()) {
            return ResultVOUtils.error(ResultEnum.PARAM_VERIFY_FALL, bindingResult.getFieldError().getDefaultMessage());
        }
        String id = deleteRequest.getId();
        boolean b = thrInvProductsService.deleteById(id);
        if (b)
            return ResultVOUtils.success("删除成功");
        else return ResultVOUtils.error(500, "无法删除！");
    }


    @RequestMapping("/udiwms/thrsys/invProducts/delAll")
    public BaseResponse delAll() {
        thrInvProductsService.deleteAll();
        return ResultVOUtils.success("删除成功");
    }

    @RequestMapping("/udiwms/thrsys/invProducts/downloadAll")
    public BaseResponse downloadAll(FilterThrInvProductsRequest filterThrInvProductsRequest) {
        String data = (String) redisUtil.get(Constant.dlThrInvProducts);
        if (data != null && data.equals("true")) {
            return ResultVOUtils.error(500, "当前任务正在下载更新库存产品信息，请稍后重试！");
        } else {
            redisUtil.set(Constant.dlThrProducts, "true", 10 * 60);
            if (filterThrInvProductsRequest.getThirdSysFk() == null) {
                return ResultVOUtils.error(500, "未选择第三方系统！");
            }
            ThrInvProductsImportLogEntity thrInvProductsImportLogEntity = new ThrInvProductsImportLogEntity();
            String genKey = CustomUtil.getId();
            thrInvProductsImportLogEntity.setGenKey(genKey);
            thrInvProductsImportLogEntity.setFromType("第三方系统获取");
            thrInvProductsImportLogEntity.setStatus(BasicProcessStatus.UDIINFO_IMPORT_PROCESS);
            thrInvProductsImportLogEntity.setUpdateTime(new Date());
            thrInvProductsImportLogEntity.setThirdSysFk(filterThrInvProductsRequest.getThirdSys());
            thrInvProductsImportLogService.insertImportLog(thrInvProductsImportLogEntity);
            thrInvProductsDlService.importProducrs(genKey,filterThrInvProductsRequest.getThirdSysFk(), null);
            return ResultVOUtils.success("后台开始下载更新，请稍后刷新查看");
        }
    }

    @PostMapping("/udiwms/thrsys/invProducts/inpisDlAll")
    public BaseResponse inpisDlAll(@RequestBody ErpOnhandRequest erpOnhandRequest) {

        String data = (String) redisUtil.get(Constant.dlThrInvProducts);
        if (data != null && data.equals("true")) {
            return ResultVOUtils.error(500, "当前任务正在下载更新库存产品信息，请稍后重试！");
        } else {
            redisUtil.set(Constant.dlThrProducts, "true", 10 * 60);
            if (erpOnhandRequest.getThirdSys() == null) {
                return ResultVOUtils.error(500, "未选择第三方系统！");
            }
            ThrInvProductsImportLogEntity thrInvProductsImportLogEntity = new ThrInvProductsImportLogEntity();
            String genKey = CustomUtil.getId();
            thrInvProductsImportLogEntity.setGenKey(genKey);
            thrInvProductsImportLogEntity.setFromType("第三方系统获取");
            thrInvProductsImportLogEntity.setStatus(BasicProcessStatus.UDIINFO_IMPORT_PROCESS);
            thrInvProductsImportLogEntity.setUpdateTime(new Date());
            thrInvProductsImportLogEntity.setThirdSysFk(erpOnhandRequest.getThirdSys());
            thrInvProductsImportLogService.insertImportLog(thrInvProductsImportLogEntity);

            if (erpOnhandRequest.getThrInvProductsEntities() != null && erpOnhandRequest.getThrInvProductsEntities().size() > 0) {//选中导出
                thrInvProductsDlService.importSelcyInvProducrs(genKey,erpOnhandRequest.getThirdSys(), erpOnhandRequest.getThrInvProductsEntities());
            } else {        //结果导出
                thrInvProductsDlService.importProducrs(genKey,erpOnhandRequest.getThirdSys(),erpOnhandRequest);
            }

            return ResultVOUtils.success("后台开始下载更新，请稍后刷新查看");
        }

    }
}
