package com.glxp.api.admin.controller.thrsys;

import com.github.pagehelper.PageInfo;
import com.glxp.api.admin.constant.BasicProcessStatus;
import com.glxp.api.admin.entity.basic.BasicThirdSysDetailEntity;
import com.glxp.api.admin.entity.thrsys.ThrInvProductsEntity;
import com.glxp.api.admin.entity.thrsys.ThrInvProductsExportLogEntity;
import com.glxp.api.admin.httpclient.ErpInvClient;
import com.glxp.api.admin.req.basic.FilterUdiEpLogRequest;
import com.glxp.api.admin.req.inout.DeleteRequest;
import com.glxp.api.admin.req.inventory.ErpOnhandRequest;
import com.glxp.api.admin.req.thrsys.FilterThrInvProductsRequest;
import com.glxp.api.admin.req.thrsys.ThrInvProductsExportRequest;
import com.glxp.api.admin.res.PageSimpleResponse;
import com.glxp.api.admin.res.inventory.ErpInvProductResponse;
import com.glxp.api.admin.service.basic.BasicThirdSysDetailService;
import com.glxp.api.admin.service.thrsys.ThrInvProductsExportLogService;
import com.glxp.api.admin.service.thrsys.ThrInvProductsService;
import com.glxp.api.admin.thread.ThrInvProductsDlService;
import com.glxp.api.admin.util.CustomUtil;
import com.glxp.api.common.enums.ResultEnum;
import com.glxp.api.common.res.BaseResponse;
import com.glxp.api.common.util.ResultVOUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.validation.BindingResult;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RestController;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletResponse;
import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.OutputStream;
import java.util.Date;
import java.util.List;
import java.util.stream.Collectors;

@RestController
public class ThrInvProductsExportLogController {
    @Resource
    ThrInvProductsExportLogService thrInvProductsExportLogService;
    @Resource
    ThrInvProductsDlService thrInvProductsDlService;
    @Resource
    BasicThirdSysDetailService basicThirdSysDetailService;
    @Resource
    private ThrInvProductsService thrInvProductsService;

    @GetMapping("/udiwms/invProducts/exportLog/filter")
    public BaseResponse filter(FilterUdiEpLogRequest filterUdiEpLogReques,
                               BindingResult bindingResult) {

        if (bindingResult.hasErrors()) {
            return ResultVOUtils.error(ResultEnum.PARAM_VERIFY_FALL, bindingResult.getFieldError().getDefaultMessage());
        }
        List<ThrInvProductsExportLogEntity> thrInvProductsExportLogEntities
                = thrInvProductsExportLogService.filterThrInvProductsExportLog(filterUdiEpLogReques);
        PageInfo<ThrInvProductsExportLogEntity> pageInfo;
        pageInfo = new PageInfo<>(thrInvProductsExportLogEntities);
        PageSimpleResponse<ThrInvProductsExportLogEntity> pageSimpleResponse = new PageSimpleResponse<>();
        pageSimpleResponse.setTotal(pageInfo.getTotal());
        pageSimpleResponse.setList(thrInvProductsExportLogEntities);
        return ResultVOUtils.success(pageSimpleResponse);
    }

    @PostMapping("/udiwms/invProducts/exportLog/deleteLog")
    public BaseResponse deleteLog(@RequestBody DeleteRequest deleteRequest, BindingResult bindingResult) {

        if (bindingResult.hasErrors()) {
            return ResultVOUtils.error(ResultEnum.PARAM_VERIFY_FALL, bindingResult.getFieldError().getDefaultMessage());
        }
        String id = deleteRequest.getId();
        thrInvProductsExportLogService.deleteById(id + "");
        return ResultVOUtils.success("删除成功");
    }

    //产品信息excel文件下载
    @GetMapping("/udiwms/invProducts/exportLog/download")
    public void download(FilterUdiEpLogRequest filterUdiEpLogRequest, HttpServletResponse res) {

        List<ThrInvProductsExportLogEntity> thrInvProductsExportLogEntities = thrInvProductsExportLogService.filterThrInvProductsExportLog(filterUdiEpLogRequest);
        ThrInvProductsExportLogEntity thrInvProductsExportLogEntity = thrInvProductsExportLogEntities.get(0);
        try {
            String filePath = thrInvProductsExportLogEntity.getFilePath();
            String fileName = "temp";
            if (filePath != null && filePath.length() > 0) {
                String[] data = filePath.split("/");
                fileName = data[data.length - 1];
            }
            res.setHeader("Content-disposition", "attachment;fileName=" + fileName);
            res.setContentType("text/plain;charset=UTF-8");
            FileInputStream input = new FileInputStream(new File(filePath));
            OutputStream out = res.getOutputStream();
            byte[] b = new byte[2048];
            int len;
            while ((len = input.read(b)) != -1) {
                out.write(b, 0, len);
            }
            input.close();
            if (thrInvProductsExportLogEntity.getDlCount() == 0) {
                thrInvProductsExportLogEntity.setDlCount(1);
            } else {
                thrInvProductsExportLogEntity.setDlCount(thrInvProductsExportLogEntity.getDlCount() + 1);
            }
            thrInvProductsExportLogService.updateThrInvProductsExportLog(thrInvProductsExportLogEntity);
        } catch (Exception ex) {
            System.out.println("下载失败！");
        }


    }

    //产品信息导出生成Excel，并创建生成记录
    @PostMapping("/udiwms/invProducts/importLog/export")
    public BaseResponse excelDownload(@RequestBody ThrInvProductsExportRequest thrInvProductsExportRequest) {

        String fileName = "D:\\udiwms\\exportFile\\" + "库存产品信息导出" + System.currentTimeMillis() + ".xls";
        File file = new File(fileName);
        if (!file.exists()) {
            try {
                file.createNewFile();
            } catch (IOException e) {
                e.printStackTrace();
            }
        }
        ThrInvProductsExportLogEntity udiInfoExportLogEntity = new ThrInvProductsExportLogEntity();
        udiInfoExportLogEntity.setStatus(BasicProcessStatus.UDIINFO_IMPORT_PROCESS);
        String genKey = CustomUtil.getId();
        udiInfoExportLogEntity.setGenKey(genKey);
        udiInfoExportLogEntity.setFilePath(fileName);
        udiInfoExportLogEntity.setUpdateTime(new Date());
        udiInfoExportLogEntity.setDlCount(0);
        thrInvProductsExportLogService.insertThrInvProductsExportLog(udiInfoExportLogEntity);
        thrInvProductsDlService.genExcel(genKey, thrInvProductsExportRequest);
        return ResultVOUtils.success("后台正在导出生成excel文件，请稍后刷新查看!");

    }


    //第三方库存产品信息接口下载（第三方使用）
    @GetMapping("/udiwms/thirdSys/invProducts/download")
    public BaseResponse getOnhand(ErpOnhandRequest onhandQueryRequest,
                                  BindingResult bindingResult) {

        if (bindingResult.hasErrors()) {
            return ResultVOUtils.error(ResultEnum.PARAM_VERIFY_FALL, bindingResult.getFieldError().getDefaultMessage());
        }
        BasicThirdSysDetailEntity basicThirdSysDetailEntity = basicThirdSysDetailService.selectByKey("invPiUrl", onhandQueryRequest.getThirdSys());
        String onHandUrl;
        if (basicThirdSysDetailEntity == null || basicThirdSysDetailEntity.getValue() == null)
            return ResultVOUtils.error(500, "ERP库存产品信息接口地址未定义");
        else
            onHandUrl = basicThirdSysDetailEntity.getValue();
        if (!basicThirdSysDetailEntity.getEnabled()) {
            return ResultVOUtils.error(500, "第三方库存信息服务未启用！");
        }
        if (basicThirdSysDetailEntity.getFromType() == 0 ) {

            BaseResponse<PageSimpleResponse<ErpInvProductResponse>> responseBaseResponse = new ErpInvClient().getInvPrdoductResponse(onhandQueryRequest, onHandUrl);
            return responseBaseResponse;
        } else {
            FilterThrInvProductsRequest filterThrInvProductsRequest = new FilterThrInvProductsRequest();
            BeanUtils.copyProperties(onhandQueryRequest, filterThrInvProductsRequest);
            filterThrInvProductsRequest.setThirdSysFk(onhandQueryRequest.getThirdSys());
            List<ThrInvProductsEntity> thrInvProductsEntities
                    = thrInvProductsService.filterThrInvProductss(filterThrInvProductsRequest);
            List<ErpInvProductResponse> erpInvProductResponseList = thrInvProductsEntities.stream().map(item ->
            {
                ErpInvProductResponse erpInvProductResponse = new ErpInvProductResponse();
                BeanUtils.copyProperties(item, erpInvProductResponse);
                erpInvProductResponse.setCode(item.getInventoryCode());
                erpInvProductResponse.setName(item.getInventoryName());
                return erpInvProductResponse;
            }).collect(Collectors.toList());
            PageInfo<ErpInvProductResponse> pageInfo;
            pageInfo = new PageInfo<>(erpInvProductResponseList);
            PageSimpleResponse<ErpInvProductResponse> pageSimpleResponse = new PageSimpleResponse<>();
            pageSimpleResponse.setTotal(pageInfo.getTotal());
            pageSimpleResponse.setList(erpInvProductResponseList);
            return ResultVOUtils.success(pageSimpleResponse);
        }

    }

}
