package com.glxp.api.admin.controller.thrsys;

import com.github.pagehelper.PageInfo;
import com.glxp.api.admin.entity.thrsys.ThrCorpImportDetailEntity;
import com.glxp.api.admin.entity.thrsys.ThrInvProductsImportDetailEntity;
import com.glxp.api.admin.entity.thrsys.ThrInvProductsImportLogEntity;
import com.glxp.api.admin.req.basic.FilterUdiIpLogRequest;
import com.glxp.api.admin.req.inout.DeleteRequest;
import com.glxp.api.admin.res.PageSimpleResponse;
import com.glxp.api.admin.service.thrsys.ThrInvProductsDetailService;
import com.glxp.api.admin.service.thrsys.ThrInvProductsImportLogService;
import com.glxp.api.common.enums.ResultEnum;
import com.glxp.api.common.res.BaseResponse;
import com.glxp.api.common.util.ResultVOUtils;
import org.springframework.validation.BindingResult;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RestController;

import javax.annotation.Resource;
import java.util.List;

@RestController
public class ThrInvProductsImportLogController {


    @Resource
    ThrInvProductsImportLogService thrInvProductsImportLogService;
    @Resource
    ThrInvProductsDetailService thrInvProductsDetailService;

    @GetMapping("/udiwms/thrInvProducts/importLog/filter")
    public BaseResponse filter(FilterUdiIpLogRequest filterUdiIpLogRequest,
                               BindingResult bindingResult) {

        if (bindingResult.hasErrors()) {
            return ResultVOUtils.error(ResultEnum.PARAM_VERIFY_FALL, bindingResult.getFieldError().getDefaultMessage());
        }
        List<ThrInvProductsImportLogEntity> thrInvProductsImportLogEntities
                = thrInvProductsImportLogService.filterImportLog(filterUdiIpLogRequest);
        PageInfo<ThrInvProductsImportLogEntity> pageInfo;
        pageInfo = new PageInfo<>(thrInvProductsImportLogEntities);
        PageSimpleResponse<ThrInvProductsImportLogEntity> pageSimpleResponse = new PageSimpleResponse<>();
        pageSimpleResponse.setTotal(pageInfo.getTotal());
        pageSimpleResponse.setList(thrInvProductsImportLogEntities);
        return ResultVOUtils.success(pageSimpleResponse);
    }

    @GetMapping("/udiwms/thrInvProducts/importLog/filterDetail")
    public BaseResponse filterDetail(FilterUdiIpLogRequest filterUdiIpLogRequest,
                                     BindingResult bindingResult) {

        if (bindingResult.hasErrors()) {
            return ResultVOUtils.error(ResultEnum.PARAM_VERIFY_FALL, bindingResult.getFieldError().getDefaultMessage());
        }
        List<ThrInvProductsImportDetailEntity> thrInvProductsImportDetailEntities
                = thrInvProductsDetailService.filterInvProductsDetailImport(filterUdiIpLogRequest);
        PageInfo<ThrInvProductsImportDetailEntity> pageInfo;
        pageInfo = new PageInfo<>(thrInvProductsImportDetailEntities);
        PageSimpleResponse<ThrInvProductsImportDetailEntity> pageSimpleResponse = new PageSimpleResponse<>();
        pageSimpleResponse.setTotal(pageInfo.getTotal());
        pageSimpleResponse.setList(thrInvProductsImportDetailEntities);
        return ResultVOUtils.success(pageSimpleResponse);
    }

    @PostMapping("/udiwms/thrInvProducts/importLog/deleteLog")
    public BaseResponse deleteLog(@RequestBody DeleteRequest deleteRequest, BindingResult bindingResult) {

        if (bindingResult.hasErrors()) {
            return ResultVOUtils.error(ResultEnum.PARAM_VERIFY_FALL, bindingResult.getFieldError().getDefaultMessage());
        }
        String id = deleteRequest.getId();
        FilterUdiIpLogRequest filterInCodeLogRequest = new FilterUdiIpLogRequest();
        filterInCodeLogRequest.setId(Integer.parseInt(id));
        List<ThrInvProductsImportLogEntity> thrInvProductsImportLogEntities = thrInvProductsImportLogService.filterImportLog(filterInCodeLogRequest);
        if (thrInvProductsImportLogEntities != null && thrInvProductsImportLogEntities.size() > 0) {
            ThrInvProductsImportLogEntity thrInvProductsImportLogEntity = thrInvProductsImportLogEntities.get(0);
            thrInvProductsImportLogService.deleteById(thrInvProductsImportLogEntity.getId() + "");
            thrInvProductsDetailService.deleteByGenkey(thrInvProductsImportLogEntity.getGenKey());
            return ResultVOUtils.success("删除成功");
        }else {
            return ResultVOUtils.error(500, "删除失败，未找到该记录");
        }


    }

}
