package com.glxp.api.admin.controller.thrsys;

import com.alibaba.fastjson.JSONObject;
import com.alibaba.fastjson.TypeReference;
import com.github.pagehelper.PageInfo;
import com.glxp.api.admin.constant.BasicProcessStatus;
import com.glxp.api.admin.entity.info.SystemParamConfigEntity;
import com.glxp.api.admin.entity.inout.ErpOrderEntity;
import com.glxp.api.admin.entity.inout.FilterErpOrderRequest;
import com.glxp.api.admin.entity.thrsys.*;
import com.glxp.api.admin.req.inout.DeleteRequest;
import com.glxp.api.admin.req.thrsys.*;
import com.glxp.api.admin.res.PageSimpleResponse;
import com.glxp.api.admin.res.inout.ErpOrderResponse;
import com.glxp.api.admin.service.info.SystemParamConfigService;
import com.glxp.api.admin.service.inout.OrderDetailService;
import com.glxp.api.admin.service.thrsys.ThrOrderDetailService;
import com.glxp.api.admin.service.thrsys.ThrOrderImportDetailService;
import com.glxp.api.admin.service.thrsys.ThrOrderImportLogService;
import com.glxp.api.admin.service.thrsys.ThrOrderService;
import com.glxp.api.admin.thread.ThrOrderImportService;
import com.glxp.api.admin.thread.ThrOrdersDlService;
import com.glxp.api.admin.util.Constant;
import com.glxp.api.admin.util.CustomUtil;
import com.glxp.api.admin.util.HttpClient;
import com.glxp.api.admin.util.RedisUtil;
import com.glxp.api.common.enums.ResultEnum;
import com.glxp.api.common.res.BaseResponse;
import com.glxp.api.common.util.ResultVOUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.validation.BindingResult;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.stream.Collectors;

@RestController
public class ThrOrderController {
    @Resource
    private ThrOrderService thrOrderService;
    @Resource
    private ThrOrderDetailService thrOrderDetailService;
    @Resource
    private ThrOrdersDlService thrOrdersDlService;
    @Resource
    private ThrOrderImportLogService thrOrderImportLogService;
    @Resource
    private ThrOrderImportDetailService thrOrderImportDetailService;
    @Resource
    RedisUtil redisUtil;
    @Resource
    private SystemParamConfigService systemParamConfigService;
    @Resource
    ThrOrderImportService thrOrderImportService;

    @Resource
    private OrderDetailService myErpOrderService;

    @GetMapping("/udiwms/thrsys/getOrders")
    public BaseResponse getOrders(FilterThrOrderRequest filterThrOrderRequest,
                                  BindingResult bindingResult) {

        if (bindingResult.hasErrors()) {
            return ResultVOUtils.error(ResultEnum.PARAM_VERIFY_FALL, bindingResult.getFieldError().getDefaultMessage());
        }
        List<ThrOrderEntity> thrOrderEntities
                = thrOrderService.filterThrOrder(filterThrOrderRequest);
        PageInfo<ThrOrderEntity> pageInfo;
        pageInfo = new PageInfo<>(thrOrderEntities);
        PageSimpleResponse<ThrOrderEntity> pageSimpleResponse = new PageSimpleResponse<>();
        pageSimpleResponse.setTotal(pageInfo.getTotal());
        pageSimpleResponse.setList(thrOrderEntities);
        return ResultVOUtils.success(pageSimpleResponse);
    }


    @GetMapping("/udiwms/thrsys/getOrderDetails")
    public BaseResponse getOrderDetails(FilterThrOrderDetailRequest filterThrOrderDetailRequest,
                                        BindingResult bindingResult) {

        if (bindingResult.hasErrors()) {
            return ResultVOUtils.error(ResultEnum.PARAM_VERIFY_FALL, bindingResult.getFieldError().getDefaultMessage());
        }
        List<ThrOrderDetailEntity> thrOrderDetailEntities
                = thrOrderDetailService.filterThrOrderDetailDetail(filterThrOrderDetailRequest);
        PageInfo<ThrOrderDetailEntity> pageInfo;
        pageInfo = new PageInfo<>(thrOrderDetailEntities);
        PageSimpleResponse<ThrOrderDetailEntity> pageSimpleResponse = new PageSimpleResponse<>();
        pageSimpleResponse.setTotal(pageInfo.getTotal());
        pageSimpleResponse.setList(thrOrderDetailEntities);
        return ResultVOUtils.success(pageSimpleResponse);
    }


    @PostMapping("/udiwms/thrsys/postOrderDetail")
    public BaseResponse postOrderDetail(@RequestBody PostThrOrderRequest postThrOrderRequest,
                                        BindingResult bindingResult) {

        if (bindingResult.hasErrors()) {
            return ResultVOUtils.error(ResultEnum.PARAM_VERIFY_FALL, bindingResult.getFieldError().getDefaultMessage());
        }
        String genKey = postThrOrderRequest.getGenKey();
        if (genKey == null) {
            genKey = CustomUtil.getId();
        }
        ThrOrderImportLogEntity thrOrderImportLogEntity = thrOrderImportLogService.selectByGenKey(genKey);
        if (thrOrderImportLogEntity == null) {
            thrOrderImportLogEntity = new ThrOrderImportLogEntity();
            thrOrderImportLogEntity.setGenKey(genKey);
            thrOrderImportLogEntity.setFromType("第三方系统上传");
            thrOrderImportLogEntity.setStatus(BasicProcessStatus.UDIINFO_IMPORT_UNPROCESS);
            thrOrderImportLogEntity.setUpdateTime(new Date());
            thrOrderImportLogEntity.setThirdSysFk(postThrOrderRequest.getThirdSys());
            thrOrderImportLogService.insertImportLog(thrOrderImportLogEntity);
        }
        List<ErpOrderResponse> erpOrderResponses = postThrOrderRequest.getDatas();
        if (erpOrderResponses != null && erpOrderResponses.size() > 0) {
            List<ThrOrderImportDetailEntity> thrOrderImportDetailEntities = new ArrayList<>();
            String finalGenKey = genKey;
            for (ErpOrderResponse erpOrderResponse : erpOrderResponses) {
                List<ErpOrderResponse.SubErpOrder> subErpOrders = erpOrderResponse.getSubErpOrders();
                if (subErpOrders != null && subErpOrders.size() > 0) {
                    for (ErpOrderResponse.SubErpOrder subErpOrder : subErpOrders) {
                        ThrOrderImportDetailEntity thrOrderDetailEntity = new ThrOrderImportDetailEntity();
                        BeanUtils.copyProperties(erpOrderResponse, thrOrderDetailEntity);
                        BeanUtils.copyProperties(subErpOrder, thrOrderDetailEntity);
                        thrOrderDetailEntity.setGenKeyFk(finalGenKey);
                        thrOrderDetailEntity.setThirdSysFk(postThrOrderRequest.getThirdSys());
                        thrOrderDetailEntity.setUpdateTime(new Date());
                        thrOrderImportDetailEntities.add(thrOrderDetailEntity);
                    }

                }
            }
            thrOrderImportDetailService.insertOrderImportDetails(thrOrderImportDetailEntities);
            thrOrderImportLogEntity.setStatus(BasicProcessStatus.UDIINFO_IMPORT_UNPROCESS);
            thrOrderImportLogEntity.setUpdateTime(new Date());
            thrOrderImportLogService.updateImportLog(thrOrderImportLogEntity);
            thrOrderImportService.importThrOrder(finalGenKey);
        } else {
            thrOrderImportLogEntity.setStatus(BasicProcessStatus.UDIINFO_IMPORT_FAIL);
            thrOrderImportLogEntity.setUpdateTime(new Date());
            thrOrderImportLogService.updateImportLog(thrOrderImportLogEntity);
            return ResultVOUtils.error(500, "上传数据为空");
        }
        return ResultVOUtils.success("单据上传成功！");
    }

    @PostMapping("/udiwms/thrsys/delOrder")
    public BaseResponse delOrder(@RequestBody DeleteRequest deleteRequest, BindingResult bindingResult) {

        if (bindingResult.hasErrors()) {
            return ResultVOUtils.error(ResultEnum.PARAM_VERIFY_FALL, bindingResult.getFieldError().getDefaultMessage());
        }
        String id = deleteRequest.getId();
        ThrOrderEntity thrOrderEntity = thrOrderService.findById(id);
        String msg = selectDelect(thrOrderEntity);
        if (msg == null) {
            boolean b = thrOrderService.deleteById(id);
            thrOrderDetailService.deleteByOrderIdFk(thrOrderEntity.getId() + "");
            if (b)
                return ResultVOUtils.success("删除成功");
            else return ResultVOUtils.error(500, "无法删除！");
        } else return ResultVOUtils.error(500, msg);

    }

    public String selectDelect(ThrOrderEntity thrOrderEntity) {
        FilterErpOrderRequest filterErpOrderRequest = new FilterErpOrderRequest();
        filterErpOrderRequest.setErpOrderId(thrOrderEntity.getBillNo());
        List<ErpOrderEntity> erpOrderEntities = myErpOrderService.filterAllMyErpOrder(filterErpOrderRequest);
        if (erpOrderEntities != null && erpOrderEntities.size() > 0) {
            return "单据" + thrOrderEntity.getBillNo() + "已于出入库单据关联，无法删除";
        } else {
            return null;
        }
    }

    @RequestMapping("/udiwms/thrsys/order/delAll")
    public BaseResponse delAll() {
        thrOrderService.deleteAll();
        thrOrderDetailService.deleteAll();
        return ResultVOUtils.success("删除成功");
    }


    @RequestMapping("/udiwms/thrsys/orders/downloadAll")
    public BaseResponse downloadAll(FilterThrOrderRequest filterThrProductsRequest) {
        String data = (String) redisUtil.get(Constant.dlThrOrders);
        if (false) {//(data != null && data.equals("true")) {
            return ResultVOUtils.error(500, "当前任务正在下载更新业务单据信息，请稍后重试！");
        } else {
            redisUtil.set(Constant.dlThrOrders, "true", 10 * 30);
            if (filterThrProductsRequest.getBillAction() == null) {
                return ResultVOUtils.error(500, "未选择业务类型！");
            }

            ThrOrderImportLogEntity thrOrderImportLogEntity = new ThrOrderImportLogEntity();
            String genKey = CustomUtil.getId();
            thrOrderImportLogEntity.setGenKey(genKey);
            thrOrderImportLogEntity.setFromType("第三方系统获取");
            thrOrderImportLogEntity.setStatus(BasicProcessStatus.UDIINFO_IMPORT_UNPROCESS);
            thrOrderImportLogEntity.setUpdateTime(new Date());
            thrOrderImportLogEntity.setThirdSysFk(filterThrProductsRequest.getThirdSysFk());
            thrOrderImportLogService.insertImportLog(thrOrderImportLogEntity);
            if (filterThrProductsRequest.getErpOrderResponses() != null) {//选中导出
                thrOrdersDlService.importSelectOrders(genKey, filterThrProductsRequest.getErpOrderResponses(), filterThrProductsRequest.getThirdSysFk());
            } else {        //结果导出
                thrOrdersDlService.importOrders(genKey, filterThrProductsRequest.getBillAction(),null);
            }
            return ResultVOUtils.success("后台开始下载更新，请稍后刷新查看");
        }
    }

    @PostMapping("/udiwms/thrsys/orders/orderDownload")
    public BaseResponse orderDownload(@RequestBody FilterThrOrderRequest filterThrProductsRequest) {
        String data = (String) redisUtil.get(Constant.dlThrOrders);
        if (false) {//(data != null && data.equals("true")) {
            return ResultVOUtils.error(500, "当前任务正在下载更新业务单据信息，请稍后重试！");
        } else {
            redisUtil.set(Constant.dlThrOrders, "true", 10 * 30);
            if (filterThrProductsRequest.getBillAction() == null) {
                return ResultVOUtils.error(500, "未选择业务类型！");
            }

            ThrOrderImportLogEntity thrOrderImportLogEntity = new ThrOrderImportLogEntity();
            String genKey = CustomUtil.getId();
            thrOrderImportLogEntity.setGenKey(genKey);
            thrOrderImportLogEntity.setFromType("第三方系统获取");
            thrOrderImportLogEntity.setStatus(BasicProcessStatus.UDIINFO_IMPORT_UNPROCESS);
            thrOrderImportLogEntity.setUpdateTime(new Date());
            thrOrderImportLogEntity.setThirdSysFk(filterThrProductsRequest.getThirdSysFk());
            thrOrderImportLogService.insertImportLog(thrOrderImportLogEntity);
            if (filterThrProductsRequest.getErpOrderResponses() != null && filterThrProductsRequest.getErpOrderResponses().size()>0) {//选中导出
                thrOrdersDlService.importSelectOrders(genKey, filterThrProductsRequest.getErpOrderResponses(), filterThrProductsRequest.getThirdSysFk());
            } else {        //结果导出
                thrOrdersDlService.importOrders(genKey, filterThrProductsRequest.getBillAction(),filterThrProductsRequest);
            }
            return ResultVOUtils.success("后台开始下载更新，请稍后刷新查看");
        }
    }
    @PostMapping("/udiwms/smp/postOrder")
    public BaseResponse postSmpOrder(@RequestBody PostSmpOrderRequest postSmpOrderRequest,
                                     BindingResult bindingResult) {

        if (bindingResult.hasErrors()) {
            return ResultVOUtils.error(ResultEnum.PARAM_VERIFY_FALL, bindingResult.getFieldError().getDefaultMessage());
        }

        SystemParamConfigEntity systemParamConfigEntity = systemParamConfigService.selectByParamKey("smp_uploadOrder_url");
        String postUrl;
        if (systemParamConfigEntity == null)
            return ResultVOUtils.error(500, "上传供应商平台接口地址未定义");
        else
            postUrl = systemParamConfigEntity.getParamValue();

        List<String> ids = postSmpOrderRequest.getIds();
        for (String id : ids) {
            ThrOrderEntity thrOrderEntity = thrOrderService.findById(id);
            UploadSmpRequest uploadSmpRequest = new UploadSmpRequest();
            BeanUtils.copyProperties(thrOrderEntity, uploadSmpRequest);
            uploadSmpRequest.setType(1);
            FilterThrOrderDetailRequest filterThrOrderDetailRequest = new FilterThrOrderDetailRequest();
            filterThrOrderDetailRequest.setOrderIdFk(id);
            List<ThrOrderDetailEntity> thrOrderDetailEntities = thrOrderDetailService.filterThrOrderDetailDetail(filterThrOrderDetailRequest);
            if (thrOrderDetailEntities != null && thrOrderDetailEntities.size() > 0) {
                List<UploadSmpRequest.SubErpOrder> subErpOrders = new ArrayList<>();
                for (ThrOrderDetailEntity thrOrderDetailEntity : thrOrderDetailEntities) {
                    UploadSmpRequest.SubErpOrder subErpOrder = new UploadSmpRequest.SubErpOrder();
                    BeanUtils.copyProperties(thrOrderDetailEntity, subErpOrder);
                    subErpOrders.add(subErpOrder);
                }
                uploadSmpRequest.setSubErpOrders(subErpOrders);

                String response = HttpClient.uCloudPost(postUrl, uploadSmpRequest);
                BaseResponse baseResponse =
                        JSONObject.parseObject(response, new TypeReference<BaseResponse>() {
                        });
                if (baseResponse.getCode() == 20000) {
                    return ResultVOUtils.success("上传成功！");
                } else {
                    return ResultVOUtils.error(baseResponse.getCode(), baseResponse.getMessage());
                }
            }
        }
        return ResultVOUtils.error(500, "上传数据成功");
    }
}
