package com.glxp.api.admin.controller.thrsys;

import com.github.pagehelper.PageInfo;
import com.glxp.api.admin.constant.BasicProcessStatus;
import com.glxp.api.admin.entity.basic.BasicThirdSysDetailEntity;
import com.glxp.api.admin.entity.thrsys.ThrOrderDetailEntity;
import com.glxp.api.admin.entity.thrsys.ThrOrderEntity;
import com.glxp.api.admin.entity.thrsys.ThrOrderExportLogEntity;
import com.glxp.api.admin.httpclient.ErpOrderClient;
import com.glxp.api.admin.req.basic.FilterUdiEpLogRequest;
import com.glxp.api.admin.req.inout.DeleteRequest;
import com.glxp.api.admin.req.inout.FilterOrderRequest;
import com.glxp.api.admin.req.thrsys.FilterThrOrderDetailRequest;
import com.glxp.api.admin.req.thrsys.FilterThrOrderRequest;
import com.glxp.api.admin.req.thrsys.ThrOrderExportRequest;
import com.glxp.api.admin.res.PageSimpleResponse;
import com.glxp.api.admin.res.inout.ErpOrderResponse;
import com.glxp.api.admin.service.basic.BussinessTypeService;
import com.glxp.api.admin.service.thrsys.ThrOrderDetailService;
import com.glxp.api.admin.service.thrsys.ThrOrderExportLogService;
import com.glxp.api.admin.service.thrsys.ThrOrderService;
import com.glxp.api.admin.thread.ThrOrdersDlService;
import com.glxp.api.admin.util.CustomUtil;
import com.glxp.api.common.enums.ResultEnum;
import com.glxp.api.common.res.BaseResponse;
import com.glxp.api.common.util.ResultVOUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.validation.BindingResult;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RestController;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletResponse;
import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.OutputStream;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

@RestController
public class ThrOrderExportLogController {
    @Resource
    ThrOrderExportLogService thrOrderExportLogService;
    @Resource
    ThrOrdersDlService thrOrdersDlService;
    @Resource
    private BussinessTypeService bussinessTypeService;
    @Resource
    private ThrOrderService thrOrderService;
    @Resource
    private ThrOrderDetailService thrOrderDetailService;

    @GetMapping("/udiwms/thrOrder/exportLog/filter")
    public BaseResponse filter(FilterUdiEpLogRequest filterUdiEpLogReques,
                               BindingResult bindingResult) {

        if (bindingResult.hasErrors()) {
            return ResultVOUtils.error(ResultEnum.PARAM_VERIFY_FALL, bindingResult.getFieldError().getDefaultMessage());
        }
        List<ThrOrderExportLogEntity> udiInfoExportLogEntities
                = thrOrderExportLogService.filterThrOrderExportLog(filterUdiEpLogReques);
        PageInfo<ThrOrderExportLogEntity> pageInfo;
        pageInfo = new PageInfo<>(udiInfoExportLogEntities);
        PageSimpleResponse<ThrOrderExportLogEntity> pageSimpleResponse = new PageSimpleResponse<>();
        pageSimpleResponse.setTotal(pageInfo.getTotal());
        pageSimpleResponse.setList(udiInfoExportLogEntities);
        return ResultVOUtils.success(pageSimpleResponse);
    }

    @PostMapping("/udiwms/thrOrder/exportLog/deleteLog")
    public BaseResponse deleteLog(@RequestBody DeleteRequest deleteRequest, BindingResult bindingResult) {

        if (bindingResult.hasErrors()) {
            return ResultVOUtils.error(ResultEnum.PARAM_VERIFY_FALL, bindingResult.getFieldError().getDefaultMessage());
        }
        String id = deleteRequest.getId();
        thrOrderExportLogService.deleteById(id + "");
        return ResultVOUtils.success("删除成功");
    }

    //业务单据excel文件下载
    @GetMapping("/udiwms/thrOrder/exportLog/download")
    public void download(FilterUdiEpLogRequest filterUdiEpLogRequest, HttpServletResponse res) {

        List<ThrOrderExportLogEntity> thrOrderExportLogEntities = thrOrderExportLogService.filterThrOrderExportLog(filterUdiEpLogRequest);
        ThrOrderExportLogEntity thrOrderExportLogEntity = thrOrderExportLogEntities.get(0);
        try {
            String filePath = thrOrderExportLogEntity.getFilePath();
            String fileName = "temp";
            if (filePath != null && filePath.length() > 0) {
                String[] data = filePath.split("/");
                fileName = data[data.length - 1];
            }
            res.setHeader("Content-disposition", "attachment;fileName=" + fileName);
            res.setContentType("text/plain;charset=UTF-8");
            FileInputStream input = new FileInputStream(new File(filePath));
            OutputStream out = res.getOutputStream();
            byte[] b = new byte[2048];
            int len;
            while ((len = input.read(b)) != -1) {
                out.write(b, 0, len);
            }
            input.close();
            if (thrOrderExportLogEntity.getDlCount() == 0) {
                thrOrderExportLogEntity.setDlCount(1);
            } else {
                thrOrderExportLogEntity.setDlCount(thrOrderExportLogEntity.getDlCount() + 1);
            }
            thrOrderExportLogService.updateThrOrderExportLog(thrOrderExportLogEntity);
        } catch (Exception ex) {
            System.out.println("下载失败！");
        }


    }

    //业务单据导出生成Excel，并创建生成记录
    @PostMapping("/udiwms/thrOrder/importLog/export")
    public BaseResponse excelDownload(@RequestBody ThrOrderExportRequest thrOrderExportRequest) {


        String fileName = "D:\\udiwms\\exportFile\\" + "业务单据导出" + System.currentTimeMillis() + ".xls";
        File file = new File(fileName);
        if (!file.exists()) {
            try {
                file.createNewFile();
            } catch (IOException e) {
                e.printStackTrace();
            }
        }
        ThrOrderExportLogEntity udiInfoExportLogEntity = new ThrOrderExportLogEntity();
        udiInfoExportLogEntity.setStatus(BasicProcessStatus.UDIINFO_IMPORT_PROCESS);
        String genKey = CustomUtil.getId();
        udiInfoExportLogEntity.setGenKey(genKey);
        udiInfoExportLogEntity.setFilePath(fileName);
        udiInfoExportLogEntity.setUpdateTime(new Date());
        udiInfoExportLogEntity.setType(BasicProcessStatus.EXPORT_EXCEL);
        udiInfoExportLogEntity.setDlCount(0);
        thrOrderExportLogService.insertThrOrderExportLog(udiInfoExportLogEntity);
        thrOrdersDlService.genExcel(genKey, thrOrderExportRequest);
        return ResultVOUtils.success("后台正在导出生成excel文件，请稍后刷新查看!");

    }

    //业务单据上传SMP，并创建生成记录
    @PostMapping("/udiwms/thrOrder/importLog/uploadSmp")
    public BaseResponse uploadSmp(@RequestBody ThrOrderExportRequest thrOrderExportRequest) {


        ThrOrderExportLogEntity udiInfoExportLogEntity = new ThrOrderExportLogEntity();
        udiInfoExportLogEntity.setStatus(BasicProcessStatus.UDIINFO_IMPORT_PROCESS);
        String genKey = CustomUtil.getId();
        udiInfoExportLogEntity.setGenKey(genKey);
        udiInfoExportLogEntity.setUpdateTime(new Date());
        udiInfoExportLogEntity.setType(BasicProcessStatus.EXPORT_SMP);
        udiInfoExportLogEntity.setDlCount(0);
        thrOrderExportLogService.insertThrOrderExportLog(udiInfoExportLogEntity);
        thrOrdersDlService.uploadSmp(genKey, thrOrderExportRequest);
        return ResultVOUtils.success("后台正在导出生成excel文件，请稍后刷新查看!");

    }

    //第三方业务单据接口下载（第三方使用）
    @GetMapping("/udiwms/thrsys/thrOrder/download")
    public BaseResponse downloadThrOrders(FilterOrderRequest filterErpOrderRequest,
                                          BindingResult bindingResult) {

        if (bindingResult.hasErrors()) {
            return ResultVOUtils.error(ResultEnum.PARAM_VERIFY_FALL, bindingResult.getFieldError().getDefaultMessage());
        }

        BasicThirdSysDetailEntity basicThirdSysDetailEntity = bussinessTypeService.findByActionKey(filterErpOrderRequest.getBillAction(), "orderQueryUrl");
        if (basicThirdSysDetailEntity.getValue() == null) {
            return ResultVOUtils.error(500, "业务单据查询接口未定义");
        }
        if (!basicThirdSysDetailEntity.getEnabled()) {
            return ResultVOUtils.error(500, "第三方业务单据服务未启用");
        }

        if (basicThirdSysDetailEntity.getFromType() == 0 ) {
            return new ErpOrderClient(basicThirdSysDetailEntity.getValue()).getErpOrderResponse(filterErpOrderRequest);
        } else {
            //todo 订单查询，需加日期
            FilterThrOrderRequest filterThrOrderRequest = new FilterThrOrderRequest();
            BeanUtils.copyProperties(filterErpOrderRequest, filterThrOrderRequest);
            filterThrOrderRequest.setPage(filterErpOrderRequest.getPage());
            filterThrOrderRequest.setLimit(filterErpOrderRequest.getLimit());
            List<ErpOrderResponse> erpOrderResponses = new ArrayList<>();
            List<ThrOrderEntity> data = thrOrderService.filterThrOrder(filterThrOrderRequest);
            if (data != null && data.size() > 0) {
                for (ThrOrderEntity thrOrderEntity : data) {
                    FilterThrOrderDetailRequest filterThrOrderDetailRequest = new FilterThrOrderDetailRequest();
                    filterThrOrderDetailRequest.setOrderIdFk(thrOrderEntity.getId() + "");
                    List<ThrOrderDetailEntity> thrOrderDetailEntities = thrOrderDetailService.filterThrOrderDetailDetail(filterThrOrderDetailRequest);
                    ErpOrderResponse erpOrderResponse = new ErpOrderResponse();
                    BeanUtils.copyProperties(thrOrderEntity, erpOrderResponse);
                    List<ErpOrderResponse.SubErpOrder> subErpOrders = new ArrayList<>();
                    if (thrOrderDetailEntities != null && thrOrderDetailEntities.size() > 0) {
                        for (ThrOrderDetailEntity thrOrderDetailEntity : thrOrderDetailEntities) {
                            ErpOrderResponse.SubErpOrder subErpOrder = new ErpOrderResponse.SubErpOrder();
                            BeanUtils.copyProperties(thrOrderDetailEntity, subErpOrder);
                            subErpOrders.add(subErpOrder);
                        }
                    }
                    erpOrderResponse.setSubErpOrders(subErpOrders);
                    erpOrderResponses.add(erpOrderResponse);
                }
            }
            PageInfo<ThrOrderEntity> pageInfo;
            pageInfo = new PageInfo<>(data);
            PageSimpleResponse<ErpOrderResponse> pageSimpleResponse = new PageSimpleResponse<>();
            pageSimpleResponse.setTotal(pageInfo.getTotal());
            pageSimpleResponse.setList(erpOrderResponses);
            return ResultVOUtils.success(pageSimpleResponse);

        }


    }

}
