package com.glxp.api.admin.controller.thrsys;


import com.github.pagehelper.PageInfo;
import com.glxp.api.admin.constant.BasicProcessStatus;
import com.glxp.api.admin.entity.basic.BussinessTypeEntity;
import com.glxp.api.admin.entity.thrsys.ThrOrderImportDetailEntity;
import com.glxp.api.admin.entity.thrsys.ThrOrderImportLogEntity;
import com.glxp.api.admin.req.basic.FilterUdiIpLogRequest;
import com.glxp.api.admin.req.inout.DeleteRequest;
import com.glxp.api.admin.res.PageSimpleResponse;
import com.glxp.api.admin.service.basic.*;
import com.glxp.api.admin.service.thrsys.ThrOrderImportDetailService;
import com.glxp.api.admin.service.thrsys.ThrOrderImportLogService;
import com.glxp.api.admin.thread.ThrOrderImportService;
import com.glxp.api.admin.util.CustomUtil;
import com.glxp.api.common.enums.ResultEnum;
import com.glxp.api.common.res.BaseResponse;
import com.glxp.api.common.util.ResultVOUtils;
import org.apache.poi.hssf.usermodel.HSSFWorkbook;
import org.apache.poi.ss.usermodel.*;
import org.apache.poi.xssf.usermodel.XSSFWorkbook;
import org.springframework.validation.BindingResult;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import javax.annotation.Resource;
import java.io.File;
import java.io.IOException;
import java.io.InputStream;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

@RestController
public class ThrOrderImportLogController {

    @Resource
    ThrOrderImportLogService thrOrderImportLogService;
    @Resource
    ThrOrderImportDetailService thrOrderImportDetailService;
    @Resource
    ThrOrderImportService thrOrderImportService;
    @Resource
    BussinessTypeService bussinessTypeService;

    @GetMapping("/udiwms/thrOrder/importLog/filter")
    public BaseResponse filter(FilterUdiIpLogRequest filterUdiIpLogRequest,
                               BindingResult bindingResult) {

        if (bindingResult.hasErrors()) {
            return ResultVOUtils.error(ResultEnum.PARAM_VERIFY_FALL, bindingResult.getFieldError().getDefaultMessage());
        }
        List<ThrOrderImportLogEntity> udiInfoImportLogEntities
                = thrOrderImportLogService.filterImportLog(filterUdiIpLogRequest);
        PageInfo<ThrOrderImportLogEntity> pageInfo;
        pageInfo = new PageInfo<>(udiInfoImportLogEntities);
        PageSimpleResponse<ThrOrderImportLogEntity> pageSimpleResponse = new PageSimpleResponse<>();
        pageSimpleResponse.setTotal(pageInfo.getTotal());
        pageSimpleResponse.setList(udiInfoImportLogEntities);
        return ResultVOUtils.success(pageSimpleResponse);
    }

    @GetMapping("/udiwms/thrOrder/importLog/filterDetail")
    public BaseResponse filterDetail(FilterUdiIpLogRequest filterUdiIpLogRequest,
                                     BindingResult bindingResult) {

        if (bindingResult.hasErrors()) {
            return ResultVOUtils.error(ResultEnum.PARAM_VERIFY_FALL, bindingResult.getFieldError().getDefaultMessage());
        }
        List<ThrOrderImportDetailEntity> thrOrderImportDetailEntities
                = thrOrderImportDetailService.filterOrderImportDetail(filterUdiIpLogRequest);
        PageInfo<ThrOrderImportDetailEntity> pageInfo;
        pageInfo = new PageInfo<>(thrOrderImportDetailEntities);
        PageSimpleResponse<ThrOrderImportDetailEntity> pageSimpleResponse = new PageSimpleResponse<>();
        pageSimpleResponse.setTotal(pageInfo.getTotal());
        pageSimpleResponse.setList(thrOrderImportDetailEntities);
        return ResultVOUtils.success(pageSimpleResponse);
    }


    @PostMapping("/udiwms/thrOrder/importLog/deleteLog")
    public BaseResponse deleteLog(@RequestBody DeleteRequest deleteRequest, BindingResult bindingResult) {

        if (bindingResult.hasErrors()) {
            return ResultVOUtils.error(ResultEnum.PARAM_VERIFY_FALL, bindingResult.getFieldError().getDefaultMessage());
        }
        String id = deleteRequest.getId();
        FilterUdiIpLogRequest filterInCodeLogRequest = new FilterUdiIpLogRequest();
        filterInCodeLogRequest.setId(Integer.parseInt(id));
        List<ThrOrderImportLogEntity> corpImportLogEntities = thrOrderImportLogService.filterImportLog(filterInCodeLogRequest);
        if (corpImportLogEntities != null && corpImportLogEntities.size() > 0) {
            ThrOrderImportLogEntity corpImportLogEntity = corpImportLogEntities.get(0);
            thrOrderImportLogService.deleteById(corpImportLogEntity.getId() + "");
            thrOrderImportDetailService.deleteByGenkey(corpImportLogEntity.getGenKey());
        }

        return ResultVOUtils.success("删除成功");
    }

    @PostMapping("/udiwms/thrOrder/importLog/delete")
    public BaseResponse delete(@RequestBody DeleteRequest deleteRequest, BindingResult bindingResult) {

        if (bindingResult.hasErrors()) {
            return ResultVOUtils.error(ResultEnum.PARAM_VERIFY_FALL, bindingResult.getFieldError().getDefaultMessage());
        }
        if (thrOrderImportDetailService.deleteById(deleteRequest.getId()))
            return ResultVOUtils.success("删除成功");
        else
            return ResultVOUtils.error(500, "删除成功");
    }


    @PostMapping("/udiwms/thrOrder/importLog/upload")
    public BaseResponse uploadProducts(@RequestParam("file") List<MultipartFile> files, @RequestParam("thirdSys") String thirdSys) {

        String filePath = "D:\\udiwms\\filePath\\";
        File createFile = new File(filePath);
        if (!createFile.exists()) {
            createFile.mkdirs();
        }
        for (int i = 0; i < files.size(); i++) {
            MultipartFile file = files.get(i);
            if (file.isEmpty()) {
                return ResultVOUtils.error(500, "上传第" + (i++) + "个文件失败");
            }
            String fileName = file.getOriginalFilename();
            try {
                InputStream inputStream = file.getInputStream();
                Workbook workbook = null;
                String filetype = fileName.substring(fileName.lastIndexOf("."));

                if (".xls".equals(filetype)) {
                    workbook = new HSSFWorkbook(inputStream);
                } else if (".xlsx".equals(filetype)) {
                    workbook = new XSSFWorkbook(inputStream);
                } else {
                    return ResultVOUtils.error(500, "请上传excel文件");
                }
                Sheet sheet = null;
                Row row = null;
                ThrOrderImportLogEntity thrOrderImportLogEntity = new ThrOrderImportLogEntity();
                String genKey = CustomUtil.getId();
                thrOrderImportLogEntity.setGenKey(genKey);
                thrOrderImportLogEntity.setThirdSysFk(thirdSys);
                thrOrderImportLogEntity.setFromType("文件导入");
                thrOrderImportLogEntity.setStatus(BasicProcessStatus.UDIINFO_IMPORT_UNPROCESS);
                thrOrderImportLogEntity.setUpdateTime(new Date(System.currentTimeMillis()));
                thrOrderImportLogService.insertImportLog(thrOrderImportLogEntity);
                List<ThrOrderImportDetailEntity> corpImportDetailEntities = new ArrayList<>();
                for (int j = 0; j < 1; j++) {//workbook.getNumberOfSheets()
                    sheet = workbook.getSheetAt(j);
                    if (sheet == null) {
                        continue;
                    }
                    // 滤过第一行标题
                    row = sheet.getRow(0);
                    if ((row.getCell(0) == null || row.getCell(1) == null || row.getCell(2) == null
                            || row.getCell(3) == null || row.getCell(4) == null || row.getCell(5) == null || row.getCell(6) == null) ||
                            (!"单据号".equals(row.getCell(0).getStringCellValue())
                                    || !"单据日期".equals(row.getCell(1).getStringCellValue())
                                    || !"往来单位ID".equals(row.getCell(2).getStringCellValue())
                                    || !"往来单位名称".equals(row.getCell(3).getStringCellValue())
                                    || !"单据类型".equals(row.getCell(4).getStringCellValue())
                                    || !"单据状态".equals(row.getCell(5).getStringCellValue())
                                    || !"产品ID".equals(row.getCell(6).getStringCellValue()))) {
                        return ResultVOUtils.error(500, "文件格式错误！");
                    }
                    for (int k = sheet.getFirstRowNum() + 1; k <= sheet.getLastRowNum(); k++) {
                        row = sheet.getRow(k);
                        if (row.getRowNum() <= 0) {
                            continue;
                        }
                        ThrOrderImportDetailEntity corpImportDetailEntity = new ThrOrderImportDetailEntity();
                        Cell cell1 = row.getCell(0);
                        if (cell1 != null) {
                            cell1.setCellType(CellType.STRING);
                            corpImportDetailEntity.setBillNo(cell1.getStringCellValue());
                        }


                        Cell cell2 = row.getCell(1);
                        if (cell2 != null) {
                            cell2.setCellType(CellType.STRING);
                            corpImportDetailEntity.setBilldate(cell2.getStringCellValue());
                        }
                        Cell cell3 = row.getCell(2);
                        if (cell3 != null) {
                            cell3.setCellType(CellType.STRING);
                            corpImportDetailEntity.setCorpId(cell3.getStringCellValue());

                        }
                        Cell cell4 = row.getCell(3);
                        if (cell4 != null) {
                            cell4.setCellType(CellType.STRING);
                            corpImportDetailEntity.setCorpName(cell4.getStringCellValue());

                        }
                        Cell cell5 = row.getCell(4);
                        if (cell5 != null) {
                            cell5.setCellType(CellType.STRING);
                            String billType = cell5.getStringCellValue();
                            BussinessTypeEntity bussinessTypeEntity = bussinessTypeService.findBTByName(billType);
                            if (bussinessTypeEntity != null) {
                                corpImportDetailEntity.setBillType(bussinessTypeEntity.getAction());
                            } else
                                corpImportDetailEntity.setBillType(billType);


                        }
                        Cell cell6 = row.getCell(5);
                        if (cell6 != null) {
                            cell6.setCellType(CellType.STRING);
                            corpImportDetailEntity.setBillFlag(cell6.getStringCellValue());
                        }

                        Cell cell7 = row.getCell(6);
                        if (cell7 != null) {
                            cell7.setCellType(CellType.STRING);
                            corpImportDetailEntity.setProductId(cell7.getStringCellValue());
                        }

                        Cell cell8 = row.getCell(7);
                        if (cell8 != null) {
                            cell8.setCellType(CellType.STRING);
                            corpImportDetailEntity.setProductName(cell8.getStringCellValue());
                        }

                        Cell cell9 = row.getCell(8);
                        if (cell9 != null) {
                            cell9.setCellType(CellType.STRING);
                            corpImportDetailEntity.setSpec(cell9.getStringCellValue());
                        }

                        Cell cell10 = row.getCell(9);
                        if (cell10 != null) {
                            cell10.setCellType(CellType.STRING);
                            corpImportDetailEntity.setBatchNo(cell10.getStringCellValue());
                        }

                        Cell cell11 = row.getCell(10);
                        if (cell11 != null) {
                            cell11.setCellType(CellType.STRING);
                            corpImportDetailEntity.setProductDate(cell11.getStringCellValue());
                        }

                        Cell cell12 = row.getCell(11);
                        if (cell12 != null) {
                            cell12.setCellType(CellType.STRING);
                            corpImportDetailEntity.setExpireDate(cell12.getStringCellValue());
                        }

                        Cell cell13 = row.getCell(12);
                        if (cell13 != null) {
                            cell13.setCellType(CellType.STRING);
                            String count = cell13.getStringCellValue();
                            if (count != null && !"".equals(count)) {
                                corpImportDetailEntity.setCount(Integer.parseInt(count));
                            }
                        }
                        Cell cell14 = row.getCell(13);
                        if (cell14 != null) {
                            cell14.setCellType(CellType.STRING);
                            String count = cell14.getStringCellValue();
                            if (count != null && !"".equals(count)) {
                                corpImportDetailEntity.setReCount(Integer.parseInt(count));
                            }
                        }

                        corpImportDetailEntity.setThirdSysFk(thirdSys);
                        corpImportDetailEntity.setGenKeyFk(genKey);
                        corpImportDetailEntity.setStatus(BasicProcessStatus.UDIINFO_IMPORT_CODE_UNPROCESS);
                        corpImportDetailEntity.setUpdateTime(new Date(System.currentTimeMillis()));
                        corpImportDetailEntities.add(corpImportDetailEntity);
                    }
                }
                thrOrderImportDetailService.insertOrderImportDetails(corpImportDetailEntities);
                thrOrderImportLogEntity.setStatus(BasicProcessStatus.UDIINFO_IMPORT_UNPROCESS);
                thrOrderImportLogService.updateImportLog(thrOrderImportLogEntity);
                workbook.close();
                thrOrderImportService.importThrOrder(genKey);
            } catch (IOException e) {
                e.printStackTrace();
            }
        }
        return ResultVOUtils.success("上传成功");
    }

}
