package com.glxp.api.admin.controller.thrsys;

import com.github.pagehelper.PageInfo;
import com.glxp.api.admin.constant.BasicProcessStatus;
import com.glxp.api.admin.entity.thrsys.ThrProductsEntity;
import com.glxp.api.admin.entity.thrsys.ThrProductsImportLogEntity;
import com.glxp.api.admin.req.basic.FilterUdiInfoRequest;
import com.glxp.api.admin.req.inout.DeleteRequest;
import com.glxp.api.admin.req.thrsys.FilterThrProductsRequest;
import com.glxp.api.admin.req.thrsys.PostThrProductsRequest;
import com.glxp.api.admin.res.PageSimpleResponse;
import com.glxp.api.admin.res.basic.ErpProductsResponse;
import com.glxp.api.admin.res.basic.UdiRelevanceResponse;
import com.glxp.api.admin.service.basic.UdiRelevanceService;
import com.glxp.api.admin.service.thrsys.ThrProductsImportLogService;
import com.glxp.api.admin.service.thrsys.ThrProductsService;
import com.glxp.api.admin.thread.ThrProductsDlService;
import com.glxp.api.admin.util.Constant;
import com.glxp.api.admin.util.CustomUtil;
import com.glxp.api.admin.util.RedisUtil;
import com.glxp.api.common.enums.ResultEnum;
import com.glxp.api.common.res.BaseResponse;
import com.glxp.api.common.util.ResultVOUtils;
import org.springframework.validation.BindingResult;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import java.util.Date;
import java.util.List;

@RestController
public class ThrProductsController {
    @Resource
    RedisUtil redisUtil;
    @Resource
    private ThrProductsService thrProductsService;
    @Resource
    private ThrProductsDlService thrProductsDlService;
    @Resource
    ThrProductsImportLogService thrProductsImportLogService;
    @Resource
    UdiRelevanceService udiRelevanceService;


    @GetMapping("/udiwms/thrsys/getThrProducts")
    public BaseResponse getThrProducts(FilterThrProductsRequest filterThrProductsRequest,
                                       BindingResult bindingResult) {

        if (bindingResult.hasErrors()) {
            return ResultVOUtils.error(ResultEnum.PARAM_VERIFY_FALL, bindingResult.getFieldError().getDefaultMessage());
        }
        List<ThrProductsEntity> thrCorpEntities
                = thrProductsService.filterThrProductsRequest(filterThrProductsRequest);
        PageInfo<ThrProductsEntity> pageInfo;
        pageInfo = new PageInfo<>(thrCorpEntities);
        PageSimpleResponse<ThrProductsEntity> pageSimpleResponse = new PageSimpleResponse<>();
        pageSimpleResponse.setTotal(pageInfo.getTotal());
        pageSimpleResponse.setList(thrCorpEntities);
        return ResultVOUtils.success(pageSimpleResponse);
    }


    @PostMapping("/udiwms/thrsys/delThrProducts")
    public BaseResponse delThrProducts(@RequestBody DeleteRequest deleteRequest, BindingResult bindingResult) {

        if (bindingResult.hasErrors()) {
            return ResultVOUtils.error(ResultEnum.PARAM_VERIFY_FALL, bindingResult.getFieldError().getDefaultMessage());
        }
        if (deleteRequest.getIds() != null && deleteRequest.getIds().size() > 0) {
            for (String id : deleteRequest.getIds()) {
                selectDetale(id);
            }
        } else {
            String id = deleteRequest.getId();
            String msg = selectDetale(id);
            if (msg != null) {
                return ResultVOUtils.error(500, msg);
            }
        }
        return ResultVOUtils.success("删除成功");
    }


    public String selectDetale(String id) {
        ThrProductsEntity thrProductsEntity = thrProductsService.selectById(id);
        if (thrProductsEntity != null) {
            FilterUdiInfoRequest filterUdiInfoRequest = new FilterUdiInfoRequest();
            if ("thirdId".equals(thrProductsEntity.getThirdSysFk())) {
                filterUdiInfoRequest.setThirdId(thrProductsEntity.getCode());
            } else if ("thirdId1".equals(thrProductsEntity.getThirdSysFk())) {
                filterUdiInfoRequest.setThirdId(thrProductsEntity.getCode());
            } else if ("thirdId2".equals(thrProductsEntity.getThirdSysFk())) {
                filterUdiInfoRequest.setThirdId(thrProductsEntity.getCode());
            } else if ("thirdId3".equals(thrProductsEntity.getThirdSysFk())) {
                filterUdiInfoRequest.setThirdId(thrProductsEntity.getCode());
            } else if ("thirdId4".equals(thrProductsEntity.getThirdSysFk())) {
                filterUdiInfoRequest.setThirdId(thrProductsEntity.getCode());
            }
            List<UdiRelevanceResponse> udiRelevanceResponses = udiRelevanceService.filterUdiRelevance(filterUdiInfoRequest);
            if (udiRelevanceResponses != null && udiRelevanceResponses.size() > 0) {
                return "该产品已与UDI码关联，无法删除！";
            }
        }
        boolean b = thrProductsService.deleteById(id);
        if (b)
            return null;
        else return "删除失败！";
    }


    @RequestMapping("/udiwms/thrsys/products/delAll")
    public BaseResponse delAll() {
        thrProductsService.deleteAll();
        return ResultVOUtils.success("删除成功");
    }


    //第三方服务上传（包括文件导入后上传）
    @PostMapping("/udiwms/thrsys/postThrProducts")
    public BaseResponse postThrProducts(@RequestBody PostThrProductsRequest postThrProductsRequest,
                                        BindingResult bindingResult) {

        if (bindingResult.hasErrors()) {
            return ResultVOUtils.error(ResultEnum.PARAM_VERIFY_FALL, bindingResult.getFieldError().getDefaultMessage());
        }
        List<ErpProductsResponse> erpProductsResponses = postThrProductsRequest.getDatas();

        String genKey = postThrProductsRequest.getGenKey();
        if (genKey == null) {
            genKey = CustomUtil.getId();
        }
        ThrProductsImportLogEntity thrProductsImportLogEntity = thrProductsImportLogService.selectByGenKey(genKey);
        if (thrProductsImportLogEntity == null) {
            thrProductsImportLogEntity = new ThrProductsImportLogEntity();
            thrProductsImportLogEntity.setGenKey(genKey);
            if (postThrProductsRequest.getUploadType() != null) {
                thrProductsImportLogEntity.setFromType(postThrProductsRequest.getUploadType());
            } else
                thrProductsImportLogEntity.setFromType("第三方系统上传");
            thrProductsImportLogEntity.setStatus(BasicProcessStatus.UDIINFO_IMPORT_PROCESS);
            thrProductsImportLogEntity.setUpdateTime(new Date());
            thrProductsImportLogEntity.setThirdSysFk(postThrProductsRequest.getThirdSys());
            thrProductsImportLogService.insertImportLog(thrProductsImportLogEntity);
        }

        if (erpProductsResponses != null && erpProductsResponses.size() > 0) {
            thrProductsDlService.insertData(genKey, erpProductsResponses, postThrProductsRequest.getThirdSys());
            return ResultVOUtils.success("产品信息上传成功！");
        }
        thrProductsImportLogEntity.setStatus(BasicProcessStatus.UDIINFO_IMPORT_FAIL);
        thrProductsImportLogEntity.setUpdateTime(new Date());
        thrProductsImportLogEntity.setRemark("上传数据为空");
        thrProductsImportLogService.updateImportLog(thrProductsImportLogEntity);
        return ResultVOUtils.error(500, "上传数据为空");
    }

    //接口一键下载全部
    @RequestMapping("/udiwms/thrsys/products/downloadAll")
    public BaseResponse downloadAll(FilterThrProductsRequest filterThrProductsRequest) {
        String data = (String) redisUtil.get(Constant.dlThrProducts);
        if (data != null && data.equals("true")) {
            return ResultVOUtils.error(500, "当前任务正在下载更新产品信息，请稍后重试！");
        } else {
            redisUtil.set(Constant.dlThrProducts, "true", 60 * 10);
            if (filterThrProductsRequest.getThirdSysFk() == null) {
                return ResultVOUtils.error(500, "未选择第三方系统！");
            }
            ThrProductsImportLogEntity thrProductsImportLogEntity = new ThrProductsImportLogEntity();
            String genKey = CustomUtil.getId();
            thrProductsImportLogEntity.setGenKey(genKey);
            thrProductsImportLogEntity.setFromType("第三方系统下载");
            thrProductsImportLogEntity.setStatus(BasicProcessStatus.UDIINFO_IMPORT_PROCESS);
            thrProductsImportLogEntity.setUpdateTime(new Date());
            thrProductsImportLogEntity.setThirdSysFk(filterThrProductsRequest.getThirdSysFk());
            thrProductsImportLogService.insertImportLog(thrProductsImportLogEntity);
            thrProductsDlService.importProducrs(genKey, filterThrProductsRequest);
            return ResultVOUtils.success("后台开始下载更新，请稍后刷新查看");
        }
    }


    //接口一键下载全部
    @PostMapping("/udiwms/thrsys/products/productsDlAll")
    public BaseResponse productsDlAll(@RequestBody FilterThrProductsRequest filterThrProductsRequest) {
        filterThrProductsRequest.setThirdSysFk(filterThrProductsRequest.getThirdSys());
        String data = (String) redisUtil.get(Constant.dlThrProducts);
        if (false) {     //data != null && data.equals("true")
            return ResultVOUtils.error(500, "当前任务正在下载更新产品信息，请稍后重试！");
        } else {
            redisUtil.set(Constant.dlThrProducts, "true", 60 * 10);
            if (filterThrProductsRequest.getThirdSysFk() == null) {
                return ResultVOUtils.error(500, "未选择第三方系统！");
            }
            ThrProductsImportLogEntity thrProductsImportLogEntity = new ThrProductsImportLogEntity();
            String genKey = CustomUtil.getId();
            thrProductsImportLogEntity.setGenKey(genKey);
            thrProductsImportLogEntity.setFromType("第三方系统下载");
            thrProductsImportLogEntity.setStatus(BasicProcessStatus.UDIINFO_IMPORT_PROCESS);
            thrProductsImportLogEntity.setUpdateTime(new Date());
            thrProductsImportLogEntity.setThirdSysFk(filterThrProductsRequest.getThirdSysFk());
            thrProductsImportLogService.insertImportLog(thrProductsImportLogEntity);

            if (filterThrProductsRequest.getThrProductsEntities() != null && filterThrProductsRequest.getThrProductsEntities().size() > 0) {//选中导出
                thrProductsDlService.importSelectProducrs(genKey, filterThrProductsRequest.getThrProductsEntities(), filterThrProductsRequest.getThirdSysFk());
            } else {        //结果导出
                thrProductsDlService.importProducrs(genKey,filterThrProductsRequest );
            }

//            thrProductsDlService.importProducrs(genKey, filterThrProductsRequest.getThirdSysFk());
            return ResultVOUtils.success("后台开始下载更新，请稍后刷新查看");
        }
    }
}
