package com.glxp.api.admin.controller.thrsys;

import com.github.pagehelper.PageInfo;
import com.glxp.api.admin.constant.BasicProcessStatus;
import com.glxp.api.admin.entity.basic.BasicThirdSysDetailEntity;
import com.glxp.api.admin.entity.thrsys.ThrProductsEntity;
import com.glxp.api.admin.entity.thrsys.ThrProductsExportLogEntity;
import com.glxp.api.admin.httpclient.ErpBasicClient;
import com.glxp.api.admin.req.basic.FilterErpGoodsRequest;
import com.glxp.api.admin.req.basic.FilterUdiEpLogRequest;
import com.glxp.api.admin.req.inout.DeleteRequest;
import com.glxp.api.admin.req.thrsys.FilterThrProductsRequest;
import com.glxp.api.admin.req.thrsys.ThrProductsExportRequest;
import com.glxp.api.admin.res.PageSimpleResponse;
import com.glxp.api.admin.res.basic.ErpProductsResponse;
import com.glxp.api.admin.service.basic.BasicThirdSysDetailService;
import com.glxp.api.admin.service.thrsys.ThrProductsExportLogService;
import com.glxp.api.admin.service.thrsys.ThrProductsService;
import com.glxp.api.admin.thread.ThrProductsDlService;
import com.glxp.api.admin.util.CustomUtil;
import com.glxp.api.common.enums.ResultEnum;
import com.glxp.api.common.res.BaseResponse;
import com.glxp.api.common.util.ResultVOUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.validation.BindingResult;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RestController;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletResponse;
import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.OutputStream;
import java.util.Date;
import java.util.List;

@RestController
public class ThrProductsExportLogController {

    @Resource
    ThrProductsExportLogService thrProductsExportLogService;
    @Resource
    ThrProductsService thrProductsService;
    @Resource
    ThrProductsDlService thrProductsDlService;
    @Resource
    BasicThirdSysDetailService basicThirdSysDetailService;

    @GetMapping("/udiwms/thrProducts/exportLog/filter")
    public BaseResponse filter(FilterUdiEpLogRequest filterUdiEpLogReques,
                               BindingResult bindingResult) {

        if (bindingResult.hasErrors()) {
            return ResultVOUtils.error(ResultEnum.PARAM_VERIFY_FALL, bindingResult.getFieldError().getDefaultMessage());
        }
        List<ThrProductsExportLogEntity> thrProductsExportLogEntities
                = thrProductsExportLogService.filterThrProductsExportLog(filterUdiEpLogReques);
        PageInfo<ThrProductsExportLogEntity> pageInfo;
        pageInfo = new PageInfo<>(thrProductsExportLogEntities);
        PageSimpleResponse<ThrProductsExportLogEntity> pageSimpleResponse = new PageSimpleResponse<>();
        pageSimpleResponse.setTotal(pageInfo.getTotal());
        pageSimpleResponse.setList(thrProductsExportLogEntities);
        return ResultVOUtils.success(pageSimpleResponse);
    }

    @PostMapping("/udiwms/thrProducts/exportLog/deleteLog")
    public BaseResponse deleteLog(@RequestBody DeleteRequest deleteRequest, BindingResult bindingResult) {

        if (bindingResult.hasErrors()) {
            return ResultVOUtils.error(ResultEnum.PARAM_VERIFY_FALL, bindingResult.getFieldError().getDefaultMessage());
        }
        String id = deleteRequest.getId();
        thrProductsExportLogService.deleteById(id + "");
        return ResultVOUtils.success("删除成功");
    }

    //产品信息excel文件下载
    @GetMapping("/udiwms/thrProducts/exportLog/download")
    public void download(FilterUdiEpLogRequest filterUdiEpLogRequest, HttpServletResponse res) {

        List<ThrProductsExportLogEntity> thrProductsExportLogEntities = thrProductsExportLogService.filterThrProductsExportLog(filterUdiEpLogRequest);
        ThrProductsExportLogEntity thrProductsExportLogEntity = thrProductsExportLogEntities.get(0);
        try {
            String filePath = thrProductsExportLogEntity.getFilePath();
            String fileName = "temp";
            if (filePath != null && filePath.length() > 0) {
                String[] data = filePath.split("/");
                fileName = data[data.length - 1];
            }
            res.setHeader("Content-disposition", "attachment;fileName=" + fileName);
            res.setContentType("text/plain;charset=UTF-8");
            FileInputStream input = new FileInputStream(new File(filePath));
            OutputStream out = res.getOutputStream();
            byte[] b = new byte[2048];
            int len;
            while ((len = input.read(b)) != -1) {
                out.write(b, 0, len);
            }
            input.close();
            if (thrProductsExportLogEntity.getDlCount() == 0) {
                thrProductsExportLogEntity.setDlCount(1);
            } else {
                thrProductsExportLogEntity.setDlCount(thrProductsExportLogEntity.getDlCount() + 1);
            }
            thrProductsExportLogService.updateThrProductsExportLog(thrProductsExportLogEntity);
        } catch (Exception ex) {
            System.out.println("下载失败！");
        }


    }

    //产品信息导出生成Excel，并创建生成记录
    @PostMapping("/udiwms/thrProducts/importLog/export")
    public BaseResponse excelDownload(@RequestBody ThrProductsExportRequest thrProductsExportRequest) {

        String fileName = "D:\\udiwms\\exportFile\\" + "产品信息导出" + System.currentTimeMillis() + ".xls";
        File file = new File(fileName);
        if (!file.exists()) {
            try {
                file.createNewFile();
            } catch (IOException e) {
                e.printStackTrace();
            }
        }
        ThrProductsExportLogEntity thrProductsExportLogEntity = new ThrProductsExportLogEntity();
        thrProductsExportLogEntity.setStatus(BasicProcessStatus.UDIINFO_IMPORT_PROCESS);
        String genKey = CustomUtil.getId();
        thrProductsExportLogEntity.setGenKey(genKey);
        thrProductsExportLogEntity.setFilePath(fileName);
        thrProductsExportLogEntity.setUpdateTime(new Date());
        thrProductsExportLogEntity.setDlCount(0);
        thrProductsExportLogEntity.setType(BasicProcessStatus.EXPORT_EXCEL);
        thrProductsExportLogService.insertThrProductsExportLog(thrProductsExportLogEntity);
        thrProductsDlService.genExcel(genKey, thrProductsExportRequest);
        return ResultVOUtils.success("后台正在导出生成excel文件，请稍后刷新查看!");

    }


    //产品信息上传供应商管理平台，并创建生成记录
    @PostMapping("/udiwms/thrProducts/importLog/uploadSmp")
    public BaseResponse uploadSmp(@RequestBody ThrProductsExportRequest thrProductsExportRequest) {

        ThrProductsExportLogEntity thrProductsExportLogEntity = new ThrProductsExportLogEntity();
        thrProductsExportLogEntity.setStatus(BasicProcessStatus.UDIINFO_IMPORT_PROCESS);
        String genKey = CustomUtil.getId();
        thrProductsExportLogEntity.setGenKey(genKey);
        thrProductsExportLogEntity.setUpdateTime(new Date());
        thrProductsExportLogEntity.setDlCount(0);
        thrProductsExportLogEntity.setType(BasicProcessStatus.EXPORT_SMP);
        thrProductsExportLogService.insertThrProductsExportLog(thrProductsExportLogEntity);
        thrProductsDlService.uploadSmp(genKey, thrProductsExportRequest);
        return ResultVOUtils.success("后台正在导出生成excel文件，请稍后刷新查看!");

    }

    //第三方产品信息接口下载（第三方使用）
    @GetMapping("/udiwms/thrsys/thrProducts/download")
    public BaseResponse getErpProducts(FilterErpGoodsRequest filterErpGoodsRequest,
                                       BindingResult bindingResult) {

        if (bindingResult.hasErrors()) {
            return ResultVOUtils.error(ResultEnum.PARAM_VERIFY_FALL, bindingResult.getFieldError().getDefaultMessage());
        }
        BasicThirdSysDetailEntity basicThirdSysDetailEntity = basicThirdSysDetailService.selectByKey("piQueryUrl", filterErpGoodsRequest.getThirdSys());
        if (basicThirdSysDetailEntity == null || basicThirdSysDetailEntity.getValue() == null)
            return ResultVOUtils.error(500, "ERP产品信息接口地址未定义");
        if (basicThirdSysDetailEntity.getEnabled()) {
            if (basicThirdSysDetailEntity.getFromType() == 0 ) {
                ErpBasicClient erpBasicHttpClient = new ErpBasicClient();
                BaseResponse<PageSimpleResponse<ErpProductsResponse>> udiDlDeviceResponse = erpBasicHttpClient.getErpProducts(filterErpGoodsRequest, basicThirdSysDetailEntity.getValue());
                if (udiDlDeviceResponse.getCode() == 20000) {
                    return udiDlDeviceResponse;
                }
                return ResultVOUtils.error(500, "未查询到产品信息");
            } else {
                FilterThrProductsRequest filterThrProductsRequest = new FilterThrProductsRequest();
                BeanUtils.copyProperties(filterErpGoodsRequest, filterThrProductsRequest);
                List<ThrProductsEntity> thrCorpEntities
                        = thrProductsService.filterThrProductsRequest(filterThrProductsRequest);
                PageInfo<ThrProductsEntity> pageInfo;
                pageInfo = new PageInfo<>(thrCorpEntities);
                PageSimpleResponse<ThrProductsEntity> pageSimpleResponse = new PageSimpleResponse<>();
                pageSimpleResponse.setTotal(pageInfo.getTotal());
                pageSimpleResponse.setList(thrCorpEntities);
                return ResultVOUtils.success(pageSimpleResponse);
            }
        } else {
            return ResultVOUtils.error(500, "第三方产品信息服务未启用");
        }
    }
}
