package com.glxp.api.admin.controller.thrsys;

import com.github.pagehelper.PageInfo;
import com.glxp.api.admin.entity.thrsys.ThrOrderImportDetailEntity;
import com.glxp.api.admin.entity.thrsys.ThrProductsImportDetailEntity;
import com.glxp.api.admin.entity.thrsys.ThrProductsImportLogEntity;
import com.glxp.api.admin.req.basic.FilterUdiIpLogRequest;
import com.glxp.api.admin.req.inout.DeleteRequest;
import com.glxp.api.admin.res.PageSimpleResponse;
import com.glxp.api.admin.service.thrsys.ThrInvProductsImportLogService;
import com.glxp.api.admin.service.thrsys.ThrProductsImportDetailService;
import com.glxp.api.admin.service.thrsys.ThrProductsImportLogService;
import com.glxp.api.common.enums.ResultEnum;
import com.glxp.api.common.res.BaseResponse;
import com.glxp.api.common.util.ResultVOUtils;
import org.springframework.validation.BindingResult;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RestController;

import javax.annotation.Resource;
import java.util.List;

@RestController
public class ThrProductsImportLogController {

    @Resource
    ThrProductsImportLogService thrProductsImportLogService;
    @Resource
    ThrProductsImportDetailService thrProductsImportDetailService;

    @GetMapping("/udiwms/thrProducts/importLog/filter")
    public BaseResponse filter(FilterUdiIpLogRequest filterUdiIpLogRequest,
                               BindingResult bindingResult) {

        if (bindingResult.hasErrors()) {
            return ResultVOUtils.error(ResultEnum.PARAM_VERIFY_FALL, bindingResult.getFieldError().getDefaultMessage());
        }
        List<ThrProductsImportLogEntity> thrProductsImportLogEntities
                = thrProductsImportLogService.filterImportLog(filterUdiIpLogRequest);
        PageInfo<ThrProductsImportLogEntity> pageInfo;
        pageInfo = new PageInfo<>(thrProductsImportLogEntities);
        PageSimpleResponse<ThrProductsImportLogEntity> pageSimpleResponse = new PageSimpleResponse<>();
        pageSimpleResponse.setTotal(pageInfo.getTotal());
        pageSimpleResponse.setList(thrProductsImportLogEntities);
        return ResultVOUtils.success(pageSimpleResponse);
    }

    @GetMapping("/udiwms/thrProducts/importLog/filterDetail")
    public BaseResponse filterDetail(FilterUdiIpLogRequest filterUdiIpLogRequest,
                                     BindingResult bindingResult) {

        if (bindingResult.hasErrors()) {
            return ResultVOUtils.error(ResultEnum.PARAM_VERIFY_FALL, bindingResult.getFieldError().getDefaultMessage());
        }
        List<ThrProductsImportDetailEntity> thrProductsImportDetailEntities
                = thrProductsImportDetailService.filterProductsDetailImport(filterUdiIpLogRequest);
        PageInfo<ThrProductsImportDetailEntity> pageInfo;
        pageInfo = new PageInfo<>(thrProductsImportDetailEntities);
        PageSimpleResponse<ThrProductsImportDetailEntity> pageSimpleResponse = new PageSimpleResponse<>();
        pageSimpleResponse.setTotal(pageInfo.getTotal());
        pageSimpleResponse.setList(thrProductsImportDetailEntities);
        return ResultVOUtils.success(pageSimpleResponse);
    }

    @PostMapping("/udiwms/thrProducts//importLog/deleteLog")
    public BaseResponse deleteLog(@RequestBody DeleteRequest deleteRequest, BindingResult bindingResult) {

        if (bindingResult.hasErrors()) {
            return ResultVOUtils.error(ResultEnum.PARAM_VERIFY_FALL, bindingResult.getFieldError().getDefaultMessage());
        }
        String id = deleteRequest.getId();
        FilterUdiIpLogRequest filterInCodeLogRequest = new FilterUdiIpLogRequest();
        filterInCodeLogRequest.setId(Integer.parseInt(id));
        List<ThrProductsImportLogEntity> thrProductsImportLogEntities = thrProductsImportLogService.filterImportLog(filterInCodeLogRequest);
        if (thrProductsImportLogEntities != null && thrProductsImportLogEntities.size() > 0) {
            ThrProductsImportLogEntity thrInvProductsImportLogEntity = thrProductsImportLogEntities.get(0);
            thrProductsImportLogService.deleteById(thrInvProductsImportLogEntity.getId() + "");
            thrProductsImportDetailService.deleteByGenkey(thrInvProductsImportLogEntity.getGenKey());
        }

        return ResultVOUtils.success("删除成功");
    }
}
