package com.glxp.api.admin.httpclient;

import com.alibaba.fastjson.JSONObject;
import com.alibaba.fastjson.TypeReference;
import com.glxp.api.admin.entity.inout.ErpOrderEntity;
import com.glxp.api.admin.req.inout.ERPPostBillRequest;
import com.glxp.api.admin.req.inout.FilterOrderRequest;
import com.glxp.api.admin.req.inout.USignRequest;
import com.glxp.api.admin.res.PageSimpleResponse;
import com.glxp.api.admin.res.inout.ErpOrderResponse;
import com.glxp.api.admin.util.DateUtil;
import com.glxp.api.admin.util.HttpClient;
import com.glxp.api.common.res.BaseResponse;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * 获取ERP出入库单据
 */
public class ErpOrderClient {
    private String url;

    public ErpOrderClient() {
    }

    public ErpOrderClient(String url) {
        this.url = url;
    }

    public List<ErpOrderEntity> getErpOrder(List<String> billCodes, String action) {
        List<ErpOrderEntity> erpOrderEntities = new ArrayList<>();
        if (billCodes != null && billCodes.size() > 0) {
            for (int i = 0; i < billCodes.size(); i++) {
                List<ErpOrderEntity> erpOrderEntitys = getErpOrder(billCodes.get(i), action);
                if (erpOrderEntitys != null) {
                    erpOrderEntities.addAll(erpOrderEntitys);
                }
            }
        }
        return erpOrderEntities;
    }

    public BaseResponse<PageSimpleResponse<ErpOrderResponse>> getErpOrderResponse(FilterOrderRequest filterOrderRequest) {
        Map<String, Object> paramMap = new HashMap<>(16);
        paramMap.put("billNo", filterOrderRequest.getBillNo());
        paramMap.put("page", filterOrderRequest.getPage());
        paramMap.put("limit", filterOrderRequest.getLimit());
        paramMap.put("billAction", filterOrderRequest.getBillAction());
        paramMap.put("billFlag", filterOrderRequest.getBillFlag());
        paramMap.put("corpName", filterOrderRequest.getCorpName());
        if (filterOrderRequest.getStartDate() == null || filterOrderRequest.getEndDate() == null) {
            Map date = DateUtil.getDayTRange();
            if (filterOrderRequest.getBillNo() != null) {
                date = DateUtil.getYearTRange();
            }
            filterOrderRequest.setStartDate((String) date.get("startDate"));
            filterOrderRequest.setEndDate((String) date.get("endDate"));
        }
        paramMap.put("startDate", filterOrderRequest.getStartDate());
        paramMap.put("endDate", filterOrderRequest.getEndDate());

        String response = "";
        response = HttpClient.mipsGet(url, paramMap);
        try {
            BaseResponse<PageSimpleResponse<ErpOrderResponse>> responseBaseResponse =
                    JSONObject.parseObject(response, new TypeReference<BaseResponse<PageSimpleResponse<ErpOrderResponse>>>() {
                    });
            return responseBaseResponse;

        } catch (
                Exception e) {
            e.printStackTrace();
        }
        return null;
    }

    public List<ErpOrderEntity> getErpOrder(FilterOrderRequest filterOrderRequest) {

        try {
            BaseResponse<PageSimpleResponse<ErpOrderResponse>> responseBaseResponse = getErpOrderResponse(filterOrderRequest);
            List<ErpOrderResponse> purchaseinResponseList = responseBaseResponse.getData().getList();
            if (purchaseinResponseList != null && purchaseinResponseList.size() > 0) {
                List<ErpOrderEntity> erpOrderEntities = new ArrayList<>();
                for (ErpOrderResponse erpOrderResponse : purchaseinResponseList) {
                    for (ErpOrderResponse.SubErpOrder subPurchase : erpOrderResponse.getSubErpOrders()) {
                        ErpOrderEntity erpOrderEntity = new ErpOrderEntity();
                        erpOrderEntity.setErpOrderId(erpOrderResponse.getBillNo());
                        erpOrderEntity.setCompanyid(erpOrderResponse.getCorpId());
                        erpOrderEntity.setCompanyname(erpOrderResponse.getCorpName());
                        erpOrderEntity.setCredate(erpOrderResponse.getBilldate());
                        erpOrderEntity.setGoodsid(subPurchase.getProductId());
                        erpOrderEntity.setGoodsname(subPurchase.getProductName());
                        erpOrderEntity.setBatchNo(subPurchase.getBatchNo());
                        erpOrderEntity.setProductDate(subPurchase.getProductDate());
                        erpOrderEntity.setExpireDate(subPurchase.getExpireDate());
                        erpOrderEntity.setErpCount(Math.abs(subPurchase.getCount()));
                        erpOrderEntity.setReCount(Math.abs(subPurchase.getReCount()));
                        erpOrderEntity.setOriginType(erpOrderResponse.getOriginType());
                        erpOrderEntity.setGoodsunit(subPurchase.getSpec());
                        erpOrderEntities.add(erpOrderEntity);
                    }
                }
                return erpOrderEntities;
            }
        } catch (
                Exception e) {
            e.printStackTrace();
        }
        return null;
    }

    public List<ErpOrderEntity> getErpOrder(String billCode, String action) {
        FilterOrderRequest filterOrderRequest = new FilterOrderRequest();
        filterOrderRequest.setPage(1);
        filterOrderRequest.setLimit(1);
        filterOrderRequest.setBillAction(action);
        filterOrderRequest.setBillNo(billCode);
        List<ErpOrderEntity> erpOrderEntities = getErpOrder(filterOrderRequest);
        return erpOrderEntities;
    }

    public BaseResponse postBill(Map<String, List<ErpOrderEntity>> postMap, String billAction, String uploadUrl) {
        ERPPostBillRequest erpPostBillRequest = new ERPPostBillRequest();
        erpPostBillRequest.setPostMap(postMap);
        erpPostBillRequest.setBillAction(billAction);
        String response = HttpClient.uCloudPost(uploadUrl, erpPostBillRequest);
        BaseResponse baseResponse =
                JSONObject.parseObject(response, new TypeReference<BaseResponse>() {
                });
        return baseResponse;
    }

    public BaseResponse postBill(ERPPostBillRequest erpPostBillRequest, String uploadUrl) {
        String response = HttpClient.uCloudPost(uploadUrl, erpPostBillRequest);
        BaseResponse baseResponse =
                JSONObject.parseObject(response, new TypeReference<BaseResponse>() {
                });
        return baseResponse;
    }

    public BaseResponse signBill(USignRequest signRequest, String url) {
        String response = HttpClient.uCloudPost(url, signRequest);
        BaseResponse baseResponse =
                JSONObject.parseObject(response, new TypeReference<BaseResponse>() {
                });
        return baseResponse;
    }

}
