package com.glxp.api.admin.service.basic.impl;

import com.github.pagehelper.PageHelper;
import com.glxp.api.admin.dao.basic.UdiRelevanceDao;
import com.glxp.api.admin.entity.basic.BasicThirdSysEntity;
import com.glxp.api.admin.entity.basic.UdiInfoEntity;
import com.glxp.api.admin.entity.basic.UdiRelevanceEntity;
import com.glxp.api.admin.req.basic.FilterUdiInfoRequest;
import com.glxp.api.admin.res.basic.UdiRelevanceResponse;
import com.glxp.api.admin.service.basic.BasicThirdSysService;
import com.glxp.api.admin.service.basic.UdiRelevanceService;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.Collections;
import java.util.List;

@Service
public class UdiRelevanceServiceImpl implements UdiRelevanceService {
    @Resource
    UdiRelevanceDao udiRelevanceDao;
    @Resource
    BasicThirdSysService basicThirdSysService;

    @Override
    public List<UdiRelevanceResponse> filterUdiRelevance(FilterUdiInfoRequest filterUdiInfoRequest) {
        if (filterUdiInfoRequest == null) {
            return Collections.emptyList();
        }
        if (filterUdiInfoRequest.getPage() != null) {
            int offset = (filterUdiInfoRequest.getPage() - 1) * filterUdiInfoRequest.getLimit();
            PageHelper.offsetPage(offset, filterUdiInfoRequest.getLimit());
        }

        List<UdiRelevanceResponse> data = udiRelevanceDao.filterUdiRelevance(filterUdiInfoRequest);
        return data;
    }

    @Override
    public boolean isExit() {
        FilterUdiInfoRequest filterUdiInfoRequest = new FilterUdiInfoRequest();
        filterUdiInfoRequest.setPage(1);
        filterUdiInfoRequest.setLimit(1);
        List<UdiRelevanceResponse> data = filterUdiRelevance(filterUdiInfoRequest);
        if (data != null && data.size() > 0) {
            return true;
        }
        return false;
    }

    @Override
    public List<UdiRelevanceResponse> filterUdiGp(FilterUdiInfoRequest filterUdiInfoRequest) {
        if (filterUdiInfoRequest == null) {
            return Collections.emptyList();
        }
        if (filterUdiInfoRequest.getPage() != null) {
            int offset = (filterUdiInfoRequest.getPage() - 1) * filterUdiInfoRequest.getLimit();
            PageHelper.offsetPage(offset, filterUdiInfoRequest.getLimit());
        }

        List<UdiRelevanceResponse> data = udiRelevanceDao.filterUdiGp(filterUdiInfoRequest);
        return data;
    }

    @Override
    public boolean isExitByUuid(String uuid) {
        List<UdiRelevanceEntity> udiRelevanceEntities = udiRelevanceDao.selectByUuid(uuid);
        if (udiRelevanceEntities != null && !udiRelevanceEntities.isEmpty()) {
            return true;
        }
        return false;
    }

    @Override
    public List<UdiRelevanceEntity> selectByUuid(String uuid) {
        return udiRelevanceDao.selectByUuid(uuid);
    }

    @Override
    public UdiRelevanceEntity selectById(String id) {
        return udiRelevanceDao.selectById(id);
    }

    @Override
    public UdiRelevanceResponse selectGroupById(String id) {
        FilterUdiInfoRequest filterUdiInfoRequest = new FilterUdiInfoRequest();
        filterUdiInfoRequest.setId(id);
        List<UdiRelevanceResponse> data = udiRelevanceDao.filterUdiRelevance(filterUdiInfoRequest);
        if (data != null)
            return data.get(0);
        return null;
    }

    @Override
    public UdiRelevanceEntity selectUpdateByUuid(String uuid) {
        FilterUdiInfoRequest filterUdiInfoRequest = new FilterUdiInfoRequest();
        filterUdiInfoRequest.setUuid(uuid);
        List<UdiRelevanceEntity> data = udiRelevanceDao.filterEntity(filterUdiInfoRequest);
        if (data != null && data.size() > 0)
            return data.get(0);
        return null;
    }

    @Override
    public UdiRelevanceEntity selectUpdateByThirdId(String thirdId) {
        FilterUdiInfoRequest filterUdiInfoRequest = new FilterUdiInfoRequest();
        if (thirdId != null) {
            BasicThirdSysEntity basicThirdSysEntity = basicThirdSysService.selectMainThrSys();
            String mainSys = basicThirdSysEntity.getThirdId();
            if (mainSys.equals("thirdId")) {
                filterUdiInfoRequest.setThirdId(thirdId);
            } else if (mainSys.equals("thirdId1")) {
                filterUdiInfoRequest.setThirdId1(thirdId);
            } else if (mainSys.equals("thirdId2")) {
                filterUdiInfoRequest.setThirdId2(thirdId);
            } else if (mainSys.equals("thirdId3")) {
                filterUdiInfoRequest.setThirdId3(thirdId);
            } else if (mainSys.equals("thirdId4")) {
                filterUdiInfoRequest.setThirdId4(thirdId);
            }
        }
        List<UdiRelevanceEntity> data = udiRelevanceDao.filterEntity(filterUdiInfoRequest);
        if (data != null && data.size() > 0)
            return data.get(0);
        return null;
    }

    @Override
    public UdiRelevanceEntity selectUpteaByAll(String uuid, String thirdId) {
        FilterUdiInfoRequest filterUdiInfoRequest = new FilterUdiInfoRequest();
        if (thirdId != null) {
            BasicThirdSysEntity basicThirdSysEntity = basicThirdSysService.selectMainThrSys();
            String mainSys = basicThirdSysEntity.getThirdId();
            filterUdiInfoRequest.setUuid(uuid);
            if (mainSys.equals("thirdId")) {
                filterUdiInfoRequest.setThirdId(thirdId);
            } else if (mainSys.equals("thirdId1")) {
                filterUdiInfoRequest.setThirdId1(thirdId);
            } else if (mainSys.equals("thirdId2")) {
                filterUdiInfoRequest.setThirdId2(thirdId);
            } else if (mainSys.equals("thirdId3")) {
                filterUdiInfoRequest.setThirdId3(thirdId);
            } else if (mainSys.equals("thirdId4")) {
                filterUdiInfoRequest.setThirdId4(thirdId);
            }
        }

        List<UdiRelevanceEntity> data = udiRelevanceDao.filterEntity(filterUdiInfoRequest);
//        if ((data == null || data.isEmpty()) && thirdId != null) {
//            filterUdiInfoRequest.setUuid(null);
//            data = udiRelevanceDao.filterEntity(filterUdiInfoRequest);
//            if ((data == null || data.isEmpty()) && uuid != null) {
//                filterUdiInfoRequest.setUuid(uuid);
//                filterUdiInfoRequest.setThirdId(null);
//                data = udiRelevanceDao.filterEntity(filterUdiInfoRequest);
//            }
//        }


        if (data != null && data.size() > 0)
            return data.get(0);
        return null;
    }

    @Override
    public UdiRelevanceEntity selectByThirdId(String thirdId, String thirdSys) {
        FilterUdiInfoRequest filterUdiInfoRequest = new FilterUdiInfoRequest();
        if ("thirdId".equals(thirdSys)) {
            filterUdiInfoRequest.setThirdId(thirdId);
        } else if ("thirdId1".equals(thirdSys)) {
            filterUdiInfoRequest.setThirdId1(thirdId);
        } else if ("thirdId2".equals(thirdSys)) {
            filterUdiInfoRequest.setThirdId2(thirdId);
        } else if ("thirdId3".equals(thirdSys)) {
            filterUdiInfoRequest.setThirdId3(thirdId);
        } else if ("thirdId4".equals(thirdSys)) {
            filterUdiInfoRequest.setThirdId4(thirdId);
        }

        return udiRelevanceDao.selectByThirdId(filterUdiInfoRequest);
    }

    @Override
    public boolean insertUdiRelevance(UdiRelevanceEntity udiRelevanceEntity) {
        return udiRelevanceDao.insertUdiRelevance(udiRelevanceEntity);
    }

    @Override
    public boolean updateUdiRelevance(UdiRelevanceEntity udiRelevanceEntity) {
        return udiRelevanceDao.updateUdiRelevance(udiRelevanceEntity);
    }

    @Override
    public boolean deleteById(String id) {
        return udiRelevanceDao.deleteById(id);
    }

    @Override
    public boolean deleteByIds(List<String> ids) {
        return udiRelevanceDao.deleteByIds(ids);
    }

    @Override
    public boolean deleteByUuid(String uuid) {
        return udiRelevanceDao.deleteByUuid(uuid);
    }
}
