package com.glxp.api.admin.service.info.impl;

import com.glxp.api.admin.dao.info.SystemPDFTemplateRelevanceDao;
import com.glxp.api.admin.entity.info.SystemPDFTemplateRelevanceEntity;
import com.glxp.api.admin.service.info.SystemPDFTemplateRelevanceService;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.Collections;
import java.util.List;
import java.util.stream.Collectors;

@Service
public class SystemPDFTemplateRelevanceServiceImpl implements SystemPDFTemplateRelevanceService {

    @Resource
    private SystemPDFTemplateRelevanceDao systemPDFTemplateRelevanceDao;

    /**
     * 根据 customerId 获取模板id
     * @param customerId
     * @return
     */
    @Override
    public List<SystemPDFTemplateRelevanceEntity> listByCustomerId(Long customerId) {
        return systemPDFTemplateRelevanceDao.listByCustomerId(customerId);
    }

    /**
     * 根据多个 customerId 查询角色列表
     * @param customerIds
     * @return
     */
    @Override
    public List<SystemPDFTemplateRelevanceEntity> listByCustomerIdIn(List<Long> customerIds) {
        if (customerIds.isEmpty()) {
            return Collections.emptyList();
        }
        return systemPDFTemplateRelevanceDao.listByCustomerIdIn(customerIds);
    }

    /**
     * 根据 roleId 获取 customerId
     * @param templateId
     * @return
     */
    @Override
    public List<SystemPDFTemplateRelevanceEntity> listByTemplateId(int templateId) {
        return systemPDFTemplateRelevanceDao.listByTemplateId(templateId);
    }

    /**
     * 批量插入
     * @param systemPDFTemplateRelevanceEntityList
     * @return
     */
    @Override
    public int insertSystemPDFTemplateRelevanceAll(List<SystemPDFTemplateRelevanceEntity> systemPDFTemplateRelevanceEntityList) {

        if (systemPDFTemplateRelevanceEntityList == null || systemPDFTemplateRelevanceEntityList.isEmpty()) {
            return 0;
        }

        return systemPDFTemplateRelevanceDao.insertSystemPDFTemplateRelevanceAll(systemPDFTemplateRelevanceEntityList);
    }

    @Override
    public boolean insert(SystemPDFTemplateRelevanceEntity systemPDFTemplateRelevanceEntity) {
        return systemPDFTemplateRelevanceDao.insert(systemPDFTemplateRelevanceEntity);
    }

    /**
     * 根据 模板ids 和 customerId 批量插入
     * @param templates
     * @param customerId
     * @return
     */
    @Override
    public int insertRolesCustomerIdAll(List<Integer> templates, Long customerId) {

        List<SystemPDFTemplateRelevanceEntity> systemPDFTemplateRelevanceEntityList = templates.stream().map(aLong -> {
            SystemPDFTemplateRelevanceEntity systemPDFTemplateRelevanceEntity = new SystemPDFTemplateRelevanceEntity();
            systemPDFTemplateRelevanceEntity.setTemplate_id(aLong);
            systemPDFTemplateRelevanceEntity.setCustomerId(customerId);
            return systemPDFTemplateRelevanceEntity;
        }).collect(Collectors.toList());
        if (!systemPDFTemplateRelevanceEntityList.isEmpty()) {
            return insertSystemPDFTemplateRelevanceAll(systemPDFTemplateRelevanceEntityList);
        }

        return 0;
    }

    /**
     * 根据 customerId 删除对应的模板关联
     * @param customerId
     * @return
     */
    @Override
    public boolean deleteByCustomerId(Long customerId) {

        return systemPDFTemplateRelevanceDao.deleteByCustomerId(customerId);
    }
}
