package com.glxp.api.admin.service.inout.impl;

import com.github.pagehelper.PageHelper;
import com.glxp.api.admin.dao.inout.CodesDao;
import com.glxp.api.admin.entity.inout.WarehouseEntity;
import com.glxp.api.admin.req.inout.WarehouseQueryRequest;
import com.glxp.api.admin.service.inout.CodesService;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.Collections;
import java.util.List;

@Service
public class CodesServiceImpl implements CodesService {

    @Resource
    CodesDao warehouseDao;

    @Override
    public List<WarehouseEntity> listWarehouse(WarehouseQueryRequest warehouseQueryRequest) {
        if (warehouseQueryRequest == null) {
            return Collections.emptyList();
        }
        int offset = (warehouseQueryRequest.getPage() - 1) * warehouseQueryRequest.getLimit();
        PageHelper.offsetPage(offset, warehouseQueryRequest.getLimit());
        return warehouseDao.listWarehouse(warehouseQueryRequest);
    }


    @Override
    public boolean insertWarehouse(WarehouseEntity warehouseEntity) {
        return warehouseDao.insertWarehouse(warehouseEntity);
    }

    @Override
    public boolean insertWarehouses(List<WarehouseEntity> warehouseEntities) {
        return warehouseDao.insertWarehouses(warehouseEntities);
    }


    @Override
    public List<WarehouseEntity> findByNo(WarehouseQueryRequest warehouseQueryRequest) {
        if (warehouseQueryRequest == null) {
            return Collections.emptyList();
        }
        int offset = (warehouseQueryRequest.getPage() - 1) * warehouseQueryRequest.getLimit();
        PageHelper.offsetPage(offset, warehouseQueryRequest.getLimit());
        return warehouseDao.findByNo(warehouseQueryRequest);
    }

    @Override
    public boolean isExits(WarehouseEntity warehouseEntity) {

        int result = warehouseDao.isExit(warehouseEntity);
        if (result == 1)
            return true;
        else
            return false;
    }

    @Override
    public List<WarehouseEntity> findByOrderId(WarehouseQueryRequest warehouseQueryRequest) {
        if (warehouseQueryRequest == null) {
            return Collections.emptyList();
        }
        if (warehouseQueryRequest.getPage() != null) {
            int offset = (warehouseQueryRequest.getPage() - 1) * warehouseQueryRequest.getLimit();
            PageHelper.offsetPage(offset, warehouseQueryRequest.getLimit());
        }

        return warehouseDao.findByOrderId(warehouseQueryRequest);
    }

    @Override
    public List<WarehouseEntity> findAllByOrderId(WarehouseQueryRequest warehouseQueryRequest) {
        if (warehouseQueryRequest == null) {
            return Collections.emptyList();
        }
        return warehouseDao.findByOrderId(warehouseQueryRequest);
    }

    @Override
    public List<WarehouseEntity> findByReceiptId(String orderId) {
        return warehouseDao.findByReceiptId(orderId);
    }


    @Override
    public List<WarehouseEntity> findByCode(String code) {

        return warehouseDao.findByCode(code);
    }

    @Override
    public boolean updateOrderId(String oldOrderId, String newOrderId) {


        return warehouseDao.updateOrderId(oldOrderId, newOrderId);
    }

    @Override
    public List<WarehouseEntity> searchCode(String code) {
        return warehouseDao.searchCode(code);
    }


    @Override
    public boolean updateUnit(String orderId, String fromCorpId, String fromCorp) {
        return warehouseDao.updateUnit(orderId, fromCorpId, fromCorp);
    }
}
