package com.glxp.api.admin.service.inout.impl;

import com.github.pagehelper.PageHelper;
import com.glxp.api.admin.dao.inout.OrderDao;
import com.glxp.api.admin.dao.inout.CodesDao;
import com.glxp.api.admin.entity.inout.OrderEntity;
import com.glxp.api.admin.entity.inout.WarehouseEntity;
import com.glxp.api.admin.req.inout.OrderFilterRequest;
import com.glxp.api.admin.req.inout.OrderQueryRequest;
import com.glxp.api.admin.service.inout.OrderService;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.Collections;
import java.util.List;

@Service
public class OrderServiceImpl implements OrderService {

    @Resource
    OrderDao orderDao;
    @Resource
    CodesDao warehouseDao;

    @Override
    public List<OrderEntity> listOrders(OrderQueryRequest orderQueryRequest) {
        if (orderQueryRequest == null) {
            return Collections.emptyList();
        }
        int offset = (orderQueryRequest.getPage() - 1) * orderQueryRequest.getLimit();
        PageHelper.offsetPage(offset, orderQueryRequest.getLimit());
        return orderDao.listOrder(orderQueryRequest);
    }


    @Override
    public boolean insertOrder(OrderEntity orderEntity) {
        return orderDao.insertOrder(orderEntity);
    }

    @Override
    public boolean updateOrder(OrderEntity orderEntity) {
        return orderDao.updateOrder(orderEntity);
    }


    @Override
    public List<OrderEntity> listOrderRepeat(OrderQueryRequest orderQueryRequest) {
        if (orderQueryRequest == null) {
            return Collections.emptyList();
        }
        int offset = (orderQueryRequest.getPage() - 1) * orderQueryRequest.getLimit();
        PageHelper.offsetPage(offset, orderQueryRequest.getLimit());
        return orderDao.listOrderRepeat(orderQueryRequest);
    }

    @Override
    public boolean deleteByOrderId(String orderId) {
        return orderDao.deleteByOrderId(orderId);
    }

    @Override
    public boolean deleteById(String id) {
        return orderDao.deleteById(id);
    }

    @Override
    public boolean updateUnit(OrderEntity orderEntity) {
        return orderDao.updateUnit(orderEntity);
    }

    @Override
    public boolean updateOrderStatus(String orderId, Integer status) {
        return orderDao.updateOrderStatus(orderId, status);
    }

    @Override
    public boolean updateExportStatus(String orderId, Integer exportStatus) {
        return orderDao.updateExportStatus(orderId, exportStatus);
    }

    @Override
    public boolean updateSignStatus(String orderId, Integer signStatus) {
        return orderDao.updateSignStatus(orderId, signStatus);
    }

    @Override
    public boolean updateContrastStatus(String orderId, String contrastStatus) {
        return orderDao.updateContrastStatus(orderId, contrastStatus);
    }

    @Override
    public boolean updateRemark(String orderId, String remark) {
        return orderDao.updateRemark(orderId, remark);
    }

    @Override
    public List<OrderEntity> findAllOrders(OrderFilterRequest orderFilterRequest) {
        return orderDao.filterNoSort(orderFilterRequest);
    }

    @Override
    public List<OrderEntity> filterList(OrderFilterRequest orderFilterRequest) {
        if (orderFilterRequest == null) {
            return Collections.emptyList();
        }
        int offset = (orderFilterRequest.getPage() - 1) * orderFilterRequest.getLimit();
        PageHelper.offsetPage(offset, orderFilterRequest.getLimit());
        return orderDao.filterList(orderFilterRequest);
    }

    @Override
    public OrderEntity findOne(OrderFilterRequest orderFilterRequest) {
        return orderDao.findOne(orderFilterRequest);
    }

    @Override
    public OrderEntity findById(String orderId) {
        OrderFilterRequest orderFilterRequest = new OrderFilterRequest();
        orderFilterRequest.setId(orderId);
        return orderDao.findOne(orderFilterRequest);
    }

    @Override
    public List<String> findCodesById(String orderId) {
        return warehouseDao.findCodesById(orderId);
    }

    @Override
    public List<OrderEntity> filterListByCode(List<WarehouseEntity> warehouseEntityList) {
        if (warehouseEntityList == null) {
            return Collections.emptyList();
        }
        return orderDao.filterListByCode(warehouseEntityList);
    }


}
