package com.glxp.api.admin.service.inout.impl;

import com.github.pagehelper.PageHelper;
import com.glxp.api.admin.dao.inout.StockOrderDao;
import com.glxp.api.admin.entity.inout.StockOrderEntity;
import com.glxp.api.admin.req.inout.StockOrderFilterRequest;
import com.glxp.api.admin.req.inout.StockOrderQueryRequest;
import com.glxp.api.admin.service.inout.StockOrderService;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.Collections;
import java.util.List;

@Service
public class StockOrderServiceImpl implements StockOrderService {

    @Resource
    StockOrderDao stockOrderDao;

    @Override
    public List<StockOrderEntity> listStockOrders(StockOrderQueryRequest stockOrderQueryRequest) {
        if (stockOrderQueryRequest == null) {
            return Collections.emptyList();
        }
        int offset = (stockOrderQueryRequest.getPage() - 1) * stockOrderQueryRequest.getLimit();
        PageHelper.offsetPage(offset, stockOrderQueryRequest.getLimit());
        return stockOrderDao.listStockOrder(stockOrderQueryRequest);
    }


    @Override
    public boolean insertStockOrder(StockOrderEntity stockOrderEntity) {
        return stockOrderDao.insertStockOrder(stockOrderEntity);
    }

    @Override
    public List<StockOrderEntity> listStockOrderNoDealWith(StockOrderQueryRequest stockOrderQueryRequest) {
        if (stockOrderQueryRequest == null) {
            return Collections.emptyList();
        }
        int offset = (stockOrderQueryRequest.getPage() - 1) * stockOrderQueryRequest.getLimit();
        PageHelper.offsetPage(offset, stockOrderQueryRequest.getLimit());
        return stockOrderDao.listStockOrderNoDealWith(stockOrderQueryRequest);
    }

    @Override
    public boolean updateById(StockOrderEntity stockOrderEntity) {
        return stockOrderDao.updateById(stockOrderEntity);
    }

    @Override
    public boolean deleteByOrderId(String orderId) {
        return stockOrderDao.deleteByOrderId(orderId);
    }

    @Override
    public boolean deleteById(String id) {
        return stockOrderDao.deleteById(id);
    }

    @Override
    public boolean updateByStatus(String orderId, Integer status) {
        return stockOrderDao.updateByStatus(orderId, status);
    }

    @Override
    public List<StockOrderEntity> findAllOrders(StockOrderFilterRequest stockOrderFilterRequest) {
        return stockOrderDao.filterListOr(stockOrderFilterRequest);
    }

    @Override
    public List<StockOrderEntity> filterListOr(StockOrderFilterRequest stockOrderFilterRequest) {
        if (stockOrderFilterRequest == null) {
            return Collections.emptyList();
        }
        int offset = (stockOrderFilterRequest.getPage() - 1) * stockOrderFilterRequest.getLimit();
        PageHelper.offsetPage(offset, stockOrderFilterRequest.getLimit());
        return stockOrderDao.filterListOr(stockOrderFilterRequest);
    }

    @Override
    public StockOrderEntity findOne(StockOrderFilterRequest stockOrderFilterRequest) {
        return stockOrderDao.findOne(stockOrderFilterRequest);
    }

    @Override
    public StockOrderEntity findById(String orderId) {
        StockOrderFilterRequest stockOrderFilterRequest = new StockOrderFilterRequest();
        stockOrderFilterRequest.setId(orderId);
        return stockOrderDao.findOne(stockOrderFilterRequest);
    }


}
