package com.glxp.api.admin.service.thrsys;

import com.glxp.api.admin.entity.basic.BasicThirdSysDetailEntity;
import com.glxp.api.admin.entity.basic.BasicThirdSysEntity;
import com.glxp.api.admin.entity.inout.ErpOrderEntity;
import com.glxp.api.admin.entity.inout.StockOrderDetailEntity;
import com.glxp.api.admin.entity.inout.StockOrderEntity;
import com.glxp.api.admin.entity.thrsys.ThrProductsEntity;
import com.glxp.api.admin.httpclient.ErpBasicClient;
import com.glxp.api.admin.req.basic.FilterErpGoodsRequest;
import com.glxp.api.admin.req.inout.StockOrderDetailFilterRequest;
import com.glxp.api.admin.req.inout.StockOrderFilterRequest;
import com.glxp.api.admin.req.thrsys.FilterThrProductsRequest;
import com.glxp.api.admin.res.PageSimpleResponse;
import com.glxp.api.admin.res.basic.ErpProductsResponse;
import com.glxp.api.admin.res.inout.ErpOrderResponse;
import com.glxp.api.admin.service.basic.BasicThirdSysDetailService;
import com.glxp.api.admin.service.basic.BasicThirdSysService;
import com.glxp.api.admin.service.inout.StockOrderDetailService;
import com.glxp.api.admin.service.inout.StockOrderService;
import com.glxp.api.admin.thread.IoModifyErpTask;
import com.glxp.api.common.res.BaseResponse;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.BeanUtils;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.List;

/**
 * 根据系统设置，获取对应的第三方数据信息
 */

@Service
public class ThrDataService {

    @Resource
    private BasicThirdSysDetailService basicThirdSysDetailService;
    @Resource
    private ThrProductsService thrProductsService;
    @Resource
    private StockOrderService stockOrderService;
    @Resource
    private StockOrderDetailService stockOrderDetailService;
    final Logger logger = LoggerFactory.getLogger(ThrDataService.class);

    public ThrProductsEntity getProducts(String code, String thirdSys) {

        BasicThirdSysDetailEntity basicThirdSysDetailEntity = basicThirdSysDetailService.selectByKey("piQueryUrl", thirdSys);

        if (basicThirdSysDetailEntity == null || !basicThirdSysDetailEntity.getEnabled()) {
            logger.info("第三方产品信息服务未启用");
            return null;
        }

        if (basicThirdSysDetailEntity.getFromType() == 0) {
            FilterErpGoodsRequest filterErpGoodsRequest = new FilterErpGoodsRequest();
            filterErpGoodsRequest.setCode(code);
            filterErpGoodsRequest.setPage(1);
            filterErpGoodsRequest.setLimit(1);
            BaseResponse<PageSimpleResponse<ErpProductsResponse>> udiDlDeviceResponse = new ErpBasicClient().getErpProducts(filterErpGoodsRequest, basicThirdSysDetailEntity.getValue());
            if (udiDlDeviceResponse.getCode() == 20000) {
                List<ErpProductsResponse> erpProductsResponses = udiDlDeviceResponse.getData().getList();
                if (erpProductsResponses != null) {
                    ThrProductsEntity thrProductsEntity = new ThrProductsEntity();
                    BeanUtils.copyProperties(erpProductsResponses.get(0), thrProductsEntity);
                    return thrProductsEntity;
                }
            }
        } else {
            FilterThrProductsRequest filterThrProductsRequest = new FilterThrProductsRequest();
            filterThrProductsRequest.setCode(code);
            filterThrProductsRequest.setThirdSysFk(thirdSys);
            List<ThrProductsEntity> thrProductsEntitys = thrProductsService.filterThrProductsRequest(filterThrProductsRequest);
            if (thrProductsEntitys != null && thrProductsEntitys.size() > 0) {
                return thrProductsEntitys.get(0);
            }
        }
        return null;
    }


    public List<ErpOrderEntity> getStockOrders(List<String> billCodes) {
        if (billCodes != null && billCodes.size() > 0) {
            for (String billCode : billCodes) {
                StockOrderFilterRequest stockOrderFilterRequest = new StockOrderFilterRequest();
                stockOrderFilterRequest.setBillNo(billCode);
                List<StockOrderEntity> data = stockOrderService.findAllOrders(stockOrderFilterRequest);
                List<ErpOrderEntity> erpOrderEntities = new ArrayList<>();
                if (data != null && data.size() > 0) {
                    for (StockOrderEntity thrOrderEntity : data) {
                        StockOrderDetailFilterRequest filterThrOrderDetailRequest = new StockOrderDetailFilterRequest();
                        filterThrOrderDetailRequest.setOrderIdFk(thrOrderEntity.getId() + "");
                        List<StockOrderDetailEntity> thrOrderDetailEntities = stockOrderDetailService.filterStockOrderDetail(filterThrOrderDetailRequest);

                        List<ErpOrderResponse.SubErpOrder> subErpOrders = new ArrayList<>();
                        if (thrOrderDetailEntities != null && thrOrderDetailEntities.size() > 0) {
                            for (StockOrderDetailEntity thrOrderDetailEntity : thrOrderDetailEntities) {
                                ErpOrderEntity erpOrderResponse = new ErpOrderEntity();
                                BeanUtils.copyProperties(thrOrderEntity, erpOrderResponse);
                                BeanUtils.copyProperties(thrOrderDetailEntity, erpOrderResponse);
                                erpOrderEntities.add(erpOrderResponse);
                            }
                        }
                    }
                }
                return erpOrderEntities;

            }

        }
        return null;
    }

    public List<ErpOrderEntity> getCacheOrders(List<String> billCodes) {
        if (billCodes != null && billCodes.size() > 0) {
            for (String billCode : billCodes) {
                StockOrderFilterRequest stockOrderFilterRequest = new StockOrderFilterRequest();
                stockOrderFilterRequest.setBillNo(billCode);
                List<StockOrderEntity> data = stockOrderService.findAllOrders(stockOrderFilterRequest);
                List<ErpOrderEntity> erpOrderEntities = new ArrayList<>();
                if (data != null && data.size() > 0) {
                    for (StockOrderEntity thrOrderEntity : data) {
                        StockOrderDetailFilterRequest filterThrOrderDetailRequest = new StockOrderDetailFilterRequest();
                        filterThrOrderDetailRequest.setOrderIdFk(thrOrderEntity.getId() + "");
                        List<StockOrderDetailEntity> thrOrderDetailEntities = stockOrderDetailService.filterStockOrderDetail(filterThrOrderDetailRequest);

                        List<ErpOrderResponse.SubErpOrder> subErpOrders = new ArrayList<>();
                        if (thrOrderDetailEntities != null && thrOrderDetailEntities.size() > 0) {
                            for (StockOrderDetailEntity thrOrderDetailEntity : thrOrderDetailEntities) {
                                ErpOrderEntity erpOrderResponse = new ErpOrderEntity();
                                BeanUtils.copyProperties(thrOrderEntity, erpOrderResponse);
                                BeanUtils.copyProperties(thrOrderDetailEntity, erpOrderResponse);
                                erpOrderEntities.add(erpOrderResponse);
                            }
                        }
                    }
                }
                return erpOrderEntities;

            }

        }
        return null;
    }
}
