package com.glxp.api.admin.service.thrsys.impl;

import com.github.pagehelper.PageHelper;
import com.glxp.api.admin.dao.thrsys.ThrOrderDao;
import com.glxp.api.admin.dao.thrsys.ThrOrderDetailDao;
import com.glxp.api.admin.entity.inout.ErpOrderEntity;
import com.glxp.api.admin.entity.thrsys.ThrOrderDetailEntity;
import com.glxp.api.admin.entity.thrsys.ThrOrderEntity;
import com.glxp.api.admin.req.thrsys.FilterThrOrderDetailRequest;
import com.glxp.api.admin.req.thrsys.FilterThrOrderRequest;
import com.glxp.api.admin.res.inout.ErpOrderResponse;
import com.glxp.api.admin.service.thrsys.ThrOrderService;
import org.springframework.beans.BeanUtils;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

@Service
public class ThrOrderServiceImpl implements ThrOrderService {
    @Resource
    private ThrOrderDao thrOrderDao;
    @Resource
    private ThrOrderDetailDao thrOrderDetailDao;

    @Override
    public List<ThrOrderEntity> filterThrOrder(FilterThrOrderRequest filterThrOrderRequest) {
        if (filterThrOrderRequest == null) {
            return Collections.emptyList();
        }
        if (filterThrOrderRequest.getPage() != null) {
            int offset = (filterThrOrderRequest.getPage() - 1) * filterThrOrderRequest.getLimit();
            PageHelper.offsetPage(offset, filterThrOrderRequest.getLimit());
        }
        List<ThrOrderEntity> data = thrOrderDao.filterThrOrder(filterThrOrderRequest);
        return data;
    }

    @Override
    public List<ErpOrderEntity> filterAllOrders(List<String> billNos, String thirdSys) {
        List<ErpOrderEntity> erpOrderEntities = new ArrayList<>();
        for (String billNo : billNos) {
            FilterThrOrderRequest filterThrOrderRequest = new FilterThrOrderRequest();
            filterThrOrderRequest.setBillNo(billNo);
            filterThrOrderRequest.setThirdSysFk(thirdSys);
            List<ThrOrderEntity> data = thrOrderDao.filterThrOrder(filterThrOrderRequest);
            if (data != null && data.size() > 0) {
                for (ThrOrderEntity thrOrderEntity : data) {
                    FilterThrOrderDetailRequest filterThrOrderDetailRequest = new FilterThrOrderDetailRequest();
                    filterThrOrderDetailRequest.setOrderIdFk(thrOrderEntity.getId() + "");
                    List<ThrOrderDetailEntity> thrOrderDetailEntities = thrOrderDetailDao.filterThrOrderDetailDetail(filterThrOrderDetailRequest);
                    if (thrOrderDetailEntities != null && thrOrderDetailEntities.size() > 0) {
                        for (ThrOrderDetailEntity thrOrderDetailEntity : thrOrderDetailEntities) {
                            ErpOrderEntity erpOrderEntity = new ErpOrderEntity();
                            BeanUtils.copyProperties(thrOrderEntity, erpOrderEntity);
                            BeanUtils.copyProperties(thrOrderDetailEntity, erpOrderEntity);
                            erpOrderEntities.add(erpOrderEntity);
                        }
                    }
                }
            }
        }
        return erpOrderEntities;
    }

    @Override
    public List<ErpOrderResponse> filterAllDetail(FilterThrOrderRequest filterThrOrderRequest) {

        List<ErpOrderResponse> erpOrderResponses = new ArrayList<>();
        List<ThrOrderEntity> data = filterThrOrder(filterThrOrderRequest);
        if (data != null && data.size() > 0) {
            for (ThrOrderEntity thrOrderEntity : data) {
                FilterThrOrderDetailRequest filterThrOrderDetailRequest = new FilterThrOrderDetailRequest();
                filterThrOrderDetailRequest.setOrderIdFk(thrOrderEntity.getId() + "");
                List<ThrOrderDetailEntity> thrOrderDetailEntities = thrOrderDetailDao.filterThrOrderDetailDetail(filterThrOrderDetailRequest);
                ErpOrderResponse erpOrderResponse = new ErpOrderResponse();
                BeanUtils.copyProperties(thrOrderEntity, erpOrderResponse);
                List<ErpOrderResponse.SubErpOrder> subErpOrders = new ArrayList<>();
                if (thrOrderDetailEntities != null && thrOrderDetailEntities.size() > 0) {
                    for (ThrOrderDetailEntity thrOrderDetailEntity : thrOrderDetailEntities) {
                        ErpOrderResponse.SubErpOrder subErpOrder = new ErpOrderResponse.SubErpOrder();
                        BeanUtils.copyProperties(thrOrderDetailEntity, subErpOrder);
                        subErpOrders.add(subErpOrder);
                    }
                }
                erpOrderResponse.setSubErpOrders(subErpOrders);
                erpOrderResponses.add(erpOrderResponse);
            }
        }

        return erpOrderResponses;
    }

    @Override
    public ThrOrderEntity findByUnique(String billNo, String thirdSysFk) {
        FilterThrOrderRequest filterThrOrderRequest = new FilterThrOrderRequest();
        filterThrOrderRequest.setBillNo(billNo);
        filterThrOrderRequest.setThirdSysFk(thirdSysFk);
        List<ThrOrderEntity> thrOrderEntities = filterThrOrder(filterThrOrderRequest);
        if (thrOrderEntities != null && thrOrderEntities.size() > 0) {
            return thrOrderEntities.get(0);
        }
        return null;
    }

    @Override
    public ThrOrderEntity findById(String id) {
        FilterThrOrderRequest filterThrOrderRequest = new FilterThrOrderRequest();
        filterThrOrderRequest.setId(id);
        List<ThrOrderEntity> thrOrderEntities = filterThrOrder(filterThrOrderRequest);
        if (thrOrderEntities != null && thrOrderEntities.size() > 0) {
            return thrOrderEntities.get(0);
        }
        return null;
    }

    @Override
    public boolean insertThrOrder(ThrOrderEntity thrCorpEntity) {
        return thrOrderDao.insertThrOrder(thrCorpEntity);
    }

    @Override
    public boolean insertThrOrders(List<ThrOrderEntity> thrOrderEntities) {
        return thrOrderDao.insertThrOrders(thrOrderEntities);
    }

    @Override
    public boolean updateThrOrder(ThrOrderEntity thrOrderEntity) {
        return thrOrderDao.updateThrOrder(thrOrderEntity);
    }

    @Override
    public boolean deleteById(String id) {
        return thrOrderDao.deleteById(id);
    }

    @Override
    public boolean deleteAll() {
        return thrOrderDao.deleteAll();
    }
}
