package com.glxp.api.admin.thread;

import com.alibaba.fastjson.JSONObject;
import com.alibaba.fastjson.TypeReference;
import com.glxp.api.admin.constant.BasicProcessStatus;
import com.glxp.api.admin.entity.basic.*;
import com.glxp.api.admin.entity.info.SystemParamConfigEntity;
import com.glxp.api.admin.req.basic.*;
import com.glxp.api.admin.res.basic.UdiRelevanceResponse;
import com.glxp.api.admin.service.basic.BasicUnitMaintainService;
import com.glxp.api.admin.service.basic.CorpExportLogService;
import com.glxp.api.admin.service.basic.UdiInfoExportLogService;
import com.glxp.api.admin.service.basic.UdiRelevanceService;
import com.glxp.api.admin.service.info.SystemParamConfigService;
import com.glxp.api.admin.util.ExcelUtil;
import com.glxp.api.admin.util.HttpClient;
import com.glxp.api.common.res.BaseResponse;
import com.glxp.api.common.util.ResultVOUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.scheduling.annotation.Async;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.io.File;
import java.io.IOException;
import java.util.ArrayList;
import java.util.List;

@Service
public class BasicGenExcelService {
    @Resource
    CorpExportLogService corpExportLogService;
    @Resource
    BasicUnitMaintainService basicUnitMaintainService;
    @Resource
    UdiInfoExportLogService udiInfoExportLogService;
    @Resource
    UdiRelevanceService udiRelevanceService;
    @Resource
    SystemParamConfigService systemParamConfigService;

    //往来单位导出excel文件
    @Async
    public void exportCorp(String genKey, CorpsExportRequest corpsExportRequest) {

        CorpExportLogEntity corpExportLogEntity = corpExportLogService.selectByGenKey(genKey);
        List<List<String>> excelData = new ArrayList<>();
        List<String> head = new ArrayList<>();
        head.add("往来单位ID");
        head.add("往来单位名称");
        head.add("往来单位拼音简写");
        head.add("地址");
        head.add("联系人");
        head.add("联系电话");
        head.add("社会信用号");
        head.add("往来单位状态");
        head.add("往来单位类型");
        head.add("thirdId");
        head.add("thirdName");
        head.add("thirdI1d");
        head.add("thirdName1");
        head.add("thirdId2");
        head.add("thirdName2");
        head.add("thirdId3");
        head.add("thirdName3");
        head.add("thirdId4");
        head.add("thirdName4");
        excelData.add(head);
        //选中导出
        if (corpsExportRequest.getIds() != null && corpsExportRequest.getIds().size() > 0) {
            List<Integer> ids = corpsExportRequest.getIds();
            for (Integer id : ids) {
                BasicUnitMaintainFilterRequest filterRequest = new BasicUnitMaintainFilterRequest();
                filterRequest.setId(id);
                List<List<String>> genDatas = genExcelData(filterRequest);
                if (genDatas != null && genDatas.size() > 0) {
                    excelData.addAll(genDatas);
                }
            }

        } else {//一键导出
            BasicUnitMaintainFilterRequest filterRequest = new BasicUnitMaintainFilterRequest();
            BeanUtils.copyProperties(corpsExportRequest, filterRequest);
            filterRequest.setPage(null);
            List<List<String>> genDatas = genExcelData(filterRequest);
            if (genDatas != null && genDatas.size() > 0) {
                excelData.addAll(genDatas);
            }
        }

        String sheetName = "往来单位信息";
        File file = new File(corpExportLogEntity.getFilePath());
        if (!file.exists()) {
            try {
                file.createNewFile();
            } catch (IOException e) {
                e.printStackTrace();
            }
        }
        new ExcelUtil().exportExcel(excelData, sheetName, corpExportLogEntity.getFilePath(), 20);
        corpExportLogEntity.setStatus(BasicProcessStatus.UDIINFO_IMPORT_SUCCESS);
        corpExportLogService.updateCorpExportLog(corpExportLogEntity);
    }

    //上传往来单位SMP
    @Async
    public void uploadCorpSmp(String genKey, CorpsExportRequest corpsExportRequest) {
        PostCorpsRequest postCorpsRequest = new PostCorpsRequest();
        postCorpsRequest.setGenKey(genKey);
        List<CorpImportDetailEntity> corpImportDetailEntities = new ArrayList<>();
        CorpExportLogEntity corpExportLogEntity = corpExportLogService.selectByGenKey(genKey);
        //选中导出
        if (corpsExportRequest.getIds() != null && corpsExportRequest.getIds().size() > 0) {
            List<Integer> ids = corpsExportRequest.getIds();
            for (Integer id : ids) {
                BasicUnitMaintainFilterRequest filterRequest = new BasicUnitMaintainFilterRequest();
                filterRequest.setId(id);
                List<BasicUnitMaintainEntity> unitMaintainEntities = basicUnitMaintainService.filterList(filterRequest);
                if (unitMaintainEntities != null && unitMaintainEntities.size() > 0) {
                    for (BasicUnitMaintainEntity basicUnitMaintainEntity : unitMaintainEntities) {
                        CorpImportDetailEntity corpImportDetailEntity = new CorpImportDetailEntity();
                        BeanUtils.copyProperties(basicUnitMaintainEntity, corpImportDetailEntity);
                        corpImportDetailEntities.add(corpImportDetailEntity);
                    }
                    postCorpsRequest.setDatas(corpImportDetailEntities);

                }
            }


        } else {//一键导出
            BasicUnitMaintainFilterRequest filterRequest = new BasicUnitMaintainFilterRequest();
            BeanUtils.copyProperties(corpsExportRequest, filterRequest);

            List<BasicUnitMaintainEntity> unitMaintainEntities = basicUnitMaintainService.filterList(filterRequest);
            if (unitMaintainEntities != null && unitMaintainEntities.size() > 0) {
                for (BasicUnitMaintainEntity basicUnitMaintainEntity : unitMaintainEntities) {
                    CorpImportDetailEntity corpImportDetailEntity = new CorpImportDetailEntity();
                    BeanUtils.copyProperties(basicUnitMaintainEntity, corpImportDetailEntity);
                    corpImportDetailEntities.add(corpImportDetailEntity);
                }
                postCorpsRequest.setDatas(corpImportDetailEntities);

            }
        }

        //todo 上传SMP
        SystemParamConfigEntity systemParamConfigEntity = systemParamConfigService.selectByParamKey("smp_service_ip");
        if(systemParamConfigEntity!=null){
            String response = HttpClient.uCloudPost(systemParamConfigEntity.getParamValue()+"/udiwms/basic/corps/upload", postCorpsRequest);
            BaseResponse baseResponse = JSONObject.parseObject(response, new TypeReference<BaseResponse>() {});
            if (baseResponse.getCode() == 20000) {
                corpExportLogEntity.setStatus(BasicProcessStatus.UDIINFO_IMPORT_SUCCESS);
            } else {
                corpExportLogEntity.setStatus(BasicProcessStatus.UDIINFO_IMPORT_FAIL);
            }
        }else {
            corpExportLogEntity.setRemark("SMP服务IP地址未定义！");
            corpExportLogEntity.setStatus(BasicProcessStatus.UDIINFO_IMPORT_FAIL);
        }



        corpExportLogService.updateCorpExportLog(corpExportLogEntity);
    }

    @Async
    public void exportUdiInfo(String genKey, UdiInfoExportRequest udiInfoExportRequest) {
        UdiInfoExportLogEntity udiInfoExportLogEntity = udiInfoExportLogService.selectByGenKey(genKey);
        List<List<String>> excelData = new ArrayList<>();
        List<String> head = new ArrayList<>();
        head.add("udiCode");
        head.add("医保编码");
        head.add("商品条码");
        head.add("thirdId");
        head.add("thirdId1");
        head.add("thirdId2");
        head.add("thirdId3");
        head.add("thirdId4");
        excelData.add(head);
        //选中导出
        if (udiInfoExportRequest.getUuids() != null && udiInfoExportRequest.getUuids().size() > 0) {
            List<String> uuids = udiInfoExportRequest.getUuids();
            for (String uuid : uuids) {
                FilterUdiInfoRequest filterUdiInfoRequest = new FilterUdiInfoRequest();
                filterUdiInfoRequest.setUuid(uuid);
                List<List<String>> genDatas = genExcelData(filterUdiInfoRequest);
                if (genDatas != null && genDatas.size() > 0) {
                    excelData.addAll(genDatas);
                }
            }

        } else {//一键导出
            FilterUdiInfoRequest filterUdiInfoRequest = new FilterUdiInfoRequest();
            BeanUtils.copyProperties(udiInfoExportRequest, filterUdiInfoRequest);
            filterUdiInfoRequest.setPage(null);
            List<List<String>> genDatas = genExcelData(filterUdiInfoRequest);
            if (genDatas != null && genDatas.size() > 0) {
                excelData.addAll(genDatas);
            }
        }

        String sheetName = "UDI产品信息对照";

        File file = new File(udiInfoExportLogEntity.getFilePath());
        if (!file.exists()) {
            try {
                file.createNewFile();
            } catch (IOException e) {
                e.printStackTrace();
            }
        }
        new ExcelUtil().exportExcel(excelData, sheetName, udiInfoExportLogEntity.getFilePath(), 20);
        udiInfoExportLogEntity.setStatus(BasicProcessStatus.UDIINFO_IMPORT_SUCCESS);
        udiInfoExportLogService.updateUdiInfoExportLog(udiInfoExportLogEntity);
    }


    @Async
    public void uploadProductsSmp(String genKey, UdiInfoExportRequest udiInfoExportRequest) {
        UdiInfoExportLogEntity udiInfoExportLogEntity = udiInfoExportLogService.selectByGenKey(genKey);
        PostUdiInfoRequest postUdiInfoRequest = new PostUdiInfoRequest();
        postUdiInfoRequest.setGenKey(genKey);
        //选中导出
        if (udiInfoExportRequest.getUuids() != null && udiInfoExportRequest.getUuids().size() > 0) {
            List<String> uuids = udiInfoExportRequest.getUuids();
            for (String uuid : uuids) {
                FilterUdiInfoRequest filterUdiInfoRequest = new FilterUdiInfoRequest();
                filterUdiInfoRequest.setUuid(uuid);
                List<UdiInfoImportDetailEntity> genDatas = genSmpData(filterUdiInfoRequest);
                postUdiInfoRequest.setDatas(genDatas);
            }

        } else {//一键导出
            FilterUdiInfoRequest filterUdiInfoRequest = new FilterUdiInfoRequest();
            BeanUtils.copyProperties(udiInfoExportRequest, filterUdiInfoRequest);
            List<UdiInfoImportDetailEntity> genDatas = genSmpData(filterUdiInfoRequest);
            postUdiInfoRequest.setDatas(genDatas);
        }
        //todo 上传SMP
        SystemParamConfigEntity systemParamConfigEntity = systemParamConfigService.selectByParamKey("smp_service_ip");
        if(systemParamConfigEntity!=null){
            String response = HttpClient.uCloudPost(systemParamConfigEntity.getParamValue()+"/udiwms/basic/products/upload", postUdiInfoRequest);
            BaseResponse baseResponse = JSONObject.parseObject(response, new TypeReference<BaseResponse>() {});
            if (baseResponse.getCode() == 20000) {
                udiInfoExportLogEntity.setStatus(BasicProcessStatus.UDIINFO_IMPORT_SUCCESS);
            } else {
                udiInfoExportLogEntity.setStatus(BasicProcessStatus.UDIINFO_IMPORT_FAIL);
            }
        }else {
            udiInfoExportLogEntity.setRemark("SMP服务IP地址未定义！");
            udiInfoExportLogEntity.setStatus(BasicProcessStatus.UDIINFO_IMPORT_FAIL);
        }
        udiInfoExportLogService.updateUdiInfoExportLog(udiInfoExportLogEntity);
    }

    public List<List<String>> genExcelData(BasicUnitMaintainFilterRequest filterUdiInfoRequest) {

        List<List<String>> excelData = new ArrayList<>();
        List<BasicUnitMaintainEntity> unitMaintainEntities = basicUnitMaintainService.filterList(filterUdiInfoRequest);
        for (BasicUnitMaintainEntity basicUnitMaintainEntity : unitMaintainEntities) {
            List<String> rows = new ArrayList<>();
            rows.add(basicUnitMaintainEntity.getErpId());
            rows.add(basicUnitMaintainEntity.getName());
            rows.add(basicUnitMaintainEntity.getSpell());
            rows.add(basicUnitMaintainEntity.getAddr());
            rows.add(basicUnitMaintainEntity.getContact());
            rows.add(basicUnitMaintainEntity.getMobile());
            rows.add(basicUnitMaintainEntity.getCreditNo());
            rows.add(basicUnitMaintainEntity.getStatus());
            rows.add(basicUnitMaintainEntity.getType());
            rows.add(basicUnitMaintainEntity.getThirdId());
            rows.add(basicUnitMaintainEntity.getThirdName());
            rows.add(basicUnitMaintainEntity.getThirdId1());
            rows.add(basicUnitMaintainEntity.getThirdName1());
            rows.add(basicUnitMaintainEntity.getThirdId2());
            rows.add(basicUnitMaintainEntity.getThirdName2());
            rows.add(basicUnitMaintainEntity.getThirdId3());
            rows.add(basicUnitMaintainEntity.getThirdName3());
            rows.add(basicUnitMaintainEntity.getThirdId4());
            rows.add(basicUnitMaintainEntity.getThirdName4());
            excelData.add(rows);
        }
        return excelData;
    }

    public List<List<String>> genExcelData(FilterUdiInfoRequest filterUdiInfoRequest) {

        List<List<String>> excelData = new ArrayList<>();
        List<UdiRelevanceResponse> udiRelevanceResponses = udiRelevanceService.filterUdiRelevance(filterUdiInfoRequest);
        for (UdiRelevanceResponse udiRelevanceResponse : udiRelevanceResponses) {
            List<String> rows = new ArrayList<>();
            rows.add(udiRelevanceResponse.getNameCode());
            rows.add(udiRelevanceResponse.getYbbm());
            rows.add(udiRelevanceResponse.getSptm());
            rows.add(udiRelevanceResponse.getThirdId());
            rows.add(udiRelevanceResponse.getThirdId1());
            rows.add(udiRelevanceResponse.getThirdId2());
            rows.add(udiRelevanceResponse.getThirdId3());
            rows.add(udiRelevanceResponse.getThirdId4());
            excelData.add(rows);
        }
        return excelData;
    }

    public List<UdiInfoImportDetailEntity> genSmpData(FilterUdiInfoRequest filterUdiInfoRequest) {
        List<UdiInfoImportDetailEntity> udiInfoImportDetailEntities = new ArrayList<>();
        List<UdiRelevanceResponse> udiRelevanceResponses = udiRelevanceService.filterUdiRelevance(filterUdiInfoRequest);
        for (UdiRelevanceResponse udiRelevanceResponse : udiRelevanceResponses) {
            UdiInfoImportDetailEntity udiInfoImportDetailEntity = new UdiInfoImportDetailEntity();
            udiInfoImportDetailEntity.setUdiCode(udiRelevanceResponse.getNameCode());
            udiInfoImportDetailEntity.setYbbm(udiRelevanceResponse.getYbbm());
            udiInfoImportDetailEntity.setSptm(udiRelevanceResponse.getSptm());
            udiInfoImportDetailEntity.setThirdId(udiRelevanceResponse.getThirdId());
            udiInfoImportDetailEntity.setThirdId1(udiRelevanceResponse.getThirdId1());
            udiInfoImportDetailEntity.setThirdId2(udiRelevanceResponse.getThirdId2());
            udiInfoImportDetailEntity.setThirdId3(udiRelevanceResponse.getThirdId3());
            udiInfoImportDetailEntity.setThirdId4(udiRelevanceResponse.getThirdId4());
            udiInfoImportDetailEntities.add(udiInfoImportDetailEntity);
        }
        return udiInfoImportDetailEntities;
    }
}
