package com.glxp.api.admin.thread;

import com.alibaba.fastjson.JSONObject;
import com.alibaba.fastjson.TypeReference;
import com.glxp.api.admin.constant.BasicProcessStatus;
import com.glxp.api.admin.constant.ConstantStatus;
import com.glxp.api.admin.entity.basic.*;
import com.glxp.api.admin.entity.info.SystemParamConfigEntity;
import com.glxp.api.admin.entity.thrsys.ThrProductsEntity;
import com.glxp.api.admin.httpclient.UdiDlHttpClient;
import com.glxp.api.admin.req.basic.FilterUdiIpLogRequest;
import com.glxp.api.admin.res.PageSimpleResponse;
import com.glxp.api.admin.res.basic.UdiRelevanceResponse;
import com.glxp.api.admin.service.basic.*;
import com.glxp.api.admin.service.info.SystemParamConfigService;
import com.glxp.api.admin.service.thrsys.ThrDataService;
import com.glxp.api.admin.util.CustomUtil;
import com.glxp.api.admin.util.DateUtil;
import com.glxp.api.admin.util.HttpClient;
import com.glxp.api.common.res.BaseResponse;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.scheduling.annotation.Async;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

@Service
public class BasicUdiInfoImportService {

    @Resource
    UdiInfoImportLogService udiInfoImportLogService;
    @Resource
    UdiInfoImportDetailService udiInfoImportService;
    @Resource
    UdiInfoService udiInfoService;
    @Resource
    UdiRelevanceService udiRelevanceService;
    @Resource
    ThrDataService thrDataService;
    @Value("${UDI_SERVER_URL}")
    private String udiUrl;
    @Resource
    SystemParamConfigService systemParamConfigService;
    @Resource
    BasicThirdSysService basicThirdSysService;

    //导入产品信息文件，生成产品信息，更新或插入对照表
    @Async
    public void importUdiInfo(String genKey) {

        UdiInfoImportLogEntity udiInfoImportLogEntity = udiInfoImportLogService.selectByGenKey(genKey);
        if (udiInfoImportLogEntity.getStatus() == BasicProcessStatus.UDIINFO_IMPORT_CODE_UNPROCESS) {
            FilterUdiIpLogRequest filterUdiIpLogRequest = new FilterUdiIpLogRequest();
            filterUdiIpLogRequest.setGenKey(genKey);
            List<UdiInfoImportDetailEntity> udiInfoImportEntities
                    = udiInfoImportService.filterUdiInfoImport(filterUdiIpLogRequest);
            if (udiInfoImportEntities != null) {
                for (UdiInfoImportDetailEntity udiInfoImportDetailEntity : udiInfoImportEntities) {
                    UdiInfoEntity udiInfoEntity = null;
                    //国家库获取取UDI信息
                    if (udiInfoImportDetailEntity.getUdiCode() != null && !udiInfoImportDetailEntity.getUdiCode().trim().isEmpty()) {
                        udiInfoEntity = udiInfoService.findByNameCode(udiInfoImportDetailEntity.getUdiCode());
                        List<UdiInfoEntity> froms = null;
                        if (udiInfoEntity == null) {
                            froms = new UdiDlHttpClient(udiUrl).getUdiByDi(udiInfoImportDetailEntity.getUdiCode());
                            if (froms != null && !froms.isEmpty()) {
                                udiInfoEntity = froms.get(0);

                            } else {
                                udiInfoImportDetailEntity.setStatus(BasicProcessStatus.UDIINFO_IMPORT_CODE_FAIL);
                                udiInfoImportDetailEntity.setRemark("未查询到UDI产品信息！");
                                udiInfoImportLogEntity.setStatus(BasicProcessStatus.UDIINFO_IMPORT_FAIL);
                                continue;
                            }
                        }
                        if (froms != null)
                            udiInfoService.insertUdiInfos(froms);
                    }
                    //判断更新
                    BasicThirdSysEntity basicThirdSysEntity = basicThirdSysService.selectMainThrSys();
                    String mainSys = basicThirdSysEntity.getThirdId();
                    String mainId = null;
                    if (mainSys.equals("thirdId")) {
                        mainId = udiInfoImportDetailEntity.getThirdId();
                    } else if (mainSys.equals("thirdId1")) {
                        mainId = udiInfoImportDetailEntity.getThirdId1();
                    } else if (mainSys.equals("thirdId2")) {
                        mainId = udiInfoImportDetailEntity.getThirdId2();
                    } else if (mainSys.equals("thirdId3")) {
                        mainId = udiInfoImportDetailEntity.getThirdId3();
                    } else if (mainSys.equals("thirdId4")) {
                        mainId = udiInfoImportDetailEntity.getThirdId4();
                    }
                    UdiRelevanceEntity udiRelevanceEntity = null;
                    if (udiInfoEntity == null) {
                        udiRelevanceEntity = udiRelevanceService.selectUpdateByThirdId(mainId);    //更新
                        if (mainId != null) {
                            ThrProductsEntity thrProductsEntity = thrDataService.getProducts(udiInfoImportDetailEntity.getThirdId(), "thirdId");
                            udiInfoEntity = new UdiInfoEntity();
                            udiInfoEntity.setNameCode(thrProductsEntity.getCode());
                            udiInfoEntity.setCpmctymc(thrProductsEntity.getName());
                            udiInfoEntity.setGgxh(thrProductsEntity.getSpec());
                            udiInfoEntity.setUuid(CustomUtil.getUUId());
                            udiInfoEntity.setProductType(ConstantStatus.PRODUCT_TYPE_THIRD);
                            udiInfoEntity.setDiType(ConstantStatus.DI_TYPE_MAIN);
                            udiInfoEntity.setManufactory(thrProductsEntity.getManufactory());
                            udiInfoEntity.setYlqxzcrbarmc(thrProductsEntity.getManufactory());
                            udiInfoEntity.setZczbhhzbapzbh(thrProductsEntity.getRegisterNo());
                            udiInfoService.insertUdiInfo(udiInfoEntity);

                        } else {
                            udiInfoImportDetailEntity.setStatus(BasicProcessStatus.UDIINFO_IMPORT_CODE_FAIL);
                            udiInfoImportDetailEntity.setRemark("格式错误！");
                            udiInfoImportLogEntity.setStatus(BasicProcessStatus.UDIINFO_IMPORT_FAIL);
                            continue;
                        }
                        if (udiRelevanceEntity == null) {
                            udiRelevanceEntity = new UdiRelevanceEntity();
                            udiRelevanceEntity.setUuid(udiInfoEntity.getUuid());
                            updateValue(udiRelevanceEntity, udiInfoImportDetailEntity, udiInfoImportLogEntity, mainId);
                            udiRelevanceService.insertUdiRelevance(udiRelevanceEntity);
                        } else {
                            udiInfoImportDetailEntity.setStatus(BasicProcessStatus.UDIINFO_IMPORT_CODE_FAIL);
                            udiInfoImportDetailEntity.setRemark("产品编码已存在，不能重复插入！");
                            udiInfoImportLogEntity.setStatus(BasicProcessStatus.UDIINFO_IMPORT_FAIL);
                        }
                    } else {
                        if (mainId == null) {//只存在UDI
                            UdiRelevanceEntity temp = udiRelevanceService.selectUpdateByUuid(udiInfoEntity.getUuid());
                            if (temp != null &&
                                    (temp.getThirdId() != null || temp.getThirdId1() != null || temp.getThirdId2() != null ||
                                            temp.getThirdId3() != null || temp.getThirdId4() != null)) {

                                udiRelevanceEntity.setUuid(udiInfoEntity.getUuid());
                                updateValue(udiRelevanceEntity, udiInfoImportDetailEntity, udiInfoImportLogEntity, mainId);
                                udiRelevanceService.insertUdiRelevance(udiRelevanceEntity);
                            } else {
                                udiInfoImportDetailEntity.setStatus(BasicProcessStatus.UDIINFO_IMPORT_CODE_FAIL);
                                udiInfoImportDetailEntity.setRemark("UDI已存在并且未关联不能重复插入！");
                                udiInfoImportLogEntity.setStatus(BasicProcessStatus.UDIINFO_IMPORT_FAIL);
                            }
                        } else {
                            UdiRelevanceEntity temp = udiRelevanceService.selectUpteaByAll(udiInfoEntity.getUuid(), mainId);
                            if (temp != null) {
                                updateValue(temp, udiInfoImportDetailEntity, udiInfoImportLogEntity, mainId);
                                udiRelevanceService.updateUdiRelevance(temp);
                            } else {
                                UdiRelevanceEntity mTemp = udiRelevanceService.selectUpdateByThirdId(mainId);
                                if(mTemp!=null){
                                    udiInfoImportDetailEntity.setStatus(BasicProcessStatus.UDIINFO_IMPORT_CODE_FAIL);
                                    udiInfoImportDetailEntity.setRemark("产品编码已存在关联不能重复插入！");
                                    udiInfoImportLogEntity.setStatus(BasicProcessStatus.UDIINFO_IMPORT_FAIL);
                                }else {
                                    UdiRelevanceEntity mTemp2 = udiRelevanceService.selectUpdateByUuid(udiInfoEntity.getUuid());
                                    if(mTemp2!=null &&  (mTemp2.getThirdId() != null || mTemp2.getThirdId1() != null || mTemp2.getThirdId2() != null ||
                                            mTemp2.getThirdId3() != null || mTemp2.getThirdId4() != null)){
                                        udiRelevanceEntity = new UdiRelevanceEntity();
                                        udiRelevanceEntity.setUuid(udiInfoEntity.getUuid());
                                        updateValue(udiRelevanceEntity,udiInfoImportDetailEntity,udiInfoImportLogEntity,mainId);
                                        udiRelevanceService.insertUdiRelevance(udiRelevanceEntity);
                                    }else {
                                        udiRelevanceEntity = new UdiRelevanceEntity();
                                        udiRelevanceEntity.setUuid(udiInfoEntity.getUuid());
                                        updateValue(udiRelevanceEntity,udiInfoImportDetailEntity,udiInfoImportLogEntity,mainId);
                                        udiRelevanceService.insertUdiRelevance(udiRelevanceEntity);
                                    }

                                }

                            }
                        }

                    }


                    //字段更新
                    udiInfoImportService.updateUdiInfoImport(udiInfoImportDetailEntity);
                }
            }
            udiInfoImportLogEntity.setStatus(BasicProcessStatus.UDIINFO_IMPORT_SUCCESS);
            udiInfoImportLogService.updateImportLog(udiInfoImportLogEntity);
        }

    }

    public void updateValue(UdiRelevanceEntity udiRelevanceEntity, UdiInfoImportDetailEntity udiInfoImportDetailEntity, UdiInfoImportLogEntity udiInfoImportLogEntity, String mainId) {
        udiRelevanceEntity.setUpdateTime(DateUtil.getDateTime());
        if (udiInfoImportDetailEntity.getYbbm() != null && !udiInfoImportDetailEntity.getYbbm().trim().equals("")) {
            udiRelevanceEntity.setYbbm(udiInfoImportDetailEntity.getYbbm());
        }
        if (udiInfoImportDetailEntity.getSptm() != null && !udiInfoImportDetailEntity.getSptm().trim().equals("")) {
            udiRelevanceEntity.setSptm(udiInfoImportDetailEntity.getSptm());
        }

        if (udiInfoImportDetailEntity.getThirdId() != null && !udiInfoImportDetailEntity.getThirdId().trim().equals("")) {
            ThrProductsEntity thrProductsEntity = thrDataService.getProducts(udiInfoImportDetailEntity.getThirdId(), "thirdId");
            if (thrProductsEntity != null) {
                if (mainId != null && udiInfoImportDetailEntity.getThirdId().equals(mainId)) {
                    udiRelevanceEntity.setManufactory(thrProductsEntity.getManufactory());
                    udiRelevanceEntity.setMeasname(thrProductsEntity.getMeasname());
                }
                udiRelevanceEntity.setThirdName(thrProductsEntity.getName());
                udiRelevanceEntity.setThirdId(udiInfoImportDetailEntity.getThirdId());
                udiInfoImportDetailEntity.setStatus(BasicProcessStatus.UDIINFO_IMPORT_CODE_SUCESS);
            } else {
                udiInfoImportDetailEntity.setStatus(BasicProcessStatus.UDIINFO_IMPORT_CODE_FAIL);
                udiInfoImportDetailEntity.setRemark("未查询到系统产品信息！");
                udiInfoImportLogEntity.setStatus(BasicProcessStatus.UDIINFO_IMPORT_FAIL);
            }

        }
        if (udiInfoImportDetailEntity.getThirdId1() != null && !udiInfoImportDetailEntity.getThirdId1().trim().equals("")) {
            ThrProductsEntity thrProductsEntity = thrDataService.getProducts(udiInfoImportDetailEntity.getThirdId1(), "thirdId1");
            if (thrProductsEntity != null) {
                if (mainId != null && udiInfoImportDetailEntity.getThirdId1().equals(mainId)) {
                    udiRelevanceEntity.setManufactory(thrProductsEntity.getManufactory());
                    udiRelevanceEntity.setMeasname(thrProductsEntity.getMeasname());
                }
                udiInfoImportDetailEntity.setStatus(BasicProcessStatus.UDIINFO_IMPORT_CODE_SUCESS);
                udiRelevanceEntity.setThirdName1(thrProductsEntity.getName());
                udiRelevanceEntity.setThirdId1(udiInfoImportDetailEntity.getThirdId1());
            } else {
                udiInfoImportDetailEntity.setStatus(BasicProcessStatus.UDIINFO_IMPORT_CODE_FAIL);
                udiInfoImportDetailEntity.setRemark("未查询到系统产品信息！");
                udiInfoImportLogEntity.setStatus(BasicProcessStatus.UDIINFO_IMPORT_FAIL);
            }

        }
        if (udiInfoImportDetailEntity.getThirdId2() != null && !udiInfoImportDetailEntity.getThirdId2().trim().equals("")) {
            ThrProductsEntity thrProductsEntity = thrDataService.getProducts(udiInfoImportDetailEntity.getThirdId2(), "thirdId2");
            if (thrProductsEntity != null) {
                if (mainId != null && udiInfoImportDetailEntity.getThirdId2().equals(mainId)) {
                    udiRelevanceEntity.setManufactory(thrProductsEntity.getManufactory());
                    udiRelevanceEntity.setMeasname(thrProductsEntity.getMeasname());
                }
                udiInfoImportDetailEntity.setStatus(BasicProcessStatus.UDIINFO_IMPORT_CODE_SUCESS);
                udiRelevanceEntity.setThirdId2(udiInfoImportDetailEntity.getThirdId2());
                udiRelevanceEntity.setThirdName2(thrProductsEntity.getName());
            } else {
                udiInfoImportDetailEntity.setStatus(BasicProcessStatus.UDIINFO_IMPORT_CODE_FAIL);
                udiRelevanceEntity.setThirdId(udiInfoImportDetailEntity.getThirdId());
                udiInfoImportDetailEntity.setRemark("未查询到系统产品信息！");
                udiInfoImportLogEntity.setStatus(BasicProcessStatus.UDIINFO_IMPORT_FAIL);
            }

        }
        if (udiInfoImportDetailEntity.getThirdId3() != null && !udiInfoImportDetailEntity.getThirdId3().trim().equals("")) {
            ThrProductsEntity thrProductsEntity = thrDataService.getProducts(udiInfoImportDetailEntity.getThirdId3(), "thirdId3");
            if (thrProductsEntity != null) {
                if (mainId != null && udiInfoImportDetailEntity.getThirdId3().equals(mainId)) {
                    udiRelevanceEntity.setManufactory(thrProductsEntity.getManufactory());
                    udiRelevanceEntity.setMeasname(thrProductsEntity.getMeasname());
                }
                udiInfoImportDetailEntity.setStatus(BasicProcessStatus.UDIINFO_IMPORT_CODE_SUCESS);
                udiRelevanceEntity.setThirdId3(udiInfoImportDetailEntity.getThirdId3());
                udiRelevanceEntity.setThirdName3(thrProductsEntity.getName());
            } else {
                udiInfoImportDetailEntity.setStatus(BasicProcessStatus.UDIINFO_IMPORT_CODE_FAIL);
                udiRelevanceEntity.setThirdId(udiInfoImportDetailEntity.getThirdId());
                udiInfoImportDetailEntity.setRemark("未查询到系统产品信息！");
                udiInfoImportLogEntity.setStatus(BasicProcessStatus.UDIINFO_IMPORT_FAIL);
            }

        }
        if (udiInfoImportDetailEntity.getThirdId4() != null && !udiInfoImportDetailEntity.getThirdId4().trim().equals("")) {
            ThrProductsEntity thrProductsEntity = thrDataService.getProducts(udiInfoImportDetailEntity.getThirdId4(), "thirdId1");
            if (thrProductsEntity != null) {
                if (mainId != null && udiInfoImportDetailEntity.getThirdId4().equals(mainId)) {
                    udiRelevanceEntity.setManufactory(thrProductsEntity.getManufactory());
                    udiRelevanceEntity.setMeasname(thrProductsEntity.getMeasname());
                }
                udiInfoImportDetailEntity.setStatus(BasicProcessStatus.UDIINFO_IMPORT_CODE_SUCESS);
                udiRelevanceEntity.setThirdName4(thrProductsEntity.getName());
                udiRelevanceEntity.setThirdId4(udiInfoImportDetailEntity.getThirdId4());
            } else {
                udiInfoImportDetailEntity.setStatus(BasicProcessStatus.UDIINFO_IMPORT_CODE_FAIL);
                udiInfoImportDetailEntity.setRemark("未查询到系统产品信息！");
                udiInfoImportLogEntity.setStatus(BasicProcessStatus.UDIINFO_IMPORT_FAIL);
            }

        }
    }

    //下载供应商产品信息，
    @Async
    public void downloadSmpUdiInfo(String genKey) {

        int page = 1;
        int limit = 100;
        String lastUpdateTime = DateUtil.formatDateTime(new Date());

        while (true) {
            List<UdiRelevanceResponse> udiRelevanceResponses = getUdiInfo(page, limit, lastUpdateTime);
            if (udiRelevanceResponses != null && udiRelevanceResponses.size() > 0) {

                for (UdiRelevanceResponse udiRelevanceResponse : udiRelevanceResponses) {
                    UdiInfoImportDetailEntity udiInfoImportEntity = new UdiInfoImportDetailEntity();
                    udiInfoImportEntity.setUdiCode(udiRelevanceResponse.getNameCode());
                    BeanUtils.copyProperties(udiRelevanceResponse, udiInfoImportEntity);
                    udiInfoImportEntity.setThirdId(udiRelevanceResponse.getErpId());
                    udiInfoImportEntity.setUpdateTime(new Date());
                    udiInfoImportEntity.setGenKeyFk(genKey);
                    udiInfoImportService.insertUdiInfoImport(udiInfoImportEntity);
                }
                if (udiRelevanceResponses.size() >= 100) {
                    page++;
                } else {
                    break;
                }

            } else {
                break;
            }
        }
        importUdiInfo(genKey);
    }

    public List<UdiRelevanceResponse> getUdiInfo(int page, int limit, String lastUpdateTime) {
        String smpUrl = "";
        SystemParamConfigEntity systemParamConfigEntity = systemParamConfigService.selectByParamKey("smp_service_ip");
        if (systemParamConfigEntity != null) {
            smpUrl = systemParamConfigEntity.getParamValue();
        }

        Map<String, Object> paramMap = new HashMap<>(16);
        paramMap.put("page", 1);
        paramMap.put("limit", 100);
        paramMap.put("lastUpdateTime", lastUpdateTime);
        String response = HttpClient.mipsGet(smpUrl + "/udiwms/basic/products/download", paramMap);
        try {
            BaseResponse<PageSimpleResponse<UdiRelevanceResponse>> udiDlDeviceResponse =
                    JSONObject.parseObject(response, new TypeReference<BaseResponse<PageSimpleResponse<UdiRelevanceResponse>>>() {

                    });
            if (udiDlDeviceResponse != null && udiDlDeviceResponse.getCode() == 20000) {
                List<UdiRelevanceResponse> udiRelevanceResponses = udiDlDeviceResponse.getData().getList();
                return udiRelevanceResponses;
            }

        } catch (Exception e) {
            e.printStackTrace();
            return null;
        }
        return null;
    }

}
