package com.glxp.api.admin.thread;

import com.glxp.api.admin.dao.info.ScheduledDao;
import com.glxp.api.admin.entity.basic.BasicThirdSysDetailEntity;
import com.glxp.api.admin.entity.basic.BasicThirdSysEntity;
import com.glxp.api.admin.entity.info.ScheduledEntity;
import com.glxp.api.admin.entity.info.SystemParamConfigEntity;
import com.glxp.api.admin.req.info.ScheduledRequest;
import com.glxp.api.admin.service.basic.BasicThirdSysDetailService;
import com.glxp.api.admin.service.basic.BasicThirdSysService;
import com.glxp.api.admin.service.info.SystemParamConfigService;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.scheduling.annotation.EnableScheduling;
import org.springframework.scheduling.annotation.SchedulingConfigurer;
import org.springframework.scheduling.config.ScheduledTaskRegistrar;
import org.springframework.scheduling.support.CronTrigger;
import org.springframework.stereotype.Component;

import javax.annotation.Resource;

@Component
@EnableScheduling
public class BasicUpdateErpDataTask implements SchedulingConfigurer {

    final Logger logger = LoggerFactory.getLogger(IoModifyErpTask.class);
    String cron;
    @Resource
    private BasicBackUpdateProductService updateproductService;
    @Resource
    protected ScheduledDao scheduledDao;
    @Resource
    SystemParamConfigService systemParamConfigService;
    @Resource
    BasicThirdSysDetailService basicThirdSysDetailService;
    @Resource
    BasicThirdSysService basicThirdSysService;

    @Override
    public void configureTasks(ScheduledTaskRegistrar scheduledTaskRegistrar) {
        scheduledTaskRegistrar.addTriggerTask(() -> process(),
                triggerContext -> {
                    ScheduledRequest scheduledRequest = new ScheduledRequest();
                    scheduledRequest.setCronName("updateErpData");
                    ScheduledEntity scheduledEntity = scheduledDao.findScheduled(scheduledRequest);
                    cron = scheduledEntity.getCron();
                    if (cron.isEmpty()) {
                        logger.error("cron is null");
                    }
                    return new CronTrigger(cron).nextExecutionTime(triggerContext);
                });
    }


    private void process() {
        logger.info("定时更新下载第三方产品信息");
        SystemParamConfigEntity isUpdate = systemParamConfigService.selectByParamKey("erp_updateErpData");
        if (isUpdate != null && "1".equals(isUpdate.getParamValue())) {

            BasicThirdSysEntity basicThirdSysEntity = basicThirdSysService.selectMainThrSys();
//            SystemParamConfigEntity thirdSys = systemParamConfigService.selectByParamKey("erp_updateErpData_thirdSys");
            String thirdSysId = null;
            if (basicThirdSysEntity != null && basicThirdSysEntity.getThirdId() != null) {
                thirdSysId = basicThirdSysEntity.getThirdId();
            } else
                thirdSysId = "thirdId";
            BasicThirdSysDetailEntity piDetailEntity = basicThirdSysDetailService.selectByKey("piQueryUrl", thirdSysId);

            BasicThirdSysDetailEntity invPiDetailEntity = basicThirdSysDetailService.selectByKey("invPiUrl", thirdSysId);
            if (piDetailEntity.getFromType() == 0) {
                if (piDetailEntity.getValue() != null && invPiDetailEntity.getValue() != null) {
                    updateproductService.addAllProducts(null,piDetailEntity.getValue(), invPiDetailEntity.getValue(), thirdSysId, true);
                } else {
                    logger.info("接口地址未设置");
                }
            } else {
                updateproductService.loadLocal(null,thirdSysId, invPiDetailEntity.getValue(), true);
            }


        } else {
            logger.info("暂未设置，不更新");
        }

    }

    public void setCron(String cron) {
        this.cron = cron;
    }

}
