package com.glxp.api.admin.thread;

import com.glxp.api.admin.config.WebSocketServer;
import com.glxp.api.admin.constant.ConstantType;
import com.glxp.api.admin.constant.IncodeStatus;
import com.glxp.api.admin.entity.basic.BasicThirdSysDetailEntity;
import com.glxp.api.admin.entity.basic.BasicThirdSysEntity;
import com.glxp.api.admin.entity.basic.UdiInfoEntity;
import com.glxp.api.admin.entity.basic.UdiRelevanceEntity;
import com.glxp.api.admin.entity.info.SystemParamConfigEntity;
import com.glxp.api.admin.entity.inout.ErpOrderEntity;
import com.glxp.api.admin.entity.inout.FilterErpOrderRequest;
import com.glxp.api.admin.entity.inout.OrderEntity;
import com.glxp.api.admin.entity.inventory.ErpStockEntity;
import com.glxp.api.admin.entity.inventory.InCodeLogEntity;
import com.glxp.api.admin.entity.inventory.StockPrintEntity;
import com.glxp.api.admin.entity.thrsys.ThrInvProductsEntity;
import com.glxp.api.admin.httpclient.ErpInvClient;
import com.glxp.api.admin.req.basic.FilterStockprintRequest;
import com.glxp.api.admin.req.basic.FilterUdiInfoRequest;
import com.glxp.api.admin.req.inout.FilterStockCheckRequest;
import com.glxp.api.admin.req.inout.OrderFilterRequest;
import com.glxp.api.admin.req.inventory.ErpOnhandRequest;
import com.glxp.api.admin.req.thrsys.FilterThrInvProductsRequest;
import com.glxp.api.admin.res.basic.UdiRelevanceResponse;
import com.glxp.api.admin.res.inventory.ErpInvProductResponse;
import com.glxp.api.admin.res.inventory.ErpOnHandResponse;
import com.glxp.api.admin.service.basic.*;
import com.glxp.api.admin.service.info.SystemParamConfigService;
import com.glxp.api.admin.service.inout.OrderDetailService;
import com.glxp.api.admin.service.inout.OrderService;
import com.glxp.api.admin.service.inventory.InCodeLogService;
import com.glxp.api.admin.service.inventory.InvStockPrintService;
import com.glxp.api.admin.service.thrsys.ThrInvProductsService;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.BeanUtils;
import org.springframework.scheduling.annotation.Async;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

@Service
public class InvImportStockService {
    @Resource
    private OrderService orderService;
    @Resource
    InvStockPrintService stockPrintService;
    @Resource
    UdiRelevanceService udiRelevanceService;
    @Resource
    private OrderDetailService myErpOrderService;
    @Resource
    private UdiInfoService productInfoService;
    @Resource
    private SystemParamConfigService systemParamConfigService;
    @Resource
    private BasicThirdSysService basicThirdSysService;
    @Resource
    InCodeLogService inCodeLogService;
    @Resource
    ThrInvProductsService thrInvProductsService;
    @Resource
    BasicThirdSysDetailService basicThirdSysDetailService;
    final Logger logger = LoggerFactory.getLogger(InvImportStockService.class);

    //产品信息生成全部
    @Async
    public void updateStock(String genKey, String thirsysFk) {

        int page = 1;
        int limit = 1000;
        InCodeLogEntity inCodeLogEntity = inCodeLogService.selectByGenKey(genKey);
        FilterUdiInfoRequest filterUdiInfoRequest = new FilterUdiInfoRequest();
        BasicThirdSysDetailEntity invPiDetailEntity = basicThirdSysDetailService.selectByKey("invPiUrl", thirsysFk);
        String url;
        if (invPiDetailEntity != null) {
            url = invPiDetailEntity.getValue();
            while (true) {
                filterUdiInfoRequest.setPage(page);
                filterUdiInfoRequest.setLimit(limit);
                List<UdiRelevanceResponse> udiRelevanceResponses = udiRelevanceService.filterUdiRelevance(filterUdiInfoRequest);
                if (udiRelevanceResponses != null && udiRelevanceResponses.size() > 0) {
                    List<StockPrintEntity> stockPrintEntities = new ArrayList<>();
                    for (UdiRelevanceResponse udiRelevanceResponse : udiRelevanceResponses) {
                        if ("thirdId".equals(thirsysFk)) {
                            generatorPrints(udiRelevanceResponse.getThirdId(), "thirdId", url, stockPrintEntities, udiRelevanceResponse, genKey);
                        } else if ("thirdId1".equals(thirsysFk)) {
                            generatorPrints(udiRelevanceResponse.getThirdId1(), "thirdId1", url, stockPrintEntities, udiRelevanceResponse, genKey);
                        } else if ("thirdId2".equals(thirsysFk)) {
                            generatorPrints(udiRelevanceResponse.getThirdId2(), "thirdId2", url, stockPrintEntities, udiRelevanceResponse, genKey);
                        } else if ("thirdId3".equals(thirsysFk)) {
                            generatorPrints(udiRelevanceResponse.getThirdId3(), "thirdId3", url, stockPrintEntities, udiRelevanceResponse, genKey);
                        } else if ("thirdId4".equals(thirsysFk)) {
                            generatorPrints(udiRelevanceResponse.getThirdId4(), "thirdId4", url, stockPrintEntities, udiRelevanceResponse, genKey);
                        }
                    }
                    stockPrintService.insertStockPrints(stockPrintEntities);
                    if (udiRelevanceResponses.size() < limit) {
                        break;
                    } else {
                        page++;
                    }
                } else {
                    break;
                }
            }
            WebSocketServer.sendInfo("后台已更新完成，请刷新查看", "sid");
            inCodeLogEntity.setStatus(IncodeStatus.FINISH);
            inCodeLogService.updateInCodeLog(inCodeLogEntity);
        } else {
            WebSocketServer.sendInfo("自动更新失败，ERP库存产品信息查询接口地址未定义", "err");
            logger.error("ERP库存产品信息查询接口地址未定义！");
            inCodeLogEntity.setStatus(IncodeStatus.ERROR);
            inCodeLogEntity.setRemark("ERP库存产品信息查询接口地址未定义！");
            inCodeLogService.updateInCodeLog(inCodeLogEntity);
        }
    }

    //根据产品ID生成
    @Async
    public void updateStockById(String genKey, List<String> ids, String thirsysFk) {
        BasicThirdSysDetailEntity invPiDetailEntity = basicThirdSysDetailService.selectByKey("invPiUrl", thirsysFk);
        String url;
        if (invPiDetailEntity != null) {
            url = invPiDetailEntity.getValue();
            if (ids != null && ids.size() > 0) {
                for (String id : ids) {
                    List<StockPrintEntity> stockPrintEntities = new ArrayList<>();
                    UdiRelevanceResponse udiRelevanceResponse = udiRelevanceService.selectGroupById(id);
                    if ("thirdId".equals(thirsysFk)) {
                        generatorPrints(udiRelevanceResponse.getThirdId(), "thirdId", url, stockPrintEntities, udiRelevanceResponse, genKey);
                    } else if ("thirdId1".equals(thirsysFk)) {
                        generatorPrints(udiRelevanceResponse.getThirdId1(), "thirdId1", url, stockPrintEntities, udiRelevanceResponse, genKey);
                    } else if ("thirdId2".equals(thirsysFk)) {
                        generatorPrints(udiRelevanceResponse.getThirdId2(), "thirdId2", url, stockPrintEntities, udiRelevanceResponse, genKey);
                    } else if ("thirdId3".equals(thirsysFk)) {
                        generatorPrints(udiRelevanceResponse.getThirdId3(), "thirdId3", url, stockPrintEntities, udiRelevanceResponse, genKey);
                    } else if ("thirdId4".equals(thirsysFk)) {
                        generatorPrints(udiRelevanceResponse.getThirdId4(), "thirdId4", url, stockPrintEntities, udiRelevanceResponse, genKey);
                    }
                    stockPrintService.insertStockPrints(stockPrintEntities);
                }

            }
        }
        InCodeLogEntity inCodeLogEntity = inCodeLogService.selectByGenKey(genKey);
        inCodeLogEntity.setStatus(IncodeStatus.FINISH);
        inCodeLogService.updateInCodeLog(inCodeLogEntity);
        WebSocketServer.sendInfo("生成内部码已完成，请刷新查看", "sid");
    }

    //根据第三方产品ID生成
    @Async
    public void updateByCode(String code, String url,String thirdSysFk) {
        UdiRelevanceEntity udiRelevanceEntity = udiRelevanceService.selectByThirdId(code,thirdSysFk);
        UdiRelevanceResponse udiRelevanceResponse = new UdiRelevanceResponse();
        BeanUtils.copyProperties(udiRelevanceEntity, udiRelevanceResponse);
        List<StockPrintEntity> stockPrintEntities = new ArrayList<>();
        ErpOnhandRequest erpOnhandRequest = new ErpOnhandRequest();
        erpOnhandRequest.setInventoryCode(udiRelevanceResponse.getThirdId());
        List<ErpInvProductResponse> uOnhandQueryResponses = new ErpInvClient().getInvPrdoduct(erpOnhandRequest, url);
        if (uOnhandQueryResponses != null && uOnhandQueryResponses.size() > 0) {

            for (ErpInvProductResponse onhandQueryResponse : uOnhandQueryResponses) {
                StockPrintEntity stockPrintEntity = new StockPrintEntity();
                stockPrintEntity.setBatchNo(onhandQueryResponse.getBatchNo());
                stockPrintEntity.setNnum(onhandQueryResponse.getCount() + "");
                stockPrintEntity.setWarehouseName(onhandQueryResponse.getWarehouseName());
                stockPrintEntity.setWarehouseCode(onhandQueryResponse.getWarehouseCode());
                stockPrintEntity.setExpireDate(onhandQueryResponse.getExpirationDate());
                stockPrintEntity.setProduceDate(onhandQueryResponse.getManufacturingDate());
                setCommonparams(stockPrintEntity, udiRelevanceResponse, stockPrintEntities);
            }

        } else {
            StockPrintEntity stockPrintEntity = new StockPrintEntity();
            setCommonparams(stockPrintEntity, udiRelevanceResponse, stockPrintEntities);
        }

    }

    //根据库存产品信息生成
    @Async
    public void updateByInvInfo(String genKey, List<String> ids, String thirsysFk) {
        if (ids != null && ids.size() > 0) {
            for (String id : ids) {

                FilterThrInvProductsRequest filterThrInvProductsRequest = new FilterThrInvProductsRequest();
                filterThrInvProductsRequest.setThirdSysFk(thirsysFk);
                filterThrInvProductsRequest.setInventoryCode(id);
                List<ThrInvProductsEntity> thrInvProductsEntities = thrInvProductsService.filterThrInvProductss(filterThrInvProductsRequest);
                if(thrInvProductsEntities!=null){
                    UdiRelevanceEntity udiRelevanceEntity = udiRelevanceService.selectByThirdId(id,thirsysFk);
                    for(ThrInvProductsEntity thrInvProductsEntity:thrInvProductsEntities){
                        StockPrintEntity stockPrintEntity = new StockPrintEntity();
                        stockPrintEntity.setNameCode(thrInvProductsEntity.getInventoryCode());
                        stockPrintEntity.setBatchNo(thrInvProductsEntity.getBatchNo());
                        stockPrintEntity.setNnum(thrInvProductsEntity.getCount() + "");
                        stockPrintEntity.setWarehouseName(thrInvProductsEntity.getWarehouseName());
                        stockPrintEntity.setWarehouseCode(thrInvProductsEntity.getWarehouseCode());
                        stockPrintEntity.setSpaceCode(thrInvProductsEntity.getSpaceCode());
                        stockPrintEntity.setSpaceName(thrInvProductsEntity.getSpaceName());
                        stockPrintEntity.setExpireDate(thrInvProductsEntity.getExpirationDate());
                        stockPrintEntity.setProduceDate(thrInvProductsEntity.getManufacturingDate());
                        stockPrintEntity.setThirdSysFk(thirsysFk);
                        stockPrintEntity.setGenKeyFk(genKey);
                        stockPrintEntity.setUdiCode(generateUdiCode(stockPrintEntity));
                        stockPrintService.insertStockPrint(stockPrintEntity);
                    }
                }
            }
            InCodeLogEntity inCodeLogEntity = inCodeLogService.selectByGenKey(genKey);
            inCodeLogEntity.setStatus(IncodeStatus.FINISH);
            inCodeLogService.updateInCodeLog(inCodeLogEntity);
            WebSocketServer.sendInfo("生成内部码已完成，请刷新查看", "sid");
        }

    }

    //根据库存产品信息全部生成
    @Async
    public void updateAllByInvInfo(String genKey, String thirsysFk) {
        int page = 1;
        int limit = 1000;
        while (true) {
            FilterThrInvProductsRequest filterThrInvProductsRequest = new FilterThrInvProductsRequest();
            filterThrInvProductsRequest.setPage(page);
            filterThrInvProductsRequest.setLimit(limit);
            List<ThrInvProductsEntity> thrInvProductsEntities = thrInvProductsService.filterThrInvProductss(filterThrInvProductsRequest);
            if (thrInvProductsEntities != null && thrInvProductsEntities.size() > 0) {
                for (ThrInvProductsEntity thrInvProductsEntity : thrInvProductsEntities) {
                    StockPrintEntity stockPrintEntity = new StockPrintEntity();
                    stockPrintEntity.setBatchNo(thrInvProductsEntity.getBatchNo());
                    stockPrintEntity.setNameCode(thrInvProductsEntity.getInventoryCode());
                    stockPrintEntity.setNnum(thrInvProductsEntity.getCount() + "");
                    stockPrintEntity.setWarehouseName(thrInvProductsEntity.getWarehouseName());
                    stockPrintEntity.setWarehouseCode(thrInvProductsEntity.getWarehouseCode());
                    stockPrintEntity.setSpaceCode(thrInvProductsEntity.getSpaceCode());
                    stockPrintEntity.setSpaceName(thrInvProductsEntity.getSpaceName());
                    stockPrintEntity.setExpireDate(thrInvProductsEntity.getExpirationDate());
                    stockPrintEntity.setProduceDate(thrInvProductsEntity.getManufacturingDate());
                    stockPrintEntity.setThirdSysFk(thirsysFk);
                    stockPrintEntity.setGenKeyFk(genKey);
                    stockPrintEntity.setUdiCode(generateUdiCode(stockPrintEntity));
                    stockPrintService.insertStockPrint(stockPrintEntity);
                }
            }
            if (thrInvProductsEntities.size() < limit) {
                break;
            } else {
                page++;
            }
        }

        InCodeLogEntity inCodeLogEntity = inCodeLogService.selectByGenKey(genKey);
        inCodeLogEntity.setStatus(IncodeStatus.FINISH);
        inCodeLogService.updateInCodeLog(inCodeLogEntity);
        WebSocketServer.sendInfo("生成内部码已完成，请刷新查看", "sid");
    }

    public void generatorPrints(String inventoryCode, String thirdSysFk, String url, List<StockPrintEntity> stockPrintEntities,
                                UdiRelevanceResponse udiRelevanceResponse, String genKey) {
        ErpOnhandRequest erpOnhandRequest = new ErpOnhandRequest();
        erpOnhandRequest.setInventoryCode(inventoryCode);
        List<ErpInvProductResponse> uOnhandQueryResponses = new ErpInvClient().getInvPrdoduct(erpOnhandRequest, url);
        if (uOnhandQueryResponses != null && uOnhandQueryResponses.size() > 0) {
            for (ErpInvProductResponse onhandQueryResponse : uOnhandQueryResponses) {
                StockPrintEntity stockPrintEntity = new StockPrintEntity();
                stockPrintEntity.setBatchNo(onhandQueryResponse.getBatchNo());
                stockPrintEntity.setNnum(onhandQueryResponse.getCount() + "");
                stockPrintEntity.setWarehouseName(onhandQueryResponse.getWarehouseName());
                stockPrintEntity.setWarehouseCode(onhandQueryResponse.getWarehouseCode());
                stockPrintEntity.setExpireDate(onhandQueryResponse.getExpirationDate());
                stockPrintEntity.setProduceDate(onhandQueryResponse.getManufacturingDate());
                stockPrintEntity.setThirdSysFk(thirdSysFk);
                stockPrintEntity.setGenKeyFk(genKey);
                setCommonparams(stockPrintEntity, udiRelevanceResponse, stockPrintEntities);
            }
        } else {
            StockPrintEntity stockPrintEntity = new StockPrintEntity();
            stockPrintEntity.setThirdSysFk(thirdSysFk);
            stockPrintEntity.setGenKeyFk(genKey);
            setCommonparams(stockPrintEntity, udiRelevanceResponse, stockPrintEntities);
        }

    }

    public void setCommonparams(StockPrintEntity stockPrintEntity, UdiRelevanceResponse udiRelevanceResponse, List<StockPrintEntity> stockPrintEntities) {


        stockPrintEntity.setCpmctymc(udiRelevanceResponse.getCpmctymc());
        stockPrintEntity.setThirdName(udiRelevanceResponse.getThirdName());
        stockPrintEntity.setGgxh(udiRelevanceResponse.getGgxh());
        stockPrintEntity.setNameCode(udiRelevanceResponse.getNameCode());
        stockPrintEntity.setThirdId(udiRelevanceResponse.getThirdId());
        stockPrintEntity.setUdiRlIdFk(udiRelevanceResponse.getId() + "");
        stockPrintEntity.setYlqxzcrbarmc(udiRelevanceResponse.getYlqxzcrbarmc());
        stockPrintEntity.setZczbhhzbapzbh(udiRelevanceResponse.getZczbhhzbapzbh());
        stockPrintEntity.setCodeType("3");
//                        if (udiRelevanceResponse.getTyshxydm() != null && !udiRelevanceResponse.getTyshxydm().equals("")) {
//
//                            if (udiRelevanceResponse.getNameCode().contains("MA")) {
//                                stockPrintEntity.setCodeType("2");
//
//                                udiCode = udiRelevanceResponse.getNameCode();
//                                if (udiRelevanceResponse.getBatchNo() != null && !udiRelevanceResponse.getBatchNo().equals("null"))
//                                    udiCode = udiCode + ".L" + udiRelevanceResponse.getBatchNo();
//                            } else {
//                                stockPrintEntity.setCodeType("1");
//                                udiCode = "01" + udiRelevanceResponse.getNameCode();
//
//                                if (udiRelevanceResponse.getBatchNo() != null && !udiRelevanceResponse.getBatchNo().equals("null"))
//                                    udiCode = udiCode + "10" + udiRelevanceResponse.getBatchNo();
//                            }
//                        } else {
//
//                        }
        stockPrintEntity.setUdiCode(generateUdiCode(stockPrintEntity));
        FilterStockprintRequest filterStockprintRequest = new FilterStockprintRequest();
        filterStockprintRequest.setUdiRlIdFk(stockPrintEntity.getUdiRlIdFk() + "");
        filterStockprintRequest.setBatchNo(stockPrintEntity.getBatchNo());
        filterStockprintRequest.setWarehouseCode(stockPrintEntity.getWarehouseCode());
        filterStockprintRequest.setSpaceCode(stockPrintEntity.getSpaceCode());
        filterStockprintRequest.setGenKeyFk(stockPrintEntity.getGenKeyFk());
        StockPrintEntity temp = stockPrintService.findByRlId(filterStockprintRequest);
        if (temp != null) {

            //如果生产日期与失效日期都有则更新
            if (stockPrintEntity.getProduceDate() != null && !stockPrintEntity.getProduceDate().equals("") && stockPrintEntity.getExpireDate() != null && !stockPrintEntity.getExpireDate().equals("")) {
                temp.setProduceDate(stockPrintEntity.getProduceDate());
                temp.setExpireDate(stockPrintEntity.getExpireDate());
            }
            if (temp.getProduceDate() == null) {
                temp.setProduceDate(stockPrintEntity.getProduceDate());
            }
            if (temp.getExpireDate() == null) {
                temp.setExpireDate(stockPrintEntity.getExpireDate());
            }
            temp.setUdiCode(generateUdiCode(temp));
            stockPrintEntity = temp;
        }
        if (udiRelevanceResponse.getTyshxydm() != null && !udiRelevanceResponse.getTyshxydm().equals("")) {
            if (udiRelevanceResponse.getDiType() == 1) {
                stockPrintEntity.setNameCode(udiRelevanceResponse.getNameCode());
                stockPrintEntities.add(stockPrintEntity);
            }
        } else
            stockPrintEntities.add(stockPrintEntity);
    }

    public String generateUdiCode(StockPrintEntity stockPrintEntity) {
        String udiCode = "";
        if (stockPrintEntity.getProduceDate() == null) {
            stockPrintEntity.setProduceDate("");
        }
        if (stockPrintEntity.getExpireDate() == null) {
            stockPrintEntity.setExpireDate("");
        }

        if (stockPrintEntity.getBatchNo() == null) {
            stockPrintEntity.setBatchNo("");
        }
        if (stockPrintEntity.getWarehouseCode() == null) {
            stockPrintEntity.setWarehouseCode("");
        }
        //货位码类型
        //#di#生产日期#失效日期#批次号#序列号#仓位号#货位号
        udiCode = "#" + stockPrintEntity.getNameCode() + "#" + stockPrintEntity.getProduceDate() + "#" + stockPrintEntity.getExpireDate() +
                "#" + stockPrintEntity.getBatchNo() + "#" + "#" + stockPrintEntity.getWarehouseCode() + "#" + stockPrintEntity.getSpaceCode();
        return udiCode;
    }

    //盘点校验
    @Async
    public void checkStock(FilterStockCheckRequest filterStockCheckRequest) {
        SystemParamConfigEntity systemParamConfigEntity = systemParamConfigService.selectByParamKey("erp_inv_product_query");
        String url;
        if (systemParamConfigEntity != null) {
            url = systemParamConfigEntity.getParamValue();

            List<ErpOrderEntity> erpOrderEntitie = getErpOrders(filterStockCheckRequest.getId() + "");
            if (erpOrderEntitie != null && erpOrderEntitie.size() > 0) {
                for (ErpOrderEntity erpOrderEntity : erpOrderEntitie) {
                    //获取UUID
                    FilterUdiInfoRequest productFilterRequest = new FilterUdiInfoRequest();
                    productFilterRequest.setNameCode(erpOrderEntity.getNameCode());
                    List<UdiInfoEntity> productInfoEntities = productInfoService.filterAllUdiInfo(productFilterRequest);
                    Map<String, List<UdiInfoEntity>> groupBy = productInfoEntities.stream().collect(Collectors.groupingBy(UdiInfoEntity::getUuid));
                    List<String> uuids = new ArrayList<>();
                    for (String uuid : groupBy.keySet()) {
                        uuids.add(uuid);
                    }
                    if (uuids.size() == 1) {
                        List<UdiRelevanceEntity> udiRelevanceEntities = udiRelevanceService.selectByUuid(uuids.get(0));
                        if (udiRelevanceEntities != null && udiRelevanceEntities.size() == 1) {
                            ErpOnhandRequest erpOnhandRequest = new ErpOnhandRequest();
                            erpOnhandRequest.setInventoryCode(udiRelevanceEntities.get(0).getThirdId());
                            List<ErpInvProductResponse> erpOnHandResponses = new ErpInvClient().getInvPrdoduct(erpOnhandRequest, url);
                            if (erpOnHandResponses != null && erpOnHandResponses.size() > 0) {
                                for (int i = 0; i < erpOnHandResponses.size(); i++) {
                                    if (erpOnHandResponses.get(i).getBatchNo().equals(erpOrderEntity.getBatchNo())
                                    ) {
                                        if (erpOrderEntity.getWarehouseCode() != null && erpOnHandResponses.get(i).getWarehouseCode().equals(erpOrderEntity.getWarehouseCode()))
                                            erpOrderEntity.setReCount(erpOnHandResponses.get(i).getCount());
                                        else {
                                            erpOrderEntity.setReCount(erpOnHandResponses.get(i).getCount());
                                        }
                                        myErpOrderService.updateErpOrder(erpOrderEntity);
                                        break;
                                    }
                                }

                            }

                        }
                    }

                }
            }
        } else {
            logger.error("ERP库存产品信息查询接口地址未定义！");
        }
    }


    private List<ErpOrderEntity> getErpOrders(String id) {
        List<String> orderIds = getOrderIds(id + "");
        List<ErpOrderEntity> erpOrderEntities = new ArrayList<>();
        if (orderIds.size() > 0) {
            FilterErpOrderRequest filterErpOrderRequest = new FilterErpOrderRequest();
            filterErpOrderRequest.setPage(1);
            filterErpOrderRequest.setLimit(10000);
            filterErpOrderRequest.setOrderIds(orderIds);
            erpOrderEntities = myErpOrderService.findByStockCheck(filterErpOrderRequest);
        }
        return erpOrderEntities;
    }

    public List<String> getOrderIds(String checkId) {
        OrderFilterRequest orderFilterRequest = new OrderFilterRequest();
        orderFilterRequest.setAction(ConstantType.TYPE_STOCK_CHECK);
        orderFilterRequest.setStockCheckFk(checkId + "");
        List<OrderEntity> orderEntities = orderService.findAllOrders(orderFilterRequest);
        List<String> orderIds = new ArrayList<>();
        if (orderEntities != null) {
            for (OrderEntity orderEntity : orderEntities) {
                orderIds.add(orderEntity.getId());
            }
        }
        return orderIds;
    }

    private boolean isRepeat(List<ErpOrderEntity> erpOrderEntities) {
        List<String> stringList = erpOrderEntities.stream().map(ErpOrderEntity::getNameCode)
                .collect(Collectors.toList());
        long count = stringList.stream().distinct().count();
        if (stringList.size() == count) {
            return false;
        } else {
            return true;
        }
    }
}
