package com.glxp.api.admin.thread;

import com.glxp.api.admin.constant.ConstantStatus;
import com.glxp.api.admin.constant.ConstantType;
import com.glxp.api.admin.entity.basic.BasicThirdSysDetailEntity;
import com.glxp.api.admin.entity.info.SystemParamConfigEntity;
import com.glxp.api.admin.httpclient.ErpOrderClient;
import com.glxp.api.admin.dao.info.ScheduledDao;
import com.glxp.api.admin.entity.info.ScheduledEntity;
import com.glxp.api.admin.entity.inout.OrderEntity;
import com.glxp.api.admin.entity.inout.ErpOrderEntity;
import com.glxp.api.admin.entity.inout.FilterErpOrderRequest;
import com.glxp.api.admin.req.inout.ERPPostBillRequest;
import com.glxp.api.admin.req.inout.OrderFilterRequest;
import com.glxp.api.admin.req.inout.USignRequest;
import com.glxp.api.admin.req.info.ScheduledRequest;
import com.glxp.api.admin.res.basic.BussinessTypResponse;
import com.glxp.api.admin.service.basic.BussinessTypeService;
import com.glxp.api.admin.service.info.SystemParamConfigService;
import com.glxp.api.admin.service.inout.OrderDetailService;
import com.glxp.api.admin.service.inout.OrderService;
import com.glxp.api.common.res.BaseResponse;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.scheduling.annotation.EnableScheduling;
import org.springframework.scheduling.annotation.SchedulingConfigurer;
import org.springframework.scheduling.config.ScheduledTaskRegistrar;
import org.springframework.scheduling.support.CronTrigger;
import org.springframework.stereotype.Component;

import javax.annotation.Resource;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

@Component
@EnableScheduling
public class IoModifyErpTask implements SchedulingConfigurer {

    final Logger logger = LoggerFactory.getLogger(IoModifyErpTask.class);

    @Resource
    protected ScheduledDao scheduledDao;
    @Resource
    private OrderService orderService;
    @Resource
    OrderDetailService myErpOrderService;
    @Resource
    private SystemParamConfigService systemParamConfigService;
    @Resource
    private BussinessTypeService bussinessTypeService;

    @Override
    public void configureTasks(ScheduledTaskRegistrar scheduledTaskRegistrar) {
        scheduledTaskRegistrar.addTriggerTask(() -> process(),
                triggerContext -> {
                    ScheduledRequest scheduledRequest = new ScheduledRequest();
                    scheduledRequest.setCronName("modifyErp");
                    ScheduledEntity scheduledEntity = scheduledDao.findScheduled(scheduledRequest);
                    String cron = scheduledEntity.getCron();
                    if (cron.isEmpty()) {
                        logger.error("cron is null");
                    }
                    return new CronTrigger(cron).nextExecutionTime(triggerContext);
                });
    }

    private void process() {

        modifyErp();
    }

    public void modifyErp() {
        SystemParamConfigEntity systemParamConfigEntity = systemParamConfigService.selectByParamKey("erpSaleOutModify");
        String isErpSaleOutModify = systemParamConfigEntity.getParamValue();
        systemParamConfigEntity = systemParamConfigService.selectByParamKey("erpPruchaseModify");
        String isErpPruchaseModify = systemParamConfigEntity.getParamValue();
        if ("1".equals(isErpSaleOutModify) || "1".equals(isErpPruchaseModify)) {
            logger.info("定时修改ERP单，定时签字");
            OrderFilterRequest orderFilterRequest = new OrderFilterRequest();
            orderFilterRequest.setExportStatus(ConstantStatus.ORDER_EXPORT_UN);
            orderFilterRequest.setContrastStatus(ConstantStatus.ORDER_CHECK_SUCCESS);
            List<OrderEntity> orderEntities = orderService.findAllOrders(orderFilterRequest);
            if (orderEntities != null && orderEntities.size() > 0) {
                for (OrderEntity orderEntity : orderEntities) {
                    FilterErpOrderRequest filterErpOrderRequest = new FilterErpOrderRequest();
                    filterErpOrderRequest.setPage(1);
                    filterErpOrderRequest.setLimit(100);
                    filterErpOrderRequest.setOrderId(orderEntity.getId());
                    List<ErpOrderEntity> erpOrderEntities = myErpOrderService.filterMyErpOrder(filterErpOrderRequest);
                    BasicThirdSysDetailEntity basicThirdSysDetailEntity = bussinessTypeService.findByActionKey(filterErpOrderRequest.getBillAction(), "orderQueryUrl");
                    if (basicThirdSysDetailEntity.getEnabled() &&  basicThirdSysDetailEntity.getFromType() == 0 ) {
                        if (basicThirdSysDetailEntity.getValue() != null) {
                            String url = basicThirdSysDetailEntity.getValue();
                            if (erpOrderEntities != null && !erpOrderEntities.isEmpty()) {
                                Map<String, List<ErpOrderEntity>> listMap = erpOrderEntities.stream().collect(Collectors.groupingBy(ErpOrderEntity::getErpOrderId));
                                if (orderEntity.getMainAction().equals(ConstantType.TYPE_OUT)) {
                                    if ("1".equals(isErpSaleOutModify)) {
                                        BaseResponse baseResponse = new ErpOrderClient().postBill(listMap, orderEntity.getAction(), url);
                                        if (baseResponse != null && baseResponse.getCode() == 20000) {
                                            orderService.updateExportStatus(orderEntity.getId(), ConstantStatus.ORDER_EXPORT_ED);
                                        }
                                    }
                                } else {
                                    if ("1".equals(isErpPruchaseModify)) {
                                        BaseResponse baseResponse = new ErpOrderClient().postBill(listMap, orderEntity.getAction(), url);
                                        if (baseResponse != null && baseResponse.getCode() == 20000) {
                                            orderService.updateExportStatus(orderEntity.getId(), ConstantStatus.ORDER_EXPORT_ED);
                                        }
                                    }
                                }
                            }
                        } else {
                            logger.error("ERP单据修改接口地址未定义");
                        }
                    }else {
                        logger.error("ERP单据修改未启用");
                    }


                }
            }


            SystemParamConfigEntity saleOutSign = systemParamConfigService.selectByParamKey("erpSaleOutSign");
            String isSaleOutSign = saleOutSign.getParamValue();
            saleOutSign = systemParamConfigService.selectByParamKey("erpPruchaseSign");
            String isPruchaseSign = saleOutSign.getParamValue();
            OrderFilterRequest signFilterRequest = new OrderFilterRequest();
            signFilterRequest.setSignStatus(ConstantStatus.ORDER_SIGN_UN);
            signFilterRequest.setContrastStatus(ConstantStatus.ORDER_CHECK_SUCCESS);
            List<OrderEntity> signsOrderEntities = orderService.findAllOrders(signFilterRequest);
            if (signsOrderEntities != null && signsOrderEntities.size() > 0) {
                for (OrderEntity orderEntity : signsOrderEntities) {

                    BasicThirdSysDetailEntity basicThirdSysDetailEntity = bussinessTypeService.findByActionKey(orderEntity.getAction(), "orderQueryUrl");
                    if (basicThirdSysDetailEntity.getEnabled() &&  basicThirdSysDetailEntity.getFromType() == 0 ) {
                        if (basicThirdSysDetailEntity.getValue() != null) {
                            String url = basicThirdSysDetailEntity.getValue();
                            FilterErpOrderRequest filterErpOrderRequest = new FilterErpOrderRequest();
                            filterErpOrderRequest.setOrderId(orderEntity.getId());
                            List<ErpOrderEntity> erpOrderEntities = myErpOrderService.filterMyErpOrder(filterErpOrderRequest);
                            if (erpOrderEntities != null && !erpOrderEntities.isEmpty()) {
                                Map<String, List<ErpOrderEntity>> listMap = erpOrderEntities.stream().collect(Collectors.groupingBy(ErpOrderEntity::getErpOrderId));
                                if (orderEntity.getMainAction().equals(ConstantType.TYPE_OUT)) {
                                    if ("1".equals(isSaleOutSign)) {
                                        if (listMap != null && listMap.size() > 0) {
                                            for (String key : listMap.keySet()) {
                                                ERPPostBillRequest signRequest = new ERPPostBillRequest();
                                                signRequest.setBillNo(key);
                                                signRequest.setBillAction(orderEntity.getAction());
                                                signRequest.setModifyType("sign");
                                                BaseResponse baseResponse = new ErpOrderClient().postBill(signRequest, url);
                                                if (baseResponse != null && baseResponse.getCode() == 20000) {
                                                    orderService.updateSignStatus(orderEntity.getId(), ConstantStatus.ORDER_SIGN_ED);
                                                }
                                            }
                                        }
                                    }
                                } else {
                                    if ("1".equals(isPruchaseSign)) {
                                        if (listMap != null && listMap.size() > 0) {
                                            for (String key : listMap.keySet()) {
                                                ERPPostBillRequest signRequest = new ERPPostBillRequest();
                                                signRequest.setBillNo(key);
                                                signRequest.setBillAction(orderEntity.getAction());
                                                signRequest.setModifyType("sign");
                                                BaseResponse baseResponse = new ErpOrderClient().postBill(signRequest, url);
                                                if (baseResponse != null && baseResponse.getCode() == 20000) {
                                                    orderService.updateSignStatus(orderEntity.getId(), ConstantStatus.ORDER_SIGN_ED);
                                                }
                                            }
                                        }
                                    }
                                }
                            }
                        } else {
                            logger.error("ERP单据修改接口地址未定义");
                        }
                    }else {
                        logger.error("ERP单据修改未启用");
                    }


                }
            }
        }
    }


}
