package com.glxp.api.admin.thread;

import com.glxp.api.admin.config.WebSocketServer;
import com.glxp.api.admin.constant.BasicProcessStatus;
import com.glxp.api.admin.entity.basic.BasicThirdSysDetailEntity;
import com.glxp.api.admin.entity.thrsys.*;
import com.glxp.api.admin.httpclient.ErpBasicClient;
import com.glxp.api.admin.req.basic.BasicUnitMaintainFilterRequest;
import com.glxp.api.admin.req.basic.FilterUdiIpLogRequest;
import com.glxp.api.admin.req.thrsys.FilterThrCorpRequest;
import com.glxp.api.admin.req.thrsys.ThrCorpExportRequest;
import com.glxp.api.admin.res.PageSimpleResponse;
import com.glxp.api.admin.res.basic.ErpUnitsResponse;
import com.glxp.api.admin.service.basic.BasicThirdSysDetailService;
import com.glxp.api.admin.service.thrsys.*;
import com.glxp.api.admin.util.Constant;
import com.glxp.api.admin.util.ExcelUtil;
import com.glxp.api.admin.util.RedisUtil;
import com.glxp.api.common.res.BaseResponse;
import org.springframework.beans.BeanUtils;
import org.springframework.scheduling.annotation.Async;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.stream.Collectors;

@Service
public class ThrCorpsDlService {

    @Resource
    ThrCorpService thrCorpService;
    @Resource
    BasicThirdSysDetailService basicThirdSysDetailService;
    @Resource
    RedisUtil redisUtil;
    @Resource
    ThrCorpImportLogService thrCorpImportLogService;
    @Resource
    ThrCorpExportLogService thrCorpExportLogService;
    @Resource
    private ThrCorpImportDetailService thrCorpImportDetailService;


    @Async
    public void importSelectCorps(String genKey, List<ErpUnitsResponse> erpUnitsResponses, String thirdSys) {
        BasicThirdSysDetailEntity piDetailEntity = basicThirdSysDetailService.selectByKey("corpUrl", thirdSys);
        if (piDetailEntity == null || piDetailEntity.getValue() == null) {
            WebSocketServer.sendInfo("往来单位接口未设置！", "sid");
            return;
        }
        ThrCorpImportLogEntity thrProductsImportLogEntity = thrCorpImportLogService.selectByGenKey(genKey);
        if (erpUnitsResponses != null && erpUnitsResponses.size() > 0) {

            List<ThrCorpImportDetailEntity> thrCorpImportDetailEntities;
            String finalGenKey = genKey;
            thrCorpImportDetailEntities = erpUnitsResponses.stream().map(
                    item -> {
                        ThrCorpImportDetailEntity thrCorpEntity = new ThrCorpImportDetailEntity();
                        BeanUtils.copyProperties(item, thrCorpEntity);
                        thrCorpEntity.setUnitId(item.getId());
                        thrCorpEntity.setGenKeyFk(finalGenKey);
                        thrCorpEntity.setUpdateTime(new Date());
                        thrCorpEntity.setThirdSysFk(thirdSys);
                        return thrCorpEntity;
                    }
            ).collect(Collectors.toList());
            thrCorpImportDetailService.insertCorpDetailImports(thrCorpImportDetailEntities);
            FilterUdiIpLogRequest filterUdiIpLogRequest = new FilterUdiIpLogRequest();
            filterUdiIpLogRequest.setGenKey(genKey);
            List<ThrCorpImportDetailEntity> temps = thrCorpImportDetailService.filterCorpDetailImport(filterUdiIpLogRequest);
            List<ThrCorpEntity> thrCorpEntities = temps.stream().map(
                    item -> {
                        ThrCorpEntity thrProductsEntity = new ThrCorpEntity();
                        BeanUtils.copyProperties(item, thrProductsEntity);
                        thrProductsEntity.setThirdSysFk(thirdSys);
                        thrProductsEntity.setUpdateTime(new Date());
                        return thrProductsEntity;
                    }
            ).collect(Collectors.toList());
            thrCorpService.insertThrCorpss(thrCorpEntities);
            redisUtil.set(Constant.dlThrProducts, "false");
            WebSocketServer.sendInfo("往来单位信息下载已完成，请刷新查看！", "sid");
            thrProductsImportLogEntity.setStatus(BasicProcessStatus.UDIINFO_IMPORT_SUCCESS);
            thrProductsImportLogEntity.setUpdateTime(new Date());
            thrCorpImportLogService.updateImportLog(thrProductsImportLogEntity);
        }
    }

    //主动下载第三方系统往来单位信息
    @Async
    public void importCorps(String genKey, BasicUnitMaintainFilterRequest filterThrProductsRequest, String thirdSys) {
        BasicThirdSysDetailEntity piDetailEntity = basicThirdSysDetailService.selectByKey("corpUrl", thirdSys);
        if (piDetailEntity == null || piDetailEntity.getValue() == null) {
            WebSocketServer.sendInfo("往来单位接口未设置！", "sid");
            return;
        }
        ThrCorpImportLogEntity thrProductsImportLogEntity = thrCorpImportLogService.selectByGenKey(genKey);

        int page = 1;
        int limit = 100;
        while (page != -1) {
            page = getCorps(page, limit, piDetailEntity.getValue(), thrProductsImportLogEntity, thirdSys, genKey, filterThrProductsRequest);
        }
        FilterUdiIpLogRequest filterUdiIpLogRequest = new FilterUdiIpLogRequest();
        filterUdiIpLogRequest.setGenKey(genKey);
        List<ThrCorpImportDetailEntity> thrCorpImportDetailEntities = thrCorpImportDetailService.filterCorpDetailImport(filterUdiIpLogRequest);
        List<ThrCorpEntity> thrCorpEntities = thrCorpImportDetailEntities.stream().map(
                item -> {
                    ThrCorpEntity thrProductsEntity = new ThrCorpEntity();
                    BeanUtils.copyProperties(item, thrProductsEntity);
                    thrProductsEntity.setUpdateTime(new Date());
                    thrProductsEntity.setThirdSysFk(thirdSys);
                    return thrProductsEntity;
                }
        ).collect(Collectors.toList());
        thrCorpService.insertThrCorpss(thrCorpEntities);
        redisUtil.set(Constant.dlThrProducts, "false");
        WebSocketServer.sendInfo("往来单位信息下载已完成，请刷新查看！", "sid");
        thrProductsImportLogEntity.setStatus(BasicProcessStatus.UDIINFO_IMPORT_SUCCESS);
        thrProductsImportLogEntity.setUpdateTime(new Date());
        thrCorpImportLogService.updateImportLog(thrProductsImportLogEntity);
    }

    //导出生成Excel文件
    @Async
    public void genExcel(String genKey, ThrCorpExportRequest thrCorpExportRequest) {
        ThrCorpExportLogEntity thrCorpExportLogEntity = thrCorpExportLogService.selectByGenKey(genKey);
        List<List<String>> excelData = new ArrayList<>();
        List<String> head = new ArrayList<>();
        head.add("往来单位ID");
        head.add("往来单位名称");
        head.add("拼音简写");
        head.add("地址");
        head.add("社会信用号");
        head.add("联系人");
        head.add("联系电话");
        excelData.add(head);
        //选中导出
        if (thrCorpExportRequest.getThrCorpEntities() != null && thrCorpExportRequest.getThrCorpEntities().size() > 0) {
            List<ThrCorpEntity> corpEntities = thrCorpExportRequest.getThrCorpEntities();
            for (ThrCorpEntity thrCorpEntity : corpEntities) {
                List<String> rows = new ArrayList<>();
                rows.add(thrCorpEntity.getUnitId());
                rows.add(thrCorpEntity.getName());
                rows.add(thrCorpEntity.getSpell());
                rows.add(thrCorpEntity.getAddr());
                rows.add(thrCorpEntity.getCreditNo());
                rows.add(thrCorpEntity.getContact());
                rows.add(thrCorpEntity.getMobile());
                excelData.add(rows);
            }
        } else {//一键导出

            //判断是否启用实时获取
            BasicThirdSysDetailEntity basicThirdSysDetailEntity = basicThirdSysDetailService.selectByKey("corpUrl", thrCorpExportRequest.getThirdSys());
            if (basicThirdSysDetailEntity == null || basicThirdSysDetailEntity.getValue() == null) {
                thrCorpExportLogEntity.setStatus(BasicProcessStatus.UDIINFO_IMPORT_FAIL);
                thrCorpExportLogEntity.setRemark("往来单位接口地址未定义");
                thrCorpExportLogService.updateThrCorpExportLog(thrCorpExportLogEntity);
                return;
            }

            if (!basicThirdSysDetailEntity.getEnabled()) {
                thrCorpExportLogEntity.setStatus(BasicProcessStatus.UDIINFO_IMPORT_FAIL);
                thrCorpExportLogEntity.setRemark("第三方往来单位服务未启用");
                thrCorpExportLogService.updateThrCorpExportLog(thrCorpExportLogEntity);
                return;
            }

            if (basicThirdSysDetailEntity.getFromType() == 0) {
                //根据查询条件一键导出第三方系统往来单位
                List<ThrCorpEntity> thrCorpEntities = exportCorps(thrCorpExportRequest);
                excelData.addAll(getRows(thrCorpEntities));
            } else {
                //根据查询条件一键导出数据库往来单位
                FilterThrCorpRequest filterThrCorpRequest = new FilterThrCorpRequest();
                BeanUtils.copyProperties(thrCorpExportRequest, filterThrCorpRequest);
                filterThrCorpRequest.setPage(null);
                List<List<String>> genDatas = genExcelData(filterThrCorpRequest);
                if (genDatas != null && genDatas.size() > 0) {
                    excelData.addAll(genDatas);
                }
            }

        }
        String sheetName = "往来单位信息";
        new ExcelUtil().exportExcel(excelData, sheetName, thrCorpExportLogEntity.getFilePath(), 20);
        thrCorpExportLogEntity.setStatus(BasicProcessStatus.UDIINFO_IMPORT_SUCCESS);
        thrCorpExportLogService.updateThrCorpExportLog(thrCorpExportLogEntity);

    }

    //接收第三方上传或文件导入数据
    @Async
    public void insertData(String genKey, String thirdSys, List<ErpUnitsResponse> erpUnitsResponses) {

        ThrCorpImportLogEntity thrCorpImportLogEntity = thrCorpImportLogService.selectByGenKey(genKey);
        //转入临时表
        List<ThrCorpEntity> thrCorpEntities;
        List<ThrCorpImportDetailEntity> thrCorpImportDetailEntities;
        String finalGenKey = genKey;
        thrCorpImportDetailEntities = erpUnitsResponses.stream().map(
                item -> {
                    ThrCorpImportDetailEntity thrCorpEntity = new ThrCorpImportDetailEntity();
                    BeanUtils.copyProperties(item, thrCorpEntity);
                    thrCorpEntity.setUnitId(item.getId());
                    thrCorpEntity.setGenKeyFk(finalGenKey);
                    thrCorpEntity.setUpdateTime(new Date());
                    thrCorpEntity.setThirdSysFk(thirdSys);
                    return thrCorpEntity;
                }
        ).collect(Collectors.toList());
        thrCorpImportDetailService.insertCorpDetailImports(thrCorpImportDetailEntities);
        thrCorpEntities = thrCorpImportDetailEntities.stream().map(
                item -> {
                    ThrCorpEntity thrCorpEntity = new ThrCorpEntity();
                    BeanUtils.copyProperties(item, thrCorpEntity);
//                        thrCorpEntity.setUnitId(item.getId());
                    thrCorpEntity.setUpdateTime(new Date());
                    thrCorpEntity.setThirdSysFk(thirdSys);
                    return thrCorpEntity;
                }
        ).collect(Collectors.toList());
        thrCorpService.insertThrCorpss(thrCorpEntities);
        thrCorpImportLogEntity.setStatus(BasicProcessStatus.UDIINFO_IMPORT_SUCCESS);
        thrCorpImportLogEntity.setUpdateTime(new Date());
        thrCorpImportLogService.updateImportLog(thrCorpImportLogEntity);
    }


    public int getCorps(int page, int limit, String corpUrl, ThrCorpImportLogEntity thrCorpImportLogEntity, String thirdSys, String genKey, BasicUnitMaintainFilterRequest basicUnitMaintainFilterRequest) {
        if (basicUnitMaintainFilterRequest == null)
            basicUnitMaintainFilterRequest = new BasicUnitMaintainFilterRequest();
        basicUnitMaintainFilterRequest.setPage(page);
        basicUnitMaintainFilterRequest.setLimit(limit);
        BaseResponse<PageSimpleResponse<ErpUnitsResponse>> erpCrops = new ErpBasicClient().getErpCrop(basicUnitMaintainFilterRequest, corpUrl);
        if (erpCrops == null || erpCrops.getCode() != 20000) {

            thrCorpImportLogEntity.setStatus(BasicProcessStatus.UDIINFO_IMPORT_FAIL);
            thrCorpImportLogEntity.setRemark(erpCrops.getMessage());
            thrCorpImportLogService.updateImportLog(thrCorpImportLogEntity);
            return -1;
        }
        List<ErpUnitsResponse> erpUnitsResponses = erpCrops.getData().getList();
        if (erpUnitsResponses != null && erpUnitsResponses.size() > 0) {

            List<ThrCorpImportDetailEntity> thrCorpImportDetailEntities;
            String finalGenKey = genKey;
            thrCorpImportDetailEntities = erpUnitsResponses.stream().map(
                    item -> {
                        ThrCorpImportDetailEntity thrCorpEntity = new ThrCorpImportDetailEntity();
                        BeanUtils.copyProperties(item, thrCorpEntity);
                        thrCorpEntity.setUnitId(item.getId());
                        thrCorpEntity.setGenKeyFk(finalGenKey);
                        thrCorpEntity.setUpdateTime(new Date());
                        thrCorpEntity.setThirdSysFk(thirdSys);
                        return thrCorpEntity;
                    }
            ).collect(Collectors.toList());
            thrCorpImportDetailService.insertCorpDetailImports(thrCorpImportDetailEntities);


            page++;
            if (page * limit < erpCrops.getData().getTotal()) {
                return page;
            }
        }
        thrCorpImportLogEntity.setStatus(BasicProcessStatus.UDIINFO_IMPORT_SUCCESS);
        thrCorpImportLogService.updateImportLog(thrCorpImportLogEntity);
        return -1;
    }


    public List<ThrCorpEntity> exportCorps(ThrCorpExportRequest thrCorpExportRequest) {
        BasicThirdSysDetailEntity corpEntity = basicThirdSysDetailService.selectByKey("corpUrl", thrCorpExportRequest.getThirdSys());
        int page = 1;
        int limit = 100;
        List<ThrCorpEntity> thrCorpEntities = new ArrayList<>();
        while (true) {
            List<ThrCorpEntity> datas = getCorps(page, limit, corpEntity.getValue(), thrCorpExportRequest);
            if (datas != null && datas.size() >= limit) {
                thrCorpEntities.addAll(datas);
            } else {
                if (datas != null) {
                    thrCorpEntities.addAll(datas);
                }
                break;
            }
        }
        return thrCorpEntities;
    }

    public List<ThrCorpEntity> getCorps(int page, int limit, String corpUrl, ThrCorpExportRequest thrCorpExportRequest) {
        BasicUnitMaintainFilterRequest basicUnitMaintainFilterRequest = new BasicUnitMaintainFilterRequest();
        basicUnitMaintainFilterRequest.setName(thrCorpExportRequest.getName());
        basicUnitMaintainFilterRequest.setUnitId(thrCorpExportRequest.getUnitId());
        basicUnitMaintainFilterRequest.setPage(page);
        basicUnitMaintainFilterRequest.setLimit(limit);
        BaseResponse<PageSimpleResponse<ErpUnitsResponse>> erpCrops = new ErpBasicClient().getErpCrop(basicUnitMaintainFilterRequest, corpUrl);
        if (erpCrops.getCode() != 20000) {
            return null;
        }
        List<ErpUnitsResponse> erpUnitsResponses = erpCrops.getData().getList();
        if (erpUnitsResponses != null && erpUnitsResponses.size() > 0) {
            List<ThrCorpEntity> thrCorpEntities = erpUnitsResponses.stream().map(
                    item -> {
                        ThrCorpEntity thrProductsEntity = new ThrCorpEntity();
                        BeanUtils.copyProperties(item, thrProductsEntity);
                        thrProductsEntity.setUnitId(item.getId());
                        return thrProductsEntity;
                    }
            ).collect(Collectors.toList());
            return thrCorpEntities;
        }
        return null;
    }

    public List<List<String>> genExcelData(FilterThrCorpRequest filterThrOrderRequest) {

        List<List<String>> excelData = new ArrayList<>();
        List<ThrCorpEntity> thrCorpEntities = thrCorpService.filterThrCorps(filterThrOrderRequest);
        excelData.addAll(getRows(thrCorpEntities));
        return excelData;
    }

    public List<List<String>> getRows(List<ThrCorpEntity> thrCorpEntities) {
        List<List<String>> excelData = new ArrayList<>();
        for (ThrCorpEntity thrCorpEntity : thrCorpEntities) {
            List<String> rows = new ArrayList<>();
            rows.add(thrCorpEntity.getUnitId());
            rows.add(thrCorpEntity.getName());
            rows.add(thrCorpEntity.getSpell());
            rows.add(thrCorpEntity.getAddr());
            rows.add(thrCorpEntity.getCreditNo());
            rows.add(thrCorpEntity.getContact());
            rows.add(thrCorpEntity.getMobile());
            excelData.add(rows);

        }
        return excelData;
    }
}
