package com.glxp.api.admin.thread;

import com.glxp.api.admin.config.WebSocketServer;
import com.glxp.api.admin.constant.BasicProcessStatus;
import com.glxp.api.admin.entity.basic.BasicThirdSysDetailEntity;
import com.glxp.api.admin.entity.thrsys.*;
import com.glxp.api.admin.httpclient.ErpInvClient;
import com.glxp.api.admin.req.basic.FilterUdiIpLogRequest;
import com.glxp.api.admin.req.inventory.ErpOnhandRequest;
import com.glxp.api.admin.req.thrsys.FilterThrInvProductsRequest;
import com.glxp.api.admin.req.thrsys.ThrInvProductsExportRequest;
import com.glxp.api.admin.res.PageSimpleResponse;
import com.glxp.api.admin.res.inventory.ErpInvProductResponse;
import com.glxp.api.admin.service.basic.BasicThirdSysDetailService;
import com.glxp.api.admin.service.thrsys.*;
import com.glxp.api.admin.util.Constant;
import com.glxp.api.admin.util.CustomUtil;
import com.glxp.api.admin.util.ExcelUtil;
import com.glxp.api.admin.util.RedisUtil;
import com.glxp.api.common.res.BaseResponse;
import org.springframework.beans.BeanUtils;
import org.springframework.scheduling.annotation.Async;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.stream.Collectors;

@Service
public class ThrInvProductsDlService {

    @Resource
    ThrInvProductsService thrInvProductsService;
    @Resource
    BasicThirdSysDetailService basicThirdSysDetailService;
    @Resource
    RedisUtil redisUtil;
    @Resource
    ThrInvProductsImportLogService thrInvProductsImportLogService;
    @Resource
    ThrInvProductsExportLogService thrInvProductsExportLogService;
    @Resource
    ThrInvProductsDetailService thrInvProductsDetailService;


    @Async
    public void importSelcyInvProducrs(String genKey, String thirdSys, List<ErpInvProductResponse> erpInvProductResponses) {
        ThrInvProductsImportLogEntity thrInvProductsImportLogEntity = thrInvProductsImportLogService.selectByGenKey(genKey);
        BasicThirdSysDetailEntity piDetailEntity = basicThirdSysDetailService.selectByKey("invPiUrl", thirdSys);
        if (piDetailEntity == null || piDetailEntity.getValue() == null) {
            WebSocketServer.sendInfo("库存产品信息接口未设置！", "sid");
            return;
        }


        if (erpInvProductResponses != null && erpInvProductResponses.size() > 0) {
            List<ThrInvProductsImportDetailEntity> thrProductsEntities = erpInvProductResponses.stream().map(
                    item -> {
                        ThrInvProductsImportDetailEntity thrProductsEntity = new ThrInvProductsImportDetailEntity();
                        BeanUtils.copyProperties(item, thrProductsEntity);
                        thrProductsEntity.setInventoryCode(item.getCode());
                        thrProductsEntity.setInventoryName(item.getName());
                        thrProductsEntity.setGenKeyFk(genKey);
                        thrProductsEntity.setThirdSysFk(thirdSys);
                        thrProductsEntity.setUpdateTime(new Date());
                        return thrProductsEntity;
                    }
            ).collect(Collectors.toList());
            thrInvProductsDetailService.insertInvProductsDetailImports(thrProductsEntities);
        }

        FilterUdiIpLogRequest filterUdiIpLogRequest = new FilterUdiIpLogRequest();
        filterUdiIpLogRequest.setGenKey(genKey);
        List<ThrInvProductsImportDetailEntity> thrInvProductsImportDetailEntities = thrInvProductsDetailService.filterInvProductsDetailImport(filterUdiIpLogRequest);
        List<ThrInvProductsEntity> thrInvProductsEntities = thrInvProductsImportDetailEntities.stream().map(
                item -> {
                    ThrInvProductsEntity thrInvProductsEntity = new ThrInvProductsEntity();
                    BeanUtils.copyProperties(item, thrInvProductsEntity);
                    thrInvProductsEntity.setUpdateTime(new Date());
                    return thrInvProductsEntity;
                }
        ).collect(Collectors.toList());
        thrInvProductsService.insertThrInvProducts(thrInvProductsEntities);
        redisUtil.set(Constant.dlThrProducts, "false");
        WebSocketServer.sendInfo("库存产品信息下载已完成，请刷新查看！", "sid");
        thrInvProductsImportLogEntity.setStatus(BasicProcessStatus.UDIINFO_IMPORT_SUCCESS);
        thrInvProductsImportLogService.updateImportLog(thrInvProductsImportLogEntity);

    }


    @Async
    public void importProducrs(String genKey, String thirdSys, ErpOnhandRequest erpOnhandRequest) {
        BasicThirdSysDetailEntity piDetailEntity = basicThirdSysDetailService.selectByKey("invPiUrl", thirdSys);
        if (piDetailEntity == null || piDetailEntity.getValue() == null) {
            WebSocketServer.sendInfo("库存产品信息接口未设置！", "sid");
            return;
        }
        ThrInvProductsImportLogEntity thrInvProductsImportLogEntity = thrInvProductsImportLogService.selectByGenKey(genKey);
        int page = 1;
        int limit = 100;
        while (page != -1) {
            page = getInvmandoc(page, limit, piDetailEntity.getValue(), thrInvProductsImportLogEntity, genKey, thirdSys, erpOnhandRequest);
        }
        FilterUdiIpLogRequest filterUdiIpLogRequest = new FilterUdiIpLogRequest();
        filterUdiIpLogRequest.setGenKey(genKey);
        List<ThrInvProductsImportDetailEntity> thrInvProductsImportDetailEntities = thrInvProductsDetailService.filterInvProductsDetailImport(filterUdiIpLogRequest);
        List<ThrInvProductsEntity> thrInvProductsEntities = thrInvProductsImportDetailEntities.stream().map(
                item -> {
                    ThrInvProductsEntity thrInvProductsEntity = new ThrInvProductsEntity();
                    BeanUtils.copyProperties(item, thrInvProductsEntity);
                    thrInvProductsEntity.setUpdateTime(new Date());
                    return thrInvProductsEntity;
                }
        ).collect(Collectors.toList());
        thrInvProductsService.insertThrInvProducts(thrInvProductsEntities);
        redisUtil.set(Constant.dlThrProducts, "false");
        WebSocketServer.sendInfo("库存产品信息下载已完成，请刷新查看！", "sid");
    }

    @Async
    public void genExcel(String genKey, ThrInvProductsExportRequest thrInvProductsExportRequest) {

        ThrInvProductsExportLogEntity thrInvProductsExportLogEntity = thrInvProductsExportLogService.selectByGenKey(genKey);
        List<List<String>> excelData = new ArrayList<>();
        List<String> head = new ArrayList<>();
        head.add("存货编码");
        head.add("存货名称");
        head.add("规格型号");
        head.add("数量");
        head.add("批次号");
        head.add("生产日期");
        head.add("失效日期");
        head.add("仓位号");
        head.add("仓位名称");
        head.add("货位号");
        head.add("货位名称");
        excelData.add(head);
        //选中导出
        if (thrInvProductsExportRequest.getThrInvProductsEntities() != null && thrInvProductsExportRequest.getThrInvProductsEntities().size() > 0) {
            List<ThrInvProductsEntity> thrInvProductsEntities = thrInvProductsExportRequest.getThrInvProductsEntities();

            for (ThrInvProductsEntity thrInvProductsEntity : thrInvProductsEntities) {
                thrInvProductsEntity.setInventoryCode(thrInvProductsEntity.getCode());
                thrInvProductsEntity.setInventoryName(thrInvProductsEntity.getName());
            }
            excelData.addAll(getRows(thrInvProductsEntities));
        } else {//一键导出
            //判断是否启用实时获取
            BasicThirdSysDetailEntity basicThirdSysDetailEntity = basicThirdSysDetailService.selectByKey("invPiUrl", thrInvProductsExportRequest.getThirdSys());
            if (basicThirdSysDetailEntity == null || basicThirdSysDetailEntity.getValue() == null) {
                thrInvProductsExportLogEntity.setStatus(BasicProcessStatus.UDIINFO_IMPORT_FAIL);
                thrInvProductsExportLogEntity.setRemark("库存产品信息接口地址未定义");
                thrInvProductsExportLogService.updateThrInvProductsExportLog(thrInvProductsExportLogEntity);
                return;
            }
            if (!basicThirdSysDetailEntity.getEnabled()) {
                thrInvProductsExportLogEntity.setStatus(BasicProcessStatus.UDIINFO_IMPORT_FAIL);
                thrInvProductsExportLogEntity.setRemark("第三方库存产品信息接口服务未启用");
                thrInvProductsExportLogService.updateThrInvProductsExportLog(thrInvProductsExportLogEntity);
                return;
            }
            if (basicThirdSysDetailEntity.getFromType() == 0) {
                //从第三方服务实时一键导出
                FilterThrInvProductsRequest filterThrInvProductsRequest = new FilterThrInvProductsRequest();
                BeanUtils.copyProperties(thrInvProductsExportRequest, filterThrInvProductsRequest);
                List<ThrInvProductsEntity> thrInvProductsEntities = exportInvProducts(filterThrInvProductsRequest);
                excelData.addAll(getRows(thrInvProductsEntities));
            } else {
                //从本地导出
                FilterThrInvProductsRequest filterThrInvProductsRequest = new FilterThrInvProductsRequest();
                BeanUtils.copyProperties(thrInvProductsExportRequest, filterThrInvProductsRequest);
                List<List<String>> genDatas = genExcelData(null, filterThrInvProductsRequest);
                if (genDatas != null && genDatas.size() > 0) {
                    excelData.addAll(genDatas);
                }
            }
        }
        String sheetName = "库存产品信息";
        new ExcelUtil().exportExcel(excelData, sheetName, thrInvProductsExportLogEntity.getFilePath(), 20);
        thrInvProductsExportLogEntity.setStatus(BasicProcessStatus.UDIINFO_IMPORT_SUCCESS);
        thrInvProductsExportLogService.updateThrInvProductsExportLog(thrInvProductsExportLogEntity);
    }

    @Async
    public void insertInvProducts(String genKey, String thirdSys, List<ErpInvProductResponse> erpOnHandResponses) {

        ThrInvProductsImportLogEntity thrInvProductsImportLogEntity = thrInvProductsImportLogService.selectByGenKey(genKey);

        List<ThrInvProductsImportDetailEntity> thrInvProductsImportDetailEntities;
        thrInvProductsImportDetailEntities = erpOnHandResponses.stream().map(
                item -> {
                    ThrInvProductsImportDetailEntity thrInvProductsImportDetailEntity = new ThrInvProductsImportDetailEntity();
                    BeanUtils.copyProperties(item, thrInvProductsImportDetailEntity);
                    thrInvProductsImportDetailEntity.setInventoryCode(item.getCode());
                    thrInvProductsImportDetailEntity.setInventoryName(item.getName());
                    thrInvProductsImportDetailEntity.setThirdSysFk(thirdSys);
                    thrInvProductsImportDetailEntity.setGenKeyFk(genKey);
                    thrInvProductsImportDetailEntity.setUpdateTime(new Date());
                    return thrInvProductsImportDetailEntity;
                }
        ).collect(Collectors.toList());
        thrInvProductsDetailService.insertInvProductsDetailImports(thrInvProductsImportDetailEntities);

        List<ThrInvProductsEntity> thrInvProductsEntities;
        thrInvProductsEntities = thrInvProductsImportDetailEntities.stream().map(
                item -> {
                    ThrInvProductsEntity thrInvProductsEntity = new ThrInvProductsEntity();
                    BeanUtils.copyProperties(item, thrInvProductsEntity);
                    thrInvProductsEntity.setUpdateTime(new Date());
                    return thrInvProductsEntity;
                }
        ).collect(Collectors.toList());
        thrInvProductsService.insertThrInvProducts(thrInvProductsEntities);

        thrInvProductsImportLogEntity.setStatus(BasicProcessStatus.UDIINFO_IMPORT_SUCCESS);
        thrInvProductsImportLogEntity.setUpdateTime(new Date());
        thrInvProductsImportLogService.updateImportLog(thrInvProductsImportLogEntity);
    }


    public int getInvmandoc(int page, int limit, String productUrl, ThrInvProductsImportLogEntity thrInvProductsImportLogEntity,
                            String genKey, String thirdSys, ErpOnhandRequest erpOnhandRequest) {
        if (erpOnhandRequest == null)
            erpOnhandRequest = new ErpOnhandRequest();
        erpOnhandRequest.setPage(page);
        erpOnhandRequest.setLimit(limit);

        BaseResponse<PageSimpleResponse<ErpInvProductResponse>> responseBaseResponse = new ErpInvClient().getInvPrdoductResponse(erpOnhandRequest, productUrl);

        if (responseBaseResponse.getCode() != 20000) {
            thrInvProductsImportLogEntity.setStatus(BasicProcessStatus.UDIINFO_IMPORT_FAIL);
            thrInvProductsImportLogEntity.setRemark(responseBaseResponse.getMessage());
            thrInvProductsImportLogService.updateImportLog(thrInvProductsImportLogEntity);
            return -1;
        }

        List<ErpInvProductResponse> erpInvProductResponses = responseBaseResponse.getData().getList();
        if (erpInvProductResponses != null && erpInvProductResponses.size() > 0) {
            List<ThrInvProductsImportDetailEntity> thrProductsEntities = erpInvProductResponses.stream().map(
                    item -> {
                        ThrInvProductsImportDetailEntity thrProductsEntity = new ThrInvProductsImportDetailEntity();
                        BeanUtils.copyProperties(item, thrProductsEntity);
                        thrProductsEntity.setInventoryCode(item.getCode());
                        thrProductsEntity.setInventoryName(item.getName());
                        thrProductsEntity.setGenKeyFk(genKey);
                        thrProductsEntity.setThirdSysFk(thirdSys);
                        thrProductsEntity.setUpdateTime(new Date());
                        return thrProductsEntity;
                    }
            ).collect(Collectors.toList());
            thrInvProductsDetailService.insertInvProductsDetailImports(thrProductsEntities);
            page++;
            if (page * limit < responseBaseResponse.getData().getTotal()) {
                return page;
            }
        }
        thrInvProductsImportLogEntity.setStatus(BasicProcessStatus.UDIINFO_IMPORT_SUCCESS);
        thrInvProductsImportLogService.updateImportLog(thrInvProductsImportLogEntity);
        return -1;
    }

    public List<ThrInvProductsEntity> exportInvProducts(FilterThrInvProductsRequest filterThrInvProductsRequest) {
        BasicThirdSysDetailEntity basicThirdSysDetailEntity = basicThirdSysDetailService.selectByKey("invPiUrl", filterThrInvProductsRequest.getThirdSys());
        int page = 1;
        int limit = 100;
        List<ThrInvProductsEntity> thrInvProductsEntities = new ArrayList<>();
        while (true) {
            List<ThrInvProductsEntity> datas = getInvProducts(page, limit, basicThirdSysDetailEntity.getValue(), filterThrInvProductsRequest);
            if (datas != null && datas.size() >= limit) {
                thrInvProductsEntities.addAll(datas);
                page++;

            } else {
                if (datas != null) {
                    thrInvProductsEntities.addAll(datas);
                }
                break;
            }
        }
        return thrInvProductsEntities;
    }

    public List<ThrInvProductsEntity> getInvProducts(int page, int limit, String invPiUrl, FilterThrInvProductsRequest filterThrInvProductsRequest) {
        ErpOnhandRequest erpOnhandRequest = new ErpOnhandRequest();
        BeanUtils.copyProperties(filterThrInvProductsRequest, erpOnhandRequest);
        erpOnhandRequest.setPage(page);
        erpOnhandRequest.setLimit(limit);

        BaseResponse<PageSimpleResponse<ErpInvProductResponse>> responseBaseResponse = new ErpInvClient().getInvPrdoductResponse(erpOnhandRequest, invPiUrl);
        if (responseBaseResponse != null && responseBaseResponse.getCode() == 20000) {
            List<ErpInvProductResponse> erpInvProductResponses = responseBaseResponse.getData().getList();
            if (erpInvProductResponses != null && erpInvProductResponses.size() > 0) {
                List<ThrInvProductsEntity> thrProductsEntities = erpInvProductResponses.stream().map(
                        item -> {
                            ThrInvProductsEntity thrProductsEntity = new ThrInvProductsEntity();
                            BeanUtils.copyProperties(item, thrProductsEntity);
                            thrProductsEntity.setInventoryCode(item.getCode());
                            thrProductsEntity.setInventoryName(item.getName());
                            return thrProductsEntity;
                        }
                ).collect(Collectors.toList());
                return thrProductsEntities;
            }
        }


        return null;
    }

    public List<List<String>> genExcelData(String id, FilterThrInvProductsRequest filterThrInvProductsRequest) {
        filterThrInvProductsRequest.setThirdSysFk(filterThrInvProductsRequest.getThirdSys());
        List<List<String>> excelData = new ArrayList<>();
        List<ThrInvProductsEntity> thrInvProductsEntities = new ArrayList<>();
        if (id != null) {
            ThrInvProductsEntity thrInvProductsEntity = thrInvProductsService.selectById(id);
            thrInvProductsEntities.add(thrInvProductsEntity);
        } else {
            thrInvProductsEntities = thrInvProductsService.filterThrInvProductss(filterThrInvProductsRequest);
        }
        excelData.addAll(getRows(thrInvProductsEntities));
        return excelData;
    }


    public List<List<String>> getRows(List<ThrInvProductsEntity> thrInvProductsEntities) {
        List<List<String>> excelData = new ArrayList<>();
        for (ThrInvProductsEntity thrInvProductsEntity : thrInvProductsEntities) {
            List<String> rows = new ArrayList<>();
            rows.add(thrInvProductsEntity.getInventoryCode());
            rows.add(thrInvProductsEntity.getInventoryName());
            rows.add(thrInvProductsEntity.getSpec());
            rows.add(thrInvProductsEntity.getCount() + "");
            rows.add(thrInvProductsEntity.getBatchNo());
            rows.add(thrInvProductsEntity.getManufacturingDate());
            rows.add(thrInvProductsEntity.getExpirationDate());
            rows.add(thrInvProductsEntity.getWarehouseCode());
            rows.add(thrInvProductsEntity.getWarehouseName());
            rows.add(thrInvProductsEntity.getSpaceCode());
            rows.add(thrInvProductsEntity.getSpaceName());
            excelData.add(rows);
        }
        return excelData;
    }
}
