package com.glxp.api.admin.thread;

import com.alibaba.fastjson.JSONObject;
import com.alibaba.fastjson.TypeReference;
import com.glxp.api.admin.config.WebSocketServer;
import com.glxp.api.admin.constant.BasicProcessStatus;
import com.glxp.api.admin.entity.basic.BasicThirdSysDetailEntity;
import com.glxp.api.admin.entity.info.SystemParamConfigEntity;
import com.glxp.api.admin.entity.thrsys.*;
import com.glxp.api.admin.httpclient.ErpOrderClient;
import com.glxp.api.admin.req.basic.FilterUdiIpLogRequest;
import com.glxp.api.admin.req.inout.FilterOrderRequest;
import com.glxp.api.admin.req.thrsys.FilterThrOrderDetailRequest;
import com.glxp.api.admin.req.thrsys.FilterThrOrderRequest;
import com.glxp.api.admin.req.thrsys.PostThrOrderRequest;
import com.glxp.api.admin.req.thrsys.ThrOrderExportRequest;
import com.glxp.api.admin.res.PageSimpleResponse;
import com.glxp.api.admin.res.inout.ErpOrderResponse;
import com.glxp.api.admin.service.basic.BasicThirdSysDetailService;
import com.glxp.api.admin.service.basic.BussinessTypeService;
import com.glxp.api.admin.service.info.SystemParamConfigService;
import com.glxp.api.admin.service.thrsys.*;
import com.glxp.api.admin.util.Constant;
import com.glxp.api.admin.util.ExcelUtil;
import com.glxp.api.admin.util.HttpClient;
import com.glxp.api.admin.util.RedisUtil;
import com.glxp.api.common.res.BaseResponse;
import org.springframework.beans.BeanUtils;
import org.springframework.scheduling.annotation.Async;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

@Service
public class ThrOrdersDlService {
    @Resource
    ThrOrderService thrOrderService;
    @Resource
    ThrOrderDetailService thrOrderDetailService;
    @Resource
    BasicThirdSysDetailService basicThirdSysDetailService;
    @Resource
    RedisUtil redisUtil;
    @Resource
    BussinessTypeService bussinessTypeService;
    @Resource
    private ThrOrderImportLogService thrOrderImportLogService;
    @Resource
    ThrOrderExportLogService thrOrderExportLogService;
    @Resource
    ThrOrderImportDetailService thrOrderImportDetailService;
    @Resource
    ThrOrderImportService thrOrderImportService;
    @Resource
    SystemParamConfigService systemParamConfigService;


    @Async
    public void importOrders(String genKey, String action, FilterThrOrderRequest filterThrProductsRequest) {

        ThrOrderImportLogEntity thrOrderImportLogEntity = thrOrderImportLogService.selectByGenKey(genKey);

        BasicThirdSysDetailEntity basicThirdSysDetailEntity = bussinessTypeService.findByActionKey(action, "orderQueryUrl");
        if (basicThirdSysDetailEntity == null || basicThirdSysDetailEntity.getValue() == null) {
            WebSocketServer.sendInfo("业务单据查询接口未设置！", "sid");
            return;
        }

        int page = 1;
        int limit = 100;
        while (page != -1) {
            page = getOrders(page, limit, basicThirdSysDetailEntity.getValue(), basicThirdSysDetailEntity.getThirdSysFk(), thrOrderImportLogEntity, action, filterThrProductsRequest);
        }

        thrOrderImportService.importThrOrder(genKey);

        redisUtil.set(Constant.dlThrProducts, "false");
        WebSocketServer.sendInfo("业务单据信息下载已完成，请刷新查看！", "sid");
        thrOrderImportLogEntity.setStatus(BasicProcessStatus.UDIINFO_IMPORT_SUCCESS);
        thrOrderImportLogService.updateImportLog(thrOrderImportLogEntity);
    }

    @Async
    public void importSelectOrders(String genKey, List<ErpOrderResponse> erpOrderResponses, String thirdSys) {
        ThrOrderImportLogEntity thrOrderImportLogEntity = thrOrderImportLogService.selectByGenKey(genKey);
        if (erpOrderResponses != null && erpOrderResponses.size() > 0) {
            for (ErpOrderResponse erpOrderResponse : erpOrderResponses) {
                ThrOrderEntity thrOrderEntity = new ThrOrderEntity();
                BeanUtils.copyProperties(erpOrderResponse, thrOrderEntity);
                thrOrderEntity.setThirdSysFk(thirdSys);
                thrOrderEntity.setUpdateTime(new Date());
                thrOrderService.insertThrOrder(thrOrderEntity);
                List<ThrOrderImportDetailEntity> thrOrderDetailEntities = new ArrayList<>();
                for (ErpOrderResponse.SubErpOrder subPurchase : erpOrderResponse.getSubErpOrders()) {
                    ThrOrderImportDetailEntity thrOrderDetailEntity = new ThrOrderImportDetailEntity();
                    BeanUtils.copyProperties(subPurchase, thrOrderDetailEntity);
                    BeanUtils.copyProperties(erpOrderResponse, thrOrderDetailEntity);
                    thrOrderDetailEntity.setGenKeyFk(thrOrderImportLogEntity.getGenKey());
                    thrOrderDetailEntity.setThirdSysFk(thirdSys);
                    thrOrderDetailEntity.setUpdateTime(new Date());
                    thrOrderDetailEntities.add(thrOrderDetailEntity);
                }
                thrOrderImportDetailService.insertOrderImportDetails(thrOrderDetailEntities);
            }
        }
        thrOrderImportService.importThrOrder(genKey);
        redisUtil.set(Constant.dlThrProducts, "false");
        WebSocketServer.sendInfo("业务单据信息下载已完成，请刷新查看！", "sid");
        thrOrderImportLogEntity.setStatus(BasicProcessStatus.UDIINFO_IMPORT_SUCCESS);
        thrOrderImportLogService.updateImportLog(thrOrderImportLogEntity);
    }

    @Async
    public void genExcel(String genKey, ThrOrderExportRequest thrOrderExportRequest) {

        ThrOrderExportLogEntity thrOrderExportLogEntity = thrOrderExportLogService.selectByGenKey(genKey);
        List<List<String>> excelData = new ArrayList<>();
        List<String> head = new ArrayList<>();
        head.add("单据号");
        head.add("单据日期");
        head.add("往来单位ID");
        head.add("往来单位名称");
        head.add("单据类型");
        head.add("单据状态");
        head.add("产品ID");
        head.add("产品名称");
        head.add("规格型号");
        head.add("批次号");
        head.add("生产日期");
        head.add("失效日期");
        head.add("订单数量");
        head.add("实际数量");
        excelData.add(head);
        //选中导出
        if (thrOrderExportRequest.getErpOrderResponses() != null && thrOrderExportRequest.getErpOrderResponses().size() > 0) {
            List<ErpOrderResponse> erpOrderResponses = thrOrderExportRequest.getErpOrderResponses();
            for (ErpOrderResponse erpOrderResponse : erpOrderResponses) {
                List<ErpOrderResponse.SubErpOrder> subErpOrders = erpOrderResponse.getSubErpOrders();
                for (ErpOrderResponse.SubErpOrder subErpOrder : subErpOrders) {
                    List<String> rows = new ArrayList<>();
                    rows.add(erpOrderResponse.getBillNo());
                    rows.add(erpOrderResponse.getBilldate());
                    rows.add(erpOrderResponse.getCorpId());
                    rows.add(erpOrderResponse.getCorpName());
                    rows.add(erpOrderResponse.getBillType());
                    rows.add(erpOrderResponse.getBillFlag());
                    rows.add(subErpOrder.getProductId());
                    rows.add(subErpOrder.getProductName());
                    rows.add(subErpOrder.getSpec());
                    rows.add(subErpOrder.getBatchNo());
                    rows.add(subErpOrder.getProductDate());
                    rows.add(subErpOrder.getExpireDate());
                    rows.add(subErpOrder.getCount() + "");
                    rows.add(subErpOrder.getReCount() + "");
                    excelData.add(rows);
                }

            }

        } else {//一键导出

            BasicThirdSysDetailEntity basicThirdSysDetailEntity = basicThirdSysDetailService.selectByKey("orderQueryUrl", thrOrderExportRequest.getThirdSysFk());
            if (basicThirdSysDetailEntity == null || basicThirdSysDetailEntity.getValue() == null) {
                thrOrderExportLogEntity.setStatus(BasicProcessStatus.UDIINFO_IMPORT_FAIL);
                thrOrderExportLogEntity.setRemark("往来单位接口地址未定义");
                thrOrderExportLogService.updateThrOrderExportLog(thrOrderExportLogEntity);
                return;
            }
            if (!basicThirdSysDetailEntity.getEnabled()) {
                thrOrderExportLogEntity.setStatus(BasicProcessStatus.UDIINFO_IMPORT_FAIL);
                thrOrderExportLogEntity.setRemark("第三方往来单位服务未启用");
                thrOrderExportLogService.updateThrOrderExportLog(thrOrderExportLogEntity);
                return;
            }

            if (basicThirdSysDetailEntity.getFromType() == 0) {
                FilterThrOrderRequest filterThrOrderRequest = new FilterThrOrderRequest();
                BeanUtils.copyProperties(thrOrderExportRequest, filterThrOrderRequest);
                List<List<String>> datas = exportThrOrders(filterThrOrderRequest);
                excelData.addAll(datas);
            } else {
                FilterThrOrderRequest filterThrOrderRequest = new FilterThrOrderRequest();
                BeanUtils.copyProperties(thrOrderExportRequest, filterThrOrderRequest);
                List<List<String>> genDatas = genExcelData(filterThrOrderRequest);
                if (genDatas != null && genDatas.size() > 0) {
                    excelData.addAll(genDatas);
                }
            }

        }
        String sheetName = "业务单据";
        new ExcelUtil().exportExcel(excelData, sheetName, thrOrderExportLogEntity.getFilePath(), 20);
        thrOrderExportLogEntity.setStatus(BasicProcessStatus.UDIINFO_IMPORT_SUCCESS);
        thrOrderExportLogService.updateThrOrderExportLog(thrOrderExportLogEntity);
    }

    @Async
    public void uploadSmp(String genKey, ThrOrderExportRequest thrOrderExportRequest) {

        ThrOrderExportLogEntity thrOrderExportLogEntity = thrOrderExportLogService.selectByGenKey(genKey);
        PostThrOrderRequest postThrOrderRequest = new PostThrOrderRequest();
        //选中导出
        if (thrOrderExportRequest.getErpOrderResponses() != null && thrOrderExportRequest.getErpOrderResponses().size() > 0) {
            List<ErpOrderResponse> erpOrderResponses = thrOrderExportRequest.getErpOrderResponses();
            postThrOrderRequest.setDatas(erpOrderResponses);
        } else {//一键导出
            List<ErpOrderResponse> erpOrderResponses = new ArrayList<>();
            BasicThirdSysDetailEntity basicThirdSysDetailEntity = basicThirdSysDetailService.selectByKey("orderQueryUrl", thrOrderExportRequest.getThirdSysFk());
            if (basicThirdSysDetailEntity == null || basicThirdSysDetailEntity.getValue() == null) {
                thrOrderExportLogEntity.setStatus(BasicProcessStatus.UDIINFO_IMPORT_FAIL);
                thrOrderExportLogEntity.setRemark("往来单位接口地址未定义");
                thrOrderExportLogService.updateThrOrderExportLog(thrOrderExportLogEntity);
                return;
            }
            if (!basicThirdSysDetailEntity.getEnabled()) {
                thrOrderExportLogEntity.setStatus(BasicProcessStatus.UDIINFO_IMPORT_FAIL);
                thrOrderExportLogEntity.setRemark("第三方往来单位服务未启用");
                thrOrderExportLogService.updateThrOrderExportLog(thrOrderExportLogEntity);
                return;
            }

            if (basicThirdSysDetailEntity.getFromType() == 0) {
                FilterThrOrderRequest filterThrOrderRequest = new FilterThrOrderRequest();
                BeanUtils.copyProperties(thrOrderExportRequest, filterThrOrderRequest);
                List<ErpOrderResponse> datas = getThrOrders(filterThrOrderRequest);
                erpOrderResponses.addAll(datas);
            } else {
                FilterThrOrderRequest filterThrOrderRequest = new FilterThrOrderRequest();
                BeanUtils.copyProperties(thrOrderExportRequest, filterThrOrderRequest);
                List<ThrOrderEntity> thrOrderEntities = thrOrderService.filterThrOrder(filterThrOrderRequest);
                for (ThrOrderEntity thrOrderEntity : thrOrderEntities) {
                    ErpOrderResponse erpOrderResponse = new ErpOrderResponse();
                    BeanUtils.copyProperties(thrOrderEntity, erpOrderResponse);
                    FilterThrOrderDetailRequest filterThrOrderDetailRequest = new FilterThrOrderDetailRequest();
                    filterThrOrderDetailRequest.setOrderIdFk(thrOrderEntity.getId() + "");
                    List<ThrOrderDetailEntity> thrOrderDetailEntities = thrOrderDetailService.filterThrOrderDetailDetail(filterThrOrderDetailRequest);
                    if (thrOrderDetailEntities != null) {
                        List<ErpOrderResponse.SubErpOrder> subErpOrders = new ArrayList<>();
                        for (ThrOrderDetailEntity thrOrderDetailEntity : thrOrderDetailEntities) {
                            ErpOrderResponse.SubErpOrder subErpOrder = new ErpOrderResponse.SubErpOrder();
                            BeanUtils.copyProperties(thrOrderDetailEntity, subErpOrder);
                            subErpOrders.add(subErpOrder);
                        }
                        erpOrderResponse.setSubErpOrders(subErpOrders);
                    }
                    erpOrderResponses.add(erpOrderResponse);
                }

            }
            postThrOrderRequest.setDatas(erpOrderResponses);
        }
        //toDo 上傳SMP
        postThrOrderRequest.setGenKey(genKey);
        postThrOrderRequest.setThirdSys(thrOrderExportRequest.getThirdSysFk());
        SystemParamConfigEntity systemParamConfigEntity = systemParamConfigService.selectByParamKey("smp_service_ip");
        if (systemParamConfigEntity != null) {
            String response = HttpClient.uCloudPost(systemParamConfigEntity.getParamValue() + "/udiwms/thrsys/postOrderDetail", postThrOrderRequest);
            BaseResponse baseResponse = JSONObject.parseObject(response, new TypeReference<BaseResponse>() {
            });
            if (baseResponse.getCode() == 20000) {
                thrOrderExportLogEntity.setStatus(BasicProcessStatus.UDIINFO_IMPORT_SUCCESS);
            } else {
                thrOrderExportLogEntity.setStatus(BasicProcessStatus.UDIINFO_IMPORT_FAIL);
            }
        } else {
            thrOrderExportLogEntity.setRemark("SMP服务IP地址未定义！");
            thrOrderExportLogEntity.setStatus(BasicProcessStatus.UDIINFO_IMPORT_FAIL);
        }

        thrOrderExportLogEntity.setStatus(BasicProcessStatus.UDIINFO_IMPORT_SUCCESS);
        thrOrderExportLogService.updateThrOrderExportLog(thrOrderExportLogEntity);
    }


    public int getOrders(int page, int limit, String orderUrl, String thirdSys, ThrOrderImportLogEntity thrOrderImportLogEntity, String action, FilterThrOrderRequest filterThrOrderRequest) {
        if (filterThrOrderRequest == null)
            filterThrOrderRequest = new FilterThrOrderRequest();
        filterThrOrderRequest.setPage(page);
        filterThrOrderRequest.setLimit(limit);
        filterThrOrderRequest.setBillAction(action);
        FilterOrderRequest filterOrderRequest = new FilterOrderRequest();
        BeanUtils.copyProperties(filterThrOrderRequest,filterOrderRequest);
        BaseResponse<PageSimpleResponse<ErpOrderResponse>> responseBaseResponse = new ErpOrderClient(orderUrl).getErpOrderResponse(filterOrderRequest);
        if (responseBaseResponse != null && responseBaseResponse.getCode() == 20000) {
            List<ErpOrderResponse> erpOrderResponses = responseBaseResponse.getData().getList();
            if (erpOrderResponses != null && erpOrderResponses.size() > 0) {
                for (ErpOrderResponse erpOrderResponse : erpOrderResponses) {
                    ThrOrderEntity thrOrderEntity = new ThrOrderEntity();
                    BeanUtils.copyProperties(erpOrderResponse, thrOrderEntity);
                    thrOrderEntity.setThirdSysFk(thirdSys);
                    thrOrderEntity.setUpdateTime(new Date());
                    thrOrderService.insertThrOrder(thrOrderEntity);
//                    thrOrderEntity = thrOrderService.findByUnique(thrOrderEntity.getBillNo(), thrOrderEntity.getThirdSysFk());
                    List<ThrOrderImportDetailEntity> thrOrderDetailEntities = new ArrayList<>();
                    for (ErpOrderResponse.SubErpOrder subPurchase : erpOrderResponse.getSubErpOrders()) {
                        ThrOrderImportDetailEntity thrOrderDetailEntity = new ThrOrderImportDetailEntity();
                        BeanUtils.copyProperties(subPurchase, thrOrderDetailEntity);
                        BeanUtils.copyProperties(erpOrderResponse, thrOrderDetailEntity);
                        thrOrderDetailEntity.setGenKeyFk(thrOrderImportLogEntity.getGenKey());
                        thrOrderDetailEntity.setThirdSysFk(thirdSys);
                        thrOrderDetailEntity.setUpdateTime(new Date());
                        thrOrderDetailEntities.add(thrOrderDetailEntity);
                    }
                    thrOrderImportDetailService.insertOrderImportDetails(thrOrderDetailEntities);
                }
                page++;
                if (page * limit < responseBaseResponse.getData().getTotal()) {
                    return page;
                }
            }
        } else {
            thrOrderImportLogEntity.setStatus(BasicProcessStatus.UDIINFO_IMPORT_FAIL);
            if (responseBaseResponse != null) {
                thrOrderImportLogEntity.setRemark(responseBaseResponse.getMessage());
            } else {
                thrOrderImportLogEntity.setRemark("第三方服务出错！");
            }
            thrOrderImportLogService.updateImportLog(thrOrderImportLogEntity);
            return -1;
        }
        return -1;
    }

    public List<List<String>> exportThrOrders(FilterThrOrderRequest filterThrOrderRequest) {
        BasicThirdSysDetailEntity basicThirdSysDetailEntity = basicThirdSysDetailService.selectByKey("orderQueryUrl", filterThrOrderRequest.getThirdSysFk());
        int page = 1;
        int limit = 100;
        List<ErpOrderResponse> erpOrderResponseList = new ArrayList<>();
        while (true) {
            List<ErpOrderResponse> datas = getThrOrders(page, limit, basicThirdSysDetailEntity.getValue(), filterThrOrderRequest);
            if (datas != null && datas.size() >= limit) {
                erpOrderResponseList.addAll(datas);
                page++;

            } else {
                if (datas != null) {
                    erpOrderResponseList.addAll(datas);
                }
                break;
            }
        }
        List<List<String>> excelData = new ArrayList<>();
        for (ErpOrderResponse erpOrderResponse : erpOrderResponseList) {
            for (ErpOrderResponse.SubErpOrder subPurchase : erpOrderResponse.getSubErpOrders()) {
                List<String> rows = new ArrayList<>();
                rows.add(erpOrderResponse.getBillNo());
                rows.add(erpOrderResponse.getBilldate());
                rows.add(erpOrderResponse.getCorpId());
                rows.add(erpOrderResponse.getCorpName());
                rows.add(erpOrderResponse.getBillType());
                rows.add(erpOrderResponse.getBillFlag());
                rows.add(subPurchase.getProductId());
                rows.add(subPurchase.getProductName());
                rows.add(subPurchase.getProductDate());
                rows.add(subPurchase.getExpireDate());
                rows.add(subPurchase.getCount() + "");
                rows.add(subPurchase.getReCount() + "");
                excelData.add(rows);
            }
        }
        return excelData;
    }

    public List<ErpOrderResponse> getThrOrders(FilterThrOrderRequest filterThrOrderRequest) {
        BasicThirdSysDetailEntity basicThirdSysDetailEntity = basicThirdSysDetailService.selectByKey("orderQueryUrl", filterThrOrderRequest.getThirdSysFk());
        int page = 1;
        int limit = 100;
        List<ErpOrderResponse> erpOrderResponseList = new ArrayList<>();
        while (true) {
            List<ErpOrderResponse> datas = getThrOrders(page, limit, basicThirdSysDetailEntity.getValue(), filterThrOrderRequest);
            if (datas != null && datas.size() >= limit) {
                erpOrderResponseList.addAll(datas);
                page++;

            } else {
                if (datas != null) {
                    erpOrderResponseList.addAll(datas);
                }
                break;
            }
        }
        return erpOrderResponseList;
    }

    public List<ErpOrderResponse> getThrOrders(int page, int limit, String orderQueryUrl, FilterThrOrderRequest filterThrOrderRequest) {
        FilterOrderRequest filterOrderRequest = new FilterOrderRequest();
        BeanUtils.copyProperties(filterThrOrderRequest, filterOrderRequest);
        filterOrderRequest.setPage(page);
        filterOrderRequest.setLimit(limit);
        BaseResponse<PageSimpleResponse<ErpOrderResponse>> responseBaseResponse = new ErpOrderClient(orderQueryUrl).getErpOrderResponse(filterOrderRequest);
        if (responseBaseResponse != null && responseBaseResponse.getCode() == 20000) {

        }
        List<ErpOrderResponse> erpOrderResponses = responseBaseResponse.getData().getList();
        if (erpOrderResponses != null && erpOrderResponses.size() > 0) {
            return erpOrderResponses;
        }
        return null;
    }

    public List<List<String>> genExcelData(FilterThrOrderRequest filterThrOrderRequest) {

        List<List<String>> excelData = new ArrayList<>();
        List<ThrOrderEntity> thrOrderEntities = thrOrderService.filterThrOrder(filterThrOrderRequest);
        for (ThrOrderEntity thrOrderEntity : thrOrderEntities) {
            FilterThrOrderDetailRequest filterThrOrderDetailRequest = new FilterThrOrderDetailRequest();
            filterThrOrderDetailRequest.setOrderIdFk(thrOrderEntity.getId() + "");
            List<ThrOrderDetailEntity> thrOrderDetailEntities = thrOrderDetailService.filterThrOrderDetailDetail(filterThrOrderDetailRequest);
            if (thrOrderDetailEntities != null) {
                for (ThrOrderDetailEntity thrOrderDetailEntity : thrOrderDetailEntities) {
                    List<String> rows = new ArrayList<>();

                    rows.add(thrOrderEntity.getBillNo());
                    rows.add(thrOrderEntity.getBilldate());
                    rows.add(thrOrderEntity.getCorpId());
                    rows.add(thrOrderEntity.getCorpName());
                    rows.add(thrOrderEntity.getBillType());
                    rows.add(thrOrderEntity.getBillFlag());
                    rows.add(thrOrderDetailEntity.getProductId());
                    rows.add(thrOrderDetailEntity.getProductName());
                    rows.add(thrOrderDetailEntity.getProductDate());
                    rows.add(thrOrderDetailEntity.getExpireDate());
                    rows.add(thrOrderDetailEntity.getCount() + "");
                    rows.add(thrOrderDetailEntity.getReCount() + "");
                    excelData.add(rows);
                }
            }

        }
        return excelData;
    }
}
