package com.glxp.api.admin.thread;

import com.alibaba.fastjson.JSONObject;
import com.alibaba.fastjson.TypeReference;
import com.glxp.api.admin.config.WebSocketServer;
import com.glxp.api.admin.constant.BasicProcessStatus;
import com.glxp.api.admin.entity.basic.BasicThirdSysDetailEntity;
import com.glxp.api.admin.entity.info.SystemParamConfigEntity;
import com.glxp.api.admin.entity.thrsys.*;
import com.glxp.api.admin.httpclient.ErpBasicClient;
import com.glxp.api.admin.req.basic.FilterErpGoodsRequest;
import com.glxp.api.admin.req.basic.FilterUdiIpLogRequest;
import com.glxp.api.admin.req.thrsys.FilterThrProductsRequest;
import com.glxp.api.admin.req.thrsys.PostThrProductsRequest;
import com.glxp.api.admin.req.thrsys.ThrProductsExportRequest;
import com.glxp.api.admin.res.PageSimpleResponse;
import com.glxp.api.admin.res.basic.ErpProductsResponse;
import com.glxp.api.admin.service.basic.BasicThirdSysDetailService;
import com.glxp.api.admin.service.info.SystemParamConfigService;
import com.glxp.api.admin.service.thrsys.ThrProductsExportLogService;
import com.glxp.api.admin.service.thrsys.ThrProductsImportDetailService;
import com.glxp.api.admin.service.thrsys.ThrProductsImportLogService;
import com.glxp.api.admin.service.thrsys.ThrProductsService;
import com.glxp.api.admin.util.Constant;
import com.glxp.api.admin.util.ExcelUtil;
import com.glxp.api.admin.util.HttpClient;
import com.glxp.api.admin.util.RedisUtil;
import com.glxp.api.common.res.BaseResponse;
import org.springframework.beans.BeanUtils;
import org.springframework.scheduling.annotation.Async;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.stream.Collectors;

@Service
public class ThrProductsDlService {
    @Resource
    ThrProductsService thrProductsService;
    @Resource
    BasicThirdSysDetailService basicThirdSysDetailService;
    @Resource
    RedisUtil redisUtil;
    @Resource
    ThrProductsImportLogService thrProductsImportLogService;
    @Resource
    ThrProductsImportDetailService thrProductsImportDetailService;
    @Resource
    ThrProductsExportLogService thrProductsExportLogService;
    @Resource
    SystemParamConfigService systemParamConfigService;

    @Async
    public void importSelectProducrs(String genKey, List<ThrProductsEntity> thrProductsEntities, String thirdSys) {
        ThrProductsImportLogEntity thrProductsImportLogEntity = thrProductsImportLogService.selectByGenKey(genKey);
        if (thrProductsEntities != null && thrProductsEntities.size() > 0) {
            List<ThrProductsImportDetailEntity> thrProductsImportDetailEntities;
            thrProductsImportDetailEntities = thrProductsEntities.stream().map(
                    item -> {
                        ThrProductsImportDetailEntity thrProductsImportDetailEntity = new ThrProductsImportDetailEntity();
                        BeanUtils.copyProperties(item, thrProductsImportDetailEntity);
                        thrProductsImportDetailEntity.setThirdSysFk(thirdSys);
                        thrProductsImportDetailEntity.setGenKeyFk(thrProductsImportLogEntity.getGenKey());
                        return thrProductsImportDetailEntity;
                    }
            ).collect(Collectors.toList());
            thrProductsImportDetailService.insertProductsDetailImports(thrProductsImportDetailEntities);
            FilterUdiIpLogRequest filterUdiIpLogRequest = new FilterUdiIpLogRequest();
            filterUdiIpLogRequest.setGenKey(genKey);
            List<ThrProductsImportDetailEntity> thrProductsImportDetailEntities1 =
                    thrProductsImportDetailService.filterProductsDetailImport(filterUdiIpLogRequest);
            List<ThrProductsEntity> thrProductsEntities1 = thrProductsImportDetailEntities1.stream().map(
                    item -> {
                        ThrProductsEntity thrProductsEntity = new ThrProductsEntity();
                        BeanUtils.copyProperties(item, thrProductsEntity);
                        thrProductsEntity.setUpdateTime(new Date());
                        return thrProductsEntity;
                    }
            ).collect(Collectors.toList());
            thrProductsService.insertThrProductss(thrProductsEntities1);

            redisUtil.set(Constant.dlThrProducts, "false");
            thrProductsImportLogEntity.setStatus(BasicProcessStatus.UDIINFO_IMPORT_SUCCESS);
            thrProductsImportLogService.updateImportLog(thrProductsImportLogEntity);
            WebSocketServer.sendInfo("产品信息下载已完成，请刷新查看！", "sid");
        }
    }

    //主动下载产品信息
    @Async
    public void importProducrs(String genKey, FilterThrProductsRequest filterThrProductsRequest) {
        ThrProductsImportLogEntity thrProductsImportLogEntity = thrProductsImportLogService.selectByGenKey(genKey);
        BasicThirdSysDetailEntity piDetailEntity = basicThirdSysDetailService.selectByKey("piQueryUrl", filterThrProductsRequest.getThirdSysFk());
        if (piDetailEntity == null || piDetailEntity.getValue() == null) {
            thrProductsImportLogEntity.setStatus(BasicProcessStatus.UDIINFO_IMPORT_FAIL);
            thrProductsImportLogEntity.setRemark("产品信息接口未设置");
            thrProductsImportLogService.updateImportLog(thrProductsImportLogEntity);
            WebSocketServer.sendInfo("产品信息接口未设置！", "sid");
            return;
        }

        int page = 1;
        int limit = 100;
        while (page != -1) {
            FilterErpGoodsRequest filterErpGoodsRequest = new FilterErpGoodsRequest();
            BeanUtils.copyProperties(filterThrProductsRequest, filterErpGoodsRequest);
            page = getInvmandoc(thrProductsImportLogEntity, page, limit, piDetailEntity.getValue(), filterErpGoodsRequest.getThirdSys(), filterErpGoodsRequest);
        }
        FilterUdiIpLogRequest filterUdiIpLogRequest = new FilterUdiIpLogRequest();
        filterUdiIpLogRequest.setGenKey(genKey);
        List<ThrProductsImportDetailEntity> thrProductsImportDetailEntities =
                thrProductsImportDetailService.filterProductsDetailImport(filterUdiIpLogRequest);
        List<ThrProductsEntity> thrProductsEntities = thrProductsImportDetailEntities.stream().map(
                item -> {
                    ThrProductsEntity thrProductsEntity = new ThrProductsEntity();
                    BeanUtils.copyProperties(item, thrProductsEntity);
                    thrProductsEntity.setUpdateTime(new Date());
                    return thrProductsEntity;
                }
        ).collect(Collectors.toList());
        thrProductsService.insertThrProductss(thrProductsEntities);

        redisUtil.set(Constant.dlThrProducts, "false");
        thrProductsImportLogEntity.setStatus(BasicProcessStatus.UDIINFO_IMPORT_SUCCESS);
        thrProductsImportLogService.updateImportLog(thrProductsImportLogEntity);
        WebSocketServer.sendInfo("产品信息下载已完成，请刷新查看！", "sid");
    }

    //导出生成Excel文件
    @Async
    public void genExcel(String genKey, ThrProductsExportRequest thrProductsExportRequest) {

        ThrProductsExportLogEntity thrProductsExportLogEntity = thrProductsExportLogService.selectByGenKey(genKey);

        List<List<String>> excelData = new ArrayList<>();
        List<String> head = new ArrayList<>();
        head.add("产品ID");
        head.add("产品名称");
        head.add("计量单位");
        head.add("规格型号");
        head.add("注册证号");
        head.add("生产厂家");
        excelData.add(head);
        //选中导出
        if (thrProductsExportRequest.getThrProductsEntities() != null && thrProductsExportRequest.getThrProductsEntities().size() > 0) {
            List<ThrProductsEntity> thrProductsEntities = thrProductsExportRequest.getThrProductsEntities();
            excelData.addAll(getRows(thrProductsEntities));

        } else {//一键导出

            BasicThirdSysDetailEntity basicThirdSysDetailEntity = basicThirdSysDetailService.selectByKey("piQueryUrl", thrProductsExportRequest.getThirdSys());
            if (basicThirdSysDetailEntity == null || basicThirdSysDetailEntity.getValue() == null) {
                thrProductsExportLogEntity.setStatus(BasicProcessStatus.UDIINFO_IMPORT_FAIL);
                thrProductsExportLogEntity.setRemark("产品信息接口未定义！");
                thrProductsExportLogService.updateThrProductsExportLog(thrProductsExportLogEntity);
                return;
            }

            if (!basicThirdSysDetailEntity.getEnabled()) {
                thrProductsExportLogEntity.setStatus(BasicProcessStatus.UDIINFO_IMPORT_FAIL);
                thrProductsExportLogEntity.setRemark("第三方产品信息服务未启用！");
                thrProductsExportLogService.updateThrProductsExportLog(thrProductsExportLogEntity);
                return;
            }

            if (basicThirdSysDetailEntity.getFromType() == 0) {
                FilterThrProductsRequest filterThrProductsRequest = new FilterThrProductsRequest();
                BeanUtils.copyProperties(thrProductsExportRequest, filterThrProductsRequest);
                List<ThrProductsEntity> thrProductsEntities = exportThrProducts(filterThrProductsRequest);
                excelData.addAll(getRows(thrProductsEntities));

            } else {
                FilterThrProductsRequest filterThrInvProductsRequest = new FilterThrProductsRequest();
                BeanUtils.copyProperties(thrProductsExportRequest, filterThrInvProductsRequest);
                List<List<String>> genDatas = genExcelData(null, filterThrInvProductsRequest);
                if (genDatas != null && genDatas.size() > 0) {
                    excelData.addAll(genDatas);
                }
            }
        }
        String sheetName = "产品信息";
        new ExcelUtil().exportExcel(excelData, sheetName, thrProductsExportLogEntity.getFilePath(), 20);
        thrProductsExportLogEntity.setStatus(BasicProcessStatus.UDIINFO_IMPORT_SUCCESS);
        thrProductsExportLogService.updateThrProductsExportLog(thrProductsExportLogEntity);

    }

    //上传SMP
    @Async
    public void uploadSmp(String genKey, ThrProductsExportRequest thrProductsExportRequest) {

        PostThrProductsRequest postThrProductsRequest = new PostThrProductsRequest();
        List<ErpProductsResponse> datas = new ArrayList<>();
        ThrProductsExportLogEntity thrProductsExportLogEntity = thrProductsExportLogService.selectByGenKey(genKey);

        //选中导出
        if (thrProductsExportRequest.getThrProductsEntities() != null && thrProductsExportRequest.getThrProductsEntities().size() > 0) {
            List<ThrProductsEntity> thrProductsEntities = thrProductsExportRequest.getThrProductsEntities();
            List<ErpProductsResponse> lists = thrProductsEntities.stream().map(
                    item -> {
                        ErpProductsResponse thrProductsEntity = new ErpProductsResponse();
                        BeanUtils.copyProperties(item, thrProductsEntity);
                        return thrProductsEntity;
                    }
            ).collect(Collectors.toList());
            datas.addAll(lists);
            postThrProductsRequest.setDatas(datas);
            postThrProductsRequest.setGenKey(genKey);
            postThrProductsRequest.setThirdSys(thrProductsExportRequest.getThirdSys());
            postThrProductsRequest.setUploadType("接口上传");


        } else {//一键导出

            BasicThirdSysDetailEntity basicThirdSysDetailEntity = basicThirdSysDetailService.selectByKey("piQueryUrl", thrProductsExportRequest.getThirdSys());
            if (basicThirdSysDetailEntity == null || basicThirdSysDetailEntity.getValue() == null) {
                thrProductsExportLogEntity.setStatus(BasicProcessStatus.UDIINFO_IMPORT_FAIL);
                thrProductsExportLogEntity.setRemark("产品信息接口未定义！");
                thrProductsExportLogService.updateThrProductsExportLog(thrProductsExportLogEntity);
                return;
            }

            if (!basicThirdSysDetailEntity.getEnabled()) {
                thrProductsExportLogEntity.setStatus(BasicProcessStatus.UDIINFO_IMPORT_FAIL);
                thrProductsExportLogEntity.setRemark("第三方产品信息服务未启用！");
                thrProductsExportLogService.updateThrProductsExportLog(thrProductsExportLogEntity);
                return;
            }
            List<ThrProductsEntity> thrProductsEntities;
            if (basicThirdSysDetailEntity.getFromType() == 0) {
                FilterThrProductsRequest filterThrProductsRequest = new FilterThrProductsRequest();
                BeanUtils.copyProperties(thrProductsExportRequest, filterThrProductsRequest);
                thrProductsEntities = exportThrProducts(filterThrProductsRequest);
            } else {
                FilterThrProductsRequest filterThrInvProductsRequest = new FilterThrProductsRequest();
                thrProductsEntities = thrProductsService.filterThrProductsRequest(filterThrInvProductsRequest);
            }
            if (thrProductsEntities != null) {
                List<ErpProductsResponse> lists = thrProductsEntities.stream().map(
                        item -> {
                            ErpProductsResponse thrProductsEntity = new ErpProductsResponse();
                            BeanUtils.copyProperties(item, thrProductsEntity);
                            return thrProductsEntity;
                        }
                ).collect(Collectors.toList());
                datas.addAll(lists);
                postThrProductsRequest.setDatas(datas);
                postThrProductsRequest.setGenKey(genKey);
                postThrProductsRequest.setThirdSys(thrProductsExportRequest.getThirdSys());
                postThrProductsRequest.setUploadType("接口上传");
            }


        }

        //Todo  接口上传SMP
        SystemParamConfigEntity systemParamConfigEntity = systemParamConfigService.selectByParamKey("smp_service_ip");
        if (systemParamConfigEntity != null) {
            String response = HttpClient.uCloudPost(systemParamConfigEntity.getParamValue() + "/udiwms/thrsys/postThrProducts", postThrProductsRequest);
            BaseResponse baseResponse = JSONObject.parseObject(response, new TypeReference<BaseResponse>() {
            });
            if (baseResponse.getCode() == 20000) {
                thrProductsExportLogEntity.setStatus(BasicProcessStatus.UDIINFO_IMPORT_SUCCESS);
            } else {
                thrProductsExportLogEntity.setStatus(BasicProcessStatus.UDIINFO_IMPORT_FAIL);
            }
        } else {
            thrProductsExportLogEntity.setRemark("SMP服务IP地址未定义！");
            thrProductsExportLogEntity.setStatus(BasicProcessStatus.UDIINFO_IMPORT_FAIL);
        }

        thrProductsExportLogEntity.setStatus(BasicProcessStatus.UDIINFO_IMPORT_SUCCESS);
        thrProductsExportLogService.updateThrProductsExportLog(thrProductsExportLogEntity);
    }

    // //接收第三方上传或文件导入数据
    @Async
    public void insertData(String genKey, List<ErpProductsResponse> erpProductsResponses, String thirdSys) {
        ThrProductsImportLogEntity thrProductsImportLogEntity = thrProductsImportLogService.selectByGenKey(genKey);
        if (erpProductsResponses != null && erpProductsResponses.size() > 0) {
            List<ThrProductsImportDetailEntity> thrProductsImportDetailEntities;
            thrProductsImportDetailEntities = erpProductsResponses.stream().map(
                    item -> {
                        ThrProductsImportDetailEntity thrProductsImportDetailEntity = new ThrProductsImportDetailEntity();
                        BeanUtils.copyProperties(item, thrProductsImportDetailEntity);
                        thrProductsImportDetailEntity.setThirdSysFk(thirdSys);
                        thrProductsImportDetailEntity.setGenKeyFk(genKey);
                        return thrProductsImportDetailEntity;
                    }
            ).collect(Collectors.toList());
            thrProductsImportDetailService.insertProductsDetailImports(thrProductsImportDetailEntities);

            List<ThrProductsEntity> thrProductsEntities;
            thrProductsEntities = thrProductsImportDetailEntities.stream().map(
                    item -> {
                        ThrProductsEntity thrProductsEntity = new ThrProductsEntity();
                        BeanUtils.copyProperties(item, thrProductsEntity);
                        thrProductsEntity.setUpdateTime(new Date());
                        return thrProductsEntity;
                    }
            ).collect(Collectors.toList());
            thrProductsService.insertThrProductss(thrProductsEntities);
            thrProductsImportLogEntity.setStatus(BasicProcessStatus.UDIINFO_IMPORT_SUCCESS);
            thrProductsImportLogEntity.setUpdateTime(new Date());
            thrProductsImportLogService.updateImportLog(thrProductsImportLogEntity);
        } else {
            thrProductsImportLogEntity.setRemark("未选择产品信息");
            thrProductsImportLogEntity.setStatus(BasicProcessStatus.UDIINFO_IMPORT_FAIL);
            thrProductsImportLogService.updateImportLog(thrProductsImportLogEntity);
        }
    }

    public int getInvmandoc(ThrProductsImportLogEntity thrProductsImportLogEntity, int page, int limit, String
            productUrl, String thirdSys, FilterErpGoodsRequest filterErpGoodsRequest) {

        filterErpGoodsRequest.setPage(page);
        filterErpGoodsRequest.setLimit(limit);
        BaseResponse<PageSimpleResponse<ErpProductsResponse>> erpProducts = new ErpBasicClient().getErpProducts(filterErpGoodsRequest, productUrl);
        if (erpProducts != null && erpProducts.getCode() == 20000) {
            List<ErpProductsResponse> erpProductsResponses = erpProducts.getData().getList();
            if (erpProductsResponses != null && erpProductsResponses.size() > 0) {
                List<ThrProductsImportDetailEntity> thrProductsImportDetailEntities;
                thrProductsImportDetailEntities = erpProductsResponses.stream().map(
                        item -> {
                            ThrProductsImportDetailEntity thrProductsImportDetailEntity = new ThrProductsImportDetailEntity();
                            BeanUtils.copyProperties(item, thrProductsImportDetailEntity);
                            thrProductsImportDetailEntity.setThirdSysFk(thirdSys);
                            thrProductsImportDetailEntity.setGenKeyFk(thrProductsImportLogEntity.getGenKey());
                            return thrProductsImportDetailEntity;
                        }
                ).collect(Collectors.toList());
                thrProductsImportDetailService.insertProductsDetailImports(thrProductsImportDetailEntities);
                page++;
                if (page * limit < erpProducts.getData().getTotal()) {
                    return page;
                }
            } else {
                return -1;
            }
        } else {
            thrProductsImportLogEntity.setRemark(erpProducts.getMessage());
            thrProductsImportLogEntity.setStatus(BasicProcessStatus.UDIINFO_IMPORT_FAIL);
            thrProductsImportLogService.updateImportLog(thrProductsImportLogEntity);
            return -1;
        }
        return -1;
    }

    public List<ThrProductsEntity> exportThrProducts(FilterThrProductsRequest filterThrProductsRequest) {
        BasicThirdSysDetailEntity basicThirdSysDetailEntity = basicThirdSysDetailService.selectByKey("piQueryUrl", filterThrProductsRequest.getThirdSys());
        int page = 1;
        int limit = 100;
        List<ThrProductsEntity> thrProductsEntities = new ArrayList<>();
        while (true) {
            List<ThrProductsEntity> datas = getThrProducrts(page, limit, basicThirdSysDetailEntity.getValue(), filterThrProductsRequest);
            if (datas != null && datas.size() >= limit) {
                thrProductsEntities.addAll(datas);
                page++;

            } else {
                if (datas != null) {
                    thrProductsEntities.addAll(datas);
                }
                break;
            }

        }
        return thrProductsEntities;
    }

    public List<ThrProductsEntity> getThrProducrts(int page, int limit, String
            productUrl, FilterThrProductsRequest filterThrProductsRequest) {

        FilterErpGoodsRequest filterErpGoodsRequest = new FilterErpGoodsRequest();
        BeanUtils.copyProperties(filterThrProductsRequest, filterErpGoodsRequest);
        filterErpGoodsRequest.setPage(page);
        filterErpGoodsRequest.setLimit(limit);
        BaseResponse<PageSimpleResponse<ErpProductsResponse>> erpProducts = new ErpBasicClient().getErpProducts(filterErpGoodsRequest, productUrl);
        if (erpProducts != null && erpProducts.getCode() == 20000) {
            List<ErpProductsResponse> erpProductsResponses = erpProducts.getData().getList();
            if (erpProductsResponses != null && erpProductsResponses.size() > 0) {
                List<ThrProductsEntity> thrProductsEntities = erpProductsResponses.stream().map(
                        item -> {
                            ThrProductsEntity thrProductsEntity = new ThrProductsEntity();
                            BeanUtils.copyProperties(item, thrProductsEntity);
                            return thrProductsEntity;
                        }
                ).collect(Collectors.toList());
                return thrProductsEntities;
            }
        }

        return null;
    }

    public List<List<String>> genExcelData(String id, FilterThrProductsRequest filterThrProductsRequest) {

        List<List<String>> excelData = new ArrayList<>();
        List<ThrProductsEntity> thrProductsEntities = new ArrayList<>();
        if (id != null) {
            ThrProductsEntity thrProductsEntity = thrProductsService.selectById(id);
            thrProductsEntities.add(thrProductsEntity);
        } else {
            thrProductsEntities = thrProductsService.filterThrProductsRequest(filterThrProductsRequest);
        }
        excelData.addAll(getRows(thrProductsEntities));
        return excelData;
    }

    public List<List<String>> getRows(List<ThrProductsEntity> thrProductsEntities) {
        List<List<String>> excelData = new ArrayList<>();
        for (ThrProductsEntity thrProductsEntity : thrProductsEntities) {
            List<String> rows = new ArrayList<>();
            rows.add(thrProductsEntity.getCode());
            rows.add(thrProductsEntity.getName());
            rows.add(thrProductsEntity.getMeasname() + "");
            rows.add(thrProductsEntity.getSpec());
            rows.add(thrProductsEntity.getRegisterNo());
            rows.add(thrProductsEntity.getManufactory());
            excelData.add(rows);
        }
        return excelData;
    }
}
